package forge.cfg;

import java.util.Collections;
import java.util.Set;

import edu.mit.csail.sdg.annotations.Returns;
import forge.program.ForgeExpression;
import forge.program.ForgeVariable;
import forge.program.ForgeExpression.Modifiable;
import forge.util.ExpressionUtil;

/**
 * A stmt which assigns an expression to a variable (local or field).
 * <p>
 * AssignStmt ::= ForgeVariable = ForgeExpression
 * 
 * @specfield var: ForgeVariable
 * @specfield expr: ForgeExpression
 * 
 * @author Greg Dennis (gdennis@mit.edu)
 */
public final class AssignStmt extends UpdateStmt {

	private final ForgeVariable var;
	private final ForgeExpression expr;
	private Set<Modifiable> refs;
	
	// derived
	private final Set<ForgeVariable> modified;
	
	AssignStmt(ForgeCFG cfg, ForgeVariable var, ForgeExpression expr) {
		super(cfg);
		program().checkProgram(var);
		program().checkProgram(expr);
		StoreException.checkStore(expr, var);
		this.var = var;
		this.expr = expr;
		modified = Collections.singleton(var);
	}
	
	@Returns("this.var")
	public ForgeVariable var() {
		return var;
	}
	
	@Returns("this.expr")
	public ForgeExpression expr() {
		return expr;
	}
	
	public Set<Modifiable> referenced() {
		if (refs == null) {
			refs = ExpressionUtil.findModifiables(expr);
		}
		return refs;
	}
	
	public Set<ForgeVariable> modified() {
		return modified;
	}
	
	@Override
	public void accept(CFGVisitor visitor) {
		visitor.visit(this);
	}
	
	void appendUpdate(StringBuilder sb) {
		sb.append(var).append(" := ").append(expr);
	}
	
}
