package forge.cfg;


/**
 * A visitor to a stmt in a CFG. The result of visiting each s can be
 * cached. If a result for a node has been cached, then the visit method on
 * that node is not called.
 *
 * @specfield cache: CFGNode -> lone T
 *
 * @author Greg Dennis (gdennis@mit.edu)
 */
public interface CFGVisitor {
	
	/**
	 * Visits an AssignStmt in the CFG.
	 */
	public void visit(AssignStmt stmt);
	
	/**
	 * Visits a CallStmt in the CFG.
	 */
	public void visit(CallStmt stmt);
	
	/**
	 * Visits a CreateStmt in the CFG.
	 */
	public void visit(CreateStmt stmt);
	
	/**
	 * Visits a BranchStmt in the CFG.
	 */
	public void visit(BranchStmt stmt);
	
	/**
	 * Visits a SpecStmt in the CFG.
	 */
	public void visit(SpecStmt stmt);
	
	/**
	 * Visits a ExitStmt in the CFG.
	 */
	public void visit(ExitStmt stmt);

}
