package forge.cfg;

import java.util.Collections;
import java.util.LinkedHashSet;
import java.util.Set;

import edu.mit.csail.sdg.annotations.Returns;
import forge.program.ForgeVariable;
import forge.program.InstanceDomain;
import forge.program.ForgeExpression.Modifiable;

/**
 * A stmt which constructs a new instance of a class and assigns that instance
 * to a local variable.
 * <p>
 * CreateStmt ::= ForgeVariable = new InstanceDomain
 * 
 * @specfield left : ForgeVariable
 * @specfield type: InstanceDomain
 *
 * @author Greg Dennis (gdennis@mit.edu)
 */
public final class CreateStmt extends UpdateStmt {

	private final ForgeVariable var;
	private final InstanceDomain type;
	
	// derived
	private final Set<Modifiable> modified, referenced;
	
	CreateStmt(ForgeCFG cfg, ForgeVariable var, InstanceDomain type) {
		super(cfg);
		program().checkProgram(var);
		program().checkProgram(type);
		StoreException.checkStore(type, var);
		this.var = var;
		this.type = type;
		Set<Modifiable> mods = new LinkedHashSet<Modifiable>(2);
		mods.add(var);
		mods.add(type);
		modified = Collections.unmodifiableSet(mods);
		referenced = Collections.<Modifiable>singleton(type);
	}
	
	@Returns("this.left")
	public ForgeVariable var() {
		return var;
	}
	
	@Returns("this.type")
	public InstanceDomain type() {
		return type;
	}
	
	@Override
	public Set<Modifiable> modified() {
		return modified;
	}
	
	@Override
	public Set<Modifiable> referenced() {
		return referenced;
	}

	@Override
	public void accept(CFGVisitor visitor) {
		visitor.visit(this);
	}
	
	void appendUpdate(StringBuilder sb) {
		sb.append(var).append(" := new ").append(type);
	}
	
}
