package forge.cfg;

import edu.mit.csail.sdg.annotations.Effects;
import edu.mit.csail.sdg.annotations.Modifies;
import edu.mit.csail.sdg.annotations.SpecField;


/**
 * A terminal stmt of a control flow graph.
 *
 * @author Greg Dennis (gdennis@mit.edu)
 */
@SpecField("succs : set CFGStmt | succs = emptySuccs.elems")
public final class ExitStmt extends CFGStmt {
	
	private final StmtSet emptySuccs;
	
	@Effects("no this.succs.elems")
	@Modifies("this.succs")
	ExitStmt(ForgeCFG cfg) {
		super(cfg);
		emptySuccs = StmtSet.empty(cfg);
	}

	@Override
	public StmtSet succs() {
		return emptySuccs;
	}
	
	@Override
	public void accept(CFGVisitor visitor) {
		visitor.visit(this);
	}
	
	void appendStmt(StringBuilder sb) {
		sb.append("exit");
	}
}
