package forge.cfg;

import java.util.Set;

import forge.program.ForgeExpression.Modifiable;

/**
 * A statement in the control-flow graph that may reference the current value
 * of modifiables.
 *
 * @author Greg Dennis (gdennis@mit.edu)
 */
public abstract class ReferenceStmt extends CFGStmt {

	ReferenceStmt(ForgeCFG cfg) {
		super(cfg);
		super.addSucc(cfg.exit(), null);
	}

	/**
	 * Returns the set of variables and instance domains referenced
	 * by the statement. The "used" modifiables, in the def-use meaning.
	 */
	public abstract Set<Modifiable> referenced();
	
	/**
	 * Returns an unmodifiable view of the successor stmts.
	 */
	public abstract StmtSet succs();
	
}
