package forge.cfg;

import forge.program.ForgeExpression;
import forge.program.ForgeType;
import forge.program.ForgeVariable;

/**
 * Checks if expression can be stored in an l-value.
 *
 * @author Greg Dennis (gdennis@mit.edu)
 */
@SuppressWarnings("serial")
public final class StoreException extends RuntimeException {
	
	private static final String NEW_LINE = System.getProperty("line.separator");
	
	/**
	 * @throws StoreException - e.type not in v.type ||
	 *                          v.function and not e.function
	 */
	static void checkStore(ForgeExpression e, ForgeVariable v) {
		checkStore(e.toString(), e.type(), v);
	}
	
	/**
	 * @throws StoreException - e.type not in v.type
	 */
	static void checkStore(ForgeType t, ForgeVariable v) {
		checkStore("expressions", t, v);
	}
	
	static void checkStore(String desc, ForgeType t, ForgeVariable v) {
		if (!t.subtypeOf(v.type())) {
			throw new StoreException(v + " of type " + v.type() + NEW_LINE +
					"cannot store " + desc + " of type " + t);
		}
	}
	
	private StoreException(String msg) {
		super(msg);
	}
	
}
