package forge.dataflow;

import edu.mit.csail.sdg.annotations.Effects;
import edu.mit.csail.sdg.annotations.Returns;

/**
 * Output data from a transfer function on a branch stmt in dataflow analysis.
 *
 * @author Greg Dennis (gdennis@mit.edu)
 */
public final class BranchData<D> {

	private final D thenData;
	private final D elseData;
	
	/**
	 * Constructs a new branch data with the specified then and else data.
	 */
	@Effects({"this.(BranchData@thenData) = thenData",
		      "this.(BranchData@elseData) = elseData"})
	public BranchData(D thenData, D elseData) {
		this.thenData = thenData;
		this.elseData = elseData;
	}
	
	/**
	 * Constructs a branch data where the then and else data are the same.
	 */
	@Effects("this.thenData = data && this.elseData = data")
	public BranchData(D data) {
		this(data, data);
	}
	
	/**
	 * Returns the then data.
	 */
	@Returns("this.thenData")
	public D thenData() {
		return thenData;
	}
	
	/**
	 * Returns the else data.
	 */
	@Returns("this.elseData")
	public D elseData() {
		return elseData;
	}
	
	public String toString() {
		return "BranchData: then = " + thenData + "; else = " + elseData;
	}
}
