package forge.dataflow;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import forge.cfg.AssignStmt;
import forge.cfg.BranchStmt;
import forge.cfg.CFGStmt;
import forge.cfg.CallStmt;
import forge.cfg.CreateStmt;
import forge.cfg.ExitStmt;
import forge.cfg.SpecStmt;

/**
 * Computes the inverse dominators for each node in the CFG.
 *
 * @author Greg Dennis (gdennis@mit.edu)
 */
public final class InverseDominatorAnalysis extends DataflowAnalysis<Set<CFGStmt>> {
	
	/** singleton. */
	public static final InverseDominatorAnalysis ANALYSIS =	new InverseDominatorAnalysis();
	
	private static final Set<CFGStmt> EMPTY_NODE_SET = Collections.emptySet();
	
	private InverseDominatorAnalysis() {
		super(true);
	}

	@Override
	public Set<CFGStmt> initial() {
		return EMPTY_NODE_SET;
	}

	@Override
	public Set<CFGStmt> transfer(AssignStmt stmt, Set<CFGStmt> input) {
		return addSet(stmt, input);
	}

	@Override
	public Set<CFGStmt> transfer(SpecStmt stmt, Set<CFGStmt> input) {
		return addSet(stmt, input);
	}

	@Override
	public BranchData<Set<CFGStmt>> transfer(BranchStmt stmt, Set<CFGStmt> input) {
		return new BranchData<Set<CFGStmt>>(addSet(stmt, input));
	}

	@Override
	public Set<CFGStmt> transfer(CallStmt stmt, Set<CFGStmt> input) {
		return addSet(stmt, input);
	}

	@Override
	public Set<CFGStmt> transfer(CreateStmt stmt, Set<CFGStmt> input) {
		return addSet(stmt, input);
	}
	
	@Override
	public Set<CFGStmt> transfer(ExitStmt stmt, Set<CFGStmt> input) {
		return addSet(stmt, input);
	}
	
	@Override
	public boolean merge(Set<CFGStmt> from, Set<CFGStmt> into) {
		return into.retainAll(from); // intersection
	}
	
	@Override
	public Set<CFGStmt> copy(Set<CFGStmt> data) {
		return new HashSet<CFGStmt>(data);
	}

	private Set<CFGStmt> addSet(CFGStmt stmt, Set<CFGStmt> input) {
		Set<CFGStmt> invDoms = new HashSet<CFGStmt>(input);
		invDoms.add(stmt);
		return invDoms;
	}
}