package forge.program;

import edu.mit.csail.sdg.annotations.Effects;
import edu.mit.csail.sdg.annotations.Modifies;
import edu.mit.csail.sdg.annotations.Returns;
import edu.mit.csail.sdg.annotations.Throws;

/**
 * A procedure in a Forge program.
 *
 * @author Greg Dennis (gdennis@mit.edu)
 */
public final class ForgeProcedure implements ProgramElement {
	
	private final ForgeProgram program;
	private final String name;
	private final LocalDecls ins, outs;
	
	/**
	 * Constructs a procedure with the given parameters and an empty body.
	 */
	@Effects("this.(ForgeProcedure@program) = program && this.(ForgeProcedure@name) = name" +
			 " && this.(ForgeProcedure@ins) = ins && this.(ForgeProcedure@outs) = outs")
	@Modifies("this.program, this.name, this.ins, this.outs")
	@Throws("NullPointerException : program = null || name = null || ins = null || outs = null")
	ForgeProcedure(ForgeProgram program, String name, LocalDecls ins, LocalDecls outs) {
		if (program == null || name == null || ins == null || outs == null) {
			throw new NullPointerException();
		}
		this.program = program;
		this.name = name;
		this.ins = ins;
		this.outs = outs;
	}
	
	@Returns("this.program")
	public ForgeProgram program() {
		return program;
	}
	
	/**
	 * Returns the name of the procedure.
	 */
	@Returns("this.name")
	public String name() {
		return name;
	}
	
	/**
	 * Returns the set of in-parameters.
	 */
	@Returns("this.ins")
	public LocalDecls inParams() {
		return ins;
	}
	
	/**
	 * Returns the set of out-parameters.
	 */
	@Returns("this.outs")
	public LocalDecls outParams() {
		return outs;
	}
	
	public String toString() {
		StringBuilder sb = new StringBuilder("proc ").append(name);
		sb.append(" (").append(ins).append(") : (").append(outs).append(")");
		return sb.toString();
	}
}
