package forge.program;

import java.util.List;
import java.util.Set;

import edu.mit.csail.sdg.annotations.Ensures;
import edu.mit.csail.sdg.annotations.Returns;
import edu.mit.csail.sdg.annotations.SpecField;

/**
 * The type of a Forge expression.
 *
 * @specfield domains: DomainType -> ... -> DomainType
 * 
 * @author Greg Dennis (gdennis@mit.edu)
 */
@SpecField("tupleTypes : set ForgeType.Tuple")
public interface ForgeType extends ForgeExpression {
	
	public boolean isDomain();
	
	@Returns("no this.tupleTypes")
	public boolean isEmpty();
	
	@Returns("this.tupleTypes in type.tupleTypes")
	public boolean subtypeOf(ForgeType type);
	
	@Ensures("return.elems = this.tupleTypes")
	public Set<? extends Tuple> tupleTypes();
	
	// projection expressions
	public ForgeType.Unary domain();
	public ForgeType.Unary range();
	public ForgeType projectType(int... columns);
	
	// binary operator expressions
	public ForgeType union(ForgeType type);
	public ForgeType difference(ForgeType type);
	public ForgeType intersection(ForgeType type);
	public ForgeType product(ForgeType type);
	public ForgeType join(ForgeType type);
	
	/**
	 * A type that is a set of classes.
	 */
	public interface Unary extends ForgeType {
		public Set<ForgeDomain> tupleTypes();
		public ForgeType.Unary union(ForgeType type);
		public ForgeType.Unary intersection(ForgeType type);
		public ForgeType.Unary difference(ForgeType type);
	}
	
	/**
	 * A type that is a tuple of classes.
	 */
	public interface Tuple extends ForgeType {
		public List<ForgeDomain> domains();
		public ForgeDomain domain();
		public ForgeDomain range();
		public ForgeType.Tuple projectType(int... columns);
		public ForgeType.Tuple product(Tuple tupleType);
	}
	
}
