package forge.program;

import edu.mit.csail.sdg.annotations.Effects;
import edu.mit.csail.sdg.annotations.Modifies;
import edu.mit.csail.sdg.annotations.Throws;

/**
 * A global or local Forge variable.
 * 
 * @author Greg Dennis (gdennis@mit.edu)
 */
public abstract class ForgeVariable extends AbstractExpression
		implements ForgeExpression.Modifiable {
	
	private final String name;
	private final ForgeType type;
	
	@Effects("this.(ForgeVariable@name) = name && this.(ForgeVariable@type) = type")
	@Modifies("this.name, this.type")
	@Throws({"NullPointerException : name == null || type == null"})
	ForgeVariable(String name, ForgeType type) {
		super(type.program());
		if (name == null) throw new NullPointerException();
		this.name = name;
		this.type = type;
	}
	
	/* inherits specification */
	public final String name() {
		return name;
	}
	
	/* inherits specification */
	public final ForgeType type() {
		return type;
	}
	
	/* inherits specification */
	public final boolean isVariable() {
		return true;
	}
	
	@Effects("return.var = this")
	public ForgeExpression old() {
		return new OldExpression(this);
	}

	/* inherits specification */
	public final <T> T accept(ExpressionVisitor<T> visitor) {
		return visitor.fullVisit(this);
	}

	/* inherits specification */
	public final String toString() {
		return name;
	}
}
