package forge.program;

import edu.mit.csail.sdg.annotations.Throws;


/**
 * Thrown when an expression is constructed from illegal arities.
 *
 * @author Greg Dennis (gdennis@mit.edu)
 */
@SuppressWarnings("serial")
public final class IllegalArityException extends RuntimeException {

	@Throws("IllegalArityException : left != right")
	static void checkMatch(int left, int right) {
		if (left != right) {
			throw new IllegalArityException("arity mismatch", left, right);
		}
	}
	
	@Throws("IllegalArityException : left = 1 && right = 1")
	static void checkJoin(int left, int right) {
		if (left == 1 && right == 1) {
			throw new IllegalArityException("illegal join arities", left, right);
		}
	}
	
	@Throws("IllegalArityException : left != 1 || right != 1")
	static void checkUnary(int left, int right) {
		if (left != 1 || right != 1) {
			throw new IllegalArityException("arities not 1", left, right);
		}
	}
	
	@Throws("IllegalArityException : arity != 1")
	static void checkUnary(int arity) {
		if (arity != 1) {
			throw new IllegalArityException("arity not 1", arity);
		}
	}
	
	@Throws("IllegalArityException : arity != 2")
	static void checkTwo(int arity) {
		if (arity != 2) {
			throw new IllegalArityException("arity not 2", arity);
		}
	}
	
	@Throws("IllegalArityException : columns.length == 0 || " +
			"(some i : int | columns[i] < 0 || columns[i] >= arity)")
	static void checkProject(int arity, int[] columns) {
		if (columns.length == 0) {
			throw new IllegalArityException("column length zero", arity);
		}
		for (int c: columns) {
			if (c < 0 || c >= arity) {
				throw new IllegalArityException(c + " not in arity", arity);
			}
		}
	}
	
	/**
	 * For a pair of arities.
	 */
	private IllegalArityException(String msg, int left, int right) {
		super(msg + ": left arity = " + left + ", right arity = " + right);
	}
	
	/**
	 * For a single arity.
	 */
	private IllegalArityException(String msg, int arity) {
		super(msg + ": arity = " + arity);
	}
	
}
