package forge.program;

import java.util.Iterator;

import edu.mit.csail.sdg.util.collections.ArrayHashSetUniqueList;
import edu.mit.csail.sdg.util.collections.SDGCollections;
import edu.mit.csail.sdg.util.collections.UniqueList;

/**
 * Interface to an ordered set of local variables
 *
 * @author Greg Dennis (gdennis@mit.edu)
 */
public interface LocalDecls extends ProgramElement {
	
	/**
	 * Returns this decl appended with the given local.
	 */
	public LocalDecls and(LocalVariable local);
	
	/**
	 * Returns the local variables in this decls.
	 */
	public UniqueList<LocalVariable> locals();
	
	/**
	 * An empty set of decls.
	 */
	static final class EmptyDecls implements LocalDecls {
		
		private static final UniqueList<LocalVariable> NO_LOCALS =
			SDGCollections.emptyUniqueList();
		
		private final ForgeProgram program;
		
		EmptyDecls(ForgeProgram program) {
			this.program = program;
		}
		
		public ForgeProgram program() {
			return program;
		}

		public LocalDecls and(LocalVariable local) {
			return local;
		}

		public UniqueList<LocalVariable> locals() {
			return NO_LOCALS;
		}
		
		public String toString() {
			return "";
		}
	}
	
	/**
	 * A set of one or more local variables.
	 */
	static final class MultiDecls implements LocalDecls {
		
		private final ForgeProgram program;
		private final UniqueList<LocalVariable> locals;
		
		MultiDecls(LocalDecls prefix, LocalVariable last) {
			program = prefix.program();
			UniqueList<LocalVariable> l = new ArrayHashSetUniqueList<LocalVariable>();
			l.addAll(prefix.locals());
			l.add(last);
			locals = SDGCollections.unmodifiableUniqueList(l);
		}
		
		public ForgeProgram program() {
			return program;
		}

		public LocalDecls and(LocalVariable local) {
			return new MultiDecls(this, local);
		}

		public UniqueList<LocalVariable> locals() {
			return locals;
		}
		
		public String toString() {
			StringBuilder sb = new StringBuilder();
			for (Iterator<LocalVariable> i = locals.iterator(); i.hasNext();) {
				sb.append(i.next());
				if (i.hasNext()) sb.append(", ");
			}
			return sb.toString();
		}
	}
}
