package forge.program;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

/**
 * An expression for a relational projection over another expression.
 *
 * @author Greg Dennis (gdennis@mit.edu)
 */
public final class ProjectionExpression extends AbstractExpression {
	
	private final ForgeExpression sub;
	private final List<Integer> columns;
	private final ForgeType type;
	
	ProjectionExpression(ForgeExpression sub, int... columns) {
		super(sub.program());
		IllegalArityException.checkProject(sub.arity(), columns);
		this.sub = sub;
		List<Integer> cols = new ArrayList<Integer>(columns.length);
		for (int i : columns) cols.add(i);
		this.columns = Collections.unmodifiableList(cols);
		type = sub.type().projectType(columns);
	}

	public boolean isFunction() {
		return false;
	}

	public ForgeType type() {
		return type;
	}
	
	public ForgeExpression sub() {
		return sub;
	}
	
	public List<Integer> columns() {
		return columns;
	}

	public <T> T accept(ExpressionVisitor<T> visitor) {
		return visitor.fullVisit(this);
	}
	
	public String toString() {
		StringBuffer sb = new StringBuffer(sub.toString());
		sb.append("<");
		for (Iterator<Integer> i = columns.iterator(); i.hasNext();) {
			sb.append(i.next());
			if (i.hasNext()) sb.append(", ");
		}
		return sb.append(">").toString();
	}

}
