package forge.solve;

import java.util.List;
import java.util.Set;

import kodkod.instance.TupleSet;
import forge.program.ForgeType;
import forge.program.ProgramElement;

/**
 * A Forge constant -- a relation of atoms.
 * 
 * @author Greg Dennis (gdennis@mit.edu)
 */
public interface ForgeConstant extends ProgramElement {
	
	/**
	 * Returns the arity of this constant.
	 */
	public int arity();
	
	/**
	 * Returns the type of this constant.
	 */
	public ForgeType type();
	
	/**
	 * Returns if the size is 0.
	 */
	public boolean isEmpty();
	
	/**
	 * Returns true if the arity is one.
	 */
	public boolean isUnary();
	
	/**
	 * Returns true if the size is one.
	 */
	public boolean isTuple();
	
	/**
	 * Returns the set of tuples in this constant
	 */
	public Set<? extends Tuple> tuples();
	
	/**
	 * Returns true if this is a subset of c
	 */
	public boolean subsetOf(ForgeConstant c);
	
	public TupleSet tupleSet();
	
	// projection operations
	public ForgeConstant.Unary domain();
	public ForgeConstant.Unary range();
	public ForgeConstant projection(int... columns);
	
//	// binary operator expressions
	public ForgeConstant union(ForgeConstant expr);
	public ForgeConstant difference(ForgeConstant expr);
	public ForgeConstant intersection(ForgeConstant expr);
	public ForgeConstant join(ForgeConstant expr);
	public ForgeConstant product(ForgeConstant expr);

	/**
	 * A constant that is a set of atoms.
	 */
	public interface Unary extends ForgeConstant {
		public ForgeType.Unary type();
		public Set<ForgeAtom> tuples();
		public ForgeConstant.Unary union(ForgeConstant expr);
		public ForgeConstant.Unary difference(ForgeConstant expr);
		public ForgeConstant.Unary intersection(ForgeConstant expr);
	}
	
	/**
	 * A constant that is a tuple of atoms.
	 */
	public interface Tuple extends ForgeConstant {
		public ForgeType type();
		public List<ForgeAtom> atoms();
		public ForgeAtom domain();
		public ForgeAtom range();
		public ForgeConstant.Tuple projection(int... columns);
		public ForgeConstant.Tuple product(Tuple tuple);
	}
	
}
