/**
 * 
 */
package forge.tests;

import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import forge.cfg.BranchStmt;
import forge.cfg.ForgeCFG;
import forge.cfg.UpdateStmt;
import forge.program.ForgeExpression;
import forge.program.ForgeProcedure;
import forge.program.ForgeProgram;
import forge.program.ForgeVariable;
import forge.program.GlobalVariable;
import forge.program.InstanceDomain;
import forge.program.LocalVariable;

/**
 * @author gdennis
 *
 */
public final class CopyTreeTest extends ForgeTest {

	CopyTreeTest() {
		super("copyTree", code, preCond, postCond, modGlobals);
	}
	
	InstanceDomain node() {
		return node;
	}
	
	private static final ForgeCFG.Impl code;
	private static final InstanceDomain node;
	private static final ForgeExpression preCond, postCond;
	private static final Set<GlobalVariable> modGlobals;
	
	static {
		ForgeProgram prog = new ForgeProgram();
		node = prog.newInstanceDomain("node");
		GlobalVariable key = prog.newGlobalVariable("key", node.product(prog.integerDomain()));
		GlobalVariable left = prog.newGlobalVariable("left", node.product(node));
		GlobalVariable right = prog.newGlobalVariable("right", node.product(node));
		LocalVariable n = prog.newLocalVariable("n", node);
		LocalVariable c = prog.newLocalVariable("c", node);
		ForgeVariable m = prog.newLocalVariable("m", node);
		ForgeProcedure proc = prog.newProcedure("copyTree", n, c);
		code = ForgeCFG.implementation(proc);
		
		BranchStmt noN     = code.newBranch(n.no());
		UpdateStmt noneC   = code.newAssign(c, prog.empty());
		UpdateStmt newC    = code.newCreate(c, node);
		UpdateStmt setKey  = code.newAssign(key, key.union(c.product(n.join(key))));
		
		List<ForgeExpression> ins = Collections.singletonList(n.join(left));
		List<ForgeVariable> outs = Collections.singletonList(m);
		UpdateStmt copyLeft = code.newCall(code, ins, outs);
		UpdateStmt setLeft = code.newAssign(left, left.union(c.product(m)));
		
		ins = Collections.singletonList(n.join(right));
		UpdateStmt copyRight = code.newCall(code, ins, outs);
		UpdateStmt setRight = code.newAssign(right, right.union(c.product(m)));
		
		code.setEntry(noN);
		noN.setThen(noneC);
		noN.setElse(newC);
		newC.setNext(setKey);
		setKey.setNext(copyLeft);
		copyLeft.setNext(setLeft);
		setLeft.setNext(copyRight);
		copyRight.setNext(setRight);
		
		preCond = n.one();
		ForgeExpression preKeys = keys(n, key.old(), left.old(), right.old());
		ForgeExpression postKeys = keys(c, key, left, right);
		postCond = c.one().and(postKeys.eq(preKeys));
		
		Set<GlobalVariable> mg = new HashSet<GlobalVariable>();
		mg.add(key); mg.add(left); mg.add(right);
		modGlobals = Collections.unmodifiableSet(mg);
	}
	
	private static ForgeExpression keys(ForgeExpression root,
			ForgeExpression key, ForgeExpression left, ForgeExpression right) {
		return root.union(root.join(left.union(right).closure())).join(key);
	}
}
