package forge.tests;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import forge.cfg.BranchStmt;
import forge.cfg.ForgeCFG;
import forge.cfg.UpdateStmt;
import forge.program.ForgeExpression;
import forge.program.ForgeProcedure;
import forge.program.ForgeProgram;
import forge.program.GlobalVariable;
import forge.program.LocalDecls;
import forge.solve.SolveOptions.SatSolver;
import forge.translate.ConstrainStrategy;
import forge.translate.HybridStrategy;
import forge.translate.InlineStrategy;
import forge.translate.SEStrategy;

/**
 * Used to distinguish the coverage offered by the three different translation
 * strategies.
 *
 * @author Greg Dennis (gdennis@mit.edu)
 */
public class CoverageTest extends ForgeTest {
	
	public static void main(String[] args) {
		CoverageTest covTest = new CoverageTest();
		System.out.println(covTest);
		covTest.setBounds(5, NO_SCOPES);
		doTest(covTest, InlineStrategy.STRATEGY);
		doTest(covTest, HybridStrategy.STRATEGY);
		doTest(covTest, ConstrainStrategy.STRATEGY);
	}
	
	private static void doTest(CoverageTest covTest, SEStrategy strategy) {
		covTest.newOptions(5).satSolver(SatSolver.MINISAT_PROVER).strategy(strategy);
		System.out.println();
		System.out.println(strategy);
		System.out.println(covTest.checkTest());
	}
	
	public CoverageTest() {
		super("Coverage", code, preCond, postCond, modGlobals);
	}
	
	private static final ForgeCFG.Impl code;
	private static final Set<GlobalVariable> modGlobals;
	private static final ForgeExpression preCond, postCond;
	
	static {
		// make the program and globals
		ForgeProgram program = new ForgeProgram();
		GlobalVariable x = program.newGlobalVariable("x", program.integerDomain());
		GlobalVariable y = program.newGlobalVariable("y", program.integerDomain());
		GlobalVariable z = program.newGlobalVariable("z", program.integerDomain());
		
		// make the procedure
		LocalDecls emptyDecls = program.emptyDecls();
		ForgeProcedure proc = program.newProcedure("test", emptyDecls, emptyDecls);
		
		// make the statements
		code = ForgeCFG.implementation(proc);
		ForgeExpression one = program.integerLiteral(1);
		BranchStmt branch = code.newBranch(program.trueLiteral());
		UpdateStmt incX = code.newAssign(x, x.plus(one));
		UpdateStmt decX = code.newAssign(x, x.minus(one));
		UpdateStmt incY = code.newAssign(y, y.plus(one));
		UpdateStmt incZ = code.newAssign(z, z.plus(one));
		
		// connect the statements
		code.setEntry(branch);
		branch.setThen(incX);
		branch.setElse(decX);
		incX.setNext(incY);
		decX.setNext(incY);
		incY.setNext(incZ);
		
		// make the pre- and post-condition
		Set<GlobalVariable> mgs = new HashSet<GlobalVariable>();
		mgs.add(x); mgs.add(y); mgs.add(z);
		modGlobals = Collections.unmodifiableSet(mgs);
		preCond = program.trueLiteral();
		postCond = x.eq(x.old().plus(one).and(z.eq(z)));
	}
}
