package forge.tests;

import java.util.Collections;
import java.util.Set;

import forge.cfg.BranchStmt;
import forge.cfg.ForgeCFG;
import forge.cfg.UpdateStmt;
import forge.program.ForgeExpression;
import forge.program.ForgeProcedure;
import forge.program.ForgeProgram;
import forge.program.GlobalVariable;
import forge.program.InstanceDomain;
import forge.program.LocalVariable;
import forge.solve.SolveOptions.SatSolver;

/**
 * 
 *
 * @author Greg Dennis (gdennis@mit.edu)
 */
public class CreateTest extends ForgeTest {
	
	public static void main(String[] args) {
		CreateTest test = new CreateTest();
		test.newOptions(5).satSolver(SatSolver.MINISAT).strategy(INLINE_STRATEGY);
		test.setBounds(6, Collections.singletonMap(bucket, 9));
		test.standardTest();
	}

	public CreateTest() {
		super("Create", code, preCond, postCond, modGlobals);
	}
	
	public static final InstanceDomain bucket;
	private static final ForgeCFG.Impl code;
	private static final ForgeExpression preCond, postCond;
	private static final Set<GlobalVariable> modGlobals;
	
	static {
		// make the procedure
		ForgeProgram prog = new ForgeProgram();
		ForgeExpression one = prog.integerLiteral(1);
		bucket = prog.newInstanceDomain("Bucket");
		GlobalVariable next = prog.newGlobalVariable("next", bucket.product(bucket));
		LocalVariable length = prog.newLocalVariable("length", prog.integerDomain());
		LocalVariable header = prog.newLocalVariable("header", bucket);
		LocalVariable p = prog.newLocalVariable("p", bucket);
		LocalVariable n = prog.newLocalVariable("n", bucket);
		ForgeProcedure proc = prog.newProcedure("makeList", length, header);
		
		// code the procedure
		code = ForgeCFG.implementation(proc);
		
		UpdateStmt newHeader    = code.newCreate(header, bucket);
		UpdateStmt setN         = code.newAssign(n, header);
		BranchStmt testLength   = code.newBranch(length.gt(one));
		UpdateStmt setP         = code.newAssign(p, n);
		UpdateStmt createBucket = code.newCreate(n, bucket);
		UpdateStmt setNext      = code.newAssign(next, next.union(p.product(n)));
		UpdateStmt decLength    = code.newAssign(length, length.minus(one));
		
		code.setEntry(newHeader);
		newHeader.setNext(setN);
		setN.setNext(testLength);
		testLength.setThen(setP);
		setP.setNext(createBucket);
		createBucket.setNext(setNext);
		setNext.setNext(decLength);
		decLength.setNext(testLength);
		
		preCond = length.one().and(length.gte(one));
		postCond = header.union(header.join(next.closure())).size().eq(length);
		modGlobals = Collections.singleton(next);
	}	
}
