package forge.tests;

import java.util.Collections;
import java.util.Set;

import forge.cfg.BranchStmt;
import forge.cfg.ForgeCFG;
import forge.cfg.UpdateStmt;
import forge.program.ForgeExpression;
import forge.program.ForgeProcedure;
import forge.program.ForgeProgram;
import forge.program.GlobalVariable;
import forge.program.InstanceDomain;
import forge.program.LocalVariable;
import forge.solve.SolveOptions.SatSolver;

public class LengthTest extends ForgeTest {
	
	public static void main(String[] args) {
		LengthTest test = new LengthTest();
		test.setBounds(4, Collections.singletonMap(node, 2));
		test.newOptions(3).strategy(CONSTRAIN_STRATEGY).satSolver(SatSolver.MINISAT_PROVER);
		test.standardTest();
	}
	
	public LengthTest() {
		super("length", code, preCond, postCond, modGlobals);
	}
	
	private static final ForgeCFG.Impl code;
	private static final ForgeExpression preCond, postCond;
	private static final Set<GlobalVariable> modGlobals;
	private static final InstanceDomain node;
	
	static {
		ForgeProgram prog = new ForgeProgram();
		node = prog.newInstanceDomain("Node");
		GlobalVariable head = prog.newGlobalVariable("head", node);
		GlobalVariable log  = prog.newGlobalVariable("log", prog.booleanDomain());
		GlobalVariable next = prog.newGlobalVariable("next", node.product(node));
		LocalVariable len   = prog.newLocalVariable("len", prog.integerDomain());
		LocalVariable curr  = prog.newLocalVariable("curr", node);
		ForgeProcedure proc = prog.newProcedure("length", prog.emptyDecls(), len);
		code = ForgeCFG.implementation(proc);
		
		UpdateStmt initLog  = code.newAssign(log, prog.trueLiteral());
		UpdateStmt initLen  = code.newAssign(len, prog.integerLiteral(0));
		UpdateStmt initCurr = code.newAssign(curr, head);
		BranchStmt someCurr = code.newBranch(curr.some());
		UpdateStmt incLen   = code.newAssign(len, len.plus(prog.integerLiteral(1)));
		UpdateStmt setCurr  = code.newAssign(curr, curr.join(next));
		
		code.setEntry(initLog);
		initLog.setNext(initLen);
		initLen.setNext(initCurr);
		initCurr.setNext(someCurr);
		someCurr.setThen(incLen);
		incLen.setNext(setCurr);
		setCurr.setNext(someCurr);
		
		LocalVariable n = prog.newLocalVariable("n", node);
		preCond = head.one().and(n.in(n.join(next.closure())).not().and(n.join(next).lone()).forAll(n));
		postCond = len.eq(head.union(head.join(next.closure())).size());
		modGlobals = Collections.singleton(log);
	}
}
