package forge.tests;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import forge.cfg.BranchStmt;
import forge.cfg.ForgeCFG;
import forge.cfg.UpdateStmt;
import forge.program.ForgeExpression;
import forge.program.ForgeLiteral;
import forge.program.ForgeProcedure;
import forge.program.ForgeProgram;
import forge.program.GlobalVariable;
import forge.program.InstanceDomain;
import forge.program.LocalVariable;
import forge.solve.ForgeSolution;
import forge.solve.SolveOptions.SatSolver;

/**
 * 
 *
 * @author Greg Dennis (gdennis@mit.edu)
 */
public final class RegisterTest extends ForgeTest {
	
	public static void main(String[] args) {
		RegisterTest test = new RegisterTest();
		test.newOptions(1).satSolver(SatSolver.MINISAT).strategy(INLINE_STRATEGY);
		Map<InstanceDomain, Integer> scopes = new HashMap<InstanceDomain, Integer>();
		scopes.put(test.stringDomain(), 10);
		scopes.put(test.userDomain(), 10);
		test.setBounds(8, scopes);
		
		int i = 0;
		for (Iterator<ForgeSolution> solIter = test.runAllTest();
				solIter.hasNext();) {
			ForgeSolution sol = solIter.next();
			System.out.println("Solution " + i + ":");
			System.out.println(sol.trace());
			System.out.println();
			i++;
			if (i >= 10) break;
		}
	}

	public RegisterTest() {
		super("Register", code, preCond, postCond, modGlobals);
	}
	
	public InstanceDomain stringDomain() {
		return string;
	}
	
	public InstanceDomain userDomain() {
		return user;
	}
	
	private static final ForgeCFG.Impl code;
	private static final ForgeExpression preCond, postCond;
	private static final Set<GlobalVariable> modGlobals;
	private static final InstanceDomain string, user;
	
	static {
		ForgeProgram prog = new ForgeProgram();
		string = prog.newInstanceDomain("String");
		user = prog.newInstanceDomain("User");
		ForgeLiteral errorUser = prog.newInstanceLiteral("ErrorUser", user);
		GlobalVariable id = prog.newGlobalVariable("id", user.product(prog.integerDomain()));
		GlobalVariable email = prog.newGlobalVariable("email", user.product(string));
		LocalVariable newUser = prog.newLocalVariable("newUser", user);
		LocalVariable newId = prog.newLocalVariable("newId", prog.integerDomain());
		LocalVariable newEmail = prog.newLocalVariable("newEmail", string);
		ForgeProcedure proc = prog.newProcedure("register", newEmail, newUser);
		
		code = ForgeCFG.implementation(proc);
		BranchStmt testEmail   = code.newBranch(newEmail.in(user.join(email)));
		UpdateStmt assignError = code.newAssign(newUser, errorUser);
		UpdateStmt createUser  = code.newCreate(newUser, user);
		UpdateStmt setEmail    = code.newAssign(email, email.union(newUser.product(newEmail)));
		UpdateStmt setId       = code.newSpec(Collections.singleton(id),
				newId.in(user.join(id.old())).not().and(
				id.eq(id.old().union(newUser.product(newId)))).forSome(newId));
		
		code.setEntry(testEmail);
		testEmail.setThen(assignError);
		testEmail.setElse(createUser);
		createUser.setNext(setEmail);
		setEmail.setNext(setId);
		
		preCond = newEmail.in(user.join(email.old())).not();
		postCond = newUser.join(email).eq(newEmail).and(newUser.join(id).in(user.join(id.old())).not());
		Set<GlobalVariable> mg = new HashSet<GlobalVariable>();
		mg.add(email);
		mg.add(id);
		modGlobals = Collections.unmodifiableSet(mg);
	}
}
