package forge.tests;

import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import forge.program.InstanceDomain;
import forge.solve.ForgeReporter;
import forge.solve.SolveOptions.SatSolver;
import forge.translate.SEStrategy;
import forge.translate.ThesisStrategy;

/**
 * Compare the different translation strategies.
 *
 * @author Greg Dennis (gdennis@mit.edu)
 */
public final class StrategyComparison {

	private StrategyComparison() {}
	
	private static SEStrategy[] STRATEGIES = new SEStrategy[] {
//		InlineStrategy.STRATEGY, HybridStrategy.STRATEGY, ConstrainStrategy.STRATEGY
		new ThesisStrategy(true,  true,  true),  new ThesisStrategy(false, true,  true),
		new ThesisStrategy(true,  true, false),  new ThesisStrategy(true,  false,  true),
		new ThesisStrategy(false, true, false),  new ThesisStrategy(false, false,  true),
		new ThesisStrategy(true,  false, false), new ThesisStrategy(false, false, false)
	};
	
	private static long start;
	
	public static void main(String[] args) {
			
		List<ForgeTest> tests = new LinkedList<ForgeTest>();
		Map<InstanceDomain, Integer> scopes = null;
		
		CopyTreeTest copyTest = new CopyTreeTest();
		copyTest.newOptions(3);
		scopes = new HashMap<InstanceDomain, Integer>();
		scopes.put(copyTest.node(), 3);
		copyTest.setBounds(3, scopes);
		tests.add(copyTest);
		
		TreeIntSetTest treeTest = new TreeIntSetTest();
		treeTest.newOptions(3);
		scopes = new HashMap<InstanceDomain, Integer>();
		scopes.put(treeTest.treeDomain(), 1);
		scopes.put(treeTest.nodeDomain(), 3);
		treeTest.setBounds(3, scopes);
		tests.add(treeTest);
		
		RegisterTest regTest = new RegisterTest();
		regTest.newOptions(1);
		scopes = new HashMap<InstanceDomain, Integer>();
		scopes.put(regTest.stringDomain(), 3);
		scopes.put(regTest.userDomain(), 3);
		regTest.setBounds(3, scopes);
		tests.add(regTest);
		
		ListTest listLoop = new ListTest.Looping();
		listLoop.newOptions(3);
		listLoop.setBounds(3, Collections.singletonMap(listLoop.listDomain(), 3));
		tests.add(listLoop);
		
		ListTest listRecurse = new ListTest.Recursive();
		listRecurse.newOptions(3);
		listRecurse.setBounds(3, Collections.singletonMap(listRecurse.listDomain(), 3));
		tests.add(listRecurse);
		
		MultiplyTest multTest = new MultiplyTest();
		multTest.newOptions(3);
		multTest.setBounds(3, ForgeTest.NO_SCOPES);
		tests.add(multTest);
		
		for (ForgeTest test: tests) {
			test.currOptions().satSolver(SatSolver.MINISAT).reporter(REPORTER);
			System.out.println("TEST: " + test.name());
			System.out.println();
			
			for(SEStrategy s: STRATEGIES) {
				test.currOptions().strategy(s);
				System.out.println("STRATEGY: " + s);
				System.out.println(test.checkTest().trace());
				long finish = System.currentTimeMillis();
				long duration = (long)((finish - start) / 1000.0);
				System.out.println(duration + " seconds");
			    System.out.println();
			}
			
			System.out.println("=======================================================");
		}
		
	}
	
	private static final ForgeReporter REPORTER = new ForgeReporter() {
		public void solvingCNF(int primaryVars, int vars, int clauses) {
			System.out.println("solving CNF: vars=" + vars + " ; clauses=" + clauses + " . . . ");
		}

		public void solvingAnalysis() {
			start = System.currentTimeMillis();
			System.out.println("startedAnalysis");
		}
	};
}
