package forge.tests;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import forge.cfg.AssignStmt;
import forge.cfg.BranchStmt;
import forge.cfg.ForgeCFG;
import forge.program.ForgeExpression;
import forge.program.ForgeProcedure;
import forge.program.ForgeProgram;
import forge.program.GlobalVariable;
import forge.solve.ForgeSolution;
import forge.solve.SolveOptions.SatSolver;
import forge.translate.ConstrainStrategy;

/**
 * 
 *
 * @author Greg Dennis (gdennis@mit.edu)
 */
public final class ZuneTest extends ForgeTest {
	
	public static void main(String[] args) {
		ZuneTest test = new ZuneTest();
		test.newOptions(1).satSolver(SatSolver.MINISAT_PROVER).strategy(ConstrainStrategy.STRATEGY);
		test.setBounds(5, NO_SCOPES);
		ForgeSolution sol = test.checkTest();
		System.out.println(sol);
	}

	public ZuneTest() {
		super("Zune test", code, preCond, postCond, modGlobals);
	}
	
	private static final ForgeCFG.Impl code;
	private static final ForgeExpression preCond;
	private static final ForgeExpression postCond;
	private static final Set<GlobalVariable> modGlobals;
	
	static {
		ForgeProgram prog = new ForgeProgram();
		ForgeExpression zero = prog.integerLiteral(0);
		ForgeExpression one = prog.integerLiteral(1);
		GlobalVariable isLeapYear = prog.newGlobalVariable("isLeapYear", prog.booleanDomain());
		GlobalVariable daysInNormYear = prog.newGlobalVariable("daysInNormYear", prog.integerDomain());
		GlobalVariable daysInLeapYear = prog.newGlobalVariable("daysInLeapYear", prog.integerDomain());
		GlobalVariable years = prog.newGlobalVariable("years", prog.integerDomain());
		GlobalVariable days = prog.newGlobalVariable("days", prog.integerDomain());
		ForgeProcedure proc = prog.newProcedure("subtractDays", prog.emptyDecls(), prog.emptyDecls());
		
		code = ForgeCFG.implementation(proc);
		AssignStmt desiredYears = code.newAssign(years, days.minus(one).divide(isLeapYear.thenElse(daysInLeapYear, daysInNormYear)));
		AssignStmt initYears = code.newAssign(years, zero);
		BranchStmt testNormDays = code.newBranch(days.gt(daysInNormYear));
		BranchStmt testLeapYear = code.newBranch(isLeapYear);
		BranchStmt testLeapDays	= code.newBranch(days.gt(daysInLeapYear));
		AssignStmt subLeapDays = code.newAssign(days, days.minus(daysInLeapYear));
		AssignStmt subNormDays = code.newAssign(days, days.minus(daysInNormYear));
		AssignStmt incYears = code.newAssign(years, years.plus(one));
		
		code.setEntry(desiredYears);
		desiredYears.setNext(initYears);
		initYears.setNext(testNormDays);
		testNormDays.setThen(testLeapYear);
		testLeapYear.setThen(testLeapDays);
		testLeapDays.setThen(subLeapDays);
		subLeapDays.setNext(incYears);
		testLeapDays.setElse(incYears);
		testLeapYear.setElse(subNormDays);
		subNormDays.setNext(incYears);
		incYears.setNext(testNormDays);
		System.out.println(code);
		
		preCond = isLeapYear.one().and(daysInNormYear.one()).
			and(daysInLeapYear.one()).and(days.old().one()).
			and(daysInLeapYear.gt(daysInNormYear)).
			and(daysInNormYear.gt(zero)).
			and(days.gt(zero)).and(years.old().one());
		postCond = years.eq(days.old().minus(one).divide(isLeapYear.thenElse(daysInLeapYear, daysInNormYear)));
		
		Set<GlobalVariable> mg = new HashSet<GlobalVariable>();
		mg.add(days);
		mg.add(years);
		modGlobals = Collections.unmodifiableSet(mg);
	}
}
