package forge.transform;

import java.util.List;
import java.util.ListIterator;

import forge.cfg.CFGStmt;
import forge.cfg.ForgeCFG;

/**
 * Applies a list of transformers in a row.
 *
 * @author Greg Dennis (gdennis@mit.edu)
 */
public final class BatchTransformer implements Transformer {
	
	private final List<Transformer> transformers;
	
	public BatchTransformer(List<Transformer> transformers) {
		this.transformers = transformers;
	}

	public ForgeCFG transform(ForgeCFG source) {
		ForgeCFG target = source;
		
//		System.out.println();
//		System.out.println("Starting CFG");
//		System.out.println(target);
//		System.out.println();
		
		for (Transformer t : transformers) {
//			System.out.println();
//			System.out.println("Transformed with " + t.getClass());
			target = t.transform(target);
//			System.out.println(target);
//			System.out.println();
		}
		
//		System.out.println();
//		System.out.println("Final CFG");
//		System.out.println(target);
//		System.out.println();
		
		return target;
	}
	
	public CFGStmt sourceStmt(CFGStmt targetStmt) {
		CFGStmt sourceStmt = targetStmt;
		for (ListIterator<Transformer> i = transformers.listIterator(
				transformers.size()); i.hasPrevious();) {
			sourceStmt = i.previous().sourceStmt(sourceStmt);
		}
		return sourceStmt;
	}

}
