package forge.transform;

import forge.program.BinaryExpression;
import forge.program.ConditionalExpression;
import forge.program.ExpressionVisitor;
import forge.program.ForgeExpression;
import forge.program.ForgeLiteral;
import forge.program.ForgeType;
import forge.program.ForgeVariable;
import forge.program.OldExpression;
import forge.program.ProjectionExpression;
import forge.program.QuantifyExpression;
import forge.program.UnaryExpression;

/**
 * Visitor where each visit method calls the default method.
 * 
 * @author Greg Dennis (gdennis@mit.edu)
 */
public abstract class ExpressionDefaulter<T> extends ExpressionVisitor<T> {
	
	protected abstract T defaultVisit(ForgeExpression expr);
	
	
	@Override
	protected T visit(ForgeType expr) {
		return defaultVisit(expr);
	}

	@Override
	protected T visit(ForgeLiteral expr) {
		return defaultVisit(expr);
	}

	@Override
	protected T visit(ForgeVariable expr) {
		return defaultVisit(expr);
	}

	@Override
	protected T visit(UnaryExpression expr) {
		return defaultVisit(expr);
	}

	@Override
	protected T visit(BinaryExpression expr) {
		return defaultVisit(expr);
	}

	@Override
	protected T visit(ConditionalExpression expr) {
		return defaultVisit(expr);
	}

	@Override
	protected T visit(ProjectionExpression expr) {
		return defaultVisit(expr);
	}

	@Override
	protected T visit(QuantifyExpression expr) {
		return defaultVisit(expr);
	}

	@Override
	protected T visit(OldExpression expr) {
		return defaultVisit(expr);
	}
}
