package forge.transform;

import forge.cfg.CFGStmt;
import forge.cfg.ForgeCFG;

/**
 * Transforms a cfg, either in place, or by creating a new cfg.
 * 
 * - no two source stmts map to the same target
 * - source update statements map only to update statements
 * - only blocks of statements may not be transformed
 * 
 * @specfield map: CFGStmt -> CFGStmt
 * @invariant all s: target.stmts | lone map.s
 * @invariant all s: source.stmts & UpdateStmt | s.map in UpdateStmt
 *
 * @author Greg Dennis (gdennis@mit.edu)
 */
public interface Transformer {
	
	/**
	 * The identity transformer.
	 */
	public static final Transformer IDENTITY = new Transformer() {
		public ForgeCFG transform(ForgeCFG source) {
			return source;
		}
		public CFGStmt sourceStmt(CFGStmt targetStmt) {
			return targetStmt;
		}
	};

	/**
	 * Performs the transformation and returns the transformed procedure.
	 * The returned procedure may be == to the give procedure.
	 */
	public abstract ForgeCFG transform(ForgeCFG source);
	
	/**
	 * Returns the source update from which the given target was transformed.
	 */
	public abstract CFGStmt sourceStmt(CFGStmt targetStmt);
}
