package forge.translate;

import java.util.HashMap;
import java.util.Map;

import kodkod.ast.Formula;
import edu.mit.csail.sdg.annotations.Returns;
import forge.cfg.ForgeCFG;
import forge.cfg.StmtSet;
import forge.program.ForgeProgram;
import forge.program.ProgramElement;
import forge.program.ForgeExpression.Modifiable;
import forge.util.ModifiableMap;

/**
 * Manage the slices in a translation.
 * 
 * @author Greg Dennis (gdennis@mit.edu)
 */
final class SliceLog implements ProgramElement {

	private final ForgeCFG cfg;
	private final ModifiableMap<StmtSet> modSlices;
	private final Map<Formula,StmtSet> formSlices;
	private final boolean unmodifiable;
	
	SliceLog(ForgeCFG cfg) {
		this(cfg, new ModifiableMap<StmtSet>(),
				new HashMap<Formula,StmtSet>(), false);
	}
	
	SliceLog branch() {
		return new SliceLog(cfg, new ModifiableMap<StmtSet>(modSlices),
				new HashMap<Formula,StmtSet>(),
				false);
	}
	
	SliceLog unmodifiable() {
		return new SliceLog(cfg, modSlices, formSlices, true);
	}
	
	private SliceLog(ForgeCFG cfg, ModifiableMap<StmtSet> modSlices,
			Map<Formula,StmtSet> formSlices, boolean unmodifiable) {
		this.cfg = cfg;
		this.modSlices = modSlices;
		this.formSlices = formSlices;
		this.unmodifiable = unmodifiable;
	}
	
	/* inherits specification */
	public ForgeProgram program() {
		return cfg.program();
	}
	
	@Returns("this.cfg")
	public ForgeCFG cfg() {
		return cfg;
	}
	
	/**
	 * Returns a view of the slice that influences the given mod.
	 */
	StmtSet getSlice(Modifiable mod) {
		StmtSet slice = modSlices.diffMap().get(mod);
		if (slice == null) {
			StmtSet base = modSlices.baseMap().get(mod);
			slice = base == null ? StmtSet.hash(cfg) : StmtSet.hash(base);
			modSlices.diffMap().put(mod, slice);
		}
		return unmodifiable ? slice.unmodifiable() : slice;
	}
	
	/**
	 * Returns a view of the slice that influences the given form.
	 */
	StmtSet getSlice(Formula form) {
		StmtSet slice = formSlices.get(form);
		if (slice == null) {
			slice = StmtSet.hash(cfg);
			formSlices.put(form, slice);
		}
		return unmodifiable ? slice.unmodifiable() : slice;
	}
	
	public String toString() {
		return modSlices.toString() + formSlices.toString();
	}
	
}
