package forge.util;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import edu.mit.csail.sdg.util.collections.OverrideMap;
import forge.program.ForgeExpression.Modifiable;

/**
 * An override map that maps modifiables to non-null values.
 *
 * @author Greg Dennis (gdennis@mit.edu)
 */
public final class ModifiableMap<V> extends OverrideMap<Modifiable,V> {
	
	private final Map<Modifiable,V> diff;
	private final boolean unmod;
	
	public ModifiableMap() {
		this(Collections.<Modifiable,V>emptyMap());
	}
	
	public ModifiableMap(Map<Modifiable, V> base) {
		this(unmodBase(base), new HashMap<Modifiable, V>(), false);
	}
	
	private static <V> Map<? extends Modifiable, ? extends V> unmodBase(
			Map<Modifiable,? extends V> base) {
		if (base instanceof ModifiableMap) {
			ModifiableMap<? extends V> diffBase = (ModifiableMap<? extends V>)base;
			if (diffBase.diff.isEmpty()) {
				return diffBase.base;
			}
		}
		return Collections.unmodifiableMap(base);
	}
	
	public ModifiableMap<V> unmodifiable() {
		return unmod ? this : new ModifiableMap<V>(base, diff, true);
	}
	
	private ModifiableMap(Map<? extends Modifiable,? extends V> base,
			Map<Modifiable,V> diff, boolean unmod) {
		super(base, diff);
		this.diff = diff;
		this.unmod = unmod;
	}
	
	/**
	 * The unmodifiable base map.
	 */
	public Map<? extends Modifiable,? extends V> baseMap() {
		return base;
	}

	/**
	 * The modifiable diff map.
	 */
	public Map<Modifiable,V> diffMap() {
		return diff;
	}

	@Override
	public V put(Modifiable key, V value) {
		if (unmod) throw new UnsupportedOperationException();
		if (value == null) throw new NullPointerException("value");
		return diff.put(key, value);
	}
	
}
