/*! \addtogroup Annotations Annotations 
 * This module defines several Squander-specific specification annotations (in addition to existing JFSL annotations). 
 * @{ 
 */
package edu.mit.csail.sdg.squander.annotations;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Specification specific options.
 * 
 * @author Aleksandar Milicevic
 */
@Retention(RetentionPolicy.RUNTIME)
@Target({ElementType.METHOD, ElementType.CONSTRUCTOR})
@Documented
public @interface Options {

    /**
     * min integer bitwidth to be used 
     */
    int bitwidth() default 4;
    
    /** 
     * whether an atom should be created for all ints (within the bitwidth) 
     * when translating to Kodkod 
     */
    boolean ensureAllInts() default false;

    /**
     * whether to search for all solutions (one at a time, of course) 
     */
    boolean solveAll() default false;
    
}
/*! @} */
