// $ANTLR 3.2 Sep 23, 2009 12:02:23 JFSL.g 2011-05-13 21:44:00

package edu.mit.csail.sdg.squander.parser;

import edu.mit.csail.sdg.squander.spec.*;


import org.antlr.runtime.*;
import java.util.Stack;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;

import org.antlr.runtime.tree.*;

/** A Java 1.5 grammar for ANTLR v3 derived from the spec
 *
 *  This is a very close representation of the spec; the changes
 *  are comestic (remove left recursion) and also fixes (the spec
 *  isn't exactly perfect).  I have run this on the 1.4.2 source
 *  and some nasty looking enums from 1.5, but have not really
 *  tested for 1.5 compatibility.
 *
 *  I built this with: java -Xmx100M org.antlr.Tool java.g 
 *  and got two errors that are ok (for now):
 *  java.g:691:9: Decision can match input such as
 *    "'0'..'9'{'E', 'e'}{'+', '-'}'0'..'9'{'D', 'F', 'd', 'f'}"
 *    using multiple alternatives: 3, 4
 *  As a result, alternative(s) 4 were disabled for that input
 *  java.g:734:35: Decision can match input such as "{'$', 'A'..'Z',
 *    '_', 'a'..'z', '\u00C0'..'\u00D6', '\u00D8'..'\u00F6',
 *    '\u00F8'..'\u1FFF', '\u3040'..'\u318F', '\u3300'..'\u337F',
 *    '\u3400'..'\u3D2D', '\u4E00'..'\u9FFF', '\uF900'..'\uFAFF'}"
 *    using multiple alternatives: 1, 2
 *  As a result, alternative(s) 2 were disabled for that input
 *
 *  You can turn enum on/off as a keyword :)
 *
 *  Version 1.0 -- initial release July 5, 2006 (requires 3.0b2 or higher)
 *
 *  Primary author: Terence Parr, July 2006
 *
 *  Version 1.0.1 -- corrections by Koen Vanderkimpen & Marko van Dooren,
 *      October 25, 2006;
 *      fixed normalInterfaceDeclaration: now uses typeParameters instead
 *          of typeParameter (according to JLS, 3rd edition)
 *      fixed castExpression: no longer allows expression next to type
 *          (according to semantics in JLS, in contrast with syntax in JLS)
 *
 *  Version 1.0.2 -- Terence Parr, Nov 27, 2006
 *      java spec I built this from had some bizarre for-loop control.
 *          Looked weird and so I looked elsewhere...Yep, it's messed up.
 *          simplified.
 *
 *  Version 1.0.3 -- Chris Hogue, Feb 26, 2007
 *      Factored out an annotationName rule and used it in the annotation rule.
 *          Not sure why, but typeName wasn't recognizing references to inner
 *          annotations (e.g. @InterfaceName.InnerAnnotation())
 *      Factored out the elementValue section of an annotation reference.  Created 
 *          elementValuePair and elementValuePairs rules, then used them in the 
 *          annotation rule.  Allows it to recognize annotation references with 
 *          multiple, comma separated attributes.
 *      Updated elementValueArrayInitializer so that it allows multiple elements.
 *          (It was only allowing 0 or 1 element).
 *      Updated localVariableDeclaration to allow annotations.  Interestingly the JLS
 *          doesn't appear to indicate this is legal, but it does work as of at least
 *          JDK 1.5.0_06.
 *      Moved the Identifier portion of annotationTypeElementRest to annotationMethodRest.
 *          Because annotationConstantRest already references variableDeclarator which 
 *          has the Identifier portion in it, the parser would fail on constants in 
 *          annotation definitions because it expected two identifiers.  
 *      Added optional trailing ';' to the alternatives in annotationTypeElementRest.
 *          Wouldn't handle an inner interface that has a trailing ';'.
 *      Swapped the expression and type rule reference order in castExpression to 
 *          make it check for genericized casts first.  It was failing to recognize a
 *          statement like  "Class<Byte> TYPE = (Class<Byte>)...;" because it was seeing
 *          'Class<Byte' in the cast expression as a less than expression, then failing 
 *          on the '>'.
 *      Changed createdName to use typeArguments instead of nonWildcardTypeArguments.
 *          Again, JLS doesn't seem to allow this, but java.lang.Class has an example of
 *          of this construct.
 *      Changed the 'this' alternative in primary to allow 'identifierSuffix' rather than
 *          just 'arguments'.  The case it couldn't handle was a call to an explicit
 *          generic method invocation (e.g. this.<E>doSomething()).  Using identifierSuffix
 *          may be overly aggressive--perhaps should create a more constrained thisSuffix rule?
 *      
 *  Version 1.0.4 -- Hiroaki Nakamura, May 3, 2007
 *
 *  Fixed formalParameterDecls, localVariableDeclaration, forInit,
 *  and forVarControl to use variableModifier* not 'final'? (annotation)?
 *
 *  Version 1.0.5 -- Terence, June 21, 2007
 *  --a[i].foo didn't work. Fixed unaryExpression
 *
 *  Version 1.0.6 -- John Ridgway, March 17, 2008
 *      Made "assert" a switchable keyword like "enum".
 *      Fixed compilationUnit to disallow "annotation importDeclaration ...".
 *      Changed "Identifier ('.' Identifier)*" to "qualifiedName" in more 
 *          places.
 *      Changed modifier* and/or variableModifier* to classOrInterfaceModifiers,
 *          modifiers or variableModifiers, as appropriate.
 *      Renamed "bound" to "typeBound" to better match language in the JLS.
 *      Added "memberDeclaration" which rewrites to methodDeclaration or 
 *      fieldDeclaration and pulled type into memberDeclaration.  So we parse 
 *          type and then move on to decide whether we're dealing with a field
 *          or a method.
 *      Modified "constructorDeclaration" to use "constructorBody" instead of
 *          "methodBody".  constructorBody starts with explicitConstructorInvocation,
 *          then goes on to blockStatement*.  Pulling explicitConstructorInvocation
 *          out of expressions allowed me to simplify "primary".
 *      Changed variableDeclarator to simplify it.
 *      Changed type to use classOrInterfaceType, thus simplifying it; of course
 *          I then had to add classOrInterfaceType, but it is used in several 
 *          places.
 *      Fixed annotations, old version allowed "@X(y,z)", which is illegal.
 *      Added optional comma to end of "elementValueArrayInitializer"; as per JLS.
 *      Changed annotationTypeElementRest to use normalClassDeclaration and 
 *          normalInterfaceDeclaration rather than classDeclaration and 
 *          interfaceDeclaration, thus getting rid of a couple of grammar ambiguities.
 *      Split localVariableDeclaration into localVariableDeclarationStatement
 *          (includes the terminating semi-colon) and localVariableDeclaration.  
 *          This allowed me to use localVariableDeclaration in "forInit" clauses,
 *           simplifying them.
 *      Changed switchBlockStatementGroup to use multiple labels.  This adds an
 *          ambiguity, but if one uses appropriately greedy parsing it yields the
 *           parse that is closest to the meaning of the switch statement.
 *      Renamed "forVarControl" to "enhancedForControl" -- JLS language.
 *      Added semantic predicates to test for shift operations rather than other
 *          things.  Thus, for instance, the string "< <" will never be treated
 *          as a left-shift operator.
 *      In "creator" we rule out "nonWildcardTypeArguments" on arrayCreation, 
 *          which are illegal.
 *      Moved "nonWildcardTypeArguments into innerCreator.
 *      Removed 'super' superSuffix from explicitGenericInvocation, since that
 *          is only used in explicitConstructorInvocation at the beginning of a
 *           constructorBody.  (This is part of the simplification of expressions
 *           mentioned earlier.)
 *      Simplified primary (got rid of those things that are only used in
 *          explicitConstructorInvocation).
 *      Lexer -- removed "Exponent?" from FloatingPointLiteral choice 4, since it
 *          led to an ambiguity.
 *
 *      This grammar successfully parses every .java file in the JDK 1.5 source 
 *          tree (excluding those whose file names include '-', which are not
 *          valid Java compilation units).
 *
 *  Known remaining problems:
 *      "Letter" and "JavaIDDigit" are wrong.  The actual specification of
 *      "Letter" should be "a character for which the method
 *      Character.isJavaIdentifierStart(int) returns true."  A "Java 
 *      letter-or-digit is a character for which the method 
 *      Character.isJavaIdentifierPart(int) returns true."
 */
@SuppressWarnings({"all"})
public class JFSLParser extends Parser {
    public static final String[] tokenNames = new String[] {
        "<invalid>", "<EOR>", "<DOWN>", "<UP>", "DECLARATION", "FRAME", "NULL", "MOD_DISJ", "MOD_NONE", "DECL_SET", "DECL_SEQ", "DECL_NONE", "FRAME_LOCATION", "FRAME_ALL", "BINARY", "UNARY", "QUANTIFY", "QUANTIFY_ENUM", "CONDITIONAL", "ARGUMENTS", "CHAIN", "PROJECTION", "JOIN", "JOIN_REFLEXIVE", "BRACKET", "CALL", "FRAME_FIELD", "FRAME_FILTER", "FRAME_DOMAIN", "OLD", "ARGUMENT", "AMBIGUOUS", "FIELD", "CAST", "CLASS_DESIGNATOR", "THIS_VAR", "SUPER_VAR", "IDENTIFIER", "VOID", "RETURN_VAR", "THROW_VAR", "LAMBDA_VAR", "DECLS", "MDECLS", "OP_EQ", "OP_NEQ", "OP_OR", "OP_AND", "OP_NOT", "OP_BIT_OR", "OP_BIT_XOR", "OP_INTERSECTION", "OP_BIT_AND_OR_INTERSECTION", "OP_TRANSPOSE", "OP_BIT_NOT_OR_TRANSPOSE", "OP_CLOSURE", "OP_GT", "OP_LT", "OP_GEQ", "OP_LEQ", "OP_SHL", "OP_SHR", "OP_USHR", "OP_INSTANCEOF", "OP_PLUS", "OP_UNION", "OP_PLUS_OR_UNION", "OP_MINUS", "OP_DIFFERENCE", "OP_MINUS_OR_DIFFERENCE", "OP_RELATIONAL_OVERRIDE", "OP_RELATIONAL_COMPOSE", "OP_TIMES", "OP_DIVIDE", "OP_MOD", "OP_EQUIV", "OP_NEQUIV", "OP_IMPLIES", "OP_SET_ONE", "OP_SET_SOME", "OP_SET_NO", "OP_SET_LONE", "OP_SET_NUM", "OP_SET_SUM", "OP_SET_ALL", "OP_SET_EXISTS", "OP_SET_COMPREHENSION", "OP_SET_COMPREHENSION_ENUM", "OP_RANGE", "OP_SET_SUBSET", "OP_NSET_SUBSET", "LIT_TRUE", "LIT_FALSE", "LIT_NULL", "TYPE_BOOLEAN", "TYPE_CHAR", "TYPE_BYTE", "TYPE_SHORT", "TYPE_INT", "TYPE_LONG", "TYPE_FLOAT", "TYPE_DOUBLE", "TYPE_REF", "TYPE_ARRAY", "FILE", "INVARIANT", "SPECFIELD", "PACKAGE", "TYPE_DECLARATION", "TYPE_PARAMETERS", "IMPORT", "METHOD", "METHOD_PARAMETERS", "SPECCASE", "SPECIFICATION", "REQUIRES", "ENSURES", "MODIFIES", "THROWS", "HELPER", "PURE", "Identifier", "FloatingPointLiteral", "CharacterLiteral", "StringLiteral", "HexLiteral", "OctalLiteral", "DecimalLiteral", "HexDigit", "IntegerTypeSuffix", "Exponent", "FloatTypeSuffix", "EscapeSequence", "UnicodeEscape", "OctalEscape", "Letter", "JavaIDDigit", "WS", "'package'", "';'", "'.'", "'import'", "'class'", "'interface'", "'{'", "'}'", "'<'", "','", "'>'", "'@Invariant'", "'('", "')'", "'@SpecField'", "'@Requires'", "'@Ensures'", "'@Throws'", "'@Modifies'", "'@Helper'", "'@Pure'", "'public'", "'private'", "':'", "'from'", "'|'", "'disj'", "'set'", "'seq'", "'*'", "'true'", "'false'", "'null'", "'boolean'", "'char'", "'byte'", "'short'", "'int'", "'long'", "'float'", "'double'", "'['", "']'", "'?'", "'='", "'!'", "'||'", "'&&'", "'^'", "'&'", "'instanceof'", "'in'", "'...'", "'+'", "'-'", "'#'", "'/'", "'%'", "'@'", "'++'", "'->'", "'~'", "'return'", "'throw'", "'this'", "'super'", "'_'", "'@old'", "'@arg'", "'one'", "'some'", "'lone'", "'no'", "'sum'", "'all'", "'exists'"
    };
    public static final int PACKAGE=107;
    public static final int T__159=159;
    public static final int T__158=158;
    public static final int OP_INSTANCEOF=63;
    public static final int OP_SET_ALL=84;
    public static final int OP_GEQ=58;
    public static final int TYPE_INT=98;
    public static final int OP_SET_LONE=81;
    public static final int OP_SET_NO=80;
    public static final int T__160=160;
    public static final int T__167=167;
    public static final int OP_INTERSECTION=51;
    public static final int EOF=-1;
    public static final int T__168=168;
    public static final int LIT_FALSE=92;
    public static final int T__165=165;
    public static final int T__166=166;
    public static final int T__163=163;
    public static final int T__164=164;
    public static final int T__161=161;
    public static final int T__162=162;
    public static final int OP_MOD=74;
    public static final int OP_EQ=44;
    public static final int IMPORT=110;
    public static final int T__148=148;
    public static final int T__147=147;
    public static final int T__149=149;
    public static final int TYPE_BYTE=96;
    public static final int DECL_SET=9;
    public static final int DECL_SEQ=10;
    public static final int TYPE_PARAMETERS=109;
    public static final int T__154=154;
    public static final int T__155=155;
    public static final int T__156=156;
    public static final int T__157=157;
    public static final int T__150=150;
    public static final int T__151=151;
    public static final int T__152=152;
    public static final int MOD_NONE=8;
    public static final int T__153=153;
    public static final int T__139=139;
    public static final int T__138=138;
    public static final int THIS_VAR=35;
    public static final int IntegerTypeSuffix=129;
    public static final int OP_DIFFERENCE=68;
    public static final int T__141=141;
    public static final int T__142=142;
    public static final int FILE=104;
    public static final int T__140=140;
    public static final int LAMBDA_VAR=41;
    public static final int T__145=145;
    public static final int T__146=146;
    public static final int T__143=143;
    public static final int T__144=144;
    public static final int METHOD_PARAMETERS=112;
    public static final int OP_SHL=60;
    public static final int TYPE_REF=102;
    public static final int WS=137;
    public static final int OP_SHR=61;
    public static final int SPECFIELD=106;
    public static final int QUANTIFY_ENUM=17;
    public static final int TYPE_CHAR=95;
    public static final int FloatingPointLiteral=122;
    public static final int JavaIDDigit=136;
    public static final int CALL=25;
    public static final int SPECIFICATION=114;
    public static final int CLASS_DESIGNATOR=34;
    public static final int CHAIN=20;
    public static final int T__213=213;
    public static final int OP_OR=46;
    public static final int AMBIGUOUS=31;
    public static final int TYPE_LONG=99;
    public static final int HexDigit=128;
    public static final int DECLARATION=4;
    public static final int T__202=202;
    public static final int ENSURES=116;
    public static final int T__203=203;
    public static final int T__204=204;
    public static final int T__205=205;
    public static final int OP_USHR=62;
    public static final int T__206=206;
    public static final int T__207=207;
    public static final int T__208=208;
    public static final int T__209=209;
    public static final int OP_RANGE=88;
    public static final int ARGUMENT=30;
    public static final int OP_NOT=48;
    public static final int OP_EQUIV=75;
    public static final int OP_SET_SOME=79;
    public static final int UNARY=15;
    public static final int OP_NEQUIV=76;
    public static final int T__210=210;
    public static final int T__212=212;
    public static final int T__211=211;
    public static final int HexLiteral=125;
    public static final int MDECLS=43;
    public static final int OP_DIVIDE=73;
    public static final int THROW_VAR=40;
    public static final int StringLiteral=124;
    public static final int OP_SET_COMPREHENSION=86;
    public static final int OP_MINUS_OR_DIFFERENCE=69;
    public static final int PROJECTION=21;
    public static final int OctalEscape=134;
    public static final int CAST=33;
    public static final int OP_BIT_AND_OR_INTERSECTION=52;
    public static final int JOIN_REFLEXIVE=23;
    public static final int OP_AND=47;
    public static final int FloatTypeSuffix=131;
    public static final int OctalLiteral=126;
    public static final int BRACKET=24;
    public static final int OP_RELATIONAL_COMPOSE=71;
    public static final int OP_BIT_XOR=50;
    public static final int LIT_NULL=93;
    public static final int MODIFIES=117;
    public static final int Identifier=121;
    public static final int TYPE_DOUBLE=101;
    public static final int QUANTIFY=16;
    public static final int OP_MINUS=67;
    public static final int VOID=38;
    public static final int OP_LT=57;
    public static final int OP_CLOSURE=55;
    public static final int DECL_NONE=11;
    public static final int OP_SET_SUM=83;
    public static final int ARGUMENTS=19;
    public static final int NULL=6;
    public static final int CONDITIONAL=18;
    public static final int INVARIANT=105;
    public static final int OLD=29;
    public static final int THROWS=118;
    public static final int FRAME_LOCATION=12;
    public static final int T__200=200;
    public static final int T__201=201;
    public static final int FIELD=32;
    public static final int OP_RELATIONAL_OVERRIDE=70;
    public static final int OP_PLUS_OR_UNION=66;
    public static final int Letter=135;
    public static final int EscapeSequence=132;
    public static final int TYPE_BOOLEAN=94;
    public static final int OP_TRANSPOSE=53;
    public static final int CharacterLiteral=123;
    public static final int SUPER_VAR=36;
    public static final int OP_GT=56;
    public static final int Exponent=130;
    public static final int OP_SET_COMPREHENSION_ENUM=87;
    public static final int LIT_TRUE=91;
    public static final int OP_BIT_NOT_OR_TRANSPOSE=54;
    public static final int T__199=199;
    public static final int TYPE_DECLARATION=108;
    public static final int PURE=120;
    public static final int T__198=198;
    public static final int T__197=197;
    public static final int T__196=196;
    public static final int T__195=195;
    public static final int T__194=194;
    public static final int FRAME_DOMAIN=28;
    public static final int T__193=193;
    public static final int T__192=192;
    public static final int T__191=191;
    public static final int T__190=190;
    public static final int IDENTIFIER=37;
    public static final int OP_BIT_OR=49;
    public static final int OP_SET_SUBSET=89;
    public static final int MOD_DISJ=7;
    public static final int SPECCASE=113;
    public static final int T__184=184;
    public static final int T__183=183;
    public static final int T__186=186;
    public static final int T__185=185;
    public static final int T__188=188;
    public static final int T__187=187;
    public static final int TYPE_SHORT=97;
    public static final int T__189=189;
    public static final int OP_PLUS=64;
    public static final int HELPER=119;
    public static final int T__180=180;
    public static final int TYPE_FLOAT=100;
    public static final int FRAME_FILTER=27;
    public static final int OP_SET_EXISTS=85;
    public static final int T__182=182;
    public static final int OP_NSET_SUBSET=90;
    public static final int T__181=181;
    public static final int DECLS=42;
    public static final int DecimalLiteral=127;
    public static final int JOIN=22;
    public static final int T__175=175;
    public static final int T__174=174;
    public static final int OP_SET_NUM=82;
    public static final int T__173=173;
    public static final int T__172=172;
    public static final int TYPE_ARRAY=103;
    public static final int T__179=179;
    public static final int REQUIRES=115;
    public static final int T__178=178;
    public static final int OP_SET_ONE=78;
    public static final int T__177=177;
    public static final int T__176=176;
    public static final int FRAME_FIELD=26;
    public static final int UnicodeEscape=133;
    public static final int OP_NEQ=45;
    public static final int T__171=171;
    public static final int T__170=170;
    public static final int FRAME=5;
    public static final int OP_LEQ=59;
    public static final int OP_TIMES=72;
    public static final int FRAME_ALL=13;
    public static final int BINARY=14;
    public static final int T__169=169;
    public static final int OP_UNION=65;
    public static final int METHOD=111;
    public static final int RETURN_VAR=39;
    public static final int OP_IMPLIES=77;

    // delegates
    // delegators


        public JFSLParser(TokenStream input) {
            this(input, new RecognizerSharedState());
        }
        public JFSLParser(TokenStream input, RecognizerSharedState state) {
            super(input, state);
            this.state.ruleMemo = new HashMap[225+1];
             
             
        }
        
    protected TreeAdaptor adaptor = new CommonTreeAdaptor();

    public void setTreeAdaptor(TreeAdaptor adaptor) {
        this.adaptor = adaptor;
    }
    public TreeAdaptor getTreeAdaptor() {
        return adaptor;
    }

    public String[] getTokenNames() { return JFSLParser.tokenNames; }
    public String getGrammarFileName() { return "JFSL.g"; }


    // Base type of nodes in the AST 
    public static final class Node extends CommonTree {
        // Used for refining ambiguous operator 
        private boolean flag;
        
        // Used for determining decision 
        private Decision decision;
        
        // Decision attributes 
        public JType jtype;
        public JField field;
        
        public Node(Token t) {
          super(t);
        }
        
        public Node(Node node) {
            super(node);
            this.flag = false;
            this.decision = null;
        }
        
        public void setFlag(boolean flag) { this.flag = flag; }    
        public boolean flag()             { return flag; }
        public void setJType(JType jtype) { this.jtype = jtype; }
        public Decision decision()        { return decision; }
                
        public void setDecision(Decision d) {
            assert this.decision == null || this.decision == d : "change of decision from " + this.decision + " to " + d;
            this.decision = d;
        }
        
        @Override
        public Tree dupNode() {
          return new Node(this);
        }       
    };

    // factory 
    public static final class NodeAdaptor extends CommonTreeAdaptor {
        @Override
        public Object create(Token payload) {
            return new Node(payload);
        }
    };

    // decision 
    public static enum Decision {LOCAL, GLOBAL, TYPE, FRAGMENT, CONST};

    public void displayRecognitionError(String[] tokenNames, RecognitionException e) {
        throw new JFSLParserException(e);
    }


    public static class clause_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "clause"
    // JFSL.g:396:1: clause : expression EOF ;
    public final JFSLParser.clause_return clause() throws RecognitionException {
        JFSLParser.clause_return retval = new JFSLParser.clause_return();
        retval.start = input.LT(1);
        int clause_StartIndex = input.index();
        Node root_0 = null;

        Token EOF2=null;
        JFSLParser.expression_return expression1 = null;


        Node EOF2_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 1) ) { return retval; }
            // JFSL.g:396:7: ( expression EOF )
            // JFSL.g:396:9: expression EOF
            {
            root_0 = (Node)adaptor.nil();

            pushFollow(FOLLOW_expression_in_clause785);
            expression1=expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, expression1.getTree());
            EOF2=(Token)match(input,EOF,FOLLOW_EOF_in_clause787); if (state.failed) return retval;

            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 1, clause_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "clause"

    public static class specField_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "specField"
    // JFSL.g:397:1: specField : declaration EOF ;
    public final JFSLParser.specField_return specField() throws RecognitionException {
        JFSLParser.specField_return retval = new JFSLParser.specField_return();
        retval.start = input.LT(1);
        int specField_StartIndex = input.index();
        Node root_0 = null;

        Token EOF4=null;
        JFSLParser.declaration_return declaration3 = null;


        Node EOF4_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 2) ) { return retval; }
            // JFSL.g:397:10: ( declaration EOF )
            // JFSL.g:397:12: declaration EOF
            {
            root_0 = (Node)adaptor.nil();

            pushFollow(FOLLOW_declaration_in_specField794);
            declaration3=declaration();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, declaration3.getTree());
            EOF4=(Token)match(input,EOF,FOLLOW_EOF_in_specField796); if (state.failed) return retval;

            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 2, specField_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "specField"

    public static class modifies_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "modifies"
    // JFSL.g:398:1: modifies : frame EOF ;
    public final JFSLParser.modifies_return modifies() throws RecognitionException {
        JFSLParser.modifies_return retval = new JFSLParser.modifies_return();
        retval.start = input.LT(1);
        int modifies_StartIndex = input.index();
        Node root_0 = null;

        Token EOF6=null;
        JFSLParser.frame_return frame5 = null;


        Node EOF6_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 3) ) { return retval; }
            // JFSL.g:398:9: ( frame EOF )
            // JFSL.g:398:11: frame EOF
            {
            root_0 = (Node)adaptor.nil();

            pushFollow(FOLLOW_frame_in_modifies803);
            frame5=frame();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, frame5.getTree());
            EOF6=(Token)match(input,EOF,FOLLOW_EOF_in_modifies805); if (state.failed) return retval;

            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 3, modifies_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "modifies"

    public static class compilationUnit_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "compilationUnit"
    // JFSL.g:403:1: compilationUnit : 'package' packageName ';' importDeclaration typeDeclaration ( ';' )* EOF -> ^( FILE packageName importDeclaration typeDeclaration ) ;
    public final JFSLParser.compilationUnit_return compilationUnit() throws RecognitionException {
        JFSLParser.compilationUnit_return retval = new JFSLParser.compilationUnit_return();
        retval.start = input.LT(1);
        int compilationUnit_StartIndex = input.index();
        Node root_0 = null;

        Token string_literal7=null;
        Token char_literal9=null;
        Token char_literal12=null;
        Token EOF13=null;
        JFSLParser.packageName_return packageName8 = null;

        JFSLParser.importDeclaration_return importDeclaration10 = null;

        JFSLParser.typeDeclaration_return typeDeclaration11 = null;


        Node string_literal7_tree=null;
        Node char_literal9_tree=null;
        Node char_literal12_tree=null;
        Node EOF13_tree=null;
        RewriteRuleTokenStream stream_138=new RewriteRuleTokenStream(adaptor,"token 138");
        RewriteRuleTokenStream stream_139=new RewriteRuleTokenStream(adaptor,"token 139");
        RewriteRuleTokenStream stream_EOF=new RewriteRuleTokenStream(adaptor,"token EOF");
        RewriteRuleSubtreeStream stream_packageName=new RewriteRuleSubtreeStream(adaptor,"rule packageName");
        RewriteRuleSubtreeStream stream_typeDeclaration=new RewriteRuleSubtreeStream(adaptor,"rule typeDeclaration");
        RewriteRuleSubtreeStream stream_importDeclaration=new RewriteRuleSubtreeStream(adaptor,"rule importDeclaration");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 4) ) { return retval; }
            // JFSL.g:404:3: ( 'package' packageName ';' importDeclaration typeDeclaration ( ';' )* EOF -> ^( FILE packageName importDeclaration typeDeclaration ) )
            // JFSL.g:404:5: 'package' packageName ';' importDeclaration typeDeclaration ( ';' )* EOF
            {
            string_literal7=(Token)match(input,138,FOLLOW_138_in_compilationUnit825); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_138.add(string_literal7);

            pushFollow(FOLLOW_packageName_in_compilationUnit827);
            packageName8=packageName();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_packageName.add(packageName8.getTree());
            char_literal9=(Token)match(input,139,FOLLOW_139_in_compilationUnit829); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_139.add(char_literal9);

            pushFollow(FOLLOW_importDeclaration_in_compilationUnit835);
            importDeclaration10=importDeclaration();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_importDeclaration.add(importDeclaration10.getTree());
            pushFollow(FOLLOW_typeDeclaration_in_compilationUnit841);
            typeDeclaration11=typeDeclaration();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_typeDeclaration.add(typeDeclaration11.getTree());
            // JFSL.g:406:21: ( ';' )*
            loop1:
            do {
                int alt1=2;
                int LA1_0 = input.LA(1);

                if ( (LA1_0==139) ) {
                    alt1=1;
                }


                switch (alt1) {
            	case 1 :
            	    // JFSL.g:0:0: ';'
            	    {
            	    char_literal12=(Token)match(input,139,FOLLOW_139_in_compilationUnit843); if (state.failed) return retval; 
            	    if ( state.backtracking==0 ) stream_139.add(char_literal12);


            	    }
            	    break;

            	default :
            	    break loop1;
                }
            } while (true);

            EOF13=(Token)match(input,EOF,FOLLOW_EOF_in_compilationUnit850); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_EOF.add(EOF13);



            // AST REWRITE
            // elements: importDeclaration, typeDeclaration, packageName
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 407:9: -> ^( FILE packageName importDeclaration typeDeclaration )
            {
                // JFSL.g:407:12: ^( FILE packageName importDeclaration typeDeclaration )
                {
                Node root_1 = (Node)adaptor.nil();
                root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(FILE, "FILE"), root_1);

                adaptor.addChild(root_1, stream_packageName.nextTree());
                adaptor.addChild(root_1, stream_importDeclaration.nextTree());
                adaptor.addChild(root_1, stream_typeDeclaration.nextTree());

                adaptor.addChild(root_0, root_1);
                }

            }

            retval.tree = root_0;}
            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 4, compilationUnit_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "compilationUnit"

    public static class packageName_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "packageName"
    // JFSL.g:410:1: packageName : Identifier ( '.' Identifier )* -> ^( PACKAGE ( Identifier )+ ) ;
    public final JFSLParser.packageName_return packageName() throws RecognitionException {
        JFSLParser.packageName_return retval = new JFSLParser.packageName_return();
        retval.start = input.LT(1);
        int packageName_StartIndex = input.index();
        Node root_0 = null;

        Token Identifier14=null;
        Token char_literal15=null;
        Token Identifier16=null;

        Node Identifier14_tree=null;
        Node char_literal15_tree=null;
        Node Identifier16_tree=null;
        RewriteRuleTokenStream stream_Identifier=new RewriteRuleTokenStream(adaptor,"token Identifier");
        RewriteRuleTokenStream stream_140=new RewriteRuleTokenStream(adaptor,"token 140");

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 5) ) { return retval; }
            // JFSL.g:411:3: ( Identifier ( '.' Identifier )* -> ^( PACKAGE ( Identifier )+ ) )
            // JFSL.g:411:5: Identifier ( '.' Identifier )*
            {
            Identifier14=(Token)match(input,Identifier,FOLLOW_Identifier_in_packageName875); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_Identifier.add(Identifier14);

            // JFSL.g:411:16: ( '.' Identifier )*
            loop2:
            do {
                int alt2=2;
                int LA2_0 = input.LA(1);

                if ( (LA2_0==140) ) {
                    alt2=1;
                }


                switch (alt2) {
            	case 1 :
            	    // JFSL.g:411:17: '.' Identifier
            	    {
            	    char_literal15=(Token)match(input,140,FOLLOW_140_in_packageName878); if (state.failed) return retval; 
            	    if ( state.backtracking==0 ) stream_140.add(char_literal15);

            	    Identifier16=(Token)match(input,Identifier,FOLLOW_Identifier_in_packageName880); if (state.failed) return retval; 
            	    if ( state.backtracking==0 ) stream_Identifier.add(Identifier16);


            	    }
            	    break;

            	default :
            	    break loop2;
                }
            } while (true);



            // AST REWRITE
            // elements: Identifier
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 411:34: -> ^( PACKAGE ( Identifier )+ )
            {
                // JFSL.g:411:37: ^( PACKAGE ( Identifier )+ )
                {
                Node root_1 = (Node)adaptor.nil();
                root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(PACKAGE, "PACKAGE"), root_1);

                if ( !(stream_Identifier.hasNext()) ) {
                    throw new RewriteEarlyExitException();
                }
                while ( stream_Identifier.hasNext() ) {
                    adaptor.addChild(root_1, stream_Identifier.nextNode());

                }
                stream_Identifier.reset();

                adaptor.addChild(root_0, root_1);
                }

            }

            retval.tree = root_0;}
            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 5, packageName_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "packageName"

    public static class importDeclaration_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "importDeclaration"
    // JFSL.g:414:1: importDeclaration : ( 'import' packageName ';' )* -> ^( IMPORT ( packageName )* ) ;
    public final JFSLParser.importDeclaration_return importDeclaration() throws RecognitionException {
        JFSLParser.importDeclaration_return retval = new JFSLParser.importDeclaration_return();
        retval.start = input.LT(1);
        int importDeclaration_StartIndex = input.index();
        Node root_0 = null;

        Token string_literal17=null;
        Token char_literal19=null;
        JFSLParser.packageName_return packageName18 = null;


        Node string_literal17_tree=null;
        Node char_literal19_tree=null;
        RewriteRuleTokenStream stream_139=new RewriteRuleTokenStream(adaptor,"token 139");
        RewriteRuleTokenStream stream_141=new RewriteRuleTokenStream(adaptor,"token 141");
        RewriteRuleSubtreeStream stream_packageName=new RewriteRuleSubtreeStream(adaptor,"rule packageName");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 6) ) { return retval; }
            // JFSL.g:415:3: ( ( 'import' packageName ';' )* -> ^( IMPORT ( packageName )* ) )
            // JFSL.g:415:5: ( 'import' packageName ';' )*
            {
            // JFSL.g:415:5: ( 'import' packageName ';' )*
            loop3:
            do {
                int alt3=2;
                int LA3_0 = input.LA(1);

                if ( (LA3_0==141) ) {
                    alt3=1;
                }


                switch (alt3) {
            	case 1 :
            	    // JFSL.g:415:6: 'import' packageName ';'
            	    {
            	    string_literal17=(Token)match(input,141,FOLLOW_141_in_importDeclaration908); if (state.failed) return retval; 
            	    if ( state.backtracking==0 ) stream_141.add(string_literal17);

            	    pushFollow(FOLLOW_packageName_in_importDeclaration910);
            	    packageName18=packageName();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_packageName.add(packageName18.getTree());
            	    char_literal19=(Token)match(input,139,FOLLOW_139_in_importDeclaration912); if (state.failed) return retval; 
            	    if ( state.backtracking==0 ) stream_139.add(char_literal19);


            	    }
            	    break;

            	default :
            	    break loop3;
                }
            } while (true);



            // AST REWRITE
            // elements: packageName
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 415:33: -> ^( IMPORT ( packageName )* )
            {
                // JFSL.g:415:36: ^( IMPORT ( packageName )* )
                {
                Node root_1 = (Node)adaptor.nil();
                root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(IMPORT, "IMPORT"), root_1);

                // JFSL.g:415:45: ( packageName )*
                while ( stream_packageName.hasNext() ) {
                    adaptor.addChild(root_1, stream_packageName.nextTree());

                }
                stream_packageName.reset();

                adaptor.addChild(root_0, root_1);
                }

            }

            retval.tree = root_0;}
            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 6, importDeclaration_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "importDeclaration"

    public static class typeDeclaration_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "typeDeclaration"
    // JFSL.g:418:1: typeDeclaration : ( 'class' | 'interface' ) Identifier ( typeParameters )? '{' ( typeBodyDeclaration )* '}' -> ^( TYPE_DECLARATION Identifier ^( TYPE_PARAMETERS ( typeParameters )? ) ( typeBodyDeclaration )* ) ;
    public final JFSLParser.typeDeclaration_return typeDeclaration() throws RecognitionException {
        JFSLParser.typeDeclaration_return retval = new JFSLParser.typeDeclaration_return();
        retval.start = input.LT(1);
        int typeDeclaration_StartIndex = input.index();
        Node root_0 = null;

        Token string_literal20=null;
        Token string_literal21=null;
        Token Identifier22=null;
        Token char_literal24=null;
        Token char_literal26=null;
        JFSLParser.typeParameters_return typeParameters23 = null;

        JFSLParser.typeBodyDeclaration_return typeBodyDeclaration25 = null;


        Node string_literal20_tree=null;
        Node string_literal21_tree=null;
        Node Identifier22_tree=null;
        Node char_literal24_tree=null;
        Node char_literal26_tree=null;
        RewriteRuleTokenStream stream_143=new RewriteRuleTokenStream(adaptor,"token 143");
        RewriteRuleTokenStream stream_144=new RewriteRuleTokenStream(adaptor,"token 144");
        RewriteRuleTokenStream stream_145=new RewriteRuleTokenStream(adaptor,"token 145");
        RewriteRuleTokenStream stream_Identifier=new RewriteRuleTokenStream(adaptor,"token Identifier");
        RewriteRuleTokenStream stream_142=new RewriteRuleTokenStream(adaptor,"token 142");
        RewriteRuleSubtreeStream stream_typeParameters=new RewriteRuleSubtreeStream(adaptor,"rule typeParameters");
        RewriteRuleSubtreeStream stream_typeBodyDeclaration=new RewriteRuleSubtreeStream(adaptor,"rule typeBodyDeclaration");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 7) ) { return retval; }
            // JFSL.g:419:3: ( ( 'class' | 'interface' ) Identifier ( typeParameters )? '{' ( typeBodyDeclaration )* '}' -> ^( TYPE_DECLARATION Identifier ^( TYPE_PARAMETERS ( typeParameters )? ) ( typeBodyDeclaration )* ) )
            // JFSL.g:419:5: ( 'class' | 'interface' ) Identifier ( typeParameters )? '{' ( typeBodyDeclaration )* '}'
            {
            // JFSL.g:419:5: ( 'class' | 'interface' )
            int alt4=2;
            int LA4_0 = input.LA(1);

            if ( (LA4_0==142) ) {
                alt4=1;
            }
            else if ( (LA4_0==143) ) {
                alt4=2;
            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 4, 0, input);

                throw nvae;
            }
            switch (alt4) {
                case 1 :
                    // JFSL.g:419:6: 'class'
                    {
                    string_literal20=(Token)match(input,142,FOLLOW_142_in_typeDeclaration939); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_142.add(string_literal20);


                    }
                    break;
                case 2 :
                    // JFSL.g:419:16: 'interface'
                    {
                    string_literal21=(Token)match(input,143,FOLLOW_143_in_typeDeclaration943); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_143.add(string_literal21);


                    }
                    break;

            }

            Identifier22=(Token)match(input,Identifier,FOLLOW_Identifier_in_typeDeclaration946); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_Identifier.add(Identifier22);

            // JFSL.g:419:40: ( typeParameters )?
            int alt5=2;
            int LA5_0 = input.LA(1);

            if ( (LA5_0==146) ) {
                alt5=1;
            }
            switch (alt5) {
                case 1 :
                    // JFSL.g:0:0: typeParameters
                    {
                    pushFollow(FOLLOW_typeParameters_in_typeDeclaration948);
                    typeParameters23=typeParameters();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_typeParameters.add(typeParameters23.getTree());

                    }
                    break;

            }

            char_literal24=(Token)match(input,144,FOLLOW_144_in_typeDeclaration955); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_144.add(char_literal24);

            // JFSL.g:420:9: ( typeBodyDeclaration )*
            loop6:
            do {
                int alt6=2;
                int LA6_0 = input.LA(1);

                if ( (LA6_0==Identifier||LA6_0==139||(LA6_0>=142 && LA6_0<=143)||LA6_0==149||LA6_0==152) ) {
                    alt6=1;
                }


                switch (alt6) {
            	case 1 :
            	    // JFSL.g:0:0: typeBodyDeclaration
            	    {
            	    pushFollow(FOLLOW_typeBodyDeclaration_in_typeDeclaration957);
            	    typeBodyDeclaration25=typeBodyDeclaration();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_typeBodyDeclaration.add(typeBodyDeclaration25.getTree());

            	    }
            	    break;

            	default :
            	    break loop6;
                }
            } while (true);

            char_literal26=(Token)match(input,145,FOLLOW_145_in_typeDeclaration960); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_145.add(char_literal26);



            // AST REWRITE
            // elements: Identifier, typeParameters, typeBodyDeclaration
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 420:34: -> ^( TYPE_DECLARATION Identifier ^( TYPE_PARAMETERS ( typeParameters )? ) ( typeBodyDeclaration )* )
            {
                // JFSL.g:420:37: ^( TYPE_DECLARATION Identifier ^( TYPE_PARAMETERS ( typeParameters )? ) ( typeBodyDeclaration )* )
                {
                Node root_1 = (Node)adaptor.nil();
                root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(TYPE_DECLARATION, "TYPE_DECLARATION"), root_1);

                adaptor.addChild(root_1, stream_Identifier.nextNode());
                // JFSL.g:420:67: ^( TYPE_PARAMETERS ( typeParameters )? )
                {
                Node root_2 = (Node)adaptor.nil();
                root_2 = (Node)adaptor.becomeRoot((Node)adaptor.create(TYPE_PARAMETERS, "TYPE_PARAMETERS"), root_2);

                // JFSL.g:420:85: ( typeParameters )?
                if ( stream_typeParameters.hasNext() ) {
                    adaptor.addChild(root_2, stream_typeParameters.nextTree());

                }
                stream_typeParameters.reset();

                adaptor.addChild(root_1, root_2);
                }
                // JFSL.g:420:102: ( typeBodyDeclaration )*
                while ( stream_typeBodyDeclaration.hasNext() ) {
                    adaptor.addChild(root_1, stream_typeBodyDeclaration.nextTree());

                }
                stream_typeBodyDeclaration.reset();

                adaptor.addChild(root_0, root_1);
                }

            }

            retval.tree = root_0;}
            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 7, typeDeclaration_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "typeDeclaration"

    public static class typeParameters_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "typeParameters"
    // JFSL.g:423:1: typeParameters : '<' Identifier ( ',' Identifier )* '>' -> ( Identifier )+ ;
    public final JFSLParser.typeParameters_return typeParameters() throws RecognitionException {
        JFSLParser.typeParameters_return retval = new JFSLParser.typeParameters_return();
        retval.start = input.LT(1);
        int typeParameters_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal27=null;
        Token Identifier28=null;
        Token char_literal29=null;
        Token Identifier30=null;
        Token char_literal31=null;

        Node char_literal27_tree=null;
        Node Identifier28_tree=null;
        Node char_literal29_tree=null;
        Node Identifier30_tree=null;
        Node char_literal31_tree=null;
        RewriteRuleTokenStream stream_146=new RewriteRuleTokenStream(adaptor,"token 146");
        RewriteRuleTokenStream stream_147=new RewriteRuleTokenStream(adaptor,"token 147");
        RewriteRuleTokenStream stream_148=new RewriteRuleTokenStream(adaptor,"token 148");
        RewriteRuleTokenStream stream_Identifier=new RewriteRuleTokenStream(adaptor,"token Identifier");

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 8) ) { return retval; }
            // JFSL.g:424:3: ( '<' Identifier ( ',' Identifier )* '>' -> ( Identifier )+ )
            // JFSL.g:424:5: '<' Identifier ( ',' Identifier )* '>'
            {
            char_literal27=(Token)match(input,146,FOLLOW_146_in_typeParameters991); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_146.add(char_literal27);

            Identifier28=(Token)match(input,Identifier,FOLLOW_Identifier_in_typeParameters993); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_Identifier.add(Identifier28);

            // JFSL.g:424:20: ( ',' Identifier )*
            loop7:
            do {
                int alt7=2;
                int LA7_0 = input.LA(1);

                if ( (LA7_0==147) ) {
                    alt7=1;
                }


                switch (alt7) {
            	case 1 :
            	    // JFSL.g:424:21: ',' Identifier
            	    {
            	    char_literal29=(Token)match(input,147,FOLLOW_147_in_typeParameters996); if (state.failed) return retval; 
            	    if ( state.backtracking==0 ) stream_147.add(char_literal29);

            	    Identifier30=(Token)match(input,Identifier,FOLLOW_Identifier_in_typeParameters998); if (state.failed) return retval; 
            	    if ( state.backtracking==0 ) stream_Identifier.add(Identifier30);


            	    }
            	    break;

            	default :
            	    break loop7;
                }
            } while (true);

            char_literal31=(Token)match(input,148,FOLLOW_148_in_typeParameters1002); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_148.add(char_literal31);



            // AST REWRITE
            // elements: Identifier
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 424:42: -> ( Identifier )+
            {
                if ( !(stream_Identifier.hasNext()) ) {
                    throw new RewriteEarlyExitException();
                }
                while ( stream_Identifier.hasNext() ) {
                    adaptor.addChild(root_0, stream_Identifier.nextNode());

                }
                stream_Identifier.reset();

            }

            retval.tree = root_0;}
            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 8, typeParameters_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "typeParameters"

    public static class typeParameters2_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "typeParameters2"
    // JFSL.g:427:1: typeParameters2 : '<' primary ( ',' primary )* '>' -> ( primary )+ ;
    public final JFSLParser.typeParameters2_return typeParameters2() throws RecognitionException {
        JFSLParser.typeParameters2_return retval = new JFSLParser.typeParameters2_return();
        retval.start = input.LT(1);
        int typeParameters2_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal32=null;
        Token char_literal34=null;
        Token char_literal36=null;
        JFSLParser.primary_return primary33 = null;

        JFSLParser.primary_return primary35 = null;


        Node char_literal32_tree=null;
        Node char_literal34_tree=null;
        Node char_literal36_tree=null;
        RewriteRuleTokenStream stream_146=new RewriteRuleTokenStream(adaptor,"token 146");
        RewriteRuleTokenStream stream_147=new RewriteRuleTokenStream(adaptor,"token 147");
        RewriteRuleTokenStream stream_148=new RewriteRuleTokenStream(adaptor,"token 148");
        RewriteRuleSubtreeStream stream_primary=new RewriteRuleSubtreeStream(adaptor,"rule primary");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 9) ) { return retval; }
            // JFSL.g:428:3: ( '<' primary ( ',' primary )* '>' -> ( primary )+ )
            // JFSL.g:428:5: '<' primary ( ',' primary )* '>'
            {
            char_literal32=(Token)match(input,146,FOLLOW_146_in_typeParameters21024); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_146.add(char_literal32);

            pushFollow(FOLLOW_primary_in_typeParameters21026);
            primary33=primary();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_primary.add(primary33.getTree());
            // JFSL.g:428:17: ( ',' primary )*
            loop8:
            do {
                int alt8=2;
                int LA8_0 = input.LA(1);

                if ( (LA8_0==147) ) {
                    alt8=1;
                }


                switch (alt8) {
            	case 1 :
            	    // JFSL.g:428:18: ',' primary
            	    {
            	    char_literal34=(Token)match(input,147,FOLLOW_147_in_typeParameters21029); if (state.failed) return retval; 
            	    if ( state.backtracking==0 ) stream_147.add(char_literal34);

            	    pushFollow(FOLLOW_primary_in_typeParameters21031);
            	    primary35=primary();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_primary.add(primary35.getTree());

            	    }
            	    break;

            	default :
            	    break loop8;
                }
            } while (true);

            char_literal36=(Token)match(input,148,FOLLOW_148_in_typeParameters21035); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_148.add(char_literal36);



            // AST REWRITE
            // elements: primary
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 428:36: -> ( primary )+
            {
                if ( !(stream_primary.hasNext()) ) {
                    throw new RewriteEarlyExitException();
                }
                while ( stream_primary.hasNext() ) {
                    adaptor.addChild(root_0, stream_primary.nextTree());

                }
                stream_primary.reset();

            }

            retval.tree = root_0;}
            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 9, typeParameters2_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "typeParameters2"

    public static class typeBodyDeclaration_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "typeBodyDeclaration"
    // JFSL.g:431:1: typeBodyDeclaration : ( ';' | typeDeclaration | '@Invariant' '(' expression ')' -> ^( INVARIANT expression ) | '@SpecField' '(' declaration ')' -> ^( SPECFIELD declaration ) | Identifier '(' methodParameters ')' '{' specCase ( ';' specCase )* ( ';' )* '}' -> ^( METHOD Identifier methodParameters ^( SPECIFICATION ( specCase )+ ) ) );
    public final JFSLParser.typeBodyDeclaration_return typeBodyDeclaration() throws RecognitionException {
        JFSLParser.typeBodyDeclaration_return retval = new JFSLParser.typeBodyDeclaration_return();
        retval.start = input.LT(1);
        int typeBodyDeclaration_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal37=null;
        Token string_literal39=null;
        Token char_literal40=null;
        Token char_literal42=null;
        Token string_literal43=null;
        Token char_literal44=null;
        Token char_literal46=null;
        Token Identifier47=null;
        Token char_literal48=null;
        Token char_literal50=null;
        Token char_literal51=null;
        Token char_literal53=null;
        Token char_literal55=null;
        Token char_literal56=null;
        JFSLParser.typeDeclaration_return typeDeclaration38 = null;

        JFSLParser.expression_return expression41 = null;

        JFSLParser.declaration_return declaration45 = null;

        JFSLParser.methodParameters_return methodParameters49 = null;

        JFSLParser.specCase_return specCase52 = null;

        JFSLParser.specCase_return specCase54 = null;


        Node char_literal37_tree=null;
        Node string_literal39_tree=null;
        Node char_literal40_tree=null;
        Node char_literal42_tree=null;
        Node string_literal43_tree=null;
        Node char_literal44_tree=null;
        Node char_literal46_tree=null;
        Node Identifier47_tree=null;
        Node char_literal48_tree=null;
        Node char_literal50_tree=null;
        Node char_literal51_tree=null;
        Node char_literal53_tree=null;
        Node char_literal55_tree=null;
        Node char_literal56_tree=null;
        RewriteRuleTokenStream stream_152=new RewriteRuleTokenStream(adaptor,"token 152");
        RewriteRuleTokenStream stream_144=new RewriteRuleTokenStream(adaptor,"token 144");
        RewriteRuleTokenStream stream_145=new RewriteRuleTokenStream(adaptor,"token 145");
        RewriteRuleTokenStream stream_150=new RewriteRuleTokenStream(adaptor,"token 150");
        RewriteRuleTokenStream stream_151=new RewriteRuleTokenStream(adaptor,"token 151");
        RewriteRuleTokenStream stream_139=new RewriteRuleTokenStream(adaptor,"token 139");
        RewriteRuleTokenStream stream_149=new RewriteRuleTokenStream(adaptor,"token 149");
        RewriteRuleTokenStream stream_Identifier=new RewriteRuleTokenStream(adaptor,"token Identifier");
        RewriteRuleSubtreeStream stream_expression=new RewriteRuleSubtreeStream(adaptor,"rule expression");
        RewriteRuleSubtreeStream stream_methodParameters=new RewriteRuleSubtreeStream(adaptor,"rule methodParameters");
        RewriteRuleSubtreeStream stream_specCase=new RewriteRuleSubtreeStream(adaptor,"rule specCase");
        RewriteRuleSubtreeStream stream_declaration=new RewriteRuleSubtreeStream(adaptor,"rule declaration");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 10) ) { return retval; }
            // JFSL.g:432:3: ( ';' | typeDeclaration | '@Invariant' '(' expression ')' -> ^( INVARIANT expression ) | '@SpecField' '(' declaration ')' -> ^( SPECFIELD declaration ) | Identifier '(' methodParameters ')' '{' specCase ( ';' specCase )* ( ';' )* '}' -> ^( METHOD Identifier methodParameters ^( SPECIFICATION ( specCase )+ ) ) )
            int alt11=5;
            switch ( input.LA(1) ) {
            case 139:
                {
                alt11=1;
                }
                break;
            case 142:
            case 143:
                {
                alt11=2;
                }
                break;
            case 149:
                {
                alt11=3;
                }
                break;
            case 152:
                {
                alt11=4;
                }
                break;
            case Identifier:
                {
                alt11=5;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 11, 0, input);

                throw nvae;
            }

            switch (alt11) {
                case 1 :
                    // JFSL.g:432:5: ';'
                    {
                    root_0 = (Node)adaptor.nil();

                    char_literal37=(Token)match(input,139,FOLLOW_139_in_typeBodyDeclaration1053); if (state.failed) return retval;

                    }
                    break;
                case 2 :
                    // JFSL.g:433:5: typeDeclaration
                    {
                    root_0 = (Node)adaptor.nil();

                    pushFollow(FOLLOW_typeDeclaration_in_typeBodyDeclaration1060);
                    typeDeclaration38=typeDeclaration();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, typeDeclaration38.getTree());

                    }
                    break;
                case 3 :
                    // JFSL.g:434:5: '@Invariant' '(' expression ')'
                    {
                    string_literal39=(Token)match(input,149,FOLLOW_149_in_typeBodyDeclaration1067); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_149.add(string_literal39);

                    char_literal40=(Token)match(input,150,FOLLOW_150_in_typeBodyDeclaration1069); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_150.add(char_literal40);

                    pushFollow(FOLLOW_expression_in_typeBodyDeclaration1071);
                    expression41=expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_expression.add(expression41.getTree());
                    char_literal42=(Token)match(input,151,FOLLOW_151_in_typeBodyDeclaration1073); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_151.add(char_literal42);



                    // AST REWRITE
                    // elements: expression
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 434:37: -> ^( INVARIANT expression )
                    {
                        // JFSL.g:434:40: ^( INVARIANT expression )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(INVARIANT, "INVARIANT"), root_1);

                        adaptor.addChild(root_1, stream_expression.nextTree());

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 4 :
                    // JFSL.g:435:5: '@SpecField' '(' declaration ')'
                    {
                    string_literal43=(Token)match(input,152,FOLLOW_152_in_typeBodyDeclaration1087); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_152.add(string_literal43);

                    char_literal44=(Token)match(input,150,FOLLOW_150_in_typeBodyDeclaration1089); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_150.add(char_literal44);

                    pushFollow(FOLLOW_declaration_in_typeBodyDeclaration1091);
                    declaration45=declaration();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_declaration.add(declaration45.getTree());
                    char_literal46=(Token)match(input,151,FOLLOW_151_in_typeBodyDeclaration1093); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_151.add(char_literal46);



                    // AST REWRITE
                    // elements: declaration
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 435:38: -> ^( SPECFIELD declaration )
                    {
                        // JFSL.g:435:41: ^( SPECFIELD declaration )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(SPECFIELD, "SPECFIELD"), root_1);

                        adaptor.addChild(root_1, stream_declaration.nextTree());

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 5 :
                    // JFSL.g:436:5: Identifier '(' methodParameters ')' '{' specCase ( ';' specCase )* ( ';' )* '}'
                    {
                    Identifier47=(Token)match(input,Identifier,FOLLOW_Identifier_in_typeBodyDeclaration1107); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_Identifier.add(Identifier47);

                    char_literal48=(Token)match(input,150,FOLLOW_150_in_typeBodyDeclaration1109); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_150.add(char_literal48);

                    pushFollow(FOLLOW_methodParameters_in_typeBodyDeclaration1111);
                    methodParameters49=methodParameters();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_methodParameters.add(methodParameters49.getTree());
                    char_literal50=(Token)match(input,151,FOLLOW_151_in_typeBodyDeclaration1113); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_151.add(char_literal50);

                    char_literal51=(Token)match(input,144,FOLLOW_144_in_typeBodyDeclaration1115); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_144.add(char_literal51);

                    pushFollow(FOLLOW_specCase_in_typeBodyDeclaration1117);
                    specCase52=specCase();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_specCase.add(specCase52.getTree());
                    // JFSL.g:436:54: ( ';' specCase )*
                    loop9:
                    do {
                        int alt9=2;
                        int LA9_0 = input.LA(1);

                        if ( (LA9_0==139) ) {
                            int LA9_1 = input.LA(2);

                            if ( (synpred13_JFSL()) ) {
                                alt9=1;
                            }


                        }


                        switch (alt9) {
                    	case 1 :
                    	    // JFSL.g:436:55: ';' specCase
                    	    {
                    	    char_literal53=(Token)match(input,139,FOLLOW_139_in_typeBodyDeclaration1120); if (state.failed) return retval; 
                    	    if ( state.backtracking==0 ) stream_139.add(char_literal53);

                    	    pushFollow(FOLLOW_specCase_in_typeBodyDeclaration1122);
                    	    specCase54=specCase();

                    	    state._fsp--;
                    	    if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) stream_specCase.add(specCase54.getTree());

                    	    }
                    	    break;

                    	default :
                    	    break loop9;
                        }
                    } while (true);

                    // JFSL.g:436:70: ( ';' )*
                    loop10:
                    do {
                        int alt10=2;
                        int LA10_0 = input.LA(1);

                        if ( (LA10_0==139) ) {
                            alt10=1;
                        }


                        switch (alt10) {
                    	case 1 :
                    	    // JFSL.g:436:71: ';'
                    	    {
                    	    char_literal55=(Token)match(input,139,FOLLOW_139_in_typeBodyDeclaration1127); if (state.failed) return retval; 
                    	    if ( state.backtracking==0 ) stream_139.add(char_literal55);


                    	    }
                    	    break;

                    	default :
                    	    break loop10;
                        }
                    } while (true);

                    char_literal56=(Token)match(input,145,FOLLOW_145_in_typeBodyDeclaration1131); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_145.add(char_literal56);



                    // AST REWRITE
                    // elements: Identifier, specCase, methodParameters
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 437:5: -> ^( METHOD Identifier methodParameters ^( SPECIFICATION ( specCase )+ ) )
                    {
                        // JFSL.g:437:8: ^( METHOD Identifier methodParameters ^( SPECIFICATION ( specCase )+ ) )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(METHOD, "METHOD"), root_1);

                        adaptor.addChild(root_1, stream_Identifier.nextNode());
                        adaptor.addChild(root_1, stream_methodParameters.nextTree());
                        // JFSL.g:437:45: ^( SPECIFICATION ( specCase )+ )
                        {
                        Node root_2 = (Node)adaptor.nil();
                        root_2 = (Node)adaptor.becomeRoot((Node)adaptor.create(SPECIFICATION, "SPECIFICATION"), root_2);

                        if ( !(stream_specCase.hasNext()) ) {
                            throw new RewriteEarlyExitException();
                        }
                        while ( stream_specCase.hasNext() ) {
                            adaptor.addChild(root_2, stream_specCase.nextTree());

                        }
                        stream_specCase.reset();

                        adaptor.addChild(root_1, root_2);
                        }

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 10, typeBodyDeclaration_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "typeBodyDeclaration"

    public static class methodParameters_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "methodParameters"
    // JFSL.g:440:1: methodParameters : ( type Identifier ( ',' type Identifier )* )? -> ^( METHOD_PARAMETERS ( type )* ( Identifier )* ) ;
    public final JFSLParser.methodParameters_return methodParameters() throws RecognitionException {
        JFSLParser.methodParameters_return retval = new JFSLParser.methodParameters_return();
        retval.start = input.LT(1);
        int methodParameters_StartIndex = input.index();
        Node root_0 = null;

        Token Identifier58=null;
        Token char_literal59=null;
        Token Identifier61=null;
        JFSLParser.type_return type57 = null;

        JFSLParser.type_return type60 = null;


        Node Identifier58_tree=null;
        Node char_literal59_tree=null;
        Node Identifier61_tree=null;
        RewriteRuleTokenStream stream_147=new RewriteRuleTokenStream(adaptor,"token 147");
        RewriteRuleTokenStream stream_Identifier=new RewriteRuleTokenStream(adaptor,"token Identifier");
        RewriteRuleSubtreeStream stream_type=new RewriteRuleSubtreeStream(adaptor,"rule type");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 11) ) { return retval; }
            // JFSL.g:441:3: ( ( type Identifier ( ',' type Identifier )* )? -> ^( METHOD_PARAMETERS ( type )* ( Identifier )* ) )
            // JFSL.g:441:5: ( type Identifier ( ',' type Identifier )* )?
            {
            // JFSL.g:441:5: ( type Identifier ( ',' type Identifier )* )?
            int alt13=2;
            int LA13_0 = input.LA(1);

            if ( (LA13_0==Identifier||(LA13_0>=171 && LA13_0<=178)) ) {
                alt13=1;
            }
            switch (alt13) {
                case 1 :
                    // JFSL.g:441:6: type Identifier ( ',' type Identifier )*
                    {
                    pushFollow(FOLLOW_type_in_methodParameters1169);
                    type57=type();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_type.add(type57.getTree());
                    Identifier58=(Token)match(input,Identifier,FOLLOW_Identifier_in_methodParameters1171); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_Identifier.add(Identifier58);

                    // JFSL.g:441:22: ( ',' type Identifier )*
                    loop12:
                    do {
                        int alt12=2;
                        int LA12_0 = input.LA(1);

                        if ( (LA12_0==147) ) {
                            alt12=1;
                        }


                        switch (alt12) {
                    	case 1 :
                    	    // JFSL.g:441:23: ',' type Identifier
                    	    {
                    	    char_literal59=(Token)match(input,147,FOLLOW_147_in_methodParameters1174); if (state.failed) return retval; 
                    	    if ( state.backtracking==0 ) stream_147.add(char_literal59);

                    	    pushFollow(FOLLOW_type_in_methodParameters1176);
                    	    type60=type();

                    	    state._fsp--;
                    	    if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) stream_type.add(type60.getTree());
                    	    Identifier61=(Token)match(input,Identifier,FOLLOW_Identifier_in_methodParameters1178); if (state.failed) return retval; 
                    	    if ( state.backtracking==0 ) stream_Identifier.add(Identifier61);


                    	    }
                    	    break;

                    	default :
                    	    break loop12;
                        }
                    } while (true);


                    }
                    break;

            }



            // AST REWRITE
            // elements: Identifier, type
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 441:47: -> ^( METHOD_PARAMETERS ( type )* ( Identifier )* )
            {
                // JFSL.g:441:50: ^( METHOD_PARAMETERS ( type )* ( Identifier )* )
                {
                Node root_1 = (Node)adaptor.nil();
                root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(METHOD_PARAMETERS, "METHOD_PARAMETERS"), root_1);

                // JFSL.g:441:70: ( type )*
                while ( stream_type.hasNext() ) {
                    adaptor.addChild(root_1, stream_type.nextTree());

                }
                stream_type.reset();
                // JFSL.g:441:76: ( Identifier )*
                while ( stream_Identifier.hasNext() ) {
                    adaptor.addChild(root_1, stream_Identifier.nextNode());

                }
                stream_Identifier.reset();

                adaptor.addChild(root_0, root_1);
                }

            }

            retval.tree = root_0;}
            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 11, methodParameters_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "methodParameters"

    public static class specCase_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "specCase"
    // JFSL.g:444:1: specCase : ( ( '@Requires' '(' requires= expression ')' )? ( '@Ensures' '(' ensures= expression ')' | ( '@Throws' '(' except= expression ')' ) )? ( '@Modifies' '(' changes= frame ')' )? -> ^( SPECCASE ^( REQUIRES ( $requires)? ) ^( ENSURES ( $ensures)? ) ^( THROWS ( $except)? ) ^( MODIFIES ( $changes)? ) ) | '@Helper' -> ^( HELPER ) | '@Pure' -> ^( PURE ) );
    public final JFSLParser.specCase_return specCase() throws RecognitionException {
        JFSLParser.specCase_return retval = new JFSLParser.specCase_return();
        retval.start = input.LT(1);
        int specCase_StartIndex = input.index();
        Node root_0 = null;

        Token string_literal62=null;
        Token char_literal63=null;
        Token char_literal64=null;
        Token string_literal65=null;
        Token char_literal66=null;
        Token char_literal67=null;
        Token string_literal68=null;
        Token char_literal69=null;
        Token char_literal70=null;
        Token string_literal71=null;
        Token char_literal72=null;
        Token char_literal73=null;
        Token string_literal74=null;
        Token string_literal75=null;
        JFSLParser.expression_return requires = null;

        JFSLParser.expression_return ensures = null;

        JFSLParser.expression_return except = null;

        JFSLParser.frame_return changes = null;


        Node string_literal62_tree=null;
        Node char_literal63_tree=null;
        Node char_literal64_tree=null;
        Node string_literal65_tree=null;
        Node char_literal66_tree=null;
        Node char_literal67_tree=null;
        Node string_literal68_tree=null;
        Node char_literal69_tree=null;
        Node char_literal70_tree=null;
        Node string_literal71_tree=null;
        Node char_literal72_tree=null;
        Node char_literal73_tree=null;
        Node string_literal74_tree=null;
        Node string_literal75_tree=null;
        RewriteRuleTokenStream stream_153=new RewriteRuleTokenStream(adaptor,"token 153");
        RewriteRuleTokenStream stream_150=new RewriteRuleTokenStream(adaptor,"token 150");
        RewriteRuleTokenStream stream_151=new RewriteRuleTokenStream(adaptor,"token 151");
        RewriteRuleTokenStream stream_158=new RewriteRuleTokenStream(adaptor,"token 158");
        RewriteRuleTokenStream stream_157=new RewriteRuleTokenStream(adaptor,"token 157");
        RewriteRuleTokenStream stream_156=new RewriteRuleTokenStream(adaptor,"token 156");
        RewriteRuleTokenStream stream_155=new RewriteRuleTokenStream(adaptor,"token 155");
        RewriteRuleTokenStream stream_154=new RewriteRuleTokenStream(adaptor,"token 154");
        RewriteRuleSubtreeStream stream_expression=new RewriteRuleSubtreeStream(adaptor,"rule expression");
        RewriteRuleSubtreeStream stream_frame=new RewriteRuleSubtreeStream(adaptor,"rule frame");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 12) ) { return retval; }
            // JFSL.g:445:3: ( ( '@Requires' '(' requires= expression ')' )? ( '@Ensures' '(' ensures= expression ')' | ( '@Throws' '(' except= expression ')' ) )? ( '@Modifies' '(' changes= frame ')' )? -> ^( SPECCASE ^( REQUIRES ( $requires)? ) ^( ENSURES ( $ensures)? ) ^( THROWS ( $except)? ) ^( MODIFIES ( $changes)? ) ) | '@Helper' -> ^( HELPER ) | '@Pure' -> ^( PURE ) )
            int alt17=3;
            switch ( input.LA(1) ) {
            case EOF:
            case 139:
            case 145:
            case 153:
            case 154:
            case 155:
            case 156:
                {
                alt17=1;
                }
                break;
            case 157:
                {
                alt17=2;
                }
                break;
            case 158:
                {
                alt17=3;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 17, 0, input);

                throw nvae;
            }

            switch (alt17) {
                case 1 :
                    // JFSL.g:445:5: ( '@Requires' '(' requires= expression ')' )? ( '@Ensures' '(' ensures= expression ')' | ( '@Throws' '(' except= expression ')' ) )? ( '@Modifies' '(' changes= frame ')' )?
                    {
                    // JFSL.g:445:5: ( '@Requires' '(' requires= expression ')' )?
                    int alt14=2;
                    int LA14_0 = input.LA(1);

                    if ( (LA14_0==153) ) {
                        alt14=1;
                    }
                    switch (alt14) {
                        case 1 :
                            // JFSL.g:445:6: '@Requires' '(' requires= expression ')'
                            {
                            string_literal62=(Token)match(input,153,FOLLOW_153_in_specCase1214); if (state.failed) return retval; 
                            if ( state.backtracking==0 ) stream_153.add(string_literal62);

                            char_literal63=(Token)match(input,150,FOLLOW_150_in_specCase1216); if (state.failed) return retval; 
                            if ( state.backtracking==0 ) stream_150.add(char_literal63);

                            pushFollow(FOLLOW_expression_in_specCase1222);
                            requires=expression();

                            state._fsp--;
                            if (state.failed) return retval;
                            if ( state.backtracking==0 ) stream_expression.add(requires.getTree());
                            char_literal64=(Token)match(input,151,FOLLOW_151_in_specCase1224); if (state.failed) return retval; 
                            if ( state.backtracking==0 ) stream_151.add(char_literal64);


                            }
                            break;

                    }

                    // JFSL.g:446:5: ( '@Ensures' '(' ensures= expression ')' | ( '@Throws' '(' except= expression ')' ) )?
                    int alt15=3;
                    int LA15_0 = input.LA(1);

                    if ( (LA15_0==154) ) {
                        alt15=1;
                    }
                    else if ( (LA15_0==155) ) {
                        alt15=2;
                    }
                    switch (alt15) {
                        case 1 :
                            // JFSL.g:446:6: '@Ensures' '(' ensures= expression ')'
                            {
                            string_literal65=(Token)match(input,154,FOLLOW_154_in_specCase1235); if (state.failed) return retval; 
                            if ( state.backtracking==0 ) stream_154.add(string_literal65);

                            char_literal66=(Token)match(input,150,FOLLOW_150_in_specCase1237); if (state.failed) return retval; 
                            if ( state.backtracking==0 ) stream_150.add(char_literal66);

                            pushFollow(FOLLOW_expression_in_specCase1243);
                            ensures=expression();

                            state._fsp--;
                            if (state.failed) return retval;
                            if ( state.backtracking==0 ) stream_expression.add(ensures.getTree());
                            char_literal67=(Token)match(input,151,FOLLOW_151_in_specCase1245); if (state.failed) return retval; 
                            if ( state.backtracking==0 ) stream_151.add(char_literal67);


                            }
                            break;
                        case 2 :
                            // JFSL.g:446:48: ( '@Throws' '(' except= expression ')' )
                            {
                            // JFSL.g:446:48: ( '@Throws' '(' except= expression ')' )
                            // JFSL.g:446:49: '@Throws' '(' except= expression ')'
                            {
                            string_literal68=(Token)match(input,155,FOLLOW_155_in_specCase1250); if (state.failed) return retval; 
                            if ( state.backtracking==0 ) stream_155.add(string_literal68);

                            char_literal69=(Token)match(input,150,FOLLOW_150_in_specCase1252); if (state.failed) return retval; 
                            if ( state.backtracking==0 ) stream_150.add(char_literal69);

                            pushFollow(FOLLOW_expression_in_specCase1258);
                            except=expression();

                            state._fsp--;
                            if (state.failed) return retval;
                            if ( state.backtracking==0 ) stream_expression.add(except.getTree());
                            char_literal70=(Token)match(input,151,FOLLOW_151_in_specCase1260); if (state.failed) return retval; 
                            if ( state.backtracking==0 ) stream_151.add(char_literal70);


                            }


                            }
                            break;

                    }

                    // JFSL.g:447:5: ( '@Modifies' '(' changes= frame ')' )?
                    int alt16=2;
                    int LA16_0 = input.LA(1);

                    if ( (LA16_0==156) ) {
                        alt16=1;
                    }
                    switch (alt16) {
                        case 1 :
                            // JFSL.g:447:6: '@Modifies' '(' changes= frame ')'
                            {
                            string_literal71=(Token)match(input,156,FOLLOW_156_in_specCase1271); if (state.failed) return retval; 
                            if ( state.backtracking==0 ) stream_156.add(string_literal71);

                            char_literal72=(Token)match(input,150,FOLLOW_150_in_specCase1273); if (state.failed) return retval; 
                            if ( state.backtracking==0 ) stream_150.add(char_literal72);

                            pushFollow(FOLLOW_frame_in_specCase1279);
                            changes=frame();

                            state._fsp--;
                            if (state.failed) return retval;
                            if ( state.backtracking==0 ) stream_frame.add(changes.getTree());
                            char_literal73=(Token)match(input,151,FOLLOW_151_in_specCase1281); if (state.failed) return retval; 
                            if ( state.backtracking==0 ) stream_151.add(char_literal73);


                            }
                            break;

                    }



                    // AST REWRITE
                    // elements: requires, ensures, except, changes
                    // token labels: 
                    // rule labels: ensures, retval, except, requires, changes
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_ensures=new RewriteRuleSubtreeStream(adaptor,"rule ensures",ensures!=null?ensures.tree:null);
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);
                    RewriteRuleSubtreeStream stream_except=new RewriteRuleSubtreeStream(adaptor,"rule except",except!=null?except.tree:null);
                    RewriteRuleSubtreeStream stream_requires=new RewriteRuleSubtreeStream(adaptor,"rule requires",requires!=null?requires.tree:null);
                    RewriteRuleSubtreeStream stream_changes=new RewriteRuleSubtreeStream(adaptor,"rule changes",changes!=null?changes.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 448:5: -> ^( SPECCASE ^( REQUIRES ( $requires)? ) ^( ENSURES ( $ensures)? ) ^( THROWS ( $except)? ) ^( MODIFIES ( $changes)? ) )
                    {
                        // JFSL.g:448:8: ^( SPECCASE ^( REQUIRES ( $requires)? ) ^( ENSURES ( $ensures)? ) ^( THROWS ( $except)? ) ^( MODIFIES ( $changes)? ) )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(SPECCASE, "SPECCASE"), root_1);

                        // JFSL.g:448:19: ^( REQUIRES ( $requires)? )
                        {
                        Node root_2 = (Node)adaptor.nil();
                        root_2 = (Node)adaptor.becomeRoot((Node)adaptor.create(REQUIRES, "REQUIRES"), root_2);

                        // JFSL.g:448:30: ( $requires)?
                        if ( stream_requires.hasNext() ) {
                            adaptor.addChild(root_2, stream_requires.nextTree());

                        }
                        stream_requires.reset();

                        adaptor.addChild(root_1, root_2);
                        }
                        // JFSL.g:448:42: ^( ENSURES ( $ensures)? )
                        {
                        Node root_2 = (Node)adaptor.nil();
                        root_2 = (Node)adaptor.becomeRoot((Node)adaptor.create(ENSURES, "ENSURES"), root_2);

                        // JFSL.g:448:52: ( $ensures)?
                        if ( stream_ensures.hasNext() ) {
                            adaptor.addChild(root_2, stream_ensures.nextTree());

                        }
                        stream_ensures.reset();

                        adaptor.addChild(root_1, root_2);
                        }
                        // JFSL.g:448:63: ^( THROWS ( $except)? )
                        {
                        Node root_2 = (Node)adaptor.nil();
                        root_2 = (Node)adaptor.becomeRoot((Node)adaptor.create(THROWS, "THROWS"), root_2);

                        // JFSL.g:448:72: ( $except)?
                        if ( stream_except.hasNext() ) {
                            adaptor.addChild(root_2, stream_except.nextTree());

                        }
                        stream_except.reset();

                        adaptor.addChild(root_1, root_2);
                        }
                        // JFSL.g:448:82: ^( MODIFIES ( $changes)? )
                        {
                        Node root_2 = (Node)adaptor.nil();
                        root_2 = (Node)adaptor.becomeRoot((Node)adaptor.create(MODIFIES, "MODIFIES"), root_2);

                        // JFSL.g:448:93: ( $changes)?
                        if ( stream_changes.hasNext() ) {
                            adaptor.addChild(root_2, stream_changes.nextTree());

                        }
                        stream_changes.reset();

                        adaptor.addChild(root_1, root_2);
                        }

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 2 :
                    // JFSL.g:449:5: '@Helper'
                    {
                    string_literal74=(Token)match(input,157,FOLLOW_157_in_specCase1332); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_157.add(string_literal74);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 449:15: -> ^( HELPER )
                    {
                        // JFSL.g:449:18: ^( HELPER )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(HELPER, "HELPER"), root_1);

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 3 :
                    // JFSL.g:450:5: '@Pure'
                    {
                    string_literal75=(Token)match(input,158,FOLLOW_158_in_specCase1347); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_158.add(string_literal75);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 450:13: -> ^( PURE )
                    {
                        // JFSL.g:450:16: ^( PURE )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(PURE, "PURE"), root_1);

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 12, specCase_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "specCase"

    public static class declaration_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "declaration"
    // JFSL.g:455:1: declaration : ( ( 'public' | 'private' )? Identifier ':' declarationMult additiveExpression 'from' frame ( '|' expression )? -> ^( DECLARATION ^( IDENTIFIER Identifier ) declarationMult additiveExpression frame ( expression )? ) | ( 'public' | 'private' )? Identifier ':' declarationMult additiveExpression ( '|' expression )? -> ^( DECLARATION ^( IDENTIFIER Identifier ) declarationMult additiveExpression NULL ( expression )? ) );
    public final JFSLParser.declaration_return declaration() throws RecognitionException {
        JFSLParser.declaration_return retval = new JFSLParser.declaration_return();
        retval.start = input.LT(1);
        int declaration_StartIndex = input.index();
        Node root_0 = null;

        Token string_literal76=null;
        Token string_literal77=null;
        Token Identifier78=null;
        Token char_literal79=null;
        Token string_literal82=null;
        Token char_literal84=null;
        Token string_literal86=null;
        Token string_literal87=null;
        Token Identifier88=null;
        Token char_literal89=null;
        Token char_literal92=null;
        JFSLParser.declarationMult_return declarationMult80 = null;

        JFSLParser.additiveExpression_return additiveExpression81 = null;

        JFSLParser.frame_return frame83 = null;

        JFSLParser.expression_return expression85 = null;

        JFSLParser.declarationMult_return declarationMult90 = null;

        JFSLParser.additiveExpression_return additiveExpression91 = null;

        JFSLParser.expression_return expression93 = null;


        Node string_literal76_tree=null;
        Node string_literal77_tree=null;
        Node Identifier78_tree=null;
        Node char_literal79_tree=null;
        Node string_literal82_tree=null;
        Node char_literal84_tree=null;
        Node string_literal86_tree=null;
        Node string_literal87_tree=null;
        Node Identifier88_tree=null;
        Node char_literal89_tree=null;
        Node char_literal92_tree=null;
        RewriteRuleTokenStream stream_161=new RewriteRuleTokenStream(adaptor,"token 161");
        RewriteRuleTokenStream stream_162=new RewriteRuleTokenStream(adaptor,"token 162");
        RewriteRuleTokenStream stream_163=new RewriteRuleTokenStream(adaptor,"token 163");
        RewriteRuleTokenStream stream_160=new RewriteRuleTokenStream(adaptor,"token 160");
        RewriteRuleTokenStream stream_159=new RewriteRuleTokenStream(adaptor,"token 159");
        RewriteRuleTokenStream stream_Identifier=new RewriteRuleTokenStream(adaptor,"token Identifier");
        RewriteRuleSubtreeStream stream_expression=new RewriteRuleSubtreeStream(adaptor,"rule expression");
        RewriteRuleSubtreeStream stream_frame=new RewriteRuleSubtreeStream(adaptor,"rule frame");
        RewriteRuleSubtreeStream stream_additiveExpression=new RewriteRuleSubtreeStream(adaptor,"rule additiveExpression");
        RewriteRuleSubtreeStream stream_declarationMult=new RewriteRuleSubtreeStream(adaptor,"rule declarationMult");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 13) ) { return retval; }
            // JFSL.g:456:3: ( ( 'public' | 'private' )? Identifier ':' declarationMult additiveExpression 'from' frame ( '|' expression )? -> ^( DECLARATION ^( IDENTIFIER Identifier ) declarationMult additiveExpression frame ( expression )? ) | ( 'public' | 'private' )? Identifier ':' declarationMult additiveExpression ( '|' expression )? -> ^( DECLARATION ^( IDENTIFIER Identifier ) declarationMult additiveExpression NULL ( expression )? ) )
            int alt22=2;
            switch ( input.LA(1) ) {
            case 159:
                {
                int LA22_1 = input.LA(2);

                if ( (synpred26_JFSL()) ) {
                    alt22=1;
                }
                else if ( (true) ) {
                    alt22=2;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 22, 1, input);

                    throw nvae;
                }
                }
                break;
            case 160:
                {
                int LA22_2 = input.LA(2);

                if ( (synpred26_JFSL()) ) {
                    alt22=1;
                }
                else if ( (true) ) {
                    alt22=2;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 22, 2, input);

                    throw nvae;
                }
                }
                break;
            case Identifier:
                {
                int LA22_3 = input.LA(2);

                if ( (synpred26_JFSL()) ) {
                    alt22=1;
                }
                else if ( (true) ) {
                    alt22=2;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 22, 3, input);

                    throw nvae;
                }
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 22, 0, input);

                throw nvae;
            }

            switch (alt22) {
                case 1 :
                    // JFSL.g:456:5: ( 'public' | 'private' )? Identifier ':' declarationMult additiveExpression 'from' frame ( '|' expression )?
                    {
                    // JFSL.g:456:5: ( 'public' | 'private' )?
                    int alt18=3;
                    int LA18_0 = input.LA(1);

                    if ( (LA18_0==159) ) {
                        alt18=1;
                    }
                    else if ( (LA18_0==160) ) {
                        alt18=2;
                    }
                    switch (alt18) {
                        case 1 :
                            // JFSL.g:456:6: 'public'
                            {
                            string_literal76=(Token)match(input,159,FOLLOW_159_in_declaration1371); if (state.failed) return retval; 
                            if ( state.backtracking==0 ) stream_159.add(string_literal76);


                            }
                            break;
                        case 2 :
                            // JFSL.g:456:17: 'private'
                            {
                            string_literal77=(Token)match(input,160,FOLLOW_160_in_declaration1375); if (state.failed) return retval; 
                            if ( state.backtracking==0 ) stream_160.add(string_literal77);


                            }
                            break;

                    }

                    Identifier78=(Token)match(input,Identifier,FOLLOW_Identifier_in_declaration1379); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_Identifier.add(Identifier78);

                    char_literal79=(Token)match(input,161,FOLLOW_161_in_declaration1381); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_161.add(char_literal79);

                    pushFollow(FOLLOW_declarationMult_in_declaration1383);
                    declarationMult80=declarationMult();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_declarationMult.add(declarationMult80.getTree());
                    pushFollow(FOLLOW_additiveExpression_in_declaration1385);
                    additiveExpression81=additiveExpression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_additiveExpression.add(additiveExpression81.getTree());
                    string_literal82=(Token)match(input,162,FOLLOW_162_in_declaration1387); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_162.add(string_literal82);

                    pushFollow(FOLLOW_frame_in_declaration1389);
                    frame83=frame();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_frame.add(frame83.getTree());
                    // JFSL.g:456:92: ( '|' expression )?
                    int alt19=2;
                    int LA19_0 = input.LA(1);

                    if ( (LA19_0==163) ) {
                        alt19=1;
                    }
                    switch (alt19) {
                        case 1 :
                            // JFSL.g:456:93: '|' expression
                            {
                            char_literal84=(Token)match(input,163,FOLLOW_163_in_declaration1392); if (state.failed) return retval; 
                            if ( state.backtracking==0 ) stream_163.add(char_literal84);

                            pushFollow(FOLLOW_expression_in_declaration1394);
                            expression85=expression();

                            state._fsp--;
                            if (state.failed) return retval;
                            if ( state.backtracking==0 ) stream_expression.add(expression85.getTree());

                            }
                            break;

                    }



                    // AST REWRITE
                    // elements: expression, Identifier, frame, additiveExpression, declarationMult
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 457:3: -> ^( DECLARATION ^( IDENTIFIER Identifier ) declarationMult additiveExpression frame ( expression )? )
                    {
                        // JFSL.g:457:6: ^( DECLARATION ^( IDENTIFIER Identifier ) declarationMult additiveExpression frame ( expression )? )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(DECLARATION, "DECLARATION"), root_1);

                        // JFSL.g:457:20: ^( IDENTIFIER Identifier )
                        {
                        Node root_2 = (Node)adaptor.nil();
                        root_2 = (Node)adaptor.becomeRoot((Node)adaptor.create(IDENTIFIER, "IDENTIFIER"), root_2);

                        adaptor.addChild(root_2, stream_Identifier.nextNode());

                        adaptor.addChild(root_1, root_2);
                        }
                        adaptor.addChild(root_1, stream_declarationMult.nextTree());
                        adaptor.addChild(root_1, stream_additiveExpression.nextTree());
                        adaptor.addChild(root_1, stream_frame.nextTree());
                        // JFSL.g:457:86: ( expression )?
                        if ( stream_expression.hasNext() ) {
                            adaptor.addChild(root_1, stream_expression.nextTree());

                        }
                        stream_expression.reset();

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 2 :
                    // JFSL.g:458:5: ( 'public' | 'private' )? Identifier ':' declarationMult additiveExpression ( '|' expression )?
                    {
                    // JFSL.g:458:5: ( 'public' | 'private' )?
                    int alt20=3;
                    int LA20_0 = input.LA(1);

                    if ( (LA20_0==159) ) {
                        alt20=1;
                    }
                    else if ( (LA20_0==160) ) {
                        alt20=2;
                    }
                    switch (alt20) {
                        case 1 :
                            // JFSL.g:458:6: 'public'
                            {
                            string_literal86=(Token)match(input,159,FOLLOW_159_in_declaration1426); if (state.failed) return retval; 
                            if ( state.backtracking==0 ) stream_159.add(string_literal86);


                            }
                            break;
                        case 2 :
                            // JFSL.g:458:17: 'private'
                            {
                            string_literal87=(Token)match(input,160,FOLLOW_160_in_declaration1430); if (state.failed) return retval; 
                            if ( state.backtracking==0 ) stream_160.add(string_literal87);


                            }
                            break;

                    }

                    Identifier88=(Token)match(input,Identifier,FOLLOW_Identifier_in_declaration1434); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_Identifier.add(Identifier88);

                    char_literal89=(Token)match(input,161,FOLLOW_161_in_declaration1436); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_161.add(char_literal89);

                    pushFollow(FOLLOW_declarationMult_in_declaration1438);
                    declarationMult90=declarationMult();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_declarationMult.add(declarationMult90.getTree());
                    pushFollow(FOLLOW_additiveExpression_in_declaration1440);
                    additiveExpression91=additiveExpression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_additiveExpression.add(additiveExpression91.getTree());
                    // JFSL.g:458:79: ( '|' expression )?
                    int alt21=2;
                    int LA21_0 = input.LA(1);

                    if ( (LA21_0==163) ) {
                        alt21=1;
                    }
                    switch (alt21) {
                        case 1 :
                            // JFSL.g:458:80: '|' expression
                            {
                            char_literal92=(Token)match(input,163,FOLLOW_163_in_declaration1443); if (state.failed) return retval; 
                            if ( state.backtracking==0 ) stream_163.add(char_literal92);

                            pushFollow(FOLLOW_expression_in_declaration1445);
                            expression93=expression();

                            state._fsp--;
                            if (state.failed) return retval;
                            if ( state.backtracking==0 ) stream_expression.add(expression93.getTree());

                            }
                            break;

                    }



                    // AST REWRITE
                    // elements: additiveExpression, declarationMult, Identifier, expression
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 459:3: -> ^( DECLARATION ^( IDENTIFIER Identifier ) declarationMult additiveExpression NULL ( expression )? )
                    {
                        // JFSL.g:459:6: ^( DECLARATION ^( IDENTIFIER Identifier ) declarationMult additiveExpression NULL ( expression )? )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(DECLARATION, "DECLARATION"), root_1);

                        // JFSL.g:459:20: ^( IDENTIFIER Identifier )
                        {
                        Node root_2 = (Node)adaptor.nil();
                        root_2 = (Node)adaptor.becomeRoot((Node)adaptor.create(IDENTIFIER, "IDENTIFIER"), root_2);

                        adaptor.addChild(root_2, stream_Identifier.nextNode());

                        adaptor.addChild(root_1, root_2);
                        }
                        adaptor.addChild(root_1, stream_declarationMult.nextTree());
                        adaptor.addChild(root_1, stream_additiveExpression.nextTree());
                        adaptor.addChild(root_1, (Node)adaptor.create(NULL, "NULL"));
                        // JFSL.g:459:85: ( expression )?
                        if ( stream_expression.hasNext() ) {
                            adaptor.addChild(root_1, stream_expression.nextTree());

                        }
                        stream_expression.reset();

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 13, declaration_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "declaration"

    public static class quantIdMod_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "quantIdMod"
    // JFSL.g:462:1: quantIdMod : ( 'disj' -> MOD_DISJ | -> MOD_NONE );
    public final JFSLParser.quantIdMod_return quantIdMod() throws RecognitionException {
        JFSLParser.quantIdMod_return retval = new JFSLParser.quantIdMod_return();
        retval.start = input.LT(1);
        int quantIdMod_StartIndex = input.index();
        Node root_0 = null;

        Token string_literal94=null;

        Node string_literal94_tree=null;
        RewriteRuleTokenStream stream_164=new RewriteRuleTokenStream(adaptor,"token 164");

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 14) ) { return retval; }
            // JFSL.g:463:3: ( 'disj' -> MOD_DISJ | -> MOD_NONE )
            int alt23=2;
            int LA23_0 = input.LA(1);

            if ( (LA23_0==164) ) {
                alt23=1;
            }
            else if ( (LA23_0==Identifier) ) {
                alt23=2;
            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 23, 0, input);

                throw nvae;
            }
            switch (alt23) {
                case 1 :
                    // JFSL.g:463:5: 'disj'
                    {
                    string_literal94=(Token)match(input,164,FOLLOW_164_in_quantIdMod1485); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_164.add(string_literal94);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 463:12: -> MOD_DISJ
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(MOD_DISJ, "MOD_DISJ"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 2 :
                    // JFSL.g:464:5: 
                    {

                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 464:5: -> MOD_NONE
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(MOD_NONE, "MOD_NONE"));

                    }

                    retval.tree = root_0;}
                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 14, quantIdMod_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "quantIdMod"

    public static class declarationMult_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "declarationMult"
    // JFSL.g:467:1: declarationMult : ( setDeclOp | 'set' -> DECL_SET | 'seq' -> DECL_SEQ | -> DECL_NONE );
    public final JFSLParser.declarationMult_return declarationMult() throws RecognitionException {
        JFSLParser.declarationMult_return retval = new JFSLParser.declarationMult_return();
        retval.start = input.LT(1);
        int declarationMult_StartIndex = input.index();
        Node root_0 = null;

        Token string_literal96=null;
        Token string_literal97=null;
        JFSLParser.setDeclOp_return setDeclOp95 = null;


        Node string_literal96_tree=null;
        Node string_literal97_tree=null;
        RewriteRuleTokenStream stream_166=new RewriteRuleTokenStream(adaptor,"token 166");
        RewriteRuleTokenStream stream_165=new RewriteRuleTokenStream(adaptor,"token 165");

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 15) ) { return retval; }
            // JFSL.g:468:3: ( setDeclOp | 'set' -> DECL_SET | 'seq' -> DECL_SEQ | -> DECL_NONE )
            int alt24=4;
            switch ( input.LA(1) ) {
            case 207:
            case 208:
            case 209:
                {
                alt24=1;
                }
                break;
            case 165:
                {
                alt24=2;
                }
                break;
            case 166:
                {
                alt24=3;
                }
                break;
            case Identifier:
            case FloatingPointLiteral:
            case CharacterLiteral:
            case StringLiteral:
            case HexLiteral:
            case OctalLiteral:
            case DecimalLiteral:
            case 144:
            case 150:
            case 168:
            case 169:
            case 170:
            case 171:
            case 172:
            case 173:
            case 174:
            case 175:
            case 176:
            case 177:
            case 178:
            case 183:
            case 186:
            case 191:
            case 192:
            case 193:
            case 196:
            case 199:
            case 200:
            case 201:
            case 202:
            case 203:
            case 204:
            case 205:
            case 206:
                {
                alt24=4;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 24, 0, input);

                throw nvae;
            }

            switch (alt24) {
                case 1 :
                    // JFSL.g:468:5: setDeclOp
                    {
                    root_0 = (Node)adaptor.nil();

                    pushFollow(FOLLOW_setDeclOp_in_declarationMult1512);
                    setDeclOp95=setDeclOp();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, setDeclOp95.getTree());

                    }
                    break;
                case 2 :
                    // JFSL.g:469:5: 'set'
                    {
                    string_literal96=(Token)match(input,165,FOLLOW_165_in_declarationMult1518); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_165.add(string_literal96);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 469:11: -> DECL_SET
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(DECL_SET, "DECL_SET"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 3 :
                    // JFSL.g:470:5: 'seq'
                    {
                    string_literal97=(Token)match(input,166,FOLLOW_166_in_declarationMult1528); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_166.add(string_literal97);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 470:11: -> DECL_SEQ
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(DECL_SEQ, "DECL_SEQ"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 4 :
                    // JFSL.g:471:5: 
                    {

                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 471:5: -> DECL_NONE
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(DECL_NONE, "DECL_NONE"));

                    }

                    retval.tree = root_0;}
                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 15, declarationMult_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "declarationMult"

    public static class frame_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "frame"
    // JFSL.g:474:1: frame : ( | storeRef ( ',' storeRef )* ) -> ^( FRAME ( storeRef )* ) ;
    public final JFSLParser.frame_return frame() throws RecognitionException {
        JFSLParser.frame_return retval = new JFSLParser.frame_return();
        retval.start = input.LT(1);
        int frame_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal99=null;
        JFSLParser.storeRef_return storeRef98 = null;

        JFSLParser.storeRef_return storeRef100 = null;


        Node char_literal99_tree=null;
        RewriteRuleTokenStream stream_147=new RewriteRuleTokenStream(adaptor,"token 147");
        RewriteRuleSubtreeStream stream_storeRef=new RewriteRuleSubtreeStream(adaptor,"rule storeRef");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 16) ) { return retval; }
            // JFSL.g:475:3: ( ( | storeRef ( ',' storeRef )* ) -> ^( FRAME ( storeRef )* ) )
            // JFSL.g:475:5: ( | storeRef ( ',' storeRef )* )
            {
            // JFSL.g:475:5: ( | storeRef ( ',' storeRef )* )
            int alt26=2;
            int LA26_0 = input.LA(1);

            if ( (LA26_0==EOF||LA26_0==151||LA26_0==163) ) {
                alt26=1;
            }
            else if ( (LA26_0==Identifier||LA26_0==144||LA26_0==150||LA26_0==186||LA26_0==196||(LA26_0>=199 && LA26_0<=206)) ) {
                alt26=2;
            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 26, 0, input);

                throw nvae;
            }
            switch (alt26) {
                case 1 :
                    // JFSL.g:475:7: 
                    {
                    }
                    break;
                case 2 :
                    // JFSL.g:475:9: storeRef ( ',' storeRef )*
                    {
                    pushFollow(FOLLOW_storeRef_in_frame1559);
                    storeRef98=storeRef();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_storeRef.add(storeRef98.getTree());
                    // JFSL.g:475:18: ( ',' storeRef )*
                    loop25:
                    do {
                        int alt25=2;
                        int LA25_0 = input.LA(1);

                        if ( (LA25_0==147) ) {
                            alt25=1;
                        }


                        switch (alt25) {
                    	case 1 :
                    	    // JFSL.g:475:19: ',' storeRef
                    	    {
                    	    char_literal99=(Token)match(input,147,FOLLOW_147_in_frame1562); if (state.failed) return retval; 
                    	    if ( state.backtracking==0 ) stream_147.add(char_literal99);

                    	    pushFollow(FOLLOW_storeRef_in_frame1564);
                    	    storeRef100=storeRef();

                    	    state._fsp--;
                    	    if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) stream_storeRef.add(storeRef100.getTree());

                    	    }
                    	    break;

                    	default :
                    	    break loop25;
                        }
                    } while (true);


                    }
                    break;

            }



            // AST REWRITE
            // elements: storeRef
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 475:35: -> ^( FRAME ( storeRef )* )
            {
                // JFSL.g:475:38: ^( FRAME ( storeRef )* )
                {
                Node root_1 = (Node)adaptor.nil();
                root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(FRAME, "FRAME"), root_1);

                // JFSL.g:475:46: ( storeRef )*
                while ( stream_storeRef.hasNext() ) {
                    adaptor.addChild(root_1, stream_storeRef.nextTree());

                }
                stream_storeRef.reset();

                adaptor.addChild(root_0, root_1);
                }

            }

            retval.tree = root_0;}
            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 16, frame_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "frame"

    public static class storeRef_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "storeRef"
    // JFSL.g:478:1: storeRef : storePrimary storeSelectors ( '|' ff= expression )? ( 'from' fd= expression )? -> ^( FRAME_LOCATION ^( FRAME_FIELD storePrimary storeSelectors ) ( ^( FRAME_FILTER $ff) )? ( ^( FRAME_DOMAIN $fd) )? ) ;
    public final JFSLParser.storeRef_return storeRef() throws RecognitionException {
        JFSLParser.storeRef_return retval = new JFSLParser.storeRef_return();
        retval.start = input.LT(1);
        int storeRef_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal103=null;
        Token string_literal104=null;
        JFSLParser.expression_return ff = null;

        JFSLParser.expression_return fd = null;

        JFSLParser.storePrimary_return storePrimary101 = null;

        JFSLParser.storeSelectors_return storeSelectors102 = null;


        Node char_literal103_tree=null;
        Node string_literal104_tree=null;
        RewriteRuleTokenStream stream_162=new RewriteRuleTokenStream(adaptor,"token 162");
        RewriteRuleTokenStream stream_163=new RewriteRuleTokenStream(adaptor,"token 163");
        RewriteRuleSubtreeStream stream_expression=new RewriteRuleSubtreeStream(adaptor,"rule expression");
        RewriteRuleSubtreeStream stream_storeSelectors=new RewriteRuleSubtreeStream(adaptor,"rule storeSelectors");
        RewriteRuleSubtreeStream stream_storePrimary=new RewriteRuleSubtreeStream(adaptor,"rule storePrimary");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 17) ) { return retval; }
            // JFSL.g:479:3: ( storePrimary storeSelectors ( '|' ff= expression )? ( 'from' fd= expression )? -> ^( FRAME_LOCATION ^( FRAME_FIELD storePrimary storeSelectors ) ( ^( FRAME_FILTER $ff) )? ( ^( FRAME_DOMAIN $fd) )? ) )
            // JFSL.g:479:5: storePrimary storeSelectors ( '|' ff= expression )? ( 'from' fd= expression )?
            {
            pushFollow(FOLLOW_storePrimary_in_storeRef1589);
            storePrimary101=storePrimary();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_storePrimary.add(storePrimary101.getTree());
            pushFollow(FOLLOW_storeSelectors_in_storeRef1591);
            storeSelectors102=storeSelectors();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_storeSelectors.add(storeSelectors102.getTree());
            // JFSL.g:479:33: ( '|' ff= expression )?
            int alt27=2;
            int LA27_0 = input.LA(1);

            if ( (LA27_0==163) ) {
                int LA27_1 = input.LA(2);

                if ( (synpred36_JFSL()) ) {
                    alt27=1;
                }
            }
            switch (alt27) {
                case 1 :
                    // JFSL.g:479:34: '|' ff= expression
                    {
                    char_literal103=(Token)match(input,163,FOLLOW_163_in_storeRef1594); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_163.add(char_literal103);

                    pushFollow(FOLLOW_expression_in_storeRef1598);
                    ff=expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_expression.add(ff.getTree());

                    }
                    break;

            }

            // JFSL.g:479:54: ( 'from' fd= expression )?
            int alt28=2;
            int LA28_0 = input.LA(1);

            if ( (LA28_0==162) ) {
                alt28=1;
            }
            switch (alt28) {
                case 1 :
                    // JFSL.g:479:55: 'from' fd= expression
                    {
                    string_literal104=(Token)match(input,162,FOLLOW_162_in_storeRef1603); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_162.add(string_literal104);

                    pushFollow(FOLLOW_expression_in_storeRef1607);
                    fd=expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_expression.add(fd.getTree());

                    }
                    break;

            }



            // AST REWRITE
            // elements: storeSelectors, storePrimary, fd, ff
            // token labels: 
            // rule labels: retval, ff, fd
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);
            RewriteRuleSubtreeStream stream_ff=new RewriteRuleSubtreeStream(adaptor,"rule ff",ff!=null?ff.tree:null);
            RewriteRuleSubtreeStream stream_fd=new RewriteRuleSubtreeStream(adaptor,"rule fd",fd!=null?fd.tree:null);

            root_0 = (Node)adaptor.nil();
            // 480:6: -> ^( FRAME_LOCATION ^( FRAME_FIELD storePrimary storeSelectors ) ( ^( FRAME_FILTER $ff) )? ( ^( FRAME_DOMAIN $fd) )? )
            {
                // JFSL.g:480:9: ^( FRAME_LOCATION ^( FRAME_FIELD storePrimary storeSelectors ) ( ^( FRAME_FILTER $ff) )? ( ^( FRAME_DOMAIN $fd) )? )
                {
                Node root_1 = (Node)adaptor.nil();
                root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(FRAME_LOCATION, "FRAME_LOCATION"), root_1);

                // JFSL.g:480:26: ^( FRAME_FIELD storePrimary storeSelectors )
                {
                Node root_2 = (Node)adaptor.nil();
                root_2 = (Node)adaptor.becomeRoot((Node)adaptor.create(FRAME_FIELD, "FRAME_FIELD"), root_2);

                adaptor.addChild(root_2, stream_storePrimary.nextTree());
                adaptor.addChild(root_2, stream_storeSelectors.nextTree());

                adaptor.addChild(root_1, root_2);
                }
                // JFSL.g:480:69: ( ^( FRAME_FILTER $ff) )?
                if ( stream_ff.hasNext() ) {
                    // JFSL.g:480:69: ^( FRAME_FILTER $ff)
                    {
                    Node root_2 = (Node)adaptor.nil();
                    root_2 = (Node)adaptor.becomeRoot((Node)adaptor.create(FRAME_FILTER, "FRAME_FILTER"), root_2);

                    adaptor.addChild(root_2, stream_ff.nextTree());

                    adaptor.addChild(root_1, root_2);
                    }

                }
                stream_ff.reset();
                // JFSL.g:480:90: ( ^( FRAME_DOMAIN $fd) )?
                if ( stream_fd.hasNext() ) {
                    // JFSL.g:480:90: ^( FRAME_DOMAIN $fd)
                    {
                    Node root_2 = (Node)adaptor.nil();
                    root_2 = (Node)adaptor.becomeRoot((Node)adaptor.create(FRAME_DOMAIN, "FRAME_DOMAIN"), root_2);

                    adaptor.addChild(root_2, stream_fd.nextTree());

                    adaptor.addChild(root_1, root_2);
                    }

                }
                stream_fd.reset();

                adaptor.addChild(root_0, root_1);
                }

            }

            retval.tree = root_0;}
            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 17, storeRef_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "storeRef"

    public static class storePrimary_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "storePrimary"
    // JFSL.g:483:1: storePrimary : ( Identifier -> ^( IDENTIFIER Identifier ) | common );
    public final JFSLParser.storePrimary_return storePrimary() throws RecognitionException {
        JFSLParser.storePrimary_return retval = new JFSLParser.storePrimary_return();
        retval.start = input.LT(1);
        int storePrimary_StartIndex = input.index();
        Node root_0 = null;

        Token Identifier105=null;
        JFSLParser.common_return common106 = null;


        Node Identifier105_tree=null;
        RewriteRuleTokenStream stream_Identifier=new RewriteRuleTokenStream(adaptor,"token Identifier");

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 18) ) { return retval; }
            // JFSL.g:484:3: ( Identifier -> ^( IDENTIFIER Identifier ) | common )
            int alt29=2;
            int LA29_0 = input.LA(1);

            if ( (LA29_0==Identifier) ) {
                alt29=1;
            }
            else if ( (LA29_0==144||LA29_0==150||LA29_0==186||LA29_0==196||(LA29_0>=199 && LA29_0<=206)) ) {
                alt29=2;
            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 29, 0, input);

                throw nvae;
            }
            switch (alt29) {
                case 1 :
                    // JFSL.g:484:5: Identifier
                    {
                    Identifier105=(Token)match(input,Identifier,FOLLOW_Identifier_in_storePrimary1660); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_Identifier.add(Identifier105);



                    // AST REWRITE
                    // elements: Identifier
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 484:16: -> ^( IDENTIFIER Identifier )
                    {
                        // JFSL.g:484:19: ^( IDENTIFIER Identifier )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(IDENTIFIER, "IDENTIFIER"), root_1);

                        adaptor.addChild(root_1, stream_Identifier.nextNode());

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 2 :
                    // JFSL.g:485:5: common
                    {
                    root_0 = (Node)adaptor.nil();

                    pushFollow(FOLLOW_common_in_storePrimary1675);
                    common106=common();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, common106.getTree());

                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 18, storePrimary_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "storePrimary"

    public static class storeSelectors_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "storeSelectors"
    // JFSL.g:488:1: storeSelectors : ( ( selector )* storeWildCard | ( selector )+ );
    public final JFSLParser.storeSelectors_return storeSelectors() throws RecognitionException {
        JFSLParser.storeSelectors_return retval = new JFSLParser.storeSelectors_return();
        retval.start = input.LT(1);
        int storeSelectors_StartIndex = input.index();
        Node root_0 = null;

        JFSLParser.selector_return selector107 = null;

        JFSLParser.storeWildCard_return storeWildCard108 = null;

        JFSLParser.selector_return selector109 = null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 19) ) { return retval; }
            // JFSL.g:489:3: ( ( selector )* storeWildCard | ( selector )+ )
            int alt32=2;
            switch ( input.LA(1) ) {
            case 140:
                {
                int LA32_1 = input.LA(2);

                if ( (synpred40_JFSL()) ) {
                    alt32=1;
                }
                else if ( (true) ) {
                    alt32=2;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 32, 1, input);

                    throw nvae;
                }
                }
                break;
            case 179:
                {
                int LA32_2 = input.LA(2);

                if ( (synpred40_JFSL()) ) {
                    alt32=1;
                }
                else if ( (true) ) {
                    alt32=2;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 32, 2, input);

                    throw nvae;
                }
                }
                break;
            case 146:
                {
                int LA32_3 = input.LA(2);

                if ( (synpred40_JFSL()) ) {
                    alt32=1;
                }
                else if ( (true) ) {
                    alt32=2;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 32, 3, input);

                    throw nvae;
                }
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 32, 0, input);

                throw nvae;
            }

            switch (alt32) {
                case 1 :
                    // JFSL.g:489:5: ( selector )* storeWildCard
                    {
                    root_0 = (Node)adaptor.nil();

                    // JFSL.g:489:5: ( selector )*
                    loop30:
                    do {
                        int alt30=2;
                        int LA30_0 = input.LA(1);

                        if ( (LA30_0==140) ) {
                            int LA30_1 = input.LA(2);

                            if ( (LA30_1==167) ) {
                                int LA30_3 = input.LA(3);

                                if ( (LA30_3==Identifier||LA30_3==144||LA30_3==150||LA30_3==186||LA30_3==196||(LA30_3>=199 && LA30_3<=206)) ) {
                                    alt30=1;
                                }


                            }
                            else if ( (LA30_1==Identifier||LA30_1==144||LA30_1==150||(LA30_1>=171 && LA30_1<=178)||LA30_1==186||LA30_1==196||(LA30_1>=199 && LA30_1<=206)) ) {
                                alt30=1;
                            }


                        }
                        else if ( (LA30_0==146||LA30_0==179) ) {
                            alt30=1;
                        }


                        switch (alt30) {
                    	case 1 :
                    	    // JFSL.g:0:0: selector
                    	    {
                    	    pushFollow(FOLLOW_selector_in_storeSelectors1690);
                    	    selector107=selector();

                    	    state._fsp--;
                    	    if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) adaptor.addChild(root_0, selector107.getTree());

                    	    }
                    	    break;

                    	default :
                    	    break loop30;
                        }
                    } while (true);

                    pushFollow(FOLLOW_storeWildCard_in_storeSelectors1693);
                    storeWildCard108=storeWildCard();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, storeWildCard108.getTree());

                    }
                    break;
                case 2 :
                    // JFSL.g:490:5: ( selector )+
                    {
                    root_0 = (Node)adaptor.nil();

                    // JFSL.g:490:5: ( selector )+
                    int cnt31=0;
                    loop31:
                    do {
                        int alt31=2;
                        int LA31_0 = input.LA(1);

                        if ( (LA31_0==140||LA31_0==146||LA31_0==179) ) {
                            alt31=1;
                        }


                        switch (alt31) {
                    	case 1 :
                    	    // JFSL.g:0:0: selector
                    	    {
                    	    pushFollow(FOLLOW_selector_in_storeSelectors1700);
                    	    selector109=selector();

                    	    state._fsp--;
                    	    if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) adaptor.addChild(root_0, selector109.getTree());

                    	    }
                    	    break;

                    	default :
                    	    if ( cnt31 >= 1 ) break loop31;
                    	    if (state.backtracking>0) {state.failed=true; return retval;}
                                EarlyExitException eee =
                                    new EarlyExitException(31, input);
                                throw eee;
                        }
                        cnt31++;
                    } while (true);


                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 19, storeSelectors_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "storeSelectors"

    public static class storeWildCard_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "storeWildCard"
    // JFSL.g:493:1: storeWildCard : ( '.' '*' ) -> FRAME_ALL ;
    public final JFSLParser.storeWildCard_return storeWildCard() throws RecognitionException {
        JFSLParser.storeWildCard_return retval = new JFSLParser.storeWildCard_return();
        retval.start = input.LT(1);
        int storeWildCard_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal110=null;
        Token char_literal111=null;

        Node char_literal110_tree=null;
        Node char_literal111_tree=null;
        RewriteRuleTokenStream stream_140=new RewriteRuleTokenStream(adaptor,"token 140");
        RewriteRuleTokenStream stream_167=new RewriteRuleTokenStream(adaptor,"token 167");

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 20) ) { return retval; }
            // JFSL.g:494:4: ( ( '.' '*' ) -> FRAME_ALL )
            // JFSL.g:494:6: ( '.' '*' )
            {
            // JFSL.g:494:6: ( '.' '*' )
            // JFSL.g:494:7: '.' '*'
            {
            char_literal110=(Token)match(input,140,FOLLOW_140_in_storeWildCard1721); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_140.add(char_literal110);

            char_literal111=(Token)match(input,167,FOLLOW_167_in_storeWildCard1723); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_167.add(char_literal111);


            }



            // AST REWRITE
            // elements: 
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 494:16: -> FRAME_ALL
            {
                adaptor.addChild(root_0, (Node)adaptor.create(FRAME_ALL, "FRAME_ALL"));

            }

            retval.tree = root_0;}
            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 20, storeWildCard_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "storeWildCard"

    public static class keywordLiteral_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "keywordLiteral"
    // JFSL.g:497:1: keywordLiteral : ( 'true' -> LIT_TRUE | 'false' -> LIT_FALSE | 'null' -> LIT_NULL );
    public final JFSLParser.keywordLiteral_return keywordLiteral() throws RecognitionException {
        JFSLParser.keywordLiteral_return retval = new JFSLParser.keywordLiteral_return();
        retval.start = input.LT(1);
        int keywordLiteral_StartIndex = input.index();
        Node root_0 = null;

        Token string_literal112=null;
        Token string_literal113=null;
        Token string_literal114=null;

        Node string_literal112_tree=null;
        Node string_literal113_tree=null;
        Node string_literal114_tree=null;
        RewriteRuleTokenStream stream_170=new RewriteRuleTokenStream(adaptor,"token 170");
        RewriteRuleTokenStream stream_169=new RewriteRuleTokenStream(adaptor,"token 169");
        RewriteRuleTokenStream stream_168=new RewriteRuleTokenStream(adaptor,"token 168");

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 21) ) { return retval; }
            // JFSL.g:498:3: ( 'true' -> LIT_TRUE | 'false' -> LIT_FALSE | 'null' -> LIT_NULL )
            int alt33=3;
            switch ( input.LA(1) ) {
            case 168:
                {
                alt33=1;
                }
                break;
            case 169:
                {
                alt33=2;
                }
                break;
            case 170:
                {
                alt33=3;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 33, 0, input);

                throw nvae;
            }

            switch (alt33) {
                case 1 :
                    // JFSL.g:498:5: 'true'
                    {
                    string_literal112=(Token)match(input,168,FOLLOW_168_in_keywordLiteral1744); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_168.add(string_literal112);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 498:12: -> LIT_TRUE
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(LIT_TRUE, "LIT_TRUE"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 2 :
                    // JFSL.g:499:5: 'false'
                    {
                    string_literal113=(Token)match(input,169,FOLLOW_169_in_keywordLiteral1756); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_169.add(string_literal113);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 499:13: -> LIT_FALSE
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(LIT_FALSE, "LIT_FALSE"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 3 :
                    // JFSL.g:500:5: 'null'
                    {
                    string_literal114=(Token)match(input,170,FOLLOW_170_in_keywordLiteral1767); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_170.add(string_literal114);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 500:12: -> LIT_NULL
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(LIT_NULL, "LIT_NULL"));

                    }

                    retval.tree = root_0;}
                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 21, keywordLiteral_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "keywordLiteral"

    public static class literal_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "literal"
    // JFSL.g:503:1: literal : ( integerLiteral | FloatingPointLiteral | CharacterLiteral | StringLiteral | keywordLiteral );
    public final JFSLParser.literal_return literal() throws RecognitionException {
        JFSLParser.literal_return retval = new JFSLParser.literal_return();
        retval.start = input.LT(1);
        int literal_StartIndex = input.index();
        Node root_0 = null;

        Token FloatingPointLiteral116=null;
        Token CharacterLiteral117=null;
        Token StringLiteral118=null;
        JFSLParser.integerLiteral_return integerLiteral115 = null;

        JFSLParser.keywordLiteral_return keywordLiteral119 = null;


        Node FloatingPointLiteral116_tree=null;
        Node CharacterLiteral117_tree=null;
        Node StringLiteral118_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 22) ) { return retval; }
            // JFSL.g:504:3: ( integerLiteral | FloatingPointLiteral | CharacterLiteral | StringLiteral | keywordLiteral )
            int alt34=5;
            switch ( input.LA(1) ) {
            case HexLiteral:
            case OctalLiteral:
            case DecimalLiteral:
                {
                alt34=1;
                }
                break;
            case FloatingPointLiteral:
                {
                alt34=2;
                }
                break;
            case CharacterLiteral:
                {
                alt34=3;
                }
                break;
            case StringLiteral:
                {
                alt34=4;
                }
                break;
            case 168:
            case 169:
            case 170:
                {
                alt34=5;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 34, 0, input);

                throw nvae;
            }

            switch (alt34) {
                case 1 :
                    // JFSL.g:504:5: integerLiteral
                    {
                    root_0 = (Node)adaptor.nil();

                    pushFollow(FOLLOW_integerLiteral_in_literal1791);
                    integerLiteral115=integerLiteral();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, integerLiteral115.getTree());

                    }
                    break;
                case 2 :
                    // JFSL.g:505:5: FloatingPointLiteral
                    {
                    root_0 = (Node)adaptor.nil();

                    FloatingPointLiteral116=(Token)match(input,FloatingPointLiteral,FOLLOW_FloatingPointLiteral_in_literal1797); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    FloatingPointLiteral116_tree = (Node)adaptor.create(FloatingPointLiteral116);
                    adaptor.addChild(root_0, FloatingPointLiteral116_tree);
                    }

                    }
                    break;
                case 3 :
                    // JFSL.g:506:5: CharacterLiteral
                    {
                    root_0 = (Node)adaptor.nil();

                    CharacterLiteral117=(Token)match(input,CharacterLiteral,FOLLOW_CharacterLiteral_in_literal1803); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    CharacterLiteral117_tree = (Node)adaptor.create(CharacterLiteral117);
                    adaptor.addChild(root_0, CharacterLiteral117_tree);
                    }

                    }
                    break;
                case 4 :
                    // JFSL.g:507:5: StringLiteral
                    {
                    root_0 = (Node)adaptor.nil();

                    StringLiteral118=(Token)match(input,StringLiteral,FOLLOW_StringLiteral_in_literal1809); if (state.failed) return retval;
                    if ( state.backtracking==0 ) {
                    StringLiteral118_tree = (Node)adaptor.create(StringLiteral118);
                    adaptor.addChild(root_0, StringLiteral118_tree);
                    }

                    }
                    break;
                case 5 :
                    // JFSL.g:508:5: keywordLiteral
                    {
                    root_0 = (Node)adaptor.nil();

                    pushFollow(FOLLOW_keywordLiteral_in_literal1815);
                    keywordLiteral119=keywordLiteral();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, keywordLiteral119.getTree());

                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 22, literal_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "literal"

    public static class integerLiteral_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "integerLiteral"
    // JFSL.g:511:1: integerLiteral : ( HexLiteral | OctalLiteral | DecimalLiteral );
    public final JFSLParser.integerLiteral_return integerLiteral() throws RecognitionException {
        JFSLParser.integerLiteral_return retval = new JFSLParser.integerLiteral_return();
        retval.start = input.LT(1);
        int integerLiteral_StartIndex = input.index();
        Node root_0 = null;

        Token set120=null;

        Node set120_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 23) ) { return retval; }
            // JFSL.g:512:3: ( HexLiteral | OctalLiteral | DecimalLiteral )
            // JFSL.g:
            {
            root_0 = (Node)adaptor.nil();

            set120=(Token)input.LT(1);
            if ( (input.LA(1)>=HexLiteral && input.LA(1)<=DecimalLiteral) ) {
                input.consume();
                if ( state.backtracking==0 ) adaptor.addChild(root_0, (Node)adaptor.create(set120));
                state.errorRecovery=false;state.failed=false;
            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                MismatchedSetException mse = new MismatchedSetException(null,input);
                throw mse;
            }


            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 23, integerLiteral_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "integerLiteral"

    public static class primitiveType_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "primitiveType"
    // JFSL.g:517:1: primitiveType : ( 'boolean' -> TYPE_BOOLEAN | 'char' -> TYPE_CHAR | 'byte' -> TYPE_BYTE | 'short' -> TYPE_SHORT | 'int' -> TYPE_INT | 'long' -> TYPE_LONG | 'float' -> TYPE_FLOAT | 'double' -> TYPE_DOUBLE );
    public final JFSLParser.primitiveType_return primitiveType() throws RecognitionException {
        JFSLParser.primitiveType_return retval = new JFSLParser.primitiveType_return();
        retval.start = input.LT(1);
        int primitiveType_StartIndex = input.index();
        Node root_0 = null;

        Token string_literal121=null;
        Token string_literal122=null;
        Token string_literal123=null;
        Token string_literal124=null;
        Token string_literal125=null;
        Token string_literal126=null;
        Token string_literal127=null;
        Token string_literal128=null;

        Node string_literal121_tree=null;
        Node string_literal122_tree=null;
        Node string_literal123_tree=null;
        Node string_literal124_tree=null;
        Node string_literal125_tree=null;
        Node string_literal126_tree=null;
        Node string_literal127_tree=null;
        Node string_literal128_tree=null;
        RewriteRuleTokenStream stream_171=new RewriteRuleTokenStream(adaptor,"token 171");
        RewriteRuleTokenStream stream_174=new RewriteRuleTokenStream(adaptor,"token 174");
        RewriteRuleTokenStream stream_175=new RewriteRuleTokenStream(adaptor,"token 175");
        RewriteRuleTokenStream stream_172=new RewriteRuleTokenStream(adaptor,"token 172");
        RewriteRuleTokenStream stream_173=new RewriteRuleTokenStream(adaptor,"token 173");
        RewriteRuleTokenStream stream_178=new RewriteRuleTokenStream(adaptor,"token 178");
        RewriteRuleTokenStream stream_177=new RewriteRuleTokenStream(adaptor,"token 177");
        RewriteRuleTokenStream stream_176=new RewriteRuleTokenStream(adaptor,"token 176");

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 24) ) { return retval; }
            // JFSL.g:518:3: ( 'boolean' -> TYPE_BOOLEAN | 'char' -> TYPE_CHAR | 'byte' -> TYPE_BYTE | 'short' -> TYPE_SHORT | 'int' -> TYPE_INT | 'long' -> TYPE_LONG | 'float' -> TYPE_FLOAT | 'double' -> TYPE_DOUBLE )
            int alt35=8;
            switch ( input.LA(1) ) {
            case 171:
                {
                alt35=1;
                }
                break;
            case 172:
                {
                alt35=2;
                }
                break;
            case 173:
                {
                alt35=3;
                }
                break;
            case 174:
                {
                alt35=4;
                }
                break;
            case 175:
                {
                alt35=5;
                }
                break;
            case 176:
                {
                alt35=6;
                }
                break;
            case 177:
                {
                alt35=7;
                }
                break;
            case 178:
                {
                alt35=8;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 35, 0, input);

                throw nvae;
            }

            switch (alt35) {
                case 1 :
                    // JFSL.g:518:5: 'boolean'
                    {
                    string_literal121=(Token)match(input,171,FOLLOW_171_in_primitiveType1869); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_171.add(string_literal121);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 518:14: -> TYPE_BOOLEAN
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(TYPE_BOOLEAN, "TYPE_BOOLEAN"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 2 :
                    // JFSL.g:519:5: 'char'
                    {
                    string_literal122=(Token)match(input,172,FOLLOW_172_in_primitiveType1878); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_172.add(string_literal122);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 519:12: -> TYPE_CHAR
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(TYPE_CHAR, "TYPE_CHAR"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 3 :
                    // JFSL.g:520:5: 'byte'
                    {
                    string_literal123=(Token)match(input,173,FOLLOW_173_in_primitiveType1888); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_173.add(string_literal123);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 520:12: -> TYPE_BYTE
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(TYPE_BYTE, "TYPE_BYTE"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 4 :
                    // JFSL.g:521:5: 'short'
                    {
                    string_literal124=(Token)match(input,174,FOLLOW_174_in_primitiveType1900); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_174.add(string_literal124);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 521:13: -> TYPE_SHORT
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(TYPE_SHORT, "TYPE_SHORT"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 5 :
                    // JFSL.g:522:5: 'int'
                    {
                    string_literal125=(Token)match(input,175,FOLLOW_175_in_primitiveType1911); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_175.add(string_literal125);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 522:11: -> TYPE_INT
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(TYPE_INT, "TYPE_INT"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 6 :
                    // JFSL.g:523:5: 'long'
                    {
                    string_literal126=(Token)match(input,176,FOLLOW_176_in_primitiveType1922); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_176.add(string_literal126);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 523:12: -> TYPE_LONG
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(TYPE_LONG, "TYPE_LONG"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 7 :
                    // JFSL.g:524:5: 'float'
                    {
                    string_literal127=(Token)match(input,177,FOLLOW_177_in_primitiveType1932); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_177.add(string_literal127);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 524:13: -> TYPE_FLOAT
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(TYPE_FLOAT, "TYPE_FLOAT"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 8 :
                    // JFSL.g:525:5: 'double'
                    {
                    string_literal128=(Token)match(input,178,FOLLOW_178_in_primitiveType1943); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_178.add(string_literal128);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 525:14: -> TYPE_DOUBLE
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(TYPE_DOUBLE, "TYPE_DOUBLE"));

                    }

                    retval.tree = root_0;}
                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 24, primitiveType_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "primitiveType"

    public static class typeName_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "typeName"
    // JFSL.g:528:1: typeName : Identifier ( '.' Identifier )* ( typeParameters )? -> ^( TYPE_REF ( ^( IDENTIFIER Identifier ) )+ ( ^( TYPE_PARAMETERS typeParameters ) )? ) ;
    public final JFSLParser.typeName_return typeName() throws RecognitionException {
        JFSLParser.typeName_return retval = new JFSLParser.typeName_return();
        retval.start = input.LT(1);
        int typeName_StartIndex = input.index();
        Node root_0 = null;

        Token Identifier129=null;
        Token char_literal130=null;
        Token Identifier131=null;
        JFSLParser.typeParameters_return typeParameters132 = null;


        Node Identifier129_tree=null;
        Node char_literal130_tree=null;
        Node Identifier131_tree=null;
        RewriteRuleTokenStream stream_Identifier=new RewriteRuleTokenStream(adaptor,"token Identifier");
        RewriteRuleTokenStream stream_140=new RewriteRuleTokenStream(adaptor,"token 140");
        RewriteRuleSubtreeStream stream_typeParameters=new RewriteRuleSubtreeStream(adaptor,"rule typeParameters");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 25) ) { return retval; }
            // JFSL.g:529:3: ( Identifier ( '.' Identifier )* ( typeParameters )? -> ^( TYPE_REF ( ^( IDENTIFIER Identifier ) )+ ( ^( TYPE_PARAMETERS typeParameters ) )? ) )
            // JFSL.g:529:5: Identifier ( '.' Identifier )* ( typeParameters )?
            {
            Identifier129=(Token)match(input,Identifier,FOLLOW_Identifier_in_typeName1960); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_Identifier.add(Identifier129);

            // JFSL.g:529:16: ( '.' Identifier )*
            loop36:
            do {
                int alt36=2;
                int LA36_0 = input.LA(1);

                if ( (LA36_0==140) ) {
                    alt36=1;
                }


                switch (alt36) {
            	case 1 :
            	    // JFSL.g:529:17: '.' Identifier
            	    {
            	    char_literal130=(Token)match(input,140,FOLLOW_140_in_typeName1963); if (state.failed) return retval; 
            	    if ( state.backtracking==0 ) stream_140.add(char_literal130);

            	    Identifier131=(Token)match(input,Identifier,FOLLOW_Identifier_in_typeName1965); if (state.failed) return retval; 
            	    if ( state.backtracking==0 ) stream_Identifier.add(Identifier131);


            	    }
            	    break;

            	default :
            	    break loop36;
                }
            } while (true);

            // JFSL.g:529:34: ( typeParameters )?
            int alt37=2;
            int LA37_0 = input.LA(1);

            if ( (LA37_0==146) ) {
                int LA37_1 = input.LA(2);

                if ( (LA37_1==Identifier) ) {
                    alt37=1;
                }
            }
            switch (alt37) {
                case 1 :
                    // JFSL.g:0:0: typeParameters
                    {
                    pushFollow(FOLLOW_typeParameters_in_typeName1969);
                    typeParameters132=typeParameters();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_typeParameters.add(typeParameters132.getTree());

                    }
                    break;

            }



            // AST REWRITE
            // elements: typeParameters, Identifier
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 529:50: -> ^( TYPE_REF ( ^( IDENTIFIER Identifier ) )+ ( ^( TYPE_PARAMETERS typeParameters ) )? )
            {
                // JFSL.g:529:53: ^( TYPE_REF ( ^( IDENTIFIER Identifier ) )+ ( ^( TYPE_PARAMETERS typeParameters ) )? )
                {
                Node root_1 = (Node)adaptor.nil();
                root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(TYPE_REF, "TYPE_REF"), root_1);

                if ( !(stream_Identifier.hasNext()) ) {
                    throw new RewriteEarlyExitException();
                }
                while ( stream_Identifier.hasNext() ) {
                    // JFSL.g:529:64: ^( IDENTIFIER Identifier )
                    {
                    Node root_2 = (Node)adaptor.nil();
                    root_2 = (Node)adaptor.becomeRoot((Node)adaptor.create(IDENTIFIER, "IDENTIFIER"), root_2);

                    adaptor.addChild(root_2, stream_Identifier.nextNode());

                    adaptor.addChild(root_1, root_2);
                    }

                }
                stream_Identifier.reset();
                // JFSL.g:529:90: ( ^( TYPE_PARAMETERS typeParameters ) )?
                if ( stream_typeParameters.hasNext() ) {
                    // JFSL.g:529:90: ^( TYPE_PARAMETERS typeParameters )
                    {
                    Node root_2 = (Node)adaptor.nil();
                    root_2 = (Node)adaptor.becomeRoot((Node)adaptor.create(TYPE_PARAMETERS, "TYPE_PARAMETERS"), root_2);

                    adaptor.addChild(root_2, stream_typeParameters.nextTree());

                    adaptor.addChild(root_1, root_2);
                    }

                }
                stream_typeParameters.reset();

                adaptor.addChild(root_0, root_1);
                }

            }

            retval.tree = root_0;}
            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 25, typeName_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "typeName"

    public static class typeDisambiguous_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "typeDisambiguous"
    // JFSL.g:532:1: typeDisambiguous : ( ( primitiveType -> primitiveType ) ( ( '[' ']' ) -> ^( TYPE_ARRAY $typeDisambiguous) )* | ( typeName -> typeName ) ( ( '[' ']' ) -> ^( TYPE_ARRAY $typeDisambiguous) )+ );
    public final JFSLParser.typeDisambiguous_return typeDisambiguous() throws RecognitionException {
        JFSLParser.typeDisambiguous_return retval = new JFSLParser.typeDisambiguous_return();
        retval.start = input.LT(1);
        int typeDisambiguous_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal134=null;
        Token char_literal135=null;
        Token char_literal137=null;
        Token char_literal138=null;
        JFSLParser.primitiveType_return primitiveType133 = null;

        JFSLParser.typeName_return typeName136 = null;


        Node char_literal134_tree=null;
        Node char_literal135_tree=null;
        Node char_literal137_tree=null;
        Node char_literal138_tree=null;
        RewriteRuleTokenStream stream_180=new RewriteRuleTokenStream(adaptor,"token 180");
        RewriteRuleTokenStream stream_179=new RewriteRuleTokenStream(adaptor,"token 179");
        RewriteRuleSubtreeStream stream_typeName=new RewriteRuleSubtreeStream(adaptor,"rule typeName");
        RewriteRuleSubtreeStream stream_primitiveType=new RewriteRuleSubtreeStream(adaptor,"rule primitiveType");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 26) ) { return retval; }
            // JFSL.g:533:3: ( ( primitiveType -> primitiveType ) ( ( '[' ']' ) -> ^( TYPE_ARRAY $typeDisambiguous) )* | ( typeName -> typeName ) ( ( '[' ']' ) -> ^( TYPE_ARRAY $typeDisambiguous) )+ )
            int alt40=2;
            int LA40_0 = input.LA(1);

            if ( ((LA40_0>=171 && LA40_0<=178)) ) {
                alt40=1;
            }
            else if ( (LA40_0==Identifier) ) {
                alt40=2;
            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 40, 0, input);

                throw nvae;
            }
            switch (alt40) {
                case 1 :
                    // JFSL.g:533:5: ( primitiveType -> primitiveType ) ( ( '[' ']' ) -> ^( TYPE_ARRAY $typeDisambiguous) )*
                    {
                    // JFSL.g:533:5: ( primitiveType -> primitiveType )
                    // JFSL.g:533:6: primitiveType
                    {
                    pushFollow(FOLLOW_primitiveType_in_typeDisambiguous2006);
                    primitiveType133=primitiveType();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_primitiveType.add(primitiveType133.getTree());


                    // AST REWRITE
                    // elements: primitiveType
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 533:20: -> primitiveType
                    {
                        adaptor.addChild(root_0, stream_primitiveType.nextTree());

                    }

                    retval.tree = root_0;}
                    }

                    // JFSL.g:533:38: ( ( '[' ']' ) -> ^( TYPE_ARRAY $typeDisambiguous) )*
                    loop38:
                    do {
                        int alt38=2;
                        int LA38_0 = input.LA(1);

                        if ( (LA38_0==179) ) {
                            int LA38_2 = input.LA(2);

                            if ( (LA38_2==180) ) {
                                int LA38_3 = input.LA(3);

                                if ( (synpred59_JFSL()) ) {
                                    alt38=1;
                                }


                            }


                        }


                        switch (alt38) {
                    	case 1 :
                    	    // JFSL.g:533:39: ( '[' ']' )
                    	    {
                    	    // JFSL.g:533:39: ( '[' ']' )
                    	    // JFSL.g:533:40: '[' ']'
                    	    {
                    	    char_literal134=(Token)match(input,179,FOLLOW_179_in_typeDisambiguous2015); if (state.failed) return retval; 
                    	    if ( state.backtracking==0 ) stream_179.add(char_literal134);

                    	    char_literal135=(Token)match(input,180,FOLLOW_180_in_typeDisambiguous2017); if (state.failed) return retval; 
                    	    if ( state.backtracking==0 ) stream_180.add(char_literal135);


                    	    }



                    	    // AST REWRITE
                    	    // elements: typeDisambiguous
                    	    // token labels: 
                    	    // rule labels: retval
                    	    // token list labels: 
                    	    // rule list labels: 
                    	    // wildcard labels: 
                    	    if ( state.backtracking==0 ) {
                    	    retval.tree = root_0;
                    	    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    	    root_0 = (Node)adaptor.nil();
                    	    // 533:49: -> ^( TYPE_ARRAY $typeDisambiguous)
                    	    {
                    	        // JFSL.g:533:52: ^( TYPE_ARRAY $typeDisambiguous)
                    	        {
                    	        Node root_1 = (Node)adaptor.nil();
                    	        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(TYPE_ARRAY, "TYPE_ARRAY"), root_1);

                    	        adaptor.addChild(root_1, stream_retval.nextTree());

                    	        adaptor.addChild(root_0, root_1);
                    	        }

                    	    }

                    	    retval.tree = root_0;}
                    	    }
                    	    break;

                    	default :
                    	    break loop38;
                        }
                    } while (true);


                    }
                    break;
                case 2 :
                    // JFSL.g:534:5: ( typeName -> typeName ) ( ( '[' ']' ) -> ^( TYPE_ARRAY $typeDisambiguous) )+
                    {
                    // JFSL.g:534:5: ( typeName -> typeName )
                    // JFSL.g:534:6: typeName
                    {
                    pushFollow(FOLLOW_typeName_in_typeDisambiguous2036);
                    typeName136=typeName();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_typeName.add(typeName136.getTree());


                    // AST REWRITE
                    // elements: typeName
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 534:15: -> typeName
                    {
                        adaptor.addChild(root_0, stream_typeName.nextTree());

                    }

                    retval.tree = root_0;}
                    }

                    // JFSL.g:534:28: ( ( '[' ']' ) -> ^( TYPE_ARRAY $typeDisambiguous) )+
                    int cnt39=0;
                    loop39:
                    do {
                        int alt39=2;
                        int LA39_0 = input.LA(1);

                        if ( (LA39_0==179) ) {
                            int LA39_2 = input.LA(2);

                            if ( (LA39_2==180) ) {
                                int LA39_3 = input.LA(3);

                                if ( (synpred61_JFSL()) ) {
                                    alt39=1;
                                }


                            }


                        }


                        switch (alt39) {
                    	case 1 :
                    	    // JFSL.g:534:29: ( '[' ']' )
                    	    {
                    	    // JFSL.g:534:29: ( '[' ']' )
                    	    // JFSL.g:534:30: '[' ']'
                    	    {
                    	    char_literal137=(Token)match(input,179,FOLLOW_179_in_typeDisambiguous2045); if (state.failed) return retval; 
                    	    if ( state.backtracking==0 ) stream_179.add(char_literal137);

                    	    char_literal138=(Token)match(input,180,FOLLOW_180_in_typeDisambiguous2047); if (state.failed) return retval; 
                    	    if ( state.backtracking==0 ) stream_180.add(char_literal138);


                    	    }



                    	    // AST REWRITE
                    	    // elements: typeDisambiguous
                    	    // token labels: 
                    	    // rule labels: retval
                    	    // token list labels: 
                    	    // rule list labels: 
                    	    // wildcard labels: 
                    	    if ( state.backtracking==0 ) {
                    	    retval.tree = root_0;
                    	    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    	    root_0 = (Node)adaptor.nil();
                    	    // 534:39: -> ^( TYPE_ARRAY $typeDisambiguous)
                    	    {
                    	        // JFSL.g:534:42: ^( TYPE_ARRAY $typeDisambiguous)
                    	        {
                    	        Node root_1 = (Node)adaptor.nil();
                    	        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(TYPE_ARRAY, "TYPE_ARRAY"), root_1);

                    	        adaptor.addChild(root_1, stream_retval.nextTree());

                    	        adaptor.addChild(root_0, root_1);
                    	        }

                    	    }

                    	    retval.tree = root_0;}
                    	    }
                    	    break;

                    	default :
                    	    if ( cnt39 >= 1 ) break loop39;
                    	    if (state.backtracking>0) {state.failed=true; return retval;}
                                EarlyExitException eee =
                                    new EarlyExitException(39, input);
                                throw eee;
                        }
                        cnt39++;
                    } while (true);


                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 26, typeDisambiguous_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "typeDisambiguous"

    public static class type_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "type"
    // JFSL.g:537:1: type : ( typeDisambiguous | typeName );
    public final JFSLParser.type_return type() throws RecognitionException {
        JFSLParser.type_return retval = new JFSLParser.type_return();
        retval.start = input.LT(1);
        int type_StartIndex = input.index();
        Node root_0 = null;

        JFSLParser.typeDisambiguous_return typeDisambiguous139 = null;

        JFSLParser.typeName_return typeName140 = null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 27) ) { return retval; }
            // JFSL.g:538:3: ( typeDisambiguous | typeName )
            int alt41=2;
            alt41 = dfa41.predict(input);
            switch (alt41) {
                case 1 :
                    // JFSL.g:538:5: typeDisambiguous
                    {
                    root_0 = (Node)adaptor.nil();

                    pushFollow(FOLLOW_typeDisambiguous_in_type2072);
                    typeDisambiguous139=typeDisambiguous();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, typeDisambiguous139.getTree());

                    }
                    break;
                case 2 :
                    // JFSL.g:539:5: typeName
                    {
                    root_0 = (Node)adaptor.nil();

                    pushFollow(FOLLOW_typeName_in_type2078);
                    typeName140=typeName();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, typeName140.getTree());

                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 27, type_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "type"

    public static class parExpression_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "parExpression"
    // JFSL.g:544:1: parExpression : '(' expression ')' ;
    public final JFSLParser.parExpression_return parExpression() throws RecognitionException {
        JFSLParser.parExpression_return retval = new JFSLParser.parExpression_return();
        retval.start = input.LT(1);
        int parExpression_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal141=null;
        Token char_literal143=null;
        JFSLParser.expression_return expression142 = null;


        Node char_literal141_tree=null;
        Node char_literal143_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 28) ) { return retval; }
            // JFSL.g:545:3: ( '(' expression ')' )
            // JFSL.g:545:5: '(' expression ')'
            {
            root_0 = (Node)adaptor.nil();

            char_literal141=(Token)match(input,150,FOLLOW_150_in_parExpression2112); if (state.failed) return retval;
            pushFollow(FOLLOW_expression_in_parExpression2115);
            expression142=expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, expression142.getTree());
            char_literal143=(Token)match(input,151,FOLLOW_151_in_parExpression2117); if (state.failed) return retval;

            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 28, parExpression_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "parExpression"

    public static class expressionList_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "expressionList"
    // JFSL.g:548:1: expressionList : expression ( ',' expression )* ;
    public final JFSLParser.expressionList_return expressionList() throws RecognitionException {
        JFSLParser.expressionList_return retval = new JFSLParser.expressionList_return();
        retval.start = input.LT(1);
        int expressionList_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal145=null;
        JFSLParser.expression_return expression144 = null;

        JFSLParser.expression_return expression146 = null;


        Node char_literal145_tree=null;

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 29) ) { return retval; }
            // JFSL.g:549:5: ( expression ( ',' expression )* )
            // JFSL.g:549:9: expression ( ',' expression )*
            {
            root_0 = (Node)adaptor.nil();

            pushFollow(FOLLOW_expression_in_expressionList2147);
            expression144=expression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, expression144.getTree());
            // JFSL.g:549:20: ( ',' expression )*
            loop42:
            do {
                int alt42=2;
                int LA42_0 = input.LA(1);

                if ( (LA42_0==147) ) {
                    alt42=1;
                }


                switch (alt42) {
            	case 1 :
            	    // JFSL.g:549:21: ',' expression
            	    {
            	    char_literal145=(Token)match(input,147,FOLLOW_147_in_expressionList2150); if (state.failed) return retval;
            	    pushFollow(FOLLOW_expression_in_expressionList2153);
            	    expression146=expression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) adaptor.addChild(root_0, expression146.getTree());

            	    }
            	    break;

            	default :
            	    break loop42;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 29, expressionList_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "expressionList"

    public static class arguments_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "arguments"
    // JFSL.g:552:1: arguments : '(' ( expressionList )? ')' -> ^( ARGUMENTS ( expressionList )? ) ;
    public final JFSLParser.arguments_return arguments() throws RecognitionException {
        JFSLParser.arguments_return retval = new JFSLParser.arguments_return();
        retval.start = input.LT(1);
        int arguments_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal147=null;
        Token char_literal149=null;
        JFSLParser.expressionList_return expressionList148 = null;


        Node char_literal147_tree=null;
        Node char_literal149_tree=null;
        RewriteRuleTokenStream stream_150=new RewriteRuleTokenStream(adaptor,"token 150");
        RewriteRuleTokenStream stream_151=new RewriteRuleTokenStream(adaptor,"token 151");
        RewriteRuleSubtreeStream stream_expressionList=new RewriteRuleSubtreeStream(adaptor,"rule expressionList");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 30) ) { return retval; }
            // JFSL.g:553:3: ( '(' ( expressionList )? ')' -> ^( ARGUMENTS ( expressionList )? ) )
            // JFSL.g:553:5: '(' ( expressionList )? ')'
            {
            char_literal147=(Token)match(input,150,FOLLOW_150_in_arguments2174); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_150.add(char_literal147);

            // JFSL.g:553:9: ( expressionList )?
            int alt43=2;
            int LA43_0 = input.LA(1);

            if ( ((LA43_0>=Identifier && LA43_0<=DecimalLiteral)||LA43_0==144||LA43_0==150||(LA43_0>=168 && LA43_0<=178)||LA43_0==183||LA43_0==186||(LA43_0>=191 && LA43_0<=193)||LA43_0==196||(LA43_0>=199 && LA43_0<=213)) ) {
                alt43=1;
            }
            switch (alt43) {
                case 1 :
                    // JFSL.g:0:0: expressionList
                    {
                    pushFollow(FOLLOW_expressionList_in_arguments2176);
                    expressionList148=expressionList();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_expressionList.add(expressionList148.getTree());

                    }
                    break;

            }

            char_literal149=(Token)match(input,151,FOLLOW_151_in_arguments2179); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_151.add(char_literal149);



            // AST REWRITE
            // elements: expressionList
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 553:29: -> ^( ARGUMENTS ( expressionList )? )
            {
                // JFSL.g:553:32: ^( ARGUMENTS ( expressionList )? )
                {
                Node root_1 = (Node)adaptor.nil();
                root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(ARGUMENTS, "ARGUMENTS"), root_1);

                // JFSL.g:553:44: ( expressionList )?
                if ( stream_expressionList.hasNext() ) {
                    adaptor.addChild(root_1, stream_expressionList.nextTree());

                }
                stream_expressionList.reset();

                adaptor.addChild(root_0, root_1);
                }

            }

            retval.tree = root_0;}
            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 30, arguments_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "arguments"

    public static class expression_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "expression"
    // JFSL.g:558:1: expression : conditionalExpression ;
    public final JFSLParser.expression_return expression() throws RecognitionException {
        JFSLParser.expression_return retval = new JFSLParser.expression_return();
        retval.start = input.LT(1);
        int expression_StartIndex = input.index();
        Node root_0 = null;

        JFSLParser.conditionalExpression_return conditionalExpression150 = null;



        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 31) ) { return retval; }
            // JFSL.g:559:3: ( conditionalExpression )
            // JFSL.g:559:5: conditionalExpression
            {
            root_0 = (Node)adaptor.nil();

            pushFollow(FOLLOW_conditionalExpression_in_expression2203);
            conditionalExpression150=conditionalExpression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) adaptor.addChild(root_0, conditionalExpression150.getTree());

            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 31, expression_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "expression"

    public static class conditionalExpression_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "conditionalExpression"
    // JFSL.g:562:1: conditionalExpression : ( quantifiedExpression -> quantifiedExpression ) ( '?' expression ':' expression -> ^( CONDITIONAL $conditionalExpression expression expression ) )? ;
    public final JFSLParser.conditionalExpression_return conditionalExpression() throws RecognitionException {
        JFSLParser.conditionalExpression_return retval = new JFSLParser.conditionalExpression_return();
        retval.start = input.LT(1);
        int conditionalExpression_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal152=null;
        Token char_literal154=null;
        JFSLParser.quantifiedExpression_return quantifiedExpression151 = null;

        JFSLParser.expression_return expression153 = null;

        JFSLParser.expression_return expression155 = null;


        Node char_literal152_tree=null;
        Node char_literal154_tree=null;
        RewriteRuleTokenStream stream_161=new RewriteRuleTokenStream(adaptor,"token 161");
        RewriteRuleTokenStream stream_181=new RewriteRuleTokenStream(adaptor,"token 181");
        RewriteRuleSubtreeStream stream_expression=new RewriteRuleSubtreeStream(adaptor,"rule expression");
        RewriteRuleSubtreeStream stream_quantifiedExpression=new RewriteRuleSubtreeStream(adaptor,"rule quantifiedExpression");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 32) ) { return retval; }
            // JFSL.g:563:3: ( ( quantifiedExpression -> quantifiedExpression ) ( '?' expression ':' expression -> ^( CONDITIONAL $conditionalExpression expression expression ) )? )
            // JFSL.g:563:5: ( quantifiedExpression -> quantifiedExpression ) ( '?' expression ':' expression -> ^( CONDITIONAL $conditionalExpression expression expression ) )?
            {
            // JFSL.g:563:5: ( quantifiedExpression -> quantifiedExpression )
            // JFSL.g:563:7: quantifiedExpression
            {
            pushFollow(FOLLOW_quantifiedExpression_in_conditionalExpression2220);
            quantifiedExpression151=quantifiedExpression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_quantifiedExpression.add(quantifiedExpression151.getTree());


            // AST REWRITE
            // elements: quantifiedExpression
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 563:28: -> quantifiedExpression
            {
                adaptor.addChild(root_0, stream_quantifiedExpression.nextTree());

            }

            retval.tree = root_0;}
            }

            // JFSL.g:564:5: ( '?' expression ':' expression -> ^( CONDITIONAL $conditionalExpression expression expression ) )?
            int alt44=2;
            int LA44_0 = input.LA(1);

            if ( (LA44_0==181) ) {
                int LA44_1 = input.LA(2);

                if ( (synpred65_JFSL()) ) {
                    alt44=1;
                }
            }
            switch (alt44) {
                case 1 :
                    // JFSL.g:564:7: '?' expression ':' expression
                    {
                    char_literal152=(Token)match(input,181,FOLLOW_181_in_conditionalExpression2233); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_181.add(char_literal152);

                    pushFollow(FOLLOW_expression_in_conditionalExpression2235);
                    expression153=expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_expression.add(expression153.getTree());
                    char_literal154=(Token)match(input,161,FOLLOW_161_in_conditionalExpression2237); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_161.add(char_literal154);

                    pushFollow(FOLLOW_expression_in_conditionalExpression2239);
                    expression155=expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_expression.add(expression155.getTree());


                    // AST REWRITE
                    // elements: conditionalExpression, expression, expression
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 564:37: -> ^( CONDITIONAL $conditionalExpression expression expression )
                    {
                        // JFSL.g:564:40: ^( CONDITIONAL $conditionalExpression expression expression )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(CONDITIONAL, "CONDITIONAL"), root_1);

                        adaptor.addChild(root_1, stream_retval.nextTree());
                        adaptor.addChild(root_1, stream_expression.nextTree());
                        adaptor.addChild(root_1, stream_expression.nextTree());

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;

            }


            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 32, conditionalExpression_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "conditionalExpression"

    public static class quantifiedExpression_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "quantifiedExpression"
    // JFSL.g:568:1: quantifiedExpression : ( setQuantOp decls '|' expression -> ^( QUANTIFY setQuantOp decls expression ) | setQuantOp mdecls '|' expression -> ^( QUANTIFY setQuantOp mdecls expression ) | logicalExpression );
    public final JFSLParser.quantifiedExpression_return quantifiedExpression() throws RecognitionException {
        JFSLParser.quantifiedExpression_return retval = new JFSLParser.quantifiedExpression_return();
        retval.start = input.LT(1);
        int quantifiedExpression_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal158=null;
        Token char_literal162=null;
        JFSLParser.setQuantOp_return setQuantOp156 = null;

        JFSLParser.decls_return decls157 = null;

        JFSLParser.expression_return expression159 = null;

        JFSLParser.setQuantOp_return setQuantOp160 = null;

        JFSLParser.mdecls_return mdecls161 = null;

        JFSLParser.expression_return expression163 = null;

        JFSLParser.logicalExpression_return logicalExpression164 = null;


        Node char_literal158_tree=null;
        Node char_literal162_tree=null;
        RewriteRuleTokenStream stream_163=new RewriteRuleTokenStream(adaptor,"token 163");
        RewriteRuleSubtreeStream stream_expression=new RewriteRuleSubtreeStream(adaptor,"rule expression");
        RewriteRuleSubtreeStream stream_setQuantOp=new RewriteRuleSubtreeStream(adaptor,"rule setQuantOp");
        RewriteRuleSubtreeStream stream_mdecls=new RewriteRuleSubtreeStream(adaptor,"rule mdecls");
        RewriteRuleSubtreeStream stream_decls=new RewriteRuleSubtreeStream(adaptor,"rule decls");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 33) ) { return retval; }
            // JFSL.g:569:3: ( setQuantOp decls '|' expression -> ^( QUANTIFY setQuantOp decls expression ) | setQuantOp mdecls '|' expression -> ^( QUANTIFY setQuantOp mdecls expression ) | logicalExpression )
            int alt45=3;
            alt45 = dfa45.predict(input);
            switch (alt45) {
                case 1 :
                    // JFSL.g:569:5: setQuantOp decls '|' expression
                    {
                    pushFollow(FOLLOW_setQuantOp_in_quantifiedExpression2275);
                    setQuantOp156=setQuantOp();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_setQuantOp.add(setQuantOp156.getTree());
                    pushFollow(FOLLOW_decls_in_quantifiedExpression2277);
                    decls157=decls();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_decls.add(decls157.getTree());
                    char_literal158=(Token)match(input,163,FOLLOW_163_in_quantifiedExpression2280); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_163.add(char_literal158);

                    pushFollow(FOLLOW_expression_in_quantifiedExpression2282);
                    expression159=expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_expression.add(expression159.getTree());


                    // AST REWRITE
                    // elements: decls, setQuantOp, expression
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 569:38: -> ^( QUANTIFY setQuantOp decls expression )
                    {
                        // JFSL.g:569:41: ^( QUANTIFY setQuantOp decls expression )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(QUANTIFY, "QUANTIFY"), root_1);

                        adaptor.addChild(root_1, stream_setQuantOp.nextTree());
                        adaptor.addChild(root_1, stream_decls.nextTree());
                        adaptor.addChild(root_1, stream_expression.nextTree());

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 2 :
                    // JFSL.g:570:5: setQuantOp mdecls '|' expression
                    {
                    pushFollow(FOLLOW_setQuantOp_in_quantifiedExpression2300);
                    setQuantOp160=setQuantOp();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_setQuantOp.add(setQuantOp160.getTree());
                    pushFollow(FOLLOW_mdecls_in_quantifiedExpression2302);
                    mdecls161=mdecls();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_mdecls.add(mdecls161.getTree());
                    char_literal162=(Token)match(input,163,FOLLOW_163_in_quantifiedExpression2304); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_163.add(char_literal162);

                    pushFollow(FOLLOW_expression_in_quantifiedExpression2306);
                    expression163=expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_expression.add(expression163.getTree());


                    // AST REWRITE
                    // elements: setQuantOp, mdecls, expression
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 570:38: -> ^( QUANTIFY setQuantOp mdecls expression )
                    {
                        // JFSL.g:570:41: ^( QUANTIFY setQuantOp mdecls expression )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(QUANTIFY, "QUANTIFY"), root_1);

                        adaptor.addChild(root_1, stream_setQuantOp.nextTree());
                        adaptor.addChild(root_1, stream_mdecls.nextTree());
                        adaptor.addChild(root_1, stream_expression.nextTree());

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 3 :
                    // JFSL.g:571:5: logicalExpression
                    {
                    root_0 = (Node)adaptor.nil();

                    pushFollow(FOLLOW_logicalExpression_in_quantifiedExpression2324);
                    logicalExpression164=logicalExpression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, logicalExpression164.getTree());

                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 33, quantifiedExpression_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "quantifiedExpression"

    public static class logicalExpression_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "logicalExpression"
    // JFSL.g:574:1: logicalExpression : ( conditionalOrExpression -> conditionalOrExpression ) ( logicalOp conditionalOrExpression -> ^( BINARY logicalOp $logicalExpression conditionalOrExpression ) )? ;
    public final JFSLParser.logicalExpression_return logicalExpression() throws RecognitionException {
        JFSLParser.logicalExpression_return retval = new JFSLParser.logicalExpression_return();
        retval.start = input.LT(1);
        int logicalExpression_StartIndex = input.index();
        Node root_0 = null;

        JFSLParser.conditionalOrExpression_return conditionalOrExpression165 = null;

        JFSLParser.logicalOp_return logicalOp166 = null;

        JFSLParser.conditionalOrExpression_return conditionalOrExpression167 = null;


        RewriteRuleSubtreeStream stream_conditionalOrExpression=new RewriteRuleSubtreeStream(adaptor,"rule conditionalOrExpression");
        RewriteRuleSubtreeStream stream_logicalOp=new RewriteRuleSubtreeStream(adaptor,"rule logicalOp");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 34) ) { return retval; }
            // JFSL.g:575:3: ( ( conditionalOrExpression -> conditionalOrExpression ) ( logicalOp conditionalOrExpression -> ^( BINARY logicalOp $logicalExpression conditionalOrExpression ) )? )
            // JFSL.g:575:5: ( conditionalOrExpression -> conditionalOrExpression ) ( logicalOp conditionalOrExpression -> ^( BINARY logicalOp $logicalExpression conditionalOrExpression ) )?
            {
            // JFSL.g:575:5: ( conditionalOrExpression -> conditionalOrExpression )
            // JFSL.g:575:6: conditionalOrExpression
            {
            pushFollow(FOLLOW_conditionalOrExpression_in_logicalExpression2340);
            conditionalOrExpression165=conditionalOrExpression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_conditionalOrExpression.add(conditionalOrExpression165.getTree());


            // AST REWRITE
            // elements: conditionalOrExpression
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 575:30: -> conditionalOrExpression
            {
                adaptor.addChild(root_0, stream_conditionalOrExpression.nextTree());

            }

            retval.tree = root_0;}
            }

            // JFSL.g:576:5: ( logicalOp conditionalOrExpression -> ^( BINARY logicalOp $logicalExpression conditionalOrExpression ) )?
            int alt46=2;
            int LA46_0 = input.LA(1);

            if ( (LA46_0==146||LA46_0==182) ) {
                alt46=1;
            }
            switch (alt46) {
                case 1 :
                    // JFSL.g:576:7: logicalOp conditionalOrExpression
                    {
                    pushFollow(FOLLOW_logicalOp_in_logicalExpression2353);
                    logicalOp166=logicalOp();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_logicalOp.add(logicalOp166.getTree());
                    pushFollow(FOLLOW_conditionalOrExpression_in_logicalExpression2355);
                    conditionalOrExpression167=conditionalOrExpression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_conditionalOrExpression.add(conditionalOrExpression167.getTree());


                    // AST REWRITE
                    // elements: logicalExpression, logicalOp, conditionalOrExpression
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 576:41: -> ^( BINARY logicalOp $logicalExpression conditionalOrExpression )
                    {
                        // JFSL.g:576:44: ^( BINARY logicalOp $logicalExpression conditionalOrExpression )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(BINARY, "BINARY"), root_1);

                        adaptor.addChild(root_1, stream_logicalOp.nextTree());
                        adaptor.addChild(root_1, stream_retval.nextTree());
                        adaptor.addChild(root_1, stream_conditionalOrExpression.nextTree());

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;

            }


            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 34, logicalExpression_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "logicalExpression"

    public static class logicalOp_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "logicalOp"
    // JFSL.g:580:1: logicalOp : ( '<' '=' '>' -> OP_EQUIV | '<' '!' '>' -> OP_NEQUIV | '=' '>' -> OP_IMPLIES );
    public final JFSLParser.logicalOp_return logicalOp() throws RecognitionException {
        JFSLParser.logicalOp_return retval = new JFSLParser.logicalOp_return();
        retval.start = input.LT(1);
        int logicalOp_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal168=null;
        Token char_literal169=null;
        Token char_literal170=null;
        Token char_literal171=null;
        Token char_literal172=null;
        Token char_literal173=null;
        Token char_literal174=null;
        Token char_literal175=null;

        Node char_literal168_tree=null;
        Node char_literal169_tree=null;
        Node char_literal170_tree=null;
        Node char_literal171_tree=null;
        Node char_literal172_tree=null;
        Node char_literal173_tree=null;
        Node char_literal174_tree=null;
        Node char_literal175_tree=null;
        RewriteRuleTokenStream stream_146=new RewriteRuleTokenStream(adaptor,"token 146");
        RewriteRuleTokenStream stream_182=new RewriteRuleTokenStream(adaptor,"token 182");
        RewriteRuleTokenStream stream_183=new RewriteRuleTokenStream(adaptor,"token 183");
        RewriteRuleTokenStream stream_148=new RewriteRuleTokenStream(adaptor,"token 148");

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 35) ) { return retval; }
            // JFSL.g:581:3: ( '<' '=' '>' -> OP_EQUIV | '<' '!' '>' -> OP_NEQUIV | '=' '>' -> OP_IMPLIES )
            int alt47=3;
            int LA47_0 = input.LA(1);

            if ( (LA47_0==146) ) {
                int LA47_1 = input.LA(2);

                if ( (LA47_1==182) ) {
                    alt47=1;
                }
                else if ( (LA47_1==183) ) {
                    alt47=2;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 47, 1, input);

                    throw nvae;
                }
            }
            else if ( (LA47_0==182) ) {
                alt47=3;
            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 47, 0, input);

                throw nvae;
            }
            switch (alt47) {
                case 1 :
                    // JFSL.g:581:7: '<' '=' '>'
                    {
                    char_literal168=(Token)match(input,146,FOLLOW_146_in_logicalOp2392); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_146.add(char_literal168);

                    char_literal169=(Token)match(input,182,FOLLOW_182_in_logicalOp2394); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_182.add(char_literal169);

                    char_literal170=(Token)match(input,148,FOLLOW_148_in_logicalOp2396); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_148.add(char_literal170);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 581:19: -> OP_EQUIV
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_EQUIV, "OP_EQUIV"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 2 :
                    // JFSL.g:581:33: '<' '!' '>'
                    {
                    char_literal171=(Token)match(input,146,FOLLOW_146_in_logicalOp2404); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_146.add(char_literal171);

                    char_literal172=(Token)match(input,183,FOLLOW_183_in_logicalOp2406); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_183.add(char_literal172);

                    char_literal173=(Token)match(input,148,FOLLOW_148_in_logicalOp2408); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_148.add(char_literal173);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 581:45: -> OP_NEQUIV
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_NEQUIV, "OP_NEQUIV"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 3 :
                    // JFSL.g:581:60: '=' '>'
                    {
                    char_literal174=(Token)match(input,182,FOLLOW_182_in_logicalOp2416); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_182.add(char_literal174);

                    char_literal175=(Token)match(input,148,FOLLOW_148_in_logicalOp2418); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_148.add(char_literal175);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 581:68: -> OP_IMPLIES
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_IMPLIES, "OP_IMPLIES"));

                    }

                    retval.tree = root_0;}
                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 35, logicalOp_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "logicalOp"

    public static class conditionalOrExpression_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "conditionalOrExpression"
    // JFSL.g:584:1: conditionalOrExpression : ( conditionalAndExpression -> conditionalAndExpression ) ( '||' a= conditionalAndExpression -> ^( BINARY OP_OR $conditionalOrExpression $a) )* ;
    public final JFSLParser.conditionalOrExpression_return conditionalOrExpression() throws RecognitionException {
        JFSLParser.conditionalOrExpression_return retval = new JFSLParser.conditionalOrExpression_return();
        retval.start = input.LT(1);
        int conditionalOrExpression_StartIndex = input.index();
        Node root_0 = null;

        Token string_literal177=null;
        JFSLParser.conditionalAndExpression_return a = null;

        JFSLParser.conditionalAndExpression_return conditionalAndExpression176 = null;


        Node string_literal177_tree=null;
        RewriteRuleTokenStream stream_184=new RewriteRuleTokenStream(adaptor,"token 184");
        RewriteRuleSubtreeStream stream_conditionalAndExpression=new RewriteRuleSubtreeStream(adaptor,"rule conditionalAndExpression");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 36) ) { return retval; }
            // JFSL.g:585:3: ( ( conditionalAndExpression -> conditionalAndExpression ) ( '||' a= conditionalAndExpression -> ^( BINARY OP_OR $conditionalOrExpression $a) )* )
            // JFSL.g:585:5: ( conditionalAndExpression -> conditionalAndExpression ) ( '||' a= conditionalAndExpression -> ^( BINARY OP_OR $conditionalOrExpression $a) )*
            {
            // JFSL.g:585:5: ( conditionalAndExpression -> conditionalAndExpression )
            // JFSL.g:585:6: conditionalAndExpression
            {
            pushFollow(FOLLOW_conditionalAndExpression_in_conditionalOrExpression2438);
            conditionalAndExpression176=conditionalAndExpression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_conditionalAndExpression.add(conditionalAndExpression176.getTree());


            // AST REWRITE
            // elements: conditionalAndExpression
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 585:31: -> conditionalAndExpression
            {
                adaptor.addChild(root_0, stream_conditionalAndExpression.nextTree());

            }

            retval.tree = root_0;}
            }

            // JFSL.g:586:7: ( '||' a= conditionalAndExpression -> ^( BINARY OP_OR $conditionalOrExpression $a) )*
            loop48:
            do {
                int alt48=2;
                int LA48_0 = input.LA(1);

                if ( (LA48_0==184) ) {
                    alt48=1;
                }


                switch (alt48) {
            	case 1 :
            	    // JFSL.g:586:9: '||' a= conditionalAndExpression
            	    {
            	    string_literal177=(Token)match(input,184,FOLLOW_184_in_conditionalOrExpression2453); if (state.failed) return retval; 
            	    if ( state.backtracking==0 ) stream_184.add(string_literal177);

            	    pushFollow(FOLLOW_conditionalAndExpression_in_conditionalOrExpression2457);
            	    a=conditionalAndExpression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_conditionalAndExpression.add(a.getTree());


            	    // AST REWRITE
            	    // elements: conditionalOrExpression, a
            	    // token labels: 
            	    // rule labels: retval, a
            	    // token list labels: 
            	    // rule list labels: 
            	    // wildcard labels: 
            	    if ( state.backtracking==0 ) {
            	    retval.tree = root_0;
            	    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);
            	    RewriteRuleSubtreeStream stream_a=new RewriteRuleSubtreeStream(adaptor,"rule a",a!=null?a.tree:null);

            	    root_0 = (Node)adaptor.nil();
            	    // 586:41: -> ^( BINARY OP_OR $conditionalOrExpression $a)
            	    {
            	        // JFSL.g:586:44: ^( BINARY OP_OR $conditionalOrExpression $a)
            	        {
            	        Node root_1 = (Node)adaptor.nil();
            	        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(BINARY, "BINARY"), root_1);

            	        adaptor.addChild(root_1, (Node)adaptor.create(OP_OR, "OP_OR"));
            	        adaptor.addChild(root_1, stream_retval.nextTree());
            	        adaptor.addChild(root_1, stream_a.nextTree());

            	        adaptor.addChild(root_0, root_1);
            	        }

            	    }

            	    retval.tree = root_0;}
            	    }
            	    break;

            	default :
            	    break loop48;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 36, conditionalOrExpression_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "conditionalOrExpression"

    public static class conditionalAndExpression_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "conditionalAndExpression"
    // JFSL.g:590:1: conditionalAndExpression : ( inclusiveOrExpression -> inclusiveOrExpression ) ( '&&' a= inclusiveOrExpression -> ^( BINARY OP_AND $conditionalAndExpression $a) )* ;
    public final JFSLParser.conditionalAndExpression_return conditionalAndExpression() throws RecognitionException {
        JFSLParser.conditionalAndExpression_return retval = new JFSLParser.conditionalAndExpression_return();
        retval.start = input.LT(1);
        int conditionalAndExpression_StartIndex = input.index();
        Node root_0 = null;

        Token string_literal179=null;
        JFSLParser.inclusiveOrExpression_return a = null;

        JFSLParser.inclusiveOrExpression_return inclusiveOrExpression178 = null;


        Node string_literal179_tree=null;
        RewriteRuleTokenStream stream_185=new RewriteRuleTokenStream(adaptor,"token 185");
        RewriteRuleSubtreeStream stream_inclusiveOrExpression=new RewriteRuleSubtreeStream(adaptor,"rule inclusiveOrExpression");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 37) ) { return retval; }
            // JFSL.g:591:3: ( ( inclusiveOrExpression -> inclusiveOrExpression ) ( '&&' a= inclusiveOrExpression -> ^( BINARY OP_AND $conditionalAndExpression $a) )* )
            // JFSL.g:591:5: ( inclusiveOrExpression -> inclusiveOrExpression ) ( '&&' a= inclusiveOrExpression -> ^( BINARY OP_AND $conditionalAndExpression $a) )*
            {
            // JFSL.g:591:5: ( inclusiveOrExpression -> inclusiveOrExpression )
            // JFSL.g:591:6: inclusiveOrExpression
            {
            pushFollow(FOLLOW_inclusiveOrExpression_in_conditionalAndExpression2494);
            inclusiveOrExpression178=inclusiveOrExpression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_inclusiveOrExpression.add(inclusiveOrExpression178.getTree());


            // AST REWRITE
            // elements: inclusiveOrExpression
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 591:28: -> inclusiveOrExpression
            {
                adaptor.addChild(root_0, stream_inclusiveOrExpression.nextTree());

            }

            retval.tree = root_0;}
            }

            // JFSL.g:592:7: ( '&&' a= inclusiveOrExpression -> ^( BINARY OP_AND $conditionalAndExpression $a) )*
            loop49:
            do {
                int alt49=2;
                int LA49_0 = input.LA(1);

                if ( (LA49_0==185) ) {
                    alt49=1;
                }


                switch (alt49) {
            	case 1 :
            	    // JFSL.g:592:9: '&&' a= inclusiveOrExpression
            	    {
            	    string_literal179=(Token)match(input,185,FOLLOW_185_in_conditionalAndExpression2509); if (state.failed) return retval; 
            	    if ( state.backtracking==0 ) stream_185.add(string_literal179);

            	    pushFollow(FOLLOW_inclusiveOrExpression_in_conditionalAndExpression2513);
            	    a=inclusiveOrExpression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_inclusiveOrExpression.add(a.getTree());


            	    // AST REWRITE
            	    // elements: a, conditionalAndExpression
            	    // token labels: 
            	    // rule labels: retval, a
            	    // token list labels: 
            	    // rule list labels: 
            	    // wildcard labels: 
            	    if ( state.backtracking==0 ) {
            	    retval.tree = root_0;
            	    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);
            	    RewriteRuleSubtreeStream stream_a=new RewriteRuleSubtreeStream(adaptor,"rule a",a!=null?a.tree:null);

            	    root_0 = (Node)adaptor.nil();
            	    // 592:38: -> ^( BINARY OP_AND $conditionalAndExpression $a)
            	    {
            	        // JFSL.g:592:41: ^( BINARY OP_AND $conditionalAndExpression $a)
            	        {
            	        Node root_1 = (Node)adaptor.nil();
            	        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(BINARY, "BINARY"), root_1);

            	        adaptor.addChild(root_1, (Node)adaptor.create(OP_AND, "OP_AND"));
            	        adaptor.addChild(root_1, stream_retval.nextTree());
            	        adaptor.addChild(root_1, stream_a.nextTree());

            	        adaptor.addChild(root_0, root_1);
            	        }

            	    }

            	    retval.tree = root_0;}
            	    }
            	    break;

            	default :
            	    break loop49;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 37, conditionalAndExpression_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "conditionalAndExpression"

    public static class inclusiveOrExpression_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "inclusiveOrExpression"
    // JFSL.g:596:1: inclusiveOrExpression : ( exclusiveOrExpression -> exclusiveOrExpression ) ( '|' a= exclusiveOrExpression -> ^( BINARY OP_BIT_OR $inclusiveOrExpression $a) )* ;
    public final JFSLParser.inclusiveOrExpression_return inclusiveOrExpression() throws RecognitionException {
        JFSLParser.inclusiveOrExpression_return retval = new JFSLParser.inclusiveOrExpression_return();
        retval.start = input.LT(1);
        int inclusiveOrExpression_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal181=null;
        JFSLParser.exclusiveOrExpression_return a = null;

        JFSLParser.exclusiveOrExpression_return exclusiveOrExpression180 = null;


        Node char_literal181_tree=null;
        RewriteRuleTokenStream stream_163=new RewriteRuleTokenStream(adaptor,"token 163");
        RewriteRuleSubtreeStream stream_exclusiveOrExpression=new RewriteRuleSubtreeStream(adaptor,"rule exclusiveOrExpression");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 38) ) { return retval; }
            // JFSL.g:597:3: ( ( exclusiveOrExpression -> exclusiveOrExpression ) ( '|' a= exclusiveOrExpression -> ^( BINARY OP_BIT_OR $inclusiveOrExpression $a) )* )
            // JFSL.g:597:5: ( exclusiveOrExpression -> exclusiveOrExpression ) ( '|' a= exclusiveOrExpression -> ^( BINARY OP_BIT_OR $inclusiveOrExpression $a) )*
            {
            // JFSL.g:597:5: ( exclusiveOrExpression -> exclusiveOrExpression )
            // JFSL.g:597:6: exclusiveOrExpression
            {
            pushFollow(FOLLOW_exclusiveOrExpression_in_inclusiveOrExpression2555);
            exclusiveOrExpression180=exclusiveOrExpression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_exclusiveOrExpression.add(exclusiveOrExpression180.getTree());


            // AST REWRITE
            // elements: exclusiveOrExpression
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 597:28: -> exclusiveOrExpression
            {
                adaptor.addChild(root_0, stream_exclusiveOrExpression.nextTree());

            }

            retval.tree = root_0;}
            }

            // JFSL.g:598:7: ( '|' a= exclusiveOrExpression -> ^( BINARY OP_BIT_OR $inclusiveOrExpression $a) )*
            loop50:
            do {
                int alt50=2;
                alt50 = dfa50.predict(input);
                switch (alt50) {
            	case 1 :
            	    // JFSL.g:598:9: '|' a= exclusiveOrExpression
            	    {
            	    char_literal181=(Token)match(input,163,FOLLOW_163_in_inclusiveOrExpression2570); if (state.failed) return retval; 
            	    if ( state.backtracking==0 ) stream_163.add(char_literal181);

            	    pushFollow(FOLLOW_exclusiveOrExpression_in_inclusiveOrExpression2574);
            	    a=exclusiveOrExpression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_exclusiveOrExpression.add(a.getTree());


            	    // AST REWRITE
            	    // elements: inclusiveOrExpression, a
            	    // token labels: 
            	    // rule labels: retval, a
            	    // token list labels: 
            	    // rule list labels: 
            	    // wildcard labels: 
            	    if ( state.backtracking==0 ) {
            	    retval.tree = root_0;
            	    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);
            	    RewriteRuleSubtreeStream stream_a=new RewriteRuleSubtreeStream(adaptor,"rule a",a!=null?a.tree:null);

            	    root_0 = (Node)adaptor.nil();
            	    // 598:37: -> ^( BINARY OP_BIT_OR $inclusiveOrExpression $a)
            	    {
            	        // JFSL.g:598:40: ^( BINARY OP_BIT_OR $inclusiveOrExpression $a)
            	        {
            	        Node root_1 = (Node)adaptor.nil();
            	        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(BINARY, "BINARY"), root_1);

            	        adaptor.addChild(root_1, (Node)adaptor.create(OP_BIT_OR, "OP_BIT_OR"));
            	        adaptor.addChild(root_1, stream_retval.nextTree());
            	        adaptor.addChild(root_1, stream_a.nextTree());

            	        adaptor.addChild(root_0, root_1);
            	        }

            	    }

            	    retval.tree = root_0;}
            	    }
            	    break;

            	default :
            	    break loop50;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 38, inclusiveOrExpression_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "inclusiveOrExpression"

    public static class exclusiveOrExpression_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "exclusiveOrExpression"
    // JFSL.g:602:1: exclusiveOrExpression : ( andExpression -> andExpression ) ( '^' a= andExpression -> ^( BINARY OP_BIT_XOR $exclusiveOrExpression $a) )* ;
    public final JFSLParser.exclusiveOrExpression_return exclusiveOrExpression() throws RecognitionException {
        JFSLParser.exclusiveOrExpression_return retval = new JFSLParser.exclusiveOrExpression_return();
        retval.start = input.LT(1);
        int exclusiveOrExpression_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal183=null;
        JFSLParser.andExpression_return a = null;

        JFSLParser.andExpression_return andExpression182 = null;


        Node char_literal183_tree=null;
        RewriteRuleTokenStream stream_186=new RewriteRuleTokenStream(adaptor,"token 186");
        RewriteRuleSubtreeStream stream_andExpression=new RewriteRuleSubtreeStream(adaptor,"rule andExpression");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 39) ) { return retval; }
            // JFSL.g:603:3: ( ( andExpression -> andExpression ) ( '^' a= andExpression -> ^( BINARY OP_BIT_XOR $exclusiveOrExpression $a) )* )
            // JFSL.g:603:5: ( andExpression -> andExpression ) ( '^' a= andExpression -> ^( BINARY OP_BIT_XOR $exclusiveOrExpression $a) )*
            {
            // JFSL.g:603:5: ( andExpression -> andExpression )
            // JFSL.g:603:6: andExpression
            {
            pushFollow(FOLLOW_andExpression_in_exclusiveOrExpression2611);
            andExpression182=andExpression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_andExpression.add(andExpression182.getTree());


            // AST REWRITE
            // elements: andExpression
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 603:20: -> andExpression
            {
                adaptor.addChild(root_0, stream_andExpression.nextTree());

            }

            retval.tree = root_0;}
            }

            // JFSL.g:604:7: ( '^' a= andExpression -> ^( BINARY OP_BIT_XOR $exclusiveOrExpression $a) )*
            loop51:
            do {
                int alt51=2;
                int LA51_0 = input.LA(1);

                if ( (LA51_0==186) ) {
                    alt51=1;
                }


                switch (alt51) {
            	case 1 :
            	    // JFSL.g:604:9: '^' a= andExpression
            	    {
            	    char_literal183=(Token)match(input,186,FOLLOW_186_in_exclusiveOrExpression2626); if (state.failed) return retval; 
            	    if ( state.backtracking==0 ) stream_186.add(char_literal183);

            	    pushFollow(FOLLOW_andExpression_in_exclusiveOrExpression2630);
            	    a=andExpression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_andExpression.add(a.getTree());


            	    // AST REWRITE
            	    // elements: a, exclusiveOrExpression
            	    // token labels: 
            	    // rule labels: retval, a
            	    // token list labels: 
            	    // rule list labels: 
            	    // wildcard labels: 
            	    if ( state.backtracking==0 ) {
            	    retval.tree = root_0;
            	    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);
            	    RewriteRuleSubtreeStream stream_a=new RewriteRuleSubtreeStream(adaptor,"rule a",a!=null?a.tree:null);

            	    root_0 = (Node)adaptor.nil();
            	    // 604:29: -> ^( BINARY OP_BIT_XOR $exclusiveOrExpression $a)
            	    {
            	        // JFSL.g:604:32: ^( BINARY OP_BIT_XOR $exclusiveOrExpression $a)
            	        {
            	        Node root_1 = (Node)adaptor.nil();
            	        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(BINARY, "BINARY"), root_1);

            	        adaptor.addChild(root_1, (Node)adaptor.create(OP_BIT_XOR, "OP_BIT_XOR"));
            	        adaptor.addChild(root_1, stream_retval.nextTree());
            	        adaptor.addChild(root_1, stream_a.nextTree());

            	        adaptor.addChild(root_0, root_1);
            	        }

            	    }

            	    retval.tree = root_0;}
            	    }
            	    break;

            	default :
            	    break loop51;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 39, exclusiveOrExpression_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "exclusiveOrExpression"

    public static class andExpression_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "andExpression"
    // JFSL.g:608:1: andExpression : ( equalityExpression -> equalityExpression ) ( bitAndOp a= equalityExpression -> ^( BINARY bitAndOp $andExpression $a) )* ;
    public final JFSLParser.andExpression_return andExpression() throws RecognitionException {
        JFSLParser.andExpression_return retval = new JFSLParser.andExpression_return();
        retval.start = input.LT(1);
        int andExpression_StartIndex = input.index();
        Node root_0 = null;

        JFSLParser.equalityExpression_return a = null;

        JFSLParser.equalityExpression_return equalityExpression184 = null;

        JFSLParser.bitAndOp_return bitAndOp185 = null;


        RewriteRuleSubtreeStream stream_equalityExpression=new RewriteRuleSubtreeStream(adaptor,"rule equalityExpression");
        RewriteRuleSubtreeStream stream_bitAndOp=new RewriteRuleSubtreeStream(adaptor,"rule bitAndOp");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 40) ) { return retval; }
            // JFSL.g:609:3: ( ( equalityExpression -> equalityExpression ) ( bitAndOp a= equalityExpression -> ^( BINARY bitAndOp $andExpression $a) )* )
            // JFSL.g:609:7: ( equalityExpression -> equalityExpression ) ( bitAndOp a= equalityExpression -> ^( BINARY bitAndOp $andExpression $a) )*
            {
            // JFSL.g:609:7: ( equalityExpression -> equalityExpression )
            // JFSL.g:609:8: equalityExpression
            {
            pushFollow(FOLLOW_equalityExpression_in_andExpression2669);
            equalityExpression184=equalityExpression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_equalityExpression.add(equalityExpression184.getTree());


            // AST REWRITE
            // elements: equalityExpression
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 609:27: -> equalityExpression
            {
                adaptor.addChild(root_0, stream_equalityExpression.nextTree());

            }

            retval.tree = root_0;}
            }

            // JFSL.g:610:7: ( bitAndOp a= equalityExpression -> ^( BINARY bitAndOp $andExpression $a) )*
            loop52:
            do {
                int alt52=2;
                int LA52_0 = input.LA(1);

                if ( (LA52_0==187) ) {
                    alt52=1;
                }


                switch (alt52) {
            	case 1 :
            	    // JFSL.g:610:9: bitAndOp a= equalityExpression
            	    {
            	    pushFollow(FOLLOW_bitAndOp_in_andExpression2684);
            	    bitAndOp185=bitAndOp();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_bitAndOp.add(bitAndOp185.getTree());
            	    pushFollow(FOLLOW_equalityExpression_in_andExpression2688);
            	    a=equalityExpression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_equalityExpression.add(a.getTree());


            	    // AST REWRITE
            	    // elements: a, bitAndOp, andExpression
            	    // token labels: 
            	    // rule labels: retval, a
            	    // token list labels: 
            	    // rule list labels: 
            	    // wildcard labels: 
            	    if ( state.backtracking==0 ) {
            	    retval.tree = root_0;
            	    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);
            	    RewriteRuleSubtreeStream stream_a=new RewriteRuleSubtreeStream(adaptor,"rule a",a!=null?a.tree:null);

            	    root_0 = (Node)adaptor.nil();
            	    // 610:39: -> ^( BINARY bitAndOp $andExpression $a)
            	    {
            	        // JFSL.g:610:42: ^( BINARY bitAndOp $andExpression $a)
            	        {
            	        Node root_1 = (Node)adaptor.nil();
            	        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(BINARY, "BINARY"), root_1);

            	        adaptor.addChild(root_1, stream_bitAndOp.nextTree());
            	        adaptor.addChild(root_1, stream_retval.nextTree());
            	        adaptor.addChild(root_1, stream_a.nextTree());

            	        adaptor.addChild(root_0, root_1);
            	        }

            	    }

            	    retval.tree = root_0;}
            	    }
            	    break;

            	default :
            	    break loop52;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 40, andExpression_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "andExpression"

    public static class bitAndOp_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "bitAndOp"
    // JFSL.g:614:1: bitAndOp : '&' -> OP_BIT_AND_OR_INTERSECTION ;
    public final JFSLParser.bitAndOp_return bitAndOp() throws RecognitionException {
        JFSLParser.bitAndOp_return retval = new JFSLParser.bitAndOp_return();
        retval.start = input.LT(1);
        int bitAndOp_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal186=null;

        Node char_literal186_tree=null;
        RewriteRuleTokenStream stream_187=new RewriteRuleTokenStream(adaptor,"token 187");

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 41) ) { return retval; }
            // JFSL.g:615:3: ( '&' -> OP_BIT_AND_OR_INTERSECTION )
            // JFSL.g:615:5: '&'
            {
            char_literal186=(Token)match(input,187,FOLLOW_187_in_bitAndOp2725); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_187.add(char_literal186);



            // AST REWRITE
            // elements: 
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 615:9: -> OP_BIT_AND_OR_INTERSECTION
            {
                adaptor.addChild(root_0, (Node)adaptor.create(OP_BIT_AND_OR_INTERSECTION, "OP_BIT_AND_OR_INTERSECTION"));

            }

            retval.tree = root_0;}
            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 41, bitAndOp_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "bitAndOp"

    public static class equalityExpression_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "equalityExpression"
    // JFSL.g:619:1: equalityExpression : ( instanceOfExpression -> instanceOfExpression ) (op= equalityOp a= instanceOfExpression -> ^( BINARY $op $equalityExpression $a) )* ;
    public final JFSLParser.equalityExpression_return equalityExpression() throws RecognitionException {
        JFSLParser.equalityExpression_return retval = new JFSLParser.equalityExpression_return();
        retval.start = input.LT(1);
        int equalityExpression_StartIndex = input.index();
        Node root_0 = null;

        JFSLParser.equalityOp_return op = null;

        JFSLParser.instanceOfExpression_return a = null;

        JFSLParser.instanceOfExpression_return instanceOfExpression187 = null;


        RewriteRuleSubtreeStream stream_instanceOfExpression=new RewriteRuleSubtreeStream(adaptor,"rule instanceOfExpression");
        RewriteRuleSubtreeStream stream_equalityOp=new RewriteRuleSubtreeStream(adaptor,"rule equalityOp");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 42) ) { return retval; }
            // JFSL.g:620:3: ( ( instanceOfExpression -> instanceOfExpression ) (op= equalityOp a= instanceOfExpression -> ^( BINARY $op $equalityExpression $a) )* )
            // JFSL.g:620:7: ( instanceOfExpression -> instanceOfExpression ) (op= equalityOp a= instanceOfExpression -> ^( BINARY $op $equalityExpression $a) )*
            {
            // JFSL.g:620:7: ( instanceOfExpression -> instanceOfExpression )
            // JFSL.g:620:8: instanceOfExpression
            {
            pushFollow(FOLLOW_instanceOfExpression_in_equalityExpression2747);
            instanceOfExpression187=instanceOfExpression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_instanceOfExpression.add(instanceOfExpression187.getTree());


            // AST REWRITE
            // elements: instanceOfExpression
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 620:29: -> instanceOfExpression
            {
                adaptor.addChild(root_0, stream_instanceOfExpression.nextTree());

            }

            retval.tree = root_0;}
            }

            // JFSL.g:621:7: (op= equalityOp a= instanceOfExpression -> ^( BINARY $op $equalityExpression $a) )*
            loop53:
            do {
                int alt53=2;
                int LA53_0 = input.LA(1);

                if ( (LA53_0==182) ) {
                    int LA53_2 = input.LA(2);

                    if ( ((LA53_2>=Identifier && LA53_2<=DecimalLiteral)||LA53_2==144||LA53_2==150||(LA53_2>=168 && LA53_2<=178)||(LA53_2>=182 && LA53_2<=183)||LA53_2==186||(LA53_2>=191 && LA53_2<=193)||LA53_2==196||(LA53_2>=199 && LA53_2<=211)) ) {
                        alt53=1;
                    }


                }
                else if ( (LA53_0==183) ) {
                    alt53=1;
                }


                switch (alt53) {
            	case 1 :
            	    // JFSL.g:621:9: op= equalityOp a= instanceOfExpression
            	    {
            	    pushFollow(FOLLOW_equalityOp_in_equalityExpression2764);
            	    op=equalityOp();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_equalityOp.add(op.getTree());
            	    pushFollow(FOLLOW_instanceOfExpression_in_equalityExpression2768);
            	    a=instanceOfExpression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_instanceOfExpression.add(a.getTree());


            	    // AST REWRITE
            	    // elements: op, a, equalityExpression
            	    // token labels: 
            	    // rule labels: retval, op, a
            	    // token list labels: 
            	    // rule list labels: 
            	    // wildcard labels: 
            	    if ( state.backtracking==0 ) {
            	    retval.tree = root_0;
            	    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);
            	    RewriteRuleSubtreeStream stream_op=new RewriteRuleSubtreeStream(adaptor,"rule op",op!=null?op.tree:null);
            	    RewriteRuleSubtreeStream stream_a=new RewriteRuleSubtreeStream(adaptor,"rule a",a!=null?a.tree:null);

            	    root_0 = (Node)adaptor.nil();
            	    // 621:46: -> ^( BINARY $op $equalityExpression $a)
            	    {
            	        // JFSL.g:621:49: ^( BINARY $op $equalityExpression $a)
            	        {
            	        Node root_1 = (Node)adaptor.nil();
            	        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(BINARY, "BINARY"), root_1);

            	        adaptor.addChild(root_1, stream_op.nextTree());
            	        adaptor.addChild(root_1, stream_retval.nextTree());
            	        adaptor.addChild(root_1, stream_a.nextTree());

            	        adaptor.addChild(root_0, root_1);
            	        }

            	    }

            	    retval.tree = root_0;}
            	    }
            	    break;

            	default :
            	    break loop53;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 42, equalityExpression_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "equalityExpression"

    public static class equalityOp_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "equalityOp"
    // JFSL.g:625:1: equalityOp : ( '=' '=' -> OP_EQ | '!' '=' -> OP_NEQ | '=' -> OP_EQ );
    public final JFSLParser.equalityOp_return equalityOp() throws RecognitionException {
        JFSLParser.equalityOp_return retval = new JFSLParser.equalityOp_return();
        retval.start = input.LT(1);
        int equalityOp_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal188=null;
        Token char_literal189=null;
        Token char_literal190=null;
        Token char_literal191=null;
        Token char_literal192=null;

        Node char_literal188_tree=null;
        Node char_literal189_tree=null;
        Node char_literal190_tree=null;
        Node char_literal191_tree=null;
        Node char_literal192_tree=null;
        RewriteRuleTokenStream stream_182=new RewriteRuleTokenStream(adaptor,"token 182");
        RewriteRuleTokenStream stream_183=new RewriteRuleTokenStream(adaptor,"token 183");

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 43) ) { return retval; }
            // JFSL.g:626:3: ( '=' '=' -> OP_EQ | '!' '=' -> OP_NEQ | '=' -> OP_EQ )
            int alt54=3;
            int LA54_0 = input.LA(1);

            if ( (LA54_0==182) ) {
                int LA54_1 = input.LA(2);

                if ( (LA54_1==182) ) {
                    alt54=1;
                }
                else if ( ((LA54_1>=Identifier && LA54_1<=DecimalLiteral)||LA54_1==144||LA54_1==150||(LA54_1>=168 && LA54_1<=178)||LA54_1==183||LA54_1==186||(LA54_1>=191 && LA54_1<=193)||LA54_1==196||(LA54_1>=199 && LA54_1<=211)) ) {
                    alt54=3;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 54, 1, input);

                    throw nvae;
                }
            }
            else if ( (LA54_0==183) ) {
                alt54=2;
            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 54, 0, input);

                throw nvae;
            }
            switch (alt54) {
                case 1 :
                    // JFSL.g:626:7: '=' '='
                    {
                    char_literal188=(Token)match(input,182,FOLLOW_182_in_equalityOp2809); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_182.add(char_literal188);

                    char_literal189=(Token)match(input,182,FOLLOW_182_in_equalityOp2811); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_182.add(char_literal189);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 626:15: -> OP_EQ
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_EQ, "OP_EQ"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 2 :
                    // JFSL.g:626:26: '!' '='
                    {
                    char_literal190=(Token)match(input,183,FOLLOW_183_in_equalityOp2819); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_183.add(char_literal190);

                    char_literal191=(Token)match(input,182,FOLLOW_182_in_equalityOp2821); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_182.add(char_literal191);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 626:34: -> OP_NEQ
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_NEQ, "OP_NEQ"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 3 :
                    // JFSL.g:626:46: '='
                    {
                    char_literal192=(Token)match(input,182,FOLLOW_182_in_equalityOp2829); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_182.add(char_literal192);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 626:50: -> OP_EQ
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_EQ, "OP_EQ"));

                    }

                    retval.tree = root_0;}
                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 43, equalityOp_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "equalityOp"

    public static class instanceOfExpression_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "instanceOfExpression"
    // JFSL.g:629:1: instanceOfExpression : ( relationalExpression -> relationalExpression ) ( 'instanceof' a= type -> ^( BINARY OP_INSTANCEOF $instanceOfExpression $a) )? ;
    public final JFSLParser.instanceOfExpression_return instanceOfExpression() throws RecognitionException {
        JFSLParser.instanceOfExpression_return retval = new JFSLParser.instanceOfExpression_return();
        retval.start = input.LT(1);
        int instanceOfExpression_StartIndex = input.index();
        Node root_0 = null;

        Token string_literal194=null;
        JFSLParser.type_return a = null;

        JFSLParser.relationalExpression_return relationalExpression193 = null;


        Node string_literal194_tree=null;
        RewriteRuleTokenStream stream_188=new RewriteRuleTokenStream(adaptor,"token 188");
        RewriteRuleSubtreeStream stream_relationalExpression=new RewriteRuleSubtreeStream(adaptor,"rule relationalExpression");
        RewriteRuleSubtreeStream stream_type=new RewriteRuleSubtreeStream(adaptor,"rule type");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 44) ) { return retval; }
            // JFSL.g:630:3: ( ( relationalExpression -> relationalExpression ) ( 'instanceof' a= type -> ^( BINARY OP_INSTANCEOF $instanceOfExpression $a) )? )
            // JFSL.g:630:7: ( relationalExpression -> relationalExpression ) ( 'instanceof' a= type -> ^( BINARY OP_INSTANCEOF $instanceOfExpression $a) )?
            {
            // JFSL.g:630:7: ( relationalExpression -> relationalExpression )
            // JFSL.g:630:8: relationalExpression
            {
            pushFollow(FOLLOW_relationalExpression_in_instanceOfExpression2851);
            relationalExpression193=relationalExpression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_relationalExpression.add(relationalExpression193.getTree());


            // AST REWRITE
            // elements: relationalExpression
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 630:29: -> relationalExpression
            {
                adaptor.addChild(root_0, stream_relationalExpression.nextTree());

            }

            retval.tree = root_0;}
            }

            // JFSL.g:631:7: ( 'instanceof' a= type -> ^( BINARY OP_INSTANCEOF $instanceOfExpression $a) )?
            int alt55=2;
            int LA55_0 = input.LA(1);

            if ( (LA55_0==188) ) {
                alt55=1;
            }
            switch (alt55) {
                case 1 :
                    // JFSL.g:631:9: 'instanceof' a= type
                    {
                    string_literal194=(Token)match(input,188,FOLLOW_188_in_instanceOfExpression2866); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_188.add(string_literal194);

                    pushFollow(FOLLOW_type_in_instanceOfExpression2870);
                    a=type();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_type.add(a.getTree());


                    // AST REWRITE
                    // elements: a, instanceOfExpression
                    // token labels: 
                    // rule labels: retval, a
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);
                    RewriteRuleSubtreeStream stream_a=new RewriteRuleSubtreeStream(adaptor,"rule a",a!=null?a.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 631:29: -> ^( BINARY OP_INSTANCEOF $instanceOfExpression $a)
                    {
                        // JFSL.g:631:32: ^( BINARY OP_INSTANCEOF $instanceOfExpression $a)
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(BINARY, "BINARY"), root_1);

                        adaptor.addChild(root_1, (Node)adaptor.create(OP_INSTANCEOF, "OP_INSTANCEOF"));
                        adaptor.addChild(root_1, stream_retval.nextTree());
                        adaptor.addChild(root_1, stream_a.nextTree());

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;

            }


            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 44, instanceOfExpression_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "instanceOfExpression"

    public static class relationalExpression_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "relationalExpression"
    // JFSL.g:635:1: relationalExpression : ( setUnaryExpression -> setUnaryExpression ) (op= relationalOp a= setUnaryExpression -> ^( BINARY $op $relationalExpression $a) )* ;
    public final JFSLParser.relationalExpression_return relationalExpression() throws RecognitionException {
        JFSLParser.relationalExpression_return retval = new JFSLParser.relationalExpression_return();
        retval.start = input.LT(1);
        int relationalExpression_StartIndex = input.index();
        Node root_0 = null;

        JFSLParser.relationalOp_return op = null;

        JFSLParser.setUnaryExpression_return a = null;

        JFSLParser.setUnaryExpression_return setUnaryExpression195 = null;


        RewriteRuleSubtreeStream stream_relationalOp=new RewriteRuleSubtreeStream(adaptor,"rule relationalOp");
        RewriteRuleSubtreeStream stream_setUnaryExpression=new RewriteRuleSubtreeStream(adaptor,"rule setUnaryExpression");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 45) ) { return retval; }
            // JFSL.g:636:3: ( ( setUnaryExpression -> setUnaryExpression ) (op= relationalOp a= setUnaryExpression -> ^( BINARY $op $relationalExpression $a) )* )
            // JFSL.g:636:7: ( setUnaryExpression -> setUnaryExpression ) (op= relationalOp a= setUnaryExpression -> ^( BINARY $op $relationalExpression $a) )*
            {
            // JFSL.g:636:7: ( setUnaryExpression -> setUnaryExpression )
            // JFSL.g:636:8: setUnaryExpression
            {
            pushFollow(FOLLOW_setUnaryExpression_in_relationalExpression2909);
            setUnaryExpression195=setUnaryExpression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_setUnaryExpression.add(setUnaryExpression195.getTree());


            // AST REWRITE
            // elements: setUnaryExpression
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 636:27: -> setUnaryExpression
            {
                adaptor.addChild(root_0, stream_setUnaryExpression.nextTree());

            }

            retval.tree = root_0;}
            }

            // JFSL.g:637:7: (op= relationalOp a= setUnaryExpression -> ^( BINARY $op $relationalExpression $a) )*
            loop56:
            do {
                int alt56=2;
                switch ( input.LA(1) ) {
                case 183:
                    {
                    int LA56_2 = input.LA(2);

                    if ( (LA56_2==189) ) {
                        alt56=1;
                    }


                    }
                    break;
                case 146:
                    {
                    switch ( input.LA(2) ) {
                    case 182:
                        {
                        int LA56_5 = input.LA(3);

                        if ( ((LA56_5>=Identifier && LA56_5<=DecimalLiteral)||LA56_5==144||LA56_5==150||(LA56_5>=168 && LA56_5<=178)||LA56_5==183||LA56_5==186||(LA56_5>=191 && LA56_5<=193)||LA56_5==196||(LA56_5>=199 && LA56_5<=211)) ) {
                            alt56=1;
                        }


                        }
                        break;
                    case 183:
                        {
                        int LA56_6 = input.LA(3);

                        if ( ((LA56_6>=Identifier && LA56_6<=DecimalLiteral)||LA56_6==144||LA56_6==150||(LA56_6>=168 && LA56_6<=178)||LA56_6==183||LA56_6==186||(LA56_6>=191 && LA56_6<=192)||LA56_6==196||(LA56_6>=199 && LA56_6<=206)) ) {
                            alt56=1;
                        }


                        }
                        break;
                    case Identifier:
                    case FloatingPointLiteral:
                    case CharacterLiteral:
                    case StringLiteral:
                    case HexLiteral:
                    case OctalLiteral:
                    case DecimalLiteral:
                    case 144:
                    case 150:
                    case 168:
                    case 169:
                    case 170:
                    case 171:
                    case 172:
                    case 173:
                    case 174:
                    case 175:
                    case 176:
                    case 177:
                    case 178:
                    case 186:
                    case 191:
                    case 192:
                    case 193:
                    case 196:
                    case 199:
                    case 200:
                    case 201:
                    case 202:
                    case 203:
                    case 204:
                    case 205:
                    case 206:
                    case 207:
                    case 208:
                    case 209:
                    case 210:
                    case 211:
                        {
                        alt56=1;
                        }
                        break;

                    }

                    }
                    break;
                case 148:
                case 189:
                    {
                    alt56=1;
                    }
                    break;

                }

                switch (alt56) {
            	case 1 :
            	    // JFSL.g:637:9: op= relationalOp a= setUnaryExpression
            	    {
            	    pushFollow(FOLLOW_relationalOp_in_relationalExpression2926);
            	    op=relationalOp();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_relationalOp.add(op.getTree());
            	    pushFollow(FOLLOW_setUnaryExpression_in_relationalExpression2930);
            	    a=setUnaryExpression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_setUnaryExpression.add(a.getTree());


            	    // AST REWRITE
            	    // elements: a, relationalExpression, op
            	    // token labels: 
            	    // rule labels: retval, op, a
            	    // token list labels: 
            	    // rule list labels: 
            	    // wildcard labels: 
            	    if ( state.backtracking==0 ) {
            	    retval.tree = root_0;
            	    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);
            	    RewriteRuleSubtreeStream stream_op=new RewriteRuleSubtreeStream(adaptor,"rule op",op!=null?op.tree:null);
            	    RewriteRuleSubtreeStream stream_a=new RewriteRuleSubtreeStream(adaptor,"rule a",a!=null?a.tree:null);

            	    root_0 = (Node)adaptor.nil();
            	    // 637:46: -> ^( BINARY $op $relationalExpression $a)
            	    {
            	        // JFSL.g:637:49: ^( BINARY $op $relationalExpression $a)
            	        {
            	        Node root_1 = (Node)adaptor.nil();
            	        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(BINARY, "BINARY"), root_1);

            	        adaptor.addChild(root_1, stream_op.nextTree());
            	        adaptor.addChild(root_1, stream_retval.nextTree());
            	        adaptor.addChild(root_1, stream_a.nextTree());

            	        adaptor.addChild(root_0, root_1);
            	        }

            	    }

            	    retval.tree = root_0;}
            	    }
            	    break;

            	default :
            	    break loop56;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 45, relationalExpression_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "relationalExpression"

    public static class inOp_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "inOp"
    // JFSL.g:641:1: inOp : 'in' -> OP_SET_SUBSET ;
    public final JFSLParser.inOp_return inOp() throws RecognitionException {
        JFSLParser.inOp_return retval = new JFSLParser.inOp_return();
        retval.start = input.LT(1);
        int inOp_StartIndex = input.index();
        Node root_0 = null;

        Token string_literal196=null;

        Node string_literal196_tree=null;
        RewriteRuleTokenStream stream_189=new RewriteRuleTokenStream(adaptor,"token 189");

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 46) ) { return retval; }
            // JFSL.g:642:3: ( 'in' -> OP_SET_SUBSET )
            // JFSL.g:642:5: 'in'
            {
            string_literal196=(Token)match(input,189,FOLLOW_189_in_inOp2969); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_189.add(string_literal196);



            // AST REWRITE
            // elements: 
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 642:10: -> OP_SET_SUBSET
            {
                adaptor.addChild(root_0, (Node)adaptor.create(OP_SET_SUBSET, "OP_SET_SUBSET"));

            }

            retval.tree = root_0;}
            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 46, inOp_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "inOp"

    public static class relationalOp_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "relationalOp"
    // JFSL.g:645:1: relationalOp : ( '<' '=' -> OP_LEQ | '>' '=' -> OP_GEQ | '<' -> OP_LT | '>' -> OP_GT | inOp | '!' 'in' -> OP_NSET_SUBSET );
    public final JFSLParser.relationalOp_return relationalOp() throws RecognitionException {
        JFSLParser.relationalOp_return retval = new JFSLParser.relationalOp_return();
        retval.start = input.LT(1);
        int relationalOp_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal197=null;
        Token char_literal198=null;
        Token char_literal199=null;
        Token char_literal200=null;
        Token char_literal201=null;
        Token char_literal202=null;
        Token char_literal204=null;
        Token string_literal205=null;
        JFSLParser.inOp_return inOp203 = null;


        Node char_literal197_tree=null;
        Node char_literal198_tree=null;
        Node char_literal199_tree=null;
        Node char_literal200_tree=null;
        Node char_literal201_tree=null;
        Node char_literal202_tree=null;
        Node char_literal204_tree=null;
        Node string_literal205_tree=null;
        RewriteRuleTokenStream stream_146=new RewriteRuleTokenStream(adaptor,"token 146");
        RewriteRuleTokenStream stream_182=new RewriteRuleTokenStream(adaptor,"token 182");
        RewriteRuleTokenStream stream_183=new RewriteRuleTokenStream(adaptor,"token 183");
        RewriteRuleTokenStream stream_148=new RewriteRuleTokenStream(adaptor,"token 148");
        RewriteRuleTokenStream stream_189=new RewriteRuleTokenStream(adaptor,"token 189");

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 47) ) { return retval; }
            // JFSL.g:646:3: ( '<' '=' -> OP_LEQ | '>' '=' -> OP_GEQ | '<' -> OP_LT | '>' -> OP_GT | inOp | '!' 'in' -> OP_NSET_SUBSET )
            int alt57=6;
            switch ( input.LA(1) ) {
            case 146:
                {
                int LA57_1 = input.LA(2);

                if ( (LA57_1==182) ) {
                    alt57=1;
                }
                else if ( ((LA57_1>=Identifier && LA57_1<=DecimalLiteral)||LA57_1==144||LA57_1==150||(LA57_1>=168 && LA57_1<=178)||LA57_1==183||LA57_1==186||(LA57_1>=191 && LA57_1<=193)||LA57_1==196||(LA57_1>=199 && LA57_1<=211)) ) {
                    alt57=3;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 57, 1, input);

                    throw nvae;
                }
                }
                break;
            case 148:
                {
                int LA57_2 = input.LA(2);

                if ( (LA57_2==182) ) {
                    alt57=2;
                }
                else if ( ((LA57_2>=Identifier && LA57_2<=DecimalLiteral)||LA57_2==144||LA57_2==150||(LA57_2>=168 && LA57_2<=178)||LA57_2==183||LA57_2==186||(LA57_2>=191 && LA57_2<=193)||LA57_2==196||(LA57_2>=199 && LA57_2<=211)) ) {
                    alt57=4;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 57, 2, input);

                    throw nvae;
                }
                }
                break;
            case 189:
                {
                alt57=5;
                }
                break;
            case 183:
                {
                alt57=6;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 57, 0, input);

                throw nvae;
            }

            switch (alt57) {
                case 1 :
                    // JFSL.g:646:5: '<' '='
                    {
                    char_literal197=(Token)match(input,146,FOLLOW_146_in_relationalOp2988); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_146.add(char_literal197);

                    char_literal198=(Token)match(input,182,FOLLOW_182_in_relationalOp2990); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_182.add(char_literal198);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 646:13: -> OP_LEQ
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_LEQ, "OP_LEQ"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 2 :
                    // JFSL.g:646:25: '>' '='
                    {
                    char_literal199=(Token)match(input,148,FOLLOW_148_in_relationalOp2998); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_148.add(char_literal199);

                    char_literal200=(Token)match(input,182,FOLLOW_182_in_relationalOp3000); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_182.add(char_literal200);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 646:33: -> OP_GEQ
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_GEQ, "OP_GEQ"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 3 :
                    // JFSL.g:646:45: '<'
                    {
                    char_literal201=(Token)match(input,146,FOLLOW_146_in_relationalOp3008); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_146.add(char_literal201);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 646:49: -> OP_LT
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_LT, "OP_LT"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 4 :
                    // JFSL.g:646:60: '>'
                    {
                    char_literal202=(Token)match(input,148,FOLLOW_148_in_relationalOp3016); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_148.add(char_literal202);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 646:64: -> OP_GT
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_GT, "OP_GT"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 5 :
                    // JFSL.g:646:75: inOp
                    {
                    root_0 = (Node)adaptor.nil();

                    pushFollow(FOLLOW_inOp_in_relationalOp3024);
                    inOp203=inOp();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, inOp203.getTree());

                    }
                    break;
                case 6 :
                    // JFSL.g:646:82: '!' 'in'
                    {
                    char_literal204=(Token)match(input,183,FOLLOW_183_in_relationalOp3028); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_183.add(char_literal204);

                    string_literal205=(Token)match(input,189,FOLLOW_189_in_relationalOp3030); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_189.add(string_literal205);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 646:91: -> OP_NSET_SUBSET
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_NSET_SUBSET, "OP_NSET_SUBSET"));

                    }

                    retval.tree = root_0;}
                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 47, relationalOp_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "relationalOp"

    public static class setUnaryExpression_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "setUnaryExpression"
    // JFSL.g:649:1: setUnaryExpression : ( setUnaryOp joinExpression -> ^( UNARY setUnaryOp joinExpression ) | shiftExpression );
    public final JFSLParser.setUnaryExpression_return setUnaryExpression() throws RecognitionException {
        JFSLParser.setUnaryExpression_return retval = new JFSLParser.setUnaryExpression_return();
        retval.start = input.LT(1);
        int setUnaryExpression_StartIndex = input.index();
        Node root_0 = null;

        JFSLParser.setUnaryOp_return setUnaryOp206 = null;

        JFSLParser.joinExpression_return joinExpression207 = null;

        JFSLParser.shiftExpression_return shiftExpression208 = null;


        RewriteRuleSubtreeStream stream_setUnaryOp=new RewriteRuleSubtreeStream(adaptor,"rule setUnaryOp");
        RewriteRuleSubtreeStream stream_joinExpression=new RewriteRuleSubtreeStream(adaptor,"rule joinExpression");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 48) ) { return retval; }
            // JFSL.g:650:3: ( setUnaryOp joinExpression -> ^( UNARY setUnaryOp joinExpression ) | shiftExpression )
            int alt58=2;
            int LA58_0 = input.LA(1);

            if ( ((LA58_0>=207 && LA58_0<=211)) ) {
                alt58=1;
            }
            else if ( ((LA58_0>=Identifier && LA58_0<=DecimalLiteral)||LA58_0==144||LA58_0==150||(LA58_0>=168 && LA58_0<=178)||LA58_0==183||LA58_0==186||(LA58_0>=191 && LA58_0<=193)||LA58_0==196||(LA58_0>=199 && LA58_0<=206)) ) {
                alt58=2;
            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 58, 0, input);

                throw nvae;
            }
            switch (alt58) {
                case 1 :
                    // JFSL.g:650:5: setUnaryOp joinExpression
                    {
                    pushFollow(FOLLOW_setUnaryOp_in_setUnaryExpression3049);
                    setUnaryOp206=setUnaryOp();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_setUnaryOp.add(setUnaryOp206.getTree());
                    pushFollow(FOLLOW_joinExpression_in_setUnaryExpression3051);
                    joinExpression207=joinExpression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_joinExpression.add(joinExpression207.getTree());


                    // AST REWRITE
                    // elements: setUnaryOp, joinExpression
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 650:31: -> ^( UNARY setUnaryOp joinExpression )
                    {
                        // JFSL.g:650:34: ^( UNARY setUnaryOp joinExpression )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(UNARY, "UNARY"), root_1);

                        adaptor.addChild(root_1, stream_setUnaryOp.nextTree());
                        adaptor.addChild(root_1, stream_joinExpression.nextTree());

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 2 :
                    // JFSL.g:651:5: shiftExpression
                    {
                    root_0 = (Node)adaptor.nil();

                    pushFollow(FOLLOW_shiftExpression_in_setUnaryExpression3067);
                    shiftExpression208=shiftExpression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, shiftExpression208.getTree());

                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 48, setUnaryExpression_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "setUnaryExpression"

    public static class shiftExpression_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "shiftExpression"
    // JFSL.g:654:1: shiftExpression : ( rangeExpression -> rangeExpression ) (op= shiftOp a= rangeExpression -> ^( BINARY $op $shiftExpression $a) )* ;
    public final JFSLParser.shiftExpression_return shiftExpression() throws RecognitionException {
        JFSLParser.shiftExpression_return retval = new JFSLParser.shiftExpression_return();
        retval.start = input.LT(1);
        int shiftExpression_StartIndex = input.index();
        Node root_0 = null;

        JFSLParser.shiftOp_return op = null;

        JFSLParser.rangeExpression_return a = null;

        JFSLParser.rangeExpression_return rangeExpression209 = null;


        RewriteRuleSubtreeStream stream_rangeExpression=new RewriteRuleSubtreeStream(adaptor,"rule rangeExpression");
        RewriteRuleSubtreeStream stream_shiftOp=new RewriteRuleSubtreeStream(adaptor,"rule shiftOp");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 49) ) { return retval; }
            // JFSL.g:655:3: ( ( rangeExpression -> rangeExpression ) (op= shiftOp a= rangeExpression -> ^( BINARY $op $shiftExpression $a) )* )
            // JFSL.g:655:7: ( rangeExpression -> rangeExpression ) (op= shiftOp a= rangeExpression -> ^( BINARY $op $shiftExpression $a) )*
            {
            // JFSL.g:655:7: ( rangeExpression -> rangeExpression )
            // JFSL.g:655:8: rangeExpression
            {
            pushFollow(FOLLOW_rangeExpression_in_shiftExpression3085);
            rangeExpression209=rangeExpression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_rangeExpression.add(rangeExpression209.getTree());


            // AST REWRITE
            // elements: rangeExpression
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 655:24: -> rangeExpression
            {
                adaptor.addChild(root_0, stream_rangeExpression.nextTree());

            }

            retval.tree = root_0;}
            }

            // JFSL.g:656:7: (op= shiftOp a= rangeExpression -> ^( BINARY $op $shiftExpression $a) )*
            loop59:
            do {
                int alt59=2;
                int LA59_0 = input.LA(1);

                if ( (LA59_0==146) ) {
                    int LA59_1 = input.LA(2);

                    if ( (LA59_1==146) ) {
                        alt59=1;
                    }


                }
                else if ( (LA59_0==148) ) {
                    int LA59_2 = input.LA(2);

                    if ( (LA59_2==148) ) {
                        alt59=1;
                    }


                }


                switch (alt59) {
            	case 1 :
            	    // JFSL.g:656:9: op= shiftOp a= rangeExpression
            	    {
            	    pushFollow(FOLLOW_shiftOp_in_shiftExpression3102);
            	    op=shiftOp();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_shiftOp.add(op.getTree());
            	    pushFollow(FOLLOW_rangeExpression_in_shiftExpression3106);
            	    a=rangeExpression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_rangeExpression.add(a.getTree());


            	    // AST REWRITE
            	    // elements: op, a, shiftExpression
            	    // token labels: 
            	    // rule labels: retval, op, a
            	    // token list labels: 
            	    // rule list labels: 
            	    // wildcard labels: 
            	    if ( state.backtracking==0 ) {
            	    retval.tree = root_0;
            	    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);
            	    RewriteRuleSubtreeStream stream_op=new RewriteRuleSubtreeStream(adaptor,"rule op",op!=null?op.tree:null);
            	    RewriteRuleSubtreeStream stream_a=new RewriteRuleSubtreeStream(adaptor,"rule a",a!=null?a.tree:null);

            	    root_0 = (Node)adaptor.nil();
            	    // 656:38: -> ^( BINARY $op $shiftExpression $a)
            	    {
            	        // JFSL.g:656:41: ^( BINARY $op $shiftExpression $a)
            	        {
            	        Node root_1 = (Node)adaptor.nil();
            	        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(BINARY, "BINARY"), root_1);

            	        adaptor.addChild(root_1, stream_op.nextTree());
            	        adaptor.addChild(root_1, stream_retval.nextTree());
            	        adaptor.addChild(root_1, stream_a.nextTree());

            	        adaptor.addChild(root_0, root_1);
            	        }

            	    }

            	    retval.tree = root_0;}
            	    }
            	    break;

            	default :
            	    break loop59;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 49, shiftExpression_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "shiftExpression"

    public static class shiftOp_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "shiftOp"
    // JFSL.g:660:1: shiftOp : ( '<' '<' -> OP_SHL | '>' '>' '>' -> OP_USHR | '>' '>' -> OP_SHR );
    public final JFSLParser.shiftOp_return shiftOp() throws RecognitionException {
        JFSLParser.shiftOp_return retval = new JFSLParser.shiftOp_return();
        retval.start = input.LT(1);
        int shiftOp_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal210=null;
        Token char_literal211=null;
        Token char_literal212=null;
        Token char_literal213=null;
        Token char_literal214=null;
        Token char_literal215=null;
        Token char_literal216=null;

        Node char_literal210_tree=null;
        Node char_literal211_tree=null;
        Node char_literal212_tree=null;
        Node char_literal213_tree=null;
        Node char_literal214_tree=null;
        Node char_literal215_tree=null;
        Node char_literal216_tree=null;
        RewriteRuleTokenStream stream_146=new RewriteRuleTokenStream(adaptor,"token 146");
        RewriteRuleTokenStream stream_148=new RewriteRuleTokenStream(adaptor,"token 148");

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 50) ) { return retval; }
            // JFSL.g:661:3: ( '<' '<' -> OP_SHL | '>' '>' '>' -> OP_USHR | '>' '>' -> OP_SHR )
            int alt60=3;
            int LA60_0 = input.LA(1);

            if ( (LA60_0==146) ) {
                alt60=1;
            }
            else if ( (LA60_0==148) ) {
                int LA60_2 = input.LA(2);

                if ( (LA60_2==148) ) {
                    int LA60_3 = input.LA(3);

                    if ( (LA60_3==148) ) {
                        alt60=2;
                    }
                    else if ( ((LA60_3>=Identifier && LA60_3<=DecimalLiteral)||LA60_3==144||LA60_3==150||(LA60_3>=168 && LA60_3<=178)||LA60_3==183||LA60_3==186||(LA60_3>=191 && LA60_3<=193)||LA60_3==196||(LA60_3>=199 && LA60_3<=206)) ) {
                        alt60=3;
                    }
                    else {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        NoViableAltException nvae =
                            new NoViableAltException("", 60, 3, input);

                        throw nvae;
                    }
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 60, 2, input);

                    throw nvae;
                }
            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 60, 0, input);

                throw nvae;
            }
            switch (alt60) {
                case 1 :
                    // JFSL.g:661:5: '<' '<'
                    {
                    char_literal210=(Token)match(input,146,FOLLOW_146_in_shiftOp3143); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_146.add(char_literal210);

                    char_literal211=(Token)match(input,146,FOLLOW_146_in_shiftOp3145); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_146.add(char_literal211);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 661:13: -> OP_SHL
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_SHL, "OP_SHL"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 2 :
                    // JFSL.g:661:25: '>' '>' '>'
                    {
                    char_literal212=(Token)match(input,148,FOLLOW_148_in_shiftOp3153); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_148.add(char_literal212);

                    char_literal213=(Token)match(input,148,FOLLOW_148_in_shiftOp3155); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_148.add(char_literal213);

                    char_literal214=(Token)match(input,148,FOLLOW_148_in_shiftOp3157); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_148.add(char_literal214);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 661:37: -> OP_USHR
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_USHR, "OP_USHR"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 3 :
                    // JFSL.g:661:50: '>' '>'
                    {
                    char_literal215=(Token)match(input,148,FOLLOW_148_in_shiftOp3165); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_148.add(char_literal215);

                    char_literal216=(Token)match(input,148,FOLLOW_148_in_shiftOp3167); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_148.add(char_literal216);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 661:58: -> OP_SHR
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_SHR, "OP_SHR"));

                    }

                    retval.tree = root_0;}
                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 50, shiftOp_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "shiftOp"

    public static class rangeExpression_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "rangeExpression"
    // JFSL.g:664:1: rangeExpression : ( additiveExpression -> additiveExpression ) ( '...' a= additiveExpression -> ^( BINARY OP_RANGE $rangeExpression $a) )* ;
    public final JFSLParser.rangeExpression_return rangeExpression() throws RecognitionException {
        JFSLParser.rangeExpression_return retval = new JFSLParser.rangeExpression_return();
        retval.start = input.LT(1);
        int rangeExpression_StartIndex = input.index();
        Node root_0 = null;

        Token string_literal218=null;
        JFSLParser.additiveExpression_return a = null;

        JFSLParser.additiveExpression_return additiveExpression217 = null;


        Node string_literal218_tree=null;
        RewriteRuleTokenStream stream_190=new RewriteRuleTokenStream(adaptor,"token 190");
        RewriteRuleSubtreeStream stream_additiveExpression=new RewriteRuleSubtreeStream(adaptor,"rule additiveExpression");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 51) ) { return retval; }
            // JFSL.g:665:3: ( ( additiveExpression -> additiveExpression ) ( '...' a= additiveExpression -> ^( BINARY OP_RANGE $rangeExpression $a) )* )
            // JFSL.g:665:5: ( additiveExpression -> additiveExpression ) ( '...' a= additiveExpression -> ^( BINARY OP_RANGE $rangeExpression $a) )*
            {
            // JFSL.g:665:5: ( additiveExpression -> additiveExpression )
            // JFSL.g:665:6: additiveExpression
            {
            pushFollow(FOLLOW_additiveExpression_in_rangeExpression3187);
            additiveExpression217=additiveExpression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_additiveExpression.add(additiveExpression217.getTree());


            // AST REWRITE
            // elements: additiveExpression
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 665:25: -> additiveExpression
            {
                adaptor.addChild(root_0, stream_additiveExpression.nextTree());

            }

            retval.tree = root_0;}
            }

            // JFSL.g:666:5: ( '...' a= additiveExpression -> ^( BINARY OP_RANGE $rangeExpression $a) )*
            loop61:
            do {
                int alt61=2;
                int LA61_0 = input.LA(1);

                if ( (LA61_0==190) ) {
                    alt61=1;
                }


                switch (alt61) {
            	case 1 :
            	    // JFSL.g:666:6: '...' a= additiveExpression
            	    {
            	    string_literal218=(Token)match(input,190,FOLLOW_190_in_rangeExpression3199); if (state.failed) return retval; 
            	    if ( state.backtracking==0 ) stream_190.add(string_literal218);

            	    pushFollow(FOLLOW_additiveExpression_in_rangeExpression3203);
            	    a=additiveExpression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_additiveExpression.add(a.getTree());


            	    // AST REWRITE
            	    // elements: a, rangeExpression
            	    // token labels: 
            	    // rule labels: retval, a
            	    // token list labels: 
            	    // rule list labels: 
            	    // wildcard labels: 
            	    if ( state.backtracking==0 ) {
            	    retval.tree = root_0;
            	    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);
            	    RewriteRuleSubtreeStream stream_a=new RewriteRuleSubtreeStream(adaptor,"rule a",a!=null?a.tree:null);

            	    root_0 = (Node)adaptor.nil();
            	    // 666:33: -> ^( BINARY OP_RANGE $rangeExpression $a)
            	    {
            	        // JFSL.g:666:36: ^( BINARY OP_RANGE $rangeExpression $a)
            	        {
            	        Node root_1 = (Node)adaptor.nil();
            	        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(BINARY, "BINARY"), root_1);

            	        adaptor.addChild(root_1, (Node)adaptor.create(OP_RANGE, "OP_RANGE"));
            	        adaptor.addChild(root_1, stream_retval.nextTree());
            	        adaptor.addChild(root_1, stream_a.nextTree());

            	        adaptor.addChild(root_0, root_1);
            	        }

            	    }

            	    retval.tree = root_0;}
            	    }
            	    break;

            	default :
            	    break loop61;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 51, rangeExpression_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "rangeExpression"

    public static class additiveExpression_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "additiveExpression"
    // JFSL.g:670:1: additiveExpression : ( sizeExpression -> sizeExpression ) (op= additiveOp a= sizeExpression -> ^( BINARY $op $additiveExpression $a) )* ;
    public final JFSLParser.additiveExpression_return additiveExpression() throws RecognitionException {
        JFSLParser.additiveExpression_return retval = new JFSLParser.additiveExpression_return();
        retval.start = input.LT(1);
        int additiveExpression_StartIndex = input.index();
        Node root_0 = null;

        JFSLParser.additiveOp_return op = null;

        JFSLParser.sizeExpression_return a = null;

        JFSLParser.sizeExpression_return sizeExpression219 = null;


        RewriteRuleSubtreeStream stream_sizeExpression=new RewriteRuleSubtreeStream(adaptor,"rule sizeExpression");
        RewriteRuleSubtreeStream stream_additiveOp=new RewriteRuleSubtreeStream(adaptor,"rule additiveOp");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 52) ) { return retval; }
            // JFSL.g:671:3: ( ( sizeExpression -> sizeExpression ) (op= additiveOp a= sizeExpression -> ^( BINARY $op $additiveExpression $a) )* )
            // JFSL.g:671:7: ( sizeExpression -> sizeExpression ) (op= additiveOp a= sizeExpression -> ^( BINARY $op $additiveExpression $a) )*
            {
            // JFSL.g:671:7: ( sizeExpression -> sizeExpression )
            // JFSL.g:671:8: sizeExpression
            {
            pushFollow(FOLLOW_sizeExpression_in_additiveExpression3241);
            sizeExpression219=sizeExpression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_sizeExpression.add(sizeExpression219.getTree());


            // AST REWRITE
            // elements: sizeExpression
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 671:23: -> sizeExpression
            {
                adaptor.addChild(root_0, stream_sizeExpression.nextTree());

            }

            retval.tree = root_0;}
            }

            // JFSL.g:672:7: (op= additiveOp a= sizeExpression -> ^( BINARY $op $additiveExpression $a) )*
            loop62:
            do {
                int alt62=2;
                int LA62_0 = input.LA(1);

                if ( ((LA62_0>=191 && LA62_0<=192)) ) {
                    alt62=1;
                }


                switch (alt62) {
            	case 1 :
            	    // JFSL.g:672:9: op= additiveOp a= sizeExpression
            	    {
            	    pushFollow(FOLLOW_additiveOp_in_additiveExpression3259);
            	    op=additiveOp();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_additiveOp.add(op.getTree());
            	    pushFollow(FOLLOW_sizeExpression_in_additiveExpression3263);
            	    a=sizeExpression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_sizeExpression.add(a.getTree());


            	    // AST REWRITE
            	    // elements: a, additiveExpression, op
            	    // token labels: 
            	    // rule labels: retval, op, a
            	    // token list labels: 
            	    // rule list labels: 
            	    // wildcard labels: 
            	    if ( state.backtracking==0 ) {
            	    retval.tree = root_0;
            	    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);
            	    RewriteRuleSubtreeStream stream_op=new RewriteRuleSubtreeStream(adaptor,"rule op",op!=null?op.tree:null);
            	    RewriteRuleSubtreeStream stream_a=new RewriteRuleSubtreeStream(adaptor,"rule a",a!=null?a.tree:null);

            	    root_0 = (Node)adaptor.nil();
            	    // 672:40: -> ^( BINARY $op $additiveExpression $a)
            	    {
            	        // JFSL.g:672:43: ^( BINARY $op $additiveExpression $a)
            	        {
            	        Node root_1 = (Node)adaptor.nil();
            	        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(BINARY, "BINARY"), root_1);

            	        adaptor.addChild(root_1, stream_op.nextTree());
            	        adaptor.addChild(root_1, stream_retval.nextTree());
            	        adaptor.addChild(root_1, stream_a.nextTree());

            	        adaptor.addChild(root_0, root_1);
            	        }

            	    }

            	    retval.tree = root_0;}
            	    }
            	    break;

            	default :
            	    break loop62;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 52, additiveExpression_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "additiveExpression"

    public static class additiveOp_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "additiveOp"
    // JFSL.g:676:1: additiveOp : ( '+' -> OP_PLUS_OR_UNION | '-' -> OP_MINUS_OR_DIFFERENCE );
    public final JFSLParser.additiveOp_return additiveOp() throws RecognitionException {
        JFSLParser.additiveOp_return retval = new JFSLParser.additiveOp_return();
        retval.start = input.LT(1);
        int additiveOp_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal220=null;
        Token char_literal221=null;

        Node char_literal220_tree=null;
        Node char_literal221_tree=null;
        RewriteRuleTokenStream stream_191=new RewriteRuleTokenStream(adaptor,"token 191");
        RewriteRuleTokenStream stream_192=new RewriteRuleTokenStream(adaptor,"token 192");

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 53) ) { return retval; }
            // JFSL.g:677:3: ( '+' -> OP_PLUS_OR_UNION | '-' -> OP_MINUS_OR_DIFFERENCE )
            int alt63=2;
            int LA63_0 = input.LA(1);

            if ( (LA63_0==191) ) {
                alt63=1;
            }
            else if ( (LA63_0==192) ) {
                alt63=2;
            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 63, 0, input);

                throw nvae;
            }
            switch (alt63) {
                case 1 :
                    // JFSL.g:677:5: '+'
                    {
                    char_literal220=(Token)match(input,191,FOLLOW_191_in_additiveOp3304); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_191.add(char_literal220);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 677:11: -> OP_PLUS_OR_UNION
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_PLUS_OR_UNION, "OP_PLUS_OR_UNION"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 2 :
                    // JFSL.g:678:5: '-'
                    {
                    char_literal221=(Token)match(input,192,FOLLOW_192_in_additiveOp3317); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_192.add(char_literal221);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 678:11: -> OP_MINUS_OR_DIFFERENCE
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_MINUS_OR_DIFFERENCE, "OP_MINUS_OR_DIFFERENCE"));

                    }

                    retval.tree = root_0;}
                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 53, additiveOp_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "additiveOp"

    public static class sizeExpression_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "sizeExpression"
    // JFSL.g:681:1: sizeExpression : ( '#' joinExpression -> ^( UNARY OP_SET_NUM joinExpression ) | multiplicativeExpression );
    public final JFSLParser.sizeExpression_return sizeExpression() throws RecognitionException {
        JFSLParser.sizeExpression_return retval = new JFSLParser.sizeExpression_return();
        retval.start = input.LT(1);
        int sizeExpression_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal222=null;
        JFSLParser.joinExpression_return joinExpression223 = null;

        JFSLParser.multiplicativeExpression_return multiplicativeExpression224 = null;


        Node char_literal222_tree=null;
        RewriteRuleTokenStream stream_193=new RewriteRuleTokenStream(adaptor,"token 193");
        RewriteRuleSubtreeStream stream_joinExpression=new RewriteRuleSubtreeStream(adaptor,"rule joinExpression");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 54) ) { return retval; }
            // JFSL.g:682:3: ( '#' joinExpression -> ^( UNARY OP_SET_NUM joinExpression ) | multiplicativeExpression )
            int alt64=2;
            int LA64_0 = input.LA(1);

            if ( (LA64_0==193) ) {
                alt64=1;
            }
            else if ( ((LA64_0>=Identifier && LA64_0<=DecimalLiteral)||LA64_0==144||LA64_0==150||(LA64_0>=168 && LA64_0<=178)||LA64_0==183||LA64_0==186||(LA64_0>=191 && LA64_0<=192)||LA64_0==196||(LA64_0>=199 && LA64_0<=206)) ) {
                alt64=2;
            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 64, 0, input);

                throw nvae;
            }
            switch (alt64) {
                case 1 :
                    // JFSL.g:682:5: '#' joinExpression
                    {
                    char_literal222=(Token)match(input,193,FOLLOW_193_in_sizeExpression3336); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_193.add(char_literal222);

                    pushFollow(FOLLOW_joinExpression_in_sizeExpression3338);
                    joinExpression223=joinExpression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_joinExpression.add(joinExpression223.getTree());


                    // AST REWRITE
                    // elements: joinExpression
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 682:24: -> ^( UNARY OP_SET_NUM joinExpression )
                    {
                        // JFSL.g:682:27: ^( UNARY OP_SET_NUM joinExpression )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(UNARY, "UNARY"), root_1);

                        adaptor.addChild(root_1, (Node)adaptor.create(OP_SET_NUM, "OP_SET_NUM"));
                        adaptor.addChild(root_1, stream_joinExpression.nextTree());

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 2 :
                    // JFSL.g:683:5: multiplicativeExpression
                    {
                    root_0 = (Node)adaptor.nil();

                    pushFollow(FOLLOW_multiplicativeExpression_in_sizeExpression3354);
                    multiplicativeExpression224=multiplicativeExpression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, multiplicativeExpression224.getTree());

                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 54, sizeExpression_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "sizeExpression"

    public static class multiplicativeExpression_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "multiplicativeExpression"
    // JFSL.g:686:1: multiplicativeExpression : ( setAdditiveExpression -> setAdditiveExpression ) (op= multOp a= setAdditiveExpression -> ^( BINARY $op $multiplicativeExpression $a) )* ;
    public final JFSLParser.multiplicativeExpression_return multiplicativeExpression() throws RecognitionException {
        JFSLParser.multiplicativeExpression_return retval = new JFSLParser.multiplicativeExpression_return();
        retval.start = input.LT(1);
        int multiplicativeExpression_StartIndex = input.index();
        Node root_0 = null;

        JFSLParser.multOp_return op = null;

        JFSLParser.setAdditiveExpression_return a = null;

        JFSLParser.setAdditiveExpression_return setAdditiveExpression225 = null;


        RewriteRuleSubtreeStream stream_multOp=new RewriteRuleSubtreeStream(adaptor,"rule multOp");
        RewriteRuleSubtreeStream stream_setAdditiveExpression=new RewriteRuleSubtreeStream(adaptor,"rule setAdditiveExpression");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 55) ) { return retval; }
            // JFSL.g:687:3: ( ( setAdditiveExpression -> setAdditiveExpression ) (op= multOp a= setAdditiveExpression -> ^( BINARY $op $multiplicativeExpression $a) )* )
            // JFSL.g:687:7: ( setAdditiveExpression -> setAdditiveExpression ) (op= multOp a= setAdditiveExpression -> ^( BINARY $op $multiplicativeExpression $a) )*
            {
            // JFSL.g:687:7: ( setAdditiveExpression -> setAdditiveExpression )
            // JFSL.g:687:8: setAdditiveExpression
            {
            pushFollow(FOLLOW_setAdditiveExpression_in_multiplicativeExpression3372);
            setAdditiveExpression225=setAdditiveExpression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_setAdditiveExpression.add(setAdditiveExpression225.getTree());


            // AST REWRITE
            // elements: setAdditiveExpression
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 687:30: -> setAdditiveExpression
            {
                adaptor.addChild(root_0, stream_setAdditiveExpression.nextTree());

            }

            retval.tree = root_0;}
            }

            // JFSL.g:688:7: (op= multOp a= setAdditiveExpression -> ^( BINARY $op $multiplicativeExpression $a) )*
            loop65:
            do {
                int alt65=2;
                int LA65_0 = input.LA(1);

                if ( (LA65_0==167||(LA65_0>=194 && LA65_0<=195)) ) {
                    alt65=1;
                }


                switch (alt65) {
            	case 1 :
            	    // JFSL.g:688:9: op= multOp a= setAdditiveExpression
            	    {
            	    pushFollow(FOLLOW_multOp_in_multiplicativeExpression3389);
            	    op=multOp();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_multOp.add(op.getTree());
            	    pushFollow(FOLLOW_setAdditiveExpression_in_multiplicativeExpression3393);
            	    a=setAdditiveExpression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_setAdditiveExpression.add(a.getTree());


            	    // AST REWRITE
            	    // elements: op, a, multiplicativeExpression
            	    // token labels: 
            	    // rule labels: retval, op, a
            	    // token list labels: 
            	    // rule list labels: 
            	    // wildcard labels: 
            	    if ( state.backtracking==0 ) {
            	    retval.tree = root_0;
            	    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);
            	    RewriteRuleSubtreeStream stream_op=new RewriteRuleSubtreeStream(adaptor,"rule op",op!=null?op.tree:null);
            	    RewriteRuleSubtreeStream stream_a=new RewriteRuleSubtreeStream(adaptor,"rule a",a!=null?a.tree:null);

            	    root_0 = (Node)adaptor.nil();
            	    // 688:43: -> ^( BINARY $op $multiplicativeExpression $a)
            	    {
            	        // JFSL.g:688:46: ^( BINARY $op $multiplicativeExpression $a)
            	        {
            	        Node root_1 = (Node)adaptor.nil();
            	        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(BINARY, "BINARY"), root_1);

            	        adaptor.addChild(root_1, stream_op.nextTree());
            	        adaptor.addChild(root_1, stream_retval.nextTree());
            	        adaptor.addChild(root_1, stream_a.nextTree());

            	        adaptor.addChild(root_0, root_1);
            	        }

            	    }

            	    retval.tree = root_0;}
            	    }
            	    break;

            	default :
            	    break loop65;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 55, multiplicativeExpression_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "multiplicativeExpression"

    public static class multOp_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "multOp"
    // JFSL.g:691:1: multOp : ( '*' -> OP_TIMES | '/' -> OP_DIVIDE | '%' -> OP_MOD );
    public final JFSLParser.multOp_return multOp() throws RecognitionException {
        JFSLParser.multOp_return retval = new JFSLParser.multOp_return();
        retval.start = input.LT(1);
        int multOp_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal226=null;
        Token char_literal227=null;
        Token char_literal228=null;

        Node char_literal226_tree=null;
        Node char_literal227_tree=null;
        Node char_literal228_tree=null;
        RewriteRuleTokenStream stream_195=new RewriteRuleTokenStream(adaptor,"token 195");
        RewriteRuleTokenStream stream_194=new RewriteRuleTokenStream(adaptor,"token 194");
        RewriteRuleTokenStream stream_167=new RewriteRuleTokenStream(adaptor,"token 167");

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 56) ) { return retval; }
            // JFSL.g:692:3: ( '*' -> OP_TIMES | '/' -> OP_DIVIDE | '%' -> OP_MOD )
            int alt66=3;
            switch ( input.LA(1) ) {
            case 167:
                {
                alt66=1;
                }
                break;
            case 194:
                {
                alt66=2;
                }
                break;
            case 195:
                {
                alt66=3;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 66, 0, input);

                throw nvae;
            }

            switch (alt66) {
                case 1 :
                    // JFSL.g:692:5: '*'
                    {
                    char_literal226=(Token)match(input,167,FOLLOW_167_in_multOp3423); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_167.add(char_literal226);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 692:11: -> OP_TIMES
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_TIMES, "OP_TIMES"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 2 :
                    // JFSL.g:693:5: '/'
                    {
                    char_literal227=(Token)match(input,194,FOLLOW_194_in_multOp3436); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_194.add(char_literal227);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 693:11: -> OP_DIVIDE
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_DIVIDE, "OP_DIVIDE"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 3 :
                    // JFSL.g:694:5: '%'
                    {
                    char_literal228=(Token)match(input,195,FOLLOW_195_in_multOp3449); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_195.add(char_literal228);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 694:11: -> OP_MOD
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_MOD, "OP_MOD"));

                    }

                    retval.tree = root_0;}
                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 56, multOp_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "multOp"

    public static class setAdditiveExpression_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "setAdditiveExpression"
    // JFSL.g:697:1: setAdditiveExpression : ( overrideExpression -> overrideExpression ) (op= setAdditiveOp a= overrideExpression -> ^( BINARY $op $setAdditiveExpression $a) )* ;
    public final JFSLParser.setAdditiveExpression_return setAdditiveExpression() throws RecognitionException {
        JFSLParser.setAdditiveExpression_return retval = new JFSLParser.setAdditiveExpression_return();
        retval.start = input.LT(1);
        int setAdditiveExpression_StartIndex = input.index();
        Node root_0 = null;

        JFSLParser.setAdditiveOp_return op = null;

        JFSLParser.overrideExpression_return a = null;

        JFSLParser.overrideExpression_return overrideExpression229 = null;


        RewriteRuleSubtreeStream stream_overrideExpression=new RewriteRuleSubtreeStream(adaptor,"rule overrideExpression");
        RewriteRuleSubtreeStream stream_setAdditiveOp=new RewriteRuleSubtreeStream(adaptor,"rule setAdditiveOp");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 57) ) { return retval; }
            // JFSL.g:698:3: ( ( overrideExpression -> overrideExpression ) (op= setAdditiveOp a= overrideExpression -> ^( BINARY $op $setAdditiveExpression $a) )* )
            // JFSL.g:698:6: ( overrideExpression -> overrideExpression ) (op= setAdditiveOp a= overrideExpression -> ^( BINARY $op $setAdditiveExpression $a) )*
            {
            // JFSL.g:698:6: ( overrideExpression -> overrideExpression )
            // JFSL.g:698:7: overrideExpression
            {
            pushFollow(FOLLOW_overrideExpression_in_setAdditiveExpression3472);
            overrideExpression229=overrideExpression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_overrideExpression.add(overrideExpression229.getTree());


            // AST REWRITE
            // elements: overrideExpression
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 698:26: -> overrideExpression
            {
                adaptor.addChild(root_0, stream_overrideExpression.nextTree());

            }

            retval.tree = root_0;}
            }

            // JFSL.g:699:9: (op= setAdditiveOp a= overrideExpression -> ^( BINARY $op $setAdditiveExpression $a) )*
            loop67:
            do {
                int alt67=2;
                int LA67_0 = input.LA(1);

                if ( (LA67_0==196) ) {
                    alt67=1;
                }


                switch (alt67) {
            	case 1 :
            	    // JFSL.g:699:11: op= setAdditiveOp a= overrideExpression
            	    {
            	    pushFollow(FOLLOW_setAdditiveOp_in_setAdditiveExpression3492);
            	    op=setAdditiveOp();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_setAdditiveOp.add(op.getTree());
            	    pushFollow(FOLLOW_overrideExpression_in_setAdditiveExpression3496);
            	    a=overrideExpression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_overrideExpression.add(a.getTree());


            	    // AST REWRITE
            	    // elements: op, setAdditiveExpression, a
            	    // token labels: 
            	    // rule labels: retval, op, a
            	    // token list labels: 
            	    // rule list labels: 
            	    // wildcard labels: 
            	    if ( state.backtracking==0 ) {
            	    retval.tree = root_0;
            	    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);
            	    RewriteRuleSubtreeStream stream_op=new RewriteRuleSubtreeStream(adaptor,"rule op",op!=null?op.tree:null);
            	    RewriteRuleSubtreeStream stream_a=new RewriteRuleSubtreeStream(adaptor,"rule a",a!=null?a.tree:null);

            	    root_0 = (Node)adaptor.nil();
            	    // 699:49: -> ^( BINARY $op $setAdditiveExpression $a)
            	    {
            	        // JFSL.g:699:52: ^( BINARY $op $setAdditiveExpression $a)
            	        {
            	        Node root_1 = (Node)adaptor.nil();
            	        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(BINARY, "BINARY"), root_1);

            	        adaptor.addChild(root_1, stream_op.nextTree());
            	        adaptor.addChild(root_1, stream_retval.nextTree());
            	        adaptor.addChild(root_1, stream_a.nextTree());

            	        adaptor.addChild(root_0, root_1);
            	        }

            	    }

            	    retval.tree = root_0;}
            	    }
            	    break;

            	default :
            	    break loop67;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 57, setAdditiveExpression_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "setAdditiveExpression"

    public static class setAdditiveOp_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "setAdditiveOp"
    // JFSL.g:703:1: setAdditiveOp : ( '@' '+' -> OP_UNION | '@' '-' -> OP_DIFFERENCE );
    public final JFSLParser.setAdditiveOp_return setAdditiveOp() throws RecognitionException {
        JFSLParser.setAdditiveOp_return retval = new JFSLParser.setAdditiveOp_return();
        retval.start = input.LT(1);
        int setAdditiveOp_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal230=null;
        Token char_literal231=null;
        Token char_literal232=null;
        Token char_literal233=null;

        Node char_literal230_tree=null;
        Node char_literal231_tree=null;
        Node char_literal232_tree=null;
        Node char_literal233_tree=null;
        RewriteRuleTokenStream stream_196=new RewriteRuleTokenStream(adaptor,"token 196");
        RewriteRuleTokenStream stream_191=new RewriteRuleTokenStream(adaptor,"token 191");
        RewriteRuleTokenStream stream_192=new RewriteRuleTokenStream(adaptor,"token 192");

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 58) ) { return retval; }
            // JFSL.g:704:3: ( '@' '+' -> OP_UNION | '@' '-' -> OP_DIFFERENCE )
            int alt68=2;
            int LA68_0 = input.LA(1);

            if ( (LA68_0==196) ) {
                int LA68_1 = input.LA(2);

                if ( (LA68_1==191) ) {
                    alt68=1;
                }
                else if ( (LA68_1==192) ) {
                    alt68=2;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 68, 1, input);

                    throw nvae;
                }
            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 68, 0, input);

                throw nvae;
            }
            switch (alt68) {
                case 1 :
                    // JFSL.g:704:5: '@' '+'
                    {
                    char_literal230=(Token)match(input,196,FOLLOW_196_in_setAdditiveOp3538); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_196.add(char_literal230);

                    char_literal231=(Token)match(input,191,FOLLOW_191_in_setAdditiveOp3540); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_191.add(char_literal231);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 704:13: -> OP_UNION
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_UNION, "OP_UNION"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 2 :
                    // JFSL.g:705:5: '@' '-'
                    {
                    char_literal232=(Token)match(input,196,FOLLOW_196_in_setAdditiveOp3551); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_196.add(char_literal232);

                    char_literal233=(Token)match(input,192,FOLLOW_192_in_setAdditiveOp3553); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_192.add(char_literal233);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 705:13: -> OP_DIFFERENCE
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_DIFFERENCE, "OP_DIFFERENCE"));

                    }

                    retval.tree = root_0;}
                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 58, setAdditiveOp_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "setAdditiveOp"

    public static class overrideExpression_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "overrideExpression"
    // JFSL.g:708:1: overrideExpression : ( intersectionExpression -> intersectionExpression ) ( '++' a= intersectionExpression -> ^( BINARY OP_RELATIONAL_OVERRIDE $overrideExpression $a) )* ;
    public final JFSLParser.overrideExpression_return overrideExpression() throws RecognitionException {
        JFSLParser.overrideExpression_return retval = new JFSLParser.overrideExpression_return();
        retval.start = input.LT(1);
        int overrideExpression_StartIndex = input.index();
        Node root_0 = null;

        Token string_literal235=null;
        JFSLParser.intersectionExpression_return a = null;

        JFSLParser.intersectionExpression_return intersectionExpression234 = null;


        Node string_literal235_tree=null;
        RewriteRuleTokenStream stream_197=new RewriteRuleTokenStream(adaptor,"token 197");
        RewriteRuleSubtreeStream stream_intersectionExpression=new RewriteRuleSubtreeStream(adaptor,"rule intersectionExpression");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 59) ) { return retval; }
            // JFSL.g:709:3: ( ( intersectionExpression -> intersectionExpression ) ( '++' a= intersectionExpression -> ^( BINARY OP_RELATIONAL_OVERRIDE $overrideExpression $a) )* )
            // JFSL.g:709:6: ( intersectionExpression -> intersectionExpression ) ( '++' a= intersectionExpression -> ^( BINARY OP_RELATIONAL_OVERRIDE $overrideExpression $a) )*
            {
            // JFSL.g:709:6: ( intersectionExpression -> intersectionExpression )
            // JFSL.g:709:7: intersectionExpression
            {
            pushFollow(FOLLOW_intersectionExpression_in_overrideExpression3576);
            intersectionExpression234=intersectionExpression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_intersectionExpression.add(intersectionExpression234.getTree());


            // AST REWRITE
            // elements: intersectionExpression
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 709:30: -> intersectionExpression
            {
                adaptor.addChild(root_0, stream_intersectionExpression.nextTree());

            }

            retval.tree = root_0;}
            }

            // JFSL.g:710:7: ( '++' a= intersectionExpression -> ^( BINARY OP_RELATIONAL_OVERRIDE $overrideExpression $a) )*
            loop69:
            do {
                int alt69=2;
                int LA69_0 = input.LA(1);

                if ( (LA69_0==197) ) {
                    alt69=1;
                }


                switch (alt69) {
            	case 1 :
            	    // JFSL.g:710:9: '++' a= intersectionExpression
            	    {
            	    string_literal235=(Token)match(input,197,FOLLOW_197_in_overrideExpression3591); if (state.failed) return retval; 
            	    if ( state.backtracking==0 ) stream_197.add(string_literal235);

            	    pushFollow(FOLLOW_intersectionExpression_in_overrideExpression3595);
            	    a=intersectionExpression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_intersectionExpression.add(a.getTree());


            	    // AST REWRITE
            	    // elements: overrideExpression, a
            	    // token labels: 
            	    // rule labels: retval, a
            	    // token list labels: 
            	    // rule list labels: 
            	    // wildcard labels: 
            	    if ( state.backtracking==0 ) {
            	    retval.tree = root_0;
            	    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);
            	    RewriteRuleSubtreeStream stream_a=new RewriteRuleSubtreeStream(adaptor,"rule a",a!=null?a.tree:null);

            	    root_0 = (Node)adaptor.nil();
            	    // 710:39: -> ^( BINARY OP_RELATIONAL_OVERRIDE $overrideExpression $a)
            	    {
            	        // JFSL.g:710:42: ^( BINARY OP_RELATIONAL_OVERRIDE $overrideExpression $a)
            	        {
            	        Node root_1 = (Node)adaptor.nil();
            	        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(BINARY, "BINARY"), root_1);

            	        adaptor.addChild(root_1, (Node)adaptor.create(OP_RELATIONAL_OVERRIDE, "OP_RELATIONAL_OVERRIDE"));
            	        adaptor.addChild(root_1, stream_retval.nextTree());
            	        adaptor.addChild(root_1, stream_a.nextTree());

            	        adaptor.addChild(root_0, root_1);
            	        }

            	    }

            	    retval.tree = root_0;}
            	    }
            	    break;

            	default :
            	    break loop69;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 59, overrideExpression_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "overrideExpression"

    public static class intersectionExpression_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "intersectionExpression"
    // JFSL.g:714:1: intersectionExpression : ( composeExpression -> composeExpression ) ( '@' '&' a= composeExpression -> ^( BINARY OP_INTERSECTION $intersectionExpression $a) )* ;
    public final JFSLParser.intersectionExpression_return intersectionExpression() throws RecognitionException {
        JFSLParser.intersectionExpression_return retval = new JFSLParser.intersectionExpression_return();
        retval.start = input.LT(1);
        int intersectionExpression_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal237=null;
        Token char_literal238=null;
        JFSLParser.composeExpression_return a = null;

        JFSLParser.composeExpression_return composeExpression236 = null;


        Node char_literal237_tree=null;
        Node char_literal238_tree=null;
        RewriteRuleTokenStream stream_187=new RewriteRuleTokenStream(adaptor,"token 187");
        RewriteRuleTokenStream stream_196=new RewriteRuleTokenStream(adaptor,"token 196");
        RewriteRuleSubtreeStream stream_composeExpression=new RewriteRuleSubtreeStream(adaptor,"rule composeExpression");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 60) ) { return retval; }
            // JFSL.g:715:3: ( ( composeExpression -> composeExpression ) ( '@' '&' a= composeExpression -> ^( BINARY OP_INTERSECTION $intersectionExpression $a) )* )
            // JFSL.g:715:6: ( composeExpression -> composeExpression ) ( '@' '&' a= composeExpression -> ^( BINARY OP_INTERSECTION $intersectionExpression $a) )*
            {
            // JFSL.g:715:6: ( composeExpression -> composeExpression )
            // JFSL.g:715:7: composeExpression
            {
            pushFollow(FOLLOW_composeExpression_in_intersectionExpression3639);
            composeExpression236=composeExpression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_composeExpression.add(composeExpression236.getTree());


            // AST REWRITE
            // elements: composeExpression
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 715:25: -> composeExpression
            {
                adaptor.addChild(root_0, stream_composeExpression.nextTree());

            }

            retval.tree = root_0;}
            }

            // JFSL.g:716:7: ( '@' '&' a= composeExpression -> ^( BINARY OP_INTERSECTION $intersectionExpression $a) )*
            loop70:
            do {
                int alt70=2;
                int LA70_0 = input.LA(1);

                if ( (LA70_0==196) ) {
                    int LA70_2 = input.LA(2);

                    if ( (LA70_2==187) ) {
                        alt70=1;
                    }


                }


                switch (alt70) {
            	case 1 :
            	    // JFSL.g:716:9: '@' '&' a= composeExpression
            	    {
            	    char_literal237=(Token)match(input,196,FOLLOW_196_in_intersectionExpression3654); if (state.failed) return retval; 
            	    if ( state.backtracking==0 ) stream_196.add(char_literal237);

            	    char_literal238=(Token)match(input,187,FOLLOW_187_in_intersectionExpression3656); if (state.failed) return retval; 
            	    if ( state.backtracking==0 ) stream_187.add(char_literal238);

            	    pushFollow(FOLLOW_composeExpression_in_intersectionExpression3660);
            	    a=composeExpression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_composeExpression.add(a.getTree());


            	    // AST REWRITE
            	    // elements: a, intersectionExpression
            	    // token labels: 
            	    // rule labels: retval, a
            	    // token list labels: 
            	    // rule list labels: 
            	    // wildcard labels: 
            	    if ( state.backtracking==0 ) {
            	    retval.tree = root_0;
            	    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);
            	    RewriteRuleSubtreeStream stream_a=new RewriteRuleSubtreeStream(adaptor,"rule a",a!=null?a.tree:null);

            	    root_0 = (Node)adaptor.nil();
            	    // 716:37: -> ^( BINARY OP_INTERSECTION $intersectionExpression $a)
            	    {
            	        // JFSL.g:716:40: ^( BINARY OP_INTERSECTION $intersectionExpression $a)
            	        {
            	        Node root_1 = (Node)adaptor.nil();
            	        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(BINARY, "BINARY"), root_1);

            	        adaptor.addChild(root_1, (Node)adaptor.create(OP_INTERSECTION, "OP_INTERSECTION"));
            	        adaptor.addChild(root_1, stream_retval.nextTree());
            	        adaptor.addChild(root_1, stream_a.nextTree());

            	        adaptor.addChild(root_0, root_1);
            	        }

            	    }

            	    retval.tree = root_0;}
            	    }
            	    break;

            	default :
            	    break loop70;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 60, intersectionExpression_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "intersectionExpression"

    public static class composeExpression_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "composeExpression"
    // JFSL.g:720:1: composeExpression : ( unaryExpression -> unaryExpression ) ( '->' a= unaryExpression -> ^( BINARY OP_RELATIONAL_COMPOSE $composeExpression $a) )* ;
    public final JFSLParser.composeExpression_return composeExpression() throws RecognitionException {
        JFSLParser.composeExpression_return retval = new JFSLParser.composeExpression_return();
        retval.start = input.LT(1);
        int composeExpression_StartIndex = input.index();
        Node root_0 = null;

        Token string_literal240=null;
        JFSLParser.unaryExpression_return a = null;

        JFSLParser.unaryExpression_return unaryExpression239 = null;


        Node string_literal240_tree=null;
        RewriteRuleTokenStream stream_198=new RewriteRuleTokenStream(adaptor,"token 198");
        RewriteRuleSubtreeStream stream_unaryExpression=new RewriteRuleSubtreeStream(adaptor,"rule unaryExpression");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 61) ) { return retval; }
            // JFSL.g:721:3: ( ( unaryExpression -> unaryExpression ) ( '->' a= unaryExpression -> ^( BINARY OP_RELATIONAL_COMPOSE $composeExpression $a) )* )
            // JFSL.g:721:6: ( unaryExpression -> unaryExpression ) ( '->' a= unaryExpression -> ^( BINARY OP_RELATIONAL_COMPOSE $composeExpression $a) )*
            {
            // JFSL.g:721:6: ( unaryExpression -> unaryExpression )
            // JFSL.g:721:7: unaryExpression
            {
            pushFollow(FOLLOW_unaryExpression_in_composeExpression3700);
            unaryExpression239=unaryExpression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_unaryExpression.add(unaryExpression239.getTree());


            // AST REWRITE
            // elements: unaryExpression
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 721:23: -> unaryExpression
            {
                adaptor.addChild(root_0, stream_unaryExpression.nextTree());

            }

            retval.tree = root_0;}
            }

            // JFSL.g:722:7: ( '->' a= unaryExpression -> ^( BINARY OP_RELATIONAL_COMPOSE $composeExpression $a) )*
            loop71:
            do {
                int alt71=2;
                int LA71_0 = input.LA(1);

                if ( (LA71_0==198) ) {
                    alt71=1;
                }


                switch (alt71) {
            	case 1 :
            	    // JFSL.g:722:9: '->' a= unaryExpression
            	    {
            	    string_literal240=(Token)match(input,198,FOLLOW_198_in_composeExpression3715); if (state.failed) return retval; 
            	    if ( state.backtracking==0 ) stream_198.add(string_literal240);

            	    pushFollow(FOLLOW_unaryExpression_in_composeExpression3719);
            	    a=unaryExpression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_unaryExpression.add(a.getTree());


            	    // AST REWRITE
            	    // elements: a, composeExpression
            	    // token labels: 
            	    // rule labels: retval, a
            	    // token list labels: 
            	    // rule list labels: 
            	    // wildcard labels: 
            	    if ( state.backtracking==0 ) {
            	    retval.tree = root_0;
            	    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);
            	    RewriteRuleSubtreeStream stream_a=new RewriteRuleSubtreeStream(adaptor,"rule a",a!=null?a.tree:null);

            	    root_0 = (Node)adaptor.nil();
            	    // 722:32: -> ^( BINARY OP_RELATIONAL_COMPOSE $composeExpression $a)
            	    {
            	        // JFSL.g:722:35: ^( BINARY OP_RELATIONAL_COMPOSE $composeExpression $a)
            	        {
            	        Node root_1 = (Node)adaptor.nil();
            	        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(BINARY, "BINARY"), root_1);

            	        adaptor.addChild(root_1, (Node)adaptor.create(OP_RELATIONAL_COMPOSE, "OP_RELATIONAL_COMPOSE"));
            	        adaptor.addChild(root_1, stream_retval.nextTree());
            	        adaptor.addChild(root_1, stream_a.nextTree());

            	        adaptor.addChild(root_0, root_1);
            	        }

            	    }

            	    retval.tree = root_0;}
            	    }
            	    break;

            	default :
            	    break loop71;
                }
            } while (true);


            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 61, composeExpression_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "composeExpression"

    public static class unaryExpression_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "unaryExpression"
    // JFSL.g:726:1: unaryExpression : ( '+' unaryExpression -> ^( UNARY OP_PLUS unaryExpression ) | '-' unaryExpression -> ^( UNARY OP_MINUS unaryExpression ) | unaryExpressionNotPlusMinus );
    public final JFSLParser.unaryExpression_return unaryExpression() throws RecognitionException {
        JFSLParser.unaryExpression_return retval = new JFSLParser.unaryExpression_return();
        retval.start = input.LT(1);
        int unaryExpression_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal241=null;
        Token char_literal243=null;
        JFSLParser.unaryExpression_return unaryExpression242 = null;

        JFSLParser.unaryExpression_return unaryExpression244 = null;

        JFSLParser.unaryExpressionNotPlusMinus_return unaryExpressionNotPlusMinus245 = null;


        Node char_literal241_tree=null;
        Node char_literal243_tree=null;
        RewriteRuleTokenStream stream_191=new RewriteRuleTokenStream(adaptor,"token 191");
        RewriteRuleTokenStream stream_192=new RewriteRuleTokenStream(adaptor,"token 192");
        RewriteRuleSubtreeStream stream_unaryExpression=new RewriteRuleSubtreeStream(adaptor,"rule unaryExpression");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 62) ) { return retval; }
            // JFSL.g:727:3: ( '+' unaryExpression -> ^( UNARY OP_PLUS unaryExpression ) | '-' unaryExpression -> ^( UNARY OP_MINUS unaryExpression ) | unaryExpressionNotPlusMinus )
            int alt72=3;
            switch ( input.LA(1) ) {
            case 191:
                {
                alt72=1;
                }
                break;
            case 192:
                {
                alt72=2;
                }
                break;
            case Identifier:
            case FloatingPointLiteral:
            case CharacterLiteral:
            case StringLiteral:
            case HexLiteral:
            case OctalLiteral:
            case DecimalLiteral:
            case 144:
            case 150:
            case 168:
            case 169:
            case 170:
            case 171:
            case 172:
            case 173:
            case 174:
            case 175:
            case 176:
            case 177:
            case 178:
            case 183:
            case 186:
            case 196:
            case 199:
            case 200:
            case 201:
            case 202:
            case 203:
            case 204:
            case 205:
            case 206:
                {
                alt72=3;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 72, 0, input);

                throw nvae;
            }

            switch (alt72) {
                case 1 :
                    // JFSL.g:727:5: '+' unaryExpression
                    {
                    char_literal241=(Token)match(input,191,FOLLOW_191_in_unaryExpression3762); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_191.add(char_literal241);

                    pushFollow(FOLLOW_unaryExpression_in_unaryExpression3764);
                    unaryExpression242=unaryExpression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_unaryExpression.add(unaryExpression242.getTree());


                    // AST REWRITE
                    // elements: unaryExpression
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 727:25: -> ^( UNARY OP_PLUS unaryExpression )
                    {
                        // JFSL.g:727:28: ^( UNARY OP_PLUS unaryExpression )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(UNARY, "UNARY"), root_1);

                        adaptor.addChild(root_1, (Node)adaptor.create(OP_PLUS, "OP_PLUS"));
                        adaptor.addChild(root_1, stream_unaryExpression.nextTree());

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 2 :
                    // JFSL.g:728:5: '-' unaryExpression
                    {
                    char_literal243=(Token)match(input,192,FOLLOW_192_in_unaryExpression3780); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_192.add(char_literal243);

                    pushFollow(FOLLOW_unaryExpression_in_unaryExpression3782);
                    unaryExpression244=unaryExpression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_unaryExpression.add(unaryExpression244.getTree());


                    // AST REWRITE
                    // elements: unaryExpression
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 728:25: -> ^( UNARY OP_MINUS unaryExpression )
                    {
                        // JFSL.g:728:28: ^( UNARY OP_MINUS unaryExpression )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(UNARY, "UNARY"), root_1);

                        adaptor.addChild(root_1, (Node)adaptor.create(OP_MINUS, "OP_MINUS"));
                        adaptor.addChild(root_1, stream_unaryExpression.nextTree());

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 3 :
                    // JFSL.g:729:5: unaryExpressionNotPlusMinus
                    {
                    root_0 = (Node)adaptor.nil();

                    pushFollow(FOLLOW_unaryExpressionNotPlusMinus_in_unaryExpression3798);
                    unaryExpressionNotPlusMinus245=unaryExpressionNotPlusMinus();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, unaryExpressionNotPlusMinus245.getTree());

                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 62, unaryExpression_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "unaryExpression"

    public static class unaryExpressionNotPlusMinus_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "unaryExpressionNotPlusMinus"
    // JFSL.g:732:1: unaryExpressionNotPlusMinus : ( '~' unaryExpression -> ^( UNARY OP_BIT_NOT_OR_TRANSPOSE unaryExpression ) | '!' unaryExpression -> ^( UNARY OP_NOT unaryExpression ) | castExpression | joinExpression );
    public final JFSLParser.unaryExpressionNotPlusMinus_return unaryExpressionNotPlusMinus() throws RecognitionException {
        JFSLParser.unaryExpressionNotPlusMinus_return retval = new JFSLParser.unaryExpressionNotPlusMinus_return();
        retval.start = input.LT(1);
        int unaryExpressionNotPlusMinus_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal246=null;
        Token char_literal248=null;
        JFSLParser.unaryExpression_return unaryExpression247 = null;

        JFSLParser.unaryExpression_return unaryExpression249 = null;

        JFSLParser.castExpression_return castExpression250 = null;

        JFSLParser.joinExpression_return joinExpression251 = null;


        Node char_literal246_tree=null;
        Node char_literal248_tree=null;
        RewriteRuleTokenStream stream_183=new RewriteRuleTokenStream(adaptor,"token 183");
        RewriteRuleTokenStream stream_199=new RewriteRuleTokenStream(adaptor,"token 199");
        RewriteRuleSubtreeStream stream_unaryExpression=new RewriteRuleSubtreeStream(adaptor,"rule unaryExpression");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 63) ) { return retval; }
            // JFSL.g:733:3: ( '~' unaryExpression -> ^( UNARY OP_BIT_NOT_OR_TRANSPOSE unaryExpression ) | '!' unaryExpression -> ^( UNARY OP_NOT unaryExpression ) | castExpression | joinExpression )
            int alt73=4;
            alt73 = dfa73.predict(input);
            switch (alt73) {
                case 1 :
                    // JFSL.g:733:5: '~' unaryExpression
                    {
                    char_literal246=(Token)match(input,199,FOLLOW_199_in_unaryExpressionNotPlusMinus3811); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_199.add(char_literal246);

                    pushFollow(FOLLOW_unaryExpression_in_unaryExpressionNotPlusMinus3813);
                    unaryExpression247=unaryExpression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_unaryExpression.add(unaryExpression247.getTree());


                    // AST REWRITE
                    // elements: unaryExpression
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 733:25: -> ^( UNARY OP_BIT_NOT_OR_TRANSPOSE unaryExpression )
                    {
                        // JFSL.g:733:28: ^( UNARY OP_BIT_NOT_OR_TRANSPOSE unaryExpression )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(UNARY, "UNARY"), root_1);

                        adaptor.addChild(root_1, (Node)adaptor.create(OP_BIT_NOT_OR_TRANSPOSE, "OP_BIT_NOT_OR_TRANSPOSE"));
                        adaptor.addChild(root_1, stream_unaryExpression.nextTree());

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 2 :
                    // JFSL.g:734:5: '!' unaryExpression
                    {
                    char_literal248=(Token)match(input,183,FOLLOW_183_in_unaryExpressionNotPlusMinus3829); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_183.add(char_literal248);

                    pushFollow(FOLLOW_unaryExpression_in_unaryExpressionNotPlusMinus3831);
                    unaryExpression249=unaryExpression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_unaryExpression.add(unaryExpression249.getTree());


                    // AST REWRITE
                    // elements: unaryExpression
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 734:25: -> ^( UNARY OP_NOT unaryExpression )
                    {
                        // JFSL.g:734:28: ^( UNARY OP_NOT unaryExpression )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(UNARY, "UNARY"), root_1);

                        adaptor.addChild(root_1, (Node)adaptor.create(OP_NOT, "OP_NOT"));
                        adaptor.addChild(root_1, stream_unaryExpression.nextTree());

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 3 :
                    // JFSL.g:735:5: castExpression
                    {
                    root_0 = (Node)adaptor.nil();

                    pushFollow(FOLLOW_castExpression_in_unaryExpressionNotPlusMinus3851);
                    castExpression250=castExpression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, castExpression250.getTree());

                    }
                    break;
                case 4 :
                    // JFSL.g:736:5: joinExpression
                    {
                    root_0 = (Node)adaptor.nil();

                    pushFollow(FOLLOW_joinExpression_in_unaryExpressionNotPlusMinus3857);
                    joinExpression251=joinExpression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, joinExpression251.getTree());

                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 63, unaryExpressionNotPlusMinus_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "unaryExpressionNotPlusMinus"

    public static class castExpression_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "castExpression"
    // JFSL.g:739:1: castExpression : '(' primitiveType ')' unaryExpression -> ^( CAST primitiveType unaryExpression ) ;
    public final JFSLParser.castExpression_return castExpression() throws RecognitionException {
        JFSLParser.castExpression_return retval = new JFSLParser.castExpression_return();
        retval.start = input.LT(1);
        int castExpression_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal252=null;
        Token char_literal254=null;
        JFSLParser.primitiveType_return primitiveType253 = null;

        JFSLParser.unaryExpression_return unaryExpression255 = null;


        Node char_literal252_tree=null;
        Node char_literal254_tree=null;
        RewriteRuleTokenStream stream_150=new RewriteRuleTokenStream(adaptor,"token 150");
        RewriteRuleTokenStream stream_151=new RewriteRuleTokenStream(adaptor,"token 151");
        RewriteRuleSubtreeStream stream_primitiveType=new RewriteRuleSubtreeStream(adaptor,"rule primitiveType");
        RewriteRuleSubtreeStream stream_unaryExpression=new RewriteRuleSubtreeStream(adaptor,"rule unaryExpression");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 64) ) { return retval; }
            // JFSL.g:740:3: ( '(' primitiveType ')' unaryExpression -> ^( CAST primitiveType unaryExpression ) )
            // JFSL.g:740:5: '(' primitiveType ')' unaryExpression
            {
            char_literal252=(Token)match(input,150,FOLLOW_150_in_castExpression3877); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_150.add(char_literal252);

            pushFollow(FOLLOW_primitiveType_in_castExpression3879);
            primitiveType253=primitiveType();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_primitiveType.add(primitiveType253.getTree());
            char_literal254=(Token)match(input,151,FOLLOW_151_in_castExpression3881); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_151.add(char_literal254);

            pushFollow(FOLLOW_unaryExpression_in_castExpression3883);
            unaryExpression255=unaryExpression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_unaryExpression.add(unaryExpression255.getTree());


            // AST REWRITE
            // elements: unaryExpression, primitiveType
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 740:43: -> ^( CAST primitiveType unaryExpression )
            {
                // JFSL.g:740:46: ^( CAST primitiveType unaryExpression )
                {
                Node root_1 = (Node)adaptor.nil();
                root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(CAST, "CAST"), root_1);

                adaptor.addChild(root_1, stream_primitiveType.nextTree());
                adaptor.addChild(root_1, stream_unaryExpression.nextTree());

                adaptor.addChild(root_0, root_1);
                }

            }

            retval.tree = root_0;}
            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 64, castExpression_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "castExpression"

    public static class decls_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "decls"
    // JFSL.g:744:1: decls : ( Identifier ( inOp | ':' ) declarationMult rangeExpression ) ( ',' ( Identifier ( inOp | ':' ) declarationMult rangeExpression ) )* -> ^( DECLS ( Identifier declarationMult rangeExpression )+ ) ;
    public final JFSLParser.decls_return decls() throws RecognitionException {
        JFSLParser.decls_return retval = new JFSLParser.decls_return();
        retval.start = input.LT(1);
        int decls_StartIndex = input.index();
        Node root_0 = null;

        Token Identifier256=null;
        Token char_literal258=null;
        Token char_literal261=null;
        Token Identifier262=null;
        Token char_literal264=null;
        JFSLParser.inOp_return inOp257 = null;

        JFSLParser.declarationMult_return declarationMult259 = null;

        JFSLParser.rangeExpression_return rangeExpression260 = null;

        JFSLParser.inOp_return inOp263 = null;

        JFSLParser.declarationMult_return declarationMult265 = null;

        JFSLParser.rangeExpression_return rangeExpression266 = null;


        Node Identifier256_tree=null;
        Node char_literal258_tree=null;
        Node char_literal261_tree=null;
        Node Identifier262_tree=null;
        Node char_literal264_tree=null;
        RewriteRuleTokenStream stream_161=new RewriteRuleTokenStream(adaptor,"token 161");
        RewriteRuleTokenStream stream_147=new RewriteRuleTokenStream(adaptor,"token 147");
        RewriteRuleTokenStream stream_Identifier=new RewriteRuleTokenStream(adaptor,"token Identifier");
        RewriteRuleSubtreeStream stream_rangeExpression=new RewriteRuleSubtreeStream(adaptor,"rule rangeExpression");
        RewriteRuleSubtreeStream stream_inOp=new RewriteRuleSubtreeStream(adaptor,"rule inOp");
        RewriteRuleSubtreeStream stream_declarationMult=new RewriteRuleSubtreeStream(adaptor,"rule declarationMult");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 65) ) { return retval; }
            // JFSL.g:745:3: ( ( Identifier ( inOp | ':' ) declarationMult rangeExpression ) ( ',' ( Identifier ( inOp | ':' ) declarationMult rangeExpression ) )* -> ^( DECLS ( Identifier declarationMult rangeExpression )+ ) )
            // JFSL.g:745:5: ( Identifier ( inOp | ':' ) declarationMult rangeExpression ) ( ',' ( Identifier ( inOp | ':' ) declarationMult rangeExpression ) )*
            {
            // JFSL.g:745:5: ( Identifier ( inOp | ':' ) declarationMult rangeExpression )
            // JFSL.g:745:6: Identifier ( inOp | ':' ) declarationMult rangeExpression
            {
            Identifier256=(Token)match(input,Identifier,FOLLOW_Identifier_in_decls3909); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_Identifier.add(Identifier256);

            // JFSL.g:745:17: ( inOp | ':' )
            int alt74=2;
            int LA74_0 = input.LA(1);

            if ( (LA74_0==189) ) {
                alt74=1;
            }
            else if ( (LA74_0==161) ) {
                alt74=2;
            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 74, 0, input);

                throw nvae;
            }
            switch (alt74) {
                case 1 :
                    // JFSL.g:745:18: inOp
                    {
                    pushFollow(FOLLOW_inOp_in_decls3912);
                    inOp257=inOp();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_inOp.add(inOp257.getTree());

                    }
                    break;
                case 2 :
                    // JFSL.g:745:25: ':'
                    {
                    char_literal258=(Token)match(input,161,FOLLOW_161_in_decls3916); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_161.add(char_literal258);


                    }
                    break;

            }

            pushFollow(FOLLOW_declarationMult_in_decls3919);
            declarationMult259=declarationMult();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_declarationMult.add(declarationMult259.getTree());
            pushFollow(FOLLOW_rangeExpression_in_decls3921);
            rangeExpression260=rangeExpression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_rangeExpression.add(rangeExpression260.getTree());

            }

            // JFSL.g:745:63: ( ',' ( Identifier ( inOp | ':' ) declarationMult rangeExpression ) )*
            loop76:
            do {
                int alt76=2;
                int LA76_0 = input.LA(1);

                if ( (LA76_0==147) ) {
                    alt76=1;
                }


                switch (alt76) {
            	case 1 :
            	    // JFSL.g:745:64: ',' ( Identifier ( inOp | ':' ) declarationMult rangeExpression )
            	    {
            	    char_literal261=(Token)match(input,147,FOLLOW_147_in_decls3925); if (state.failed) return retval; 
            	    if ( state.backtracking==0 ) stream_147.add(char_literal261);

            	    // JFSL.g:745:68: ( Identifier ( inOp | ':' ) declarationMult rangeExpression )
            	    // JFSL.g:745:69: Identifier ( inOp | ':' ) declarationMult rangeExpression
            	    {
            	    Identifier262=(Token)match(input,Identifier,FOLLOW_Identifier_in_decls3928); if (state.failed) return retval; 
            	    if ( state.backtracking==0 ) stream_Identifier.add(Identifier262);

            	    // JFSL.g:745:80: ( inOp | ':' )
            	    int alt75=2;
            	    int LA75_0 = input.LA(1);

            	    if ( (LA75_0==189) ) {
            	        alt75=1;
            	    }
            	    else if ( (LA75_0==161) ) {
            	        alt75=2;
            	    }
            	    else {
            	        if (state.backtracking>0) {state.failed=true; return retval;}
            	        NoViableAltException nvae =
            	            new NoViableAltException("", 75, 0, input);

            	        throw nvae;
            	    }
            	    switch (alt75) {
            	        case 1 :
            	            // JFSL.g:745:81: inOp
            	            {
            	            pushFollow(FOLLOW_inOp_in_decls3931);
            	            inOp263=inOp();

            	            state._fsp--;
            	            if (state.failed) return retval;
            	            if ( state.backtracking==0 ) stream_inOp.add(inOp263.getTree());

            	            }
            	            break;
            	        case 2 :
            	            // JFSL.g:745:88: ':'
            	            {
            	            char_literal264=(Token)match(input,161,FOLLOW_161_in_decls3935); if (state.failed) return retval; 
            	            if ( state.backtracking==0 ) stream_161.add(char_literal264);


            	            }
            	            break;

            	    }

            	    pushFollow(FOLLOW_declarationMult_in_decls3938);
            	    declarationMult265=declarationMult();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_declarationMult.add(declarationMult265.getTree());
            	    pushFollow(FOLLOW_rangeExpression_in_decls3940);
            	    rangeExpression266=rangeExpression();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_rangeExpression.add(rangeExpression266.getTree());

            	    }


            	    }
            	    break;

            	default :
            	    break loop76;
                }
            } while (true);



            // AST REWRITE
            // elements: declarationMult, Identifier, rangeExpression
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 746:10: -> ^( DECLS ( Identifier declarationMult rangeExpression )+ )
            {
                // JFSL.g:746:13: ^( DECLS ( Identifier declarationMult rangeExpression )+ )
                {
                Node root_1 = (Node)adaptor.nil();
                root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(DECLS, "DECLS"), root_1);

                if ( !(stream_declarationMult.hasNext()||stream_Identifier.hasNext()||stream_rangeExpression.hasNext()) ) {
                    throw new RewriteEarlyExitException();
                }
                while ( stream_declarationMult.hasNext()||stream_Identifier.hasNext()||stream_rangeExpression.hasNext() ) {
                    adaptor.addChild(root_1, stream_Identifier.nextNode());
                    adaptor.addChild(root_1, stream_declarationMult.nextTree());
                    adaptor.addChild(root_1, stream_rangeExpression.nextTree());

                }
                stream_declarationMult.reset();
                stream_Identifier.reset();
                stream_rangeExpression.reset();

                adaptor.addChild(root_0, root_1);
                }

            }

            retval.tree = root_0;}
            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 65, decls_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "decls"

    public static class mdecls_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "mdecls"
    // JFSL.g:749:1: mdecls : ( quantIdMod Identifier ( ',' Identifier )* ( inOp | ':' ) declarationMult rangeExpression ) -> ^( MDECLS quantIdMod ( Identifier )+ declarationMult rangeExpression ) ;
    public final JFSLParser.mdecls_return mdecls() throws RecognitionException {
        JFSLParser.mdecls_return retval = new JFSLParser.mdecls_return();
        retval.start = input.LT(1);
        int mdecls_StartIndex = input.index();
        Node root_0 = null;

        Token Identifier268=null;
        Token char_literal269=null;
        Token Identifier270=null;
        Token char_literal272=null;
        JFSLParser.quantIdMod_return quantIdMod267 = null;

        JFSLParser.inOp_return inOp271 = null;

        JFSLParser.declarationMult_return declarationMult273 = null;

        JFSLParser.rangeExpression_return rangeExpression274 = null;


        Node Identifier268_tree=null;
        Node char_literal269_tree=null;
        Node Identifier270_tree=null;
        Node char_literal272_tree=null;
        RewriteRuleTokenStream stream_161=new RewriteRuleTokenStream(adaptor,"token 161");
        RewriteRuleTokenStream stream_147=new RewriteRuleTokenStream(adaptor,"token 147");
        RewriteRuleTokenStream stream_Identifier=new RewriteRuleTokenStream(adaptor,"token Identifier");
        RewriteRuleSubtreeStream stream_rangeExpression=new RewriteRuleSubtreeStream(adaptor,"rule rangeExpression");
        RewriteRuleSubtreeStream stream_inOp=new RewriteRuleSubtreeStream(adaptor,"rule inOp");
        RewriteRuleSubtreeStream stream_quantIdMod=new RewriteRuleSubtreeStream(adaptor,"rule quantIdMod");
        RewriteRuleSubtreeStream stream_declarationMult=new RewriteRuleSubtreeStream(adaptor,"rule declarationMult");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 66) ) { return retval; }
            // JFSL.g:750:3: ( ( quantIdMod Identifier ( ',' Identifier )* ( inOp | ':' ) declarationMult rangeExpression ) -> ^( MDECLS quantIdMod ( Identifier )+ declarationMult rangeExpression ) )
            // JFSL.g:750:5: ( quantIdMod Identifier ( ',' Identifier )* ( inOp | ':' ) declarationMult rangeExpression )
            {
            // JFSL.g:750:5: ( quantIdMod Identifier ( ',' Identifier )* ( inOp | ':' ) declarationMult rangeExpression )
            // JFSL.g:750:6: quantIdMod Identifier ( ',' Identifier )* ( inOp | ':' ) declarationMult rangeExpression
            {
            pushFollow(FOLLOW_quantIdMod_in_mdecls3983);
            quantIdMod267=quantIdMod();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_quantIdMod.add(quantIdMod267.getTree());
            Identifier268=(Token)match(input,Identifier,FOLLOW_Identifier_in_mdecls3985); if (state.failed) return retval; 
            if ( state.backtracking==0 ) stream_Identifier.add(Identifier268);

            // JFSL.g:750:28: ( ',' Identifier )*
            loop77:
            do {
                int alt77=2;
                int LA77_0 = input.LA(1);

                if ( (LA77_0==147) ) {
                    alt77=1;
                }


                switch (alt77) {
            	case 1 :
            	    // JFSL.g:750:29: ',' Identifier
            	    {
            	    char_literal269=(Token)match(input,147,FOLLOW_147_in_mdecls3988); if (state.failed) return retval; 
            	    if ( state.backtracking==0 ) stream_147.add(char_literal269);

            	    Identifier270=(Token)match(input,Identifier,FOLLOW_Identifier_in_mdecls3990); if (state.failed) return retval; 
            	    if ( state.backtracking==0 ) stream_Identifier.add(Identifier270);


            	    }
            	    break;

            	default :
            	    break loop77;
                }
            } while (true);

            // JFSL.g:750:46: ( inOp | ':' )
            int alt78=2;
            int LA78_0 = input.LA(1);

            if ( (LA78_0==189) ) {
                alt78=1;
            }
            else if ( (LA78_0==161) ) {
                alt78=2;
            }
            else {
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 78, 0, input);

                throw nvae;
            }
            switch (alt78) {
                case 1 :
                    // JFSL.g:750:47: inOp
                    {
                    pushFollow(FOLLOW_inOp_in_mdecls3995);
                    inOp271=inOp();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_inOp.add(inOp271.getTree());

                    }
                    break;
                case 2 :
                    // JFSL.g:750:54: ':'
                    {
                    char_literal272=(Token)match(input,161,FOLLOW_161_in_mdecls3999); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_161.add(char_literal272);


                    }
                    break;

            }

            pushFollow(FOLLOW_declarationMult_in_mdecls4002);
            declarationMult273=declarationMult();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_declarationMult.add(declarationMult273.getTree());
            pushFollow(FOLLOW_rangeExpression_in_mdecls4004);
            rangeExpression274=rangeExpression();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_rangeExpression.add(rangeExpression274.getTree());

            }



            // AST REWRITE
            // elements: declarationMult, Identifier, rangeExpression, quantIdMod
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 751:10: -> ^( MDECLS quantIdMod ( Identifier )+ declarationMult rangeExpression )
            {
                // JFSL.g:751:13: ^( MDECLS quantIdMod ( Identifier )+ declarationMult rangeExpression )
                {
                Node root_1 = (Node)adaptor.nil();
                root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(MDECLS, "MDECLS"), root_1);

                adaptor.addChild(root_1, stream_quantIdMod.nextTree());
                if ( !(stream_Identifier.hasNext()) ) {
                    throw new RewriteEarlyExitException();
                }
                while ( stream_Identifier.hasNext() ) {
                    adaptor.addChild(root_1, stream_Identifier.nextNode());

                }
                stream_Identifier.reset();
                adaptor.addChild(root_1, stream_declarationMult.nextTree());
                adaptor.addChild(root_1, stream_rangeExpression.nextTree());

                adaptor.addChild(root_0, root_1);
                }

            }

            retval.tree = root_0;}
            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 66, mdecls_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "mdecls"

    public static class relationalUnaryExpression_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "relationalUnaryExpression"
    // JFSL.g:754:1: relationalUnaryExpression : ( specUnaryOp parExpression -> ^( UNARY specUnaryOp parExpression ) | specUnaryOp Identifier -> ^( UNARY specUnaryOp ^( IDENTIFIER Identifier ) ) );
    public final JFSLParser.relationalUnaryExpression_return relationalUnaryExpression() throws RecognitionException {
        JFSLParser.relationalUnaryExpression_return retval = new JFSLParser.relationalUnaryExpression_return();
        retval.start = input.LT(1);
        int relationalUnaryExpression_StartIndex = input.index();
        Node root_0 = null;

        Token Identifier278=null;
        JFSLParser.specUnaryOp_return specUnaryOp275 = null;

        JFSLParser.parExpression_return parExpression276 = null;

        JFSLParser.specUnaryOp_return specUnaryOp277 = null;


        Node Identifier278_tree=null;
        RewriteRuleTokenStream stream_Identifier=new RewriteRuleTokenStream(adaptor,"token Identifier");
        RewriteRuleSubtreeStream stream_specUnaryOp=new RewriteRuleSubtreeStream(adaptor,"rule specUnaryOp");
        RewriteRuleSubtreeStream stream_parExpression=new RewriteRuleSubtreeStream(adaptor,"rule parExpression");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 67) ) { return retval; }
            // JFSL.g:755:3: ( specUnaryOp parExpression -> ^( UNARY specUnaryOp parExpression ) | specUnaryOp Identifier -> ^( UNARY specUnaryOp ^( IDENTIFIER Identifier ) ) )
            int alt79=2;
            switch ( input.LA(1) ) {
            case 196:
                {
                int LA79_1 = input.LA(2);

                if ( (LA79_1==186) ) {
                    int LA79_4 = input.LA(3);

                    if ( (LA79_4==150) ) {
                        alt79=1;
                    }
                    else if ( (LA79_4==Identifier) ) {
                        alt79=2;
                    }
                    else {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        NoViableAltException nvae =
                            new NoViableAltException("", 79, 4, input);

                        throw nvae;
                    }
                }
                else if ( (LA79_1==199) ) {
                    int LA79_5 = input.LA(3);

                    if ( (LA79_5==Identifier) ) {
                        alt79=2;
                    }
                    else if ( (LA79_5==150) ) {
                        alt79=1;
                    }
                    else {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        NoViableAltException nvae =
                            new NoViableAltException("", 79, 5, input);

                        throw nvae;
                    }
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 79, 1, input);

                    throw nvae;
                }
                }
                break;
            case 186:
                {
                int LA79_2 = input.LA(2);

                if ( (LA79_2==150) ) {
                    alt79=1;
                }
                else if ( (LA79_2==Identifier) ) {
                    alt79=2;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 79, 2, input);

                    throw nvae;
                }
                }
                break;
            case 199:
                {
                int LA79_3 = input.LA(2);

                if ( (LA79_3==Identifier) ) {
                    alt79=2;
                }
                else if ( (LA79_3==150) ) {
                    alt79=1;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 79, 3, input);

                    throw nvae;
                }
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 79, 0, input);

                throw nvae;
            }

            switch (alt79) {
                case 1 :
                    // JFSL.g:755:5: specUnaryOp parExpression
                    {
                    pushFollow(FOLLOW_specUnaryOp_in_relationalUnaryExpression4043);
                    specUnaryOp275=specUnaryOp();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_specUnaryOp.add(specUnaryOp275.getTree());
                    pushFollow(FOLLOW_parExpression_in_relationalUnaryExpression4045);
                    parExpression276=parExpression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_parExpression.add(parExpression276.getTree());


                    // AST REWRITE
                    // elements: parExpression, specUnaryOp
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 755:31: -> ^( UNARY specUnaryOp parExpression )
                    {
                        // JFSL.g:755:34: ^( UNARY specUnaryOp parExpression )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(UNARY, "UNARY"), root_1);

                        adaptor.addChild(root_1, stream_specUnaryOp.nextTree());
                        adaptor.addChild(root_1, stream_parExpression.nextTree());

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 2 :
                    // JFSL.g:756:5: specUnaryOp Identifier
                    {
                    pushFollow(FOLLOW_specUnaryOp_in_relationalUnaryExpression4061);
                    specUnaryOp277=specUnaryOp();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_specUnaryOp.add(specUnaryOp277.getTree());
                    Identifier278=(Token)match(input,Identifier,FOLLOW_Identifier_in_relationalUnaryExpression4063); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_Identifier.add(Identifier278);



                    // AST REWRITE
                    // elements: specUnaryOp, Identifier
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 756:28: -> ^( UNARY specUnaryOp ^( IDENTIFIER Identifier ) )
                    {
                        // JFSL.g:756:31: ^( UNARY specUnaryOp ^( IDENTIFIER Identifier ) )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(UNARY, "UNARY"), root_1);

                        adaptor.addChild(root_1, stream_specUnaryOp.nextTree());
                        // JFSL.g:756:51: ^( IDENTIFIER Identifier )
                        {
                        Node root_2 = (Node)adaptor.nil();
                        root_2 = (Node)adaptor.becomeRoot((Node)adaptor.create(IDENTIFIER, "IDENTIFIER"), root_2);

                        adaptor.addChild(root_2, stream_Identifier.nextNode());

                        adaptor.addChild(root_1, root_2);
                        }

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 67, relationalUnaryExpression_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "relationalUnaryExpression"

    public static class joinExpression_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "joinExpression"
    // JFSL.g:759:1: joinExpression : primary ( selector )* -> ^( CHAIN primary ( selector )* ) ;
    public final JFSLParser.joinExpression_return joinExpression() throws RecognitionException {
        JFSLParser.joinExpression_return retval = new JFSLParser.joinExpression_return();
        retval.start = input.LT(1);
        int joinExpression_StartIndex = input.index();
        Node root_0 = null;

        JFSLParser.primary_return primary279 = null;

        JFSLParser.selector_return selector280 = null;


        RewriteRuleSubtreeStream stream_selector=new RewriteRuleSubtreeStream(adaptor,"rule selector");
        RewriteRuleSubtreeStream stream_primary=new RewriteRuleSubtreeStream(adaptor,"rule primary");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 68) ) { return retval; }
            // JFSL.g:760:3: ( primary ( selector )* -> ^( CHAIN primary ( selector )* ) )
            // JFSL.g:760:7: primary ( selector )*
            {
            pushFollow(FOLLOW_primary_in_joinExpression4096);
            primary279=primary();

            state._fsp--;
            if (state.failed) return retval;
            if ( state.backtracking==0 ) stream_primary.add(primary279.getTree());
            // JFSL.g:760:15: ( selector )*
            loop80:
            do {
                int alt80=2;
                alt80 = dfa80.predict(input);
                switch (alt80) {
            	case 1 :
            	    // JFSL.g:0:0: selector
            	    {
            	    pushFollow(FOLLOW_selector_in_joinExpression4098);
            	    selector280=selector();

            	    state._fsp--;
            	    if (state.failed) return retval;
            	    if ( state.backtracking==0 ) stream_selector.add(selector280.getTree());

            	    }
            	    break;

            	default :
            	    break loop80;
                }
            } while (true);



            // AST REWRITE
            // elements: primary, selector
            // token labels: 
            // rule labels: retval
            // token list labels: 
            // rule list labels: 
            // wildcard labels: 
            if ( state.backtracking==0 ) {
            retval.tree = root_0;
            RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

            root_0 = (Node)adaptor.nil();
            // 760:24: -> ^( CHAIN primary ( selector )* )
            {
                // JFSL.g:760:27: ^( CHAIN primary ( selector )* )
                {
                Node root_1 = (Node)adaptor.nil();
                root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(CHAIN, "CHAIN"), root_1);

                adaptor.addChild(root_1, stream_primary.nextTree());
                // JFSL.g:760:43: ( selector )*
                while ( stream_selector.hasNext() ) {
                    adaptor.addChild(root_1, stream_selector.nextTree());

                }
                stream_selector.reset();

                adaptor.addChild(root_0, root_1);
                }

            }

            retval.tree = root_0;}
            }

            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 68, joinExpression_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "joinExpression"

    public static class common_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "common"
    // JFSL.g:763:1: common : ( parExpression | 'return' -> RETURN_VAR | 'throw' -> THROW_VAR | 'this' -> THIS_VAR | 'super' -> SUPER_VAR | '_' -> LAMBDA_VAR | '@old' '(' expression ')' -> ^( OLD expression ) | '@arg' '(' integerLiteral ')' -> ^( ARGUMENT integerLiteral ) | relationalUnaryExpression | '{' decls '|' expression '}' -> ^( QUANTIFY OP_SET_COMPREHENSION decls expression ) | '{' rangeExpression ( ',' rangeExpression )* '}' -> ^( OP_SET_COMPREHENSION_ENUM ( rangeExpression )+ ) );
    public final JFSLParser.common_return common() throws RecognitionException {
        JFSLParser.common_return retval = new JFSLParser.common_return();
        retval.start = input.LT(1);
        int common_StartIndex = input.index();
        Node root_0 = null;

        Token string_literal282=null;
        Token string_literal283=null;
        Token string_literal284=null;
        Token string_literal285=null;
        Token char_literal286=null;
        Token string_literal287=null;
        Token char_literal288=null;
        Token char_literal290=null;
        Token string_literal291=null;
        Token char_literal292=null;
        Token char_literal294=null;
        Token char_literal296=null;
        Token char_literal298=null;
        Token char_literal300=null;
        Token char_literal301=null;
        Token char_literal303=null;
        Token char_literal305=null;
        JFSLParser.parExpression_return parExpression281 = null;

        JFSLParser.expression_return expression289 = null;

        JFSLParser.integerLiteral_return integerLiteral293 = null;

        JFSLParser.relationalUnaryExpression_return relationalUnaryExpression295 = null;

        JFSLParser.decls_return decls297 = null;

        JFSLParser.expression_return expression299 = null;

        JFSLParser.rangeExpression_return rangeExpression302 = null;

        JFSLParser.rangeExpression_return rangeExpression304 = null;


        Node string_literal282_tree=null;
        Node string_literal283_tree=null;
        Node string_literal284_tree=null;
        Node string_literal285_tree=null;
        Node char_literal286_tree=null;
        Node string_literal287_tree=null;
        Node char_literal288_tree=null;
        Node char_literal290_tree=null;
        Node string_literal291_tree=null;
        Node char_literal292_tree=null;
        Node char_literal294_tree=null;
        Node char_literal296_tree=null;
        Node char_literal298_tree=null;
        Node char_literal300_tree=null;
        Node char_literal301_tree=null;
        Node char_literal303_tree=null;
        Node char_literal305_tree=null;
        RewriteRuleTokenStream stream_202=new RewriteRuleTokenStream(adaptor,"token 202");
        RewriteRuleTokenStream stream_203=new RewriteRuleTokenStream(adaptor,"token 203");
        RewriteRuleTokenStream stream_144=new RewriteRuleTokenStream(adaptor,"token 144");
        RewriteRuleTokenStream stream_204=new RewriteRuleTokenStream(adaptor,"token 204");
        RewriteRuleTokenStream stream_145=new RewriteRuleTokenStream(adaptor,"token 145");
        RewriteRuleTokenStream stream_205=new RewriteRuleTokenStream(adaptor,"token 205");
        RewriteRuleTokenStream stream_147=new RewriteRuleTokenStream(adaptor,"token 147");
        RewriteRuleTokenStream stream_200=new RewriteRuleTokenStream(adaptor,"token 200");
        RewriteRuleTokenStream stream_201=new RewriteRuleTokenStream(adaptor,"token 201");
        RewriteRuleTokenStream stream_206=new RewriteRuleTokenStream(adaptor,"token 206");
        RewriteRuleTokenStream stream_150=new RewriteRuleTokenStream(adaptor,"token 150");
        RewriteRuleTokenStream stream_163=new RewriteRuleTokenStream(adaptor,"token 163");
        RewriteRuleTokenStream stream_151=new RewriteRuleTokenStream(adaptor,"token 151");
        RewriteRuleSubtreeStream stream_expression=new RewriteRuleSubtreeStream(adaptor,"rule expression");
        RewriteRuleSubtreeStream stream_rangeExpression=new RewriteRuleSubtreeStream(adaptor,"rule rangeExpression");
        RewriteRuleSubtreeStream stream_decls=new RewriteRuleSubtreeStream(adaptor,"rule decls");
        RewriteRuleSubtreeStream stream_integerLiteral=new RewriteRuleSubtreeStream(adaptor,"rule integerLiteral");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 69) ) { return retval; }
            // JFSL.g:764:3: ( parExpression | 'return' -> RETURN_VAR | 'throw' -> THROW_VAR | 'this' -> THIS_VAR | 'super' -> SUPER_VAR | '_' -> LAMBDA_VAR | '@old' '(' expression ')' -> ^( OLD expression ) | '@arg' '(' integerLiteral ')' -> ^( ARGUMENT integerLiteral ) | relationalUnaryExpression | '{' decls '|' expression '}' -> ^( QUANTIFY OP_SET_COMPREHENSION decls expression ) | '{' rangeExpression ( ',' rangeExpression )* '}' -> ^( OP_SET_COMPREHENSION_ENUM ( rangeExpression )+ ) )
            int alt82=11;
            alt82 = dfa82.predict(input);
            switch (alt82) {
                case 1 :
                    // JFSL.g:764:5: parExpression
                    {
                    root_0 = (Node)adaptor.nil();

                    pushFollow(FOLLOW_parExpression_in_common4127);
                    parExpression281=parExpression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, parExpression281.getTree());

                    }
                    break;
                case 2 :
                    // JFSL.g:765:5: 'return'
                    {
                    string_literal282=(Token)match(input,200,FOLLOW_200_in_common4133); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_200.add(string_literal282);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 765:14: -> RETURN_VAR
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(RETURN_VAR, "RETURN_VAR"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 3 :
                    // JFSL.g:766:5: 'throw'
                    {
                    string_literal283=(Token)match(input,201,FOLLOW_201_in_common4143); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_201.add(string_literal283);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 766:14: -> THROW_VAR
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(THROW_VAR, "THROW_VAR"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 4 :
                    // JFSL.g:767:5: 'this'
                    {
                    string_literal284=(Token)match(input,202,FOLLOW_202_in_common4154); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_202.add(string_literal284);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 767:14: -> THIS_VAR
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(THIS_VAR, "THIS_VAR"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 5 :
                    // JFSL.g:768:5: 'super'
                    {
                    string_literal285=(Token)match(input,203,FOLLOW_203_in_common4166); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_203.add(string_literal285);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 768:14: -> SUPER_VAR
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(SUPER_VAR, "SUPER_VAR"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 6 :
                    // JFSL.g:769:5: '_'
                    {
                    char_literal286=(Token)match(input,204,FOLLOW_204_in_common4177); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_204.add(char_literal286);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 769:14: -> LAMBDA_VAR
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(LAMBDA_VAR, "LAMBDA_VAR"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 7 :
                    // JFSL.g:770:5: '@old' '(' expression ')'
                    {
                    string_literal287=(Token)match(input,205,FOLLOW_205_in_common4192); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_205.add(string_literal287);

                    char_literal288=(Token)match(input,150,FOLLOW_150_in_common4194); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_150.add(char_literal288);

                    pushFollow(FOLLOW_expression_in_common4196);
                    expression289=expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_expression.add(expression289.getTree());
                    char_literal290=(Token)match(input,151,FOLLOW_151_in_common4198); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_151.add(char_literal290);



                    // AST REWRITE
                    // elements: expression
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 770:31: -> ^( OLD expression )
                    {
                        // JFSL.g:770:34: ^( OLD expression )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(OLD, "OLD"), root_1);

                        adaptor.addChild(root_1, stream_expression.nextTree());

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 8 :
                    // JFSL.g:771:5: '@arg' '(' integerLiteral ')'
                    {
                    string_literal291=(Token)match(input,206,FOLLOW_206_in_common4212); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_206.add(string_literal291);

                    char_literal292=(Token)match(input,150,FOLLOW_150_in_common4214); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_150.add(char_literal292);

                    pushFollow(FOLLOW_integerLiteral_in_common4216);
                    integerLiteral293=integerLiteral();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_integerLiteral.add(integerLiteral293.getTree());
                    char_literal294=(Token)match(input,151,FOLLOW_151_in_common4218); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_151.add(char_literal294);



                    // AST REWRITE
                    // elements: integerLiteral
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 771:35: -> ^( ARGUMENT integerLiteral )
                    {
                        // JFSL.g:771:38: ^( ARGUMENT integerLiteral )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(ARGUMENT, "ARGUMENT"), root_1);

                        adaptor.addChild(root_1, stream_integerLiteral.nextTree());

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 9 :
                    // JFSL.g:772:5: relationalUnaryExpression
                    {
                    root_0 = (Node)adaptor.nil();

                    pushFollow(FOLLOW_relationalUnaryExpression_in_common4232);
                    relationalUnaryExpression295=relationalUnaryExpression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, relationalUnaryExpression295.getTree());

                    }
                    break;
                case 10 :
                    // JFSL.g:773:5: '{' decls '|' expression '}'
                    {
                    char_literal296=(Token)match(input,144,FOLLOW_144_in_common4238); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_144.add(char_literal296);

                    pushFollow(FOLLOW_decls_in_common4240);
                    decls297=decls();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_decls.add(decls297.getTree());
                    char_literal298=(Token)match(input,163,FOLLOW_163_in_common4242); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_163.add(char_literal298);

                    pushFollow(FOLLOW_expression_in_common4244);
                    expression299=expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_expression.add(expression299.getTree());
                    char_literal300=(Token)match(input,145,FOLLOW_145_in_common4246); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_145.add(char_literal300);



                    // AST REWRITE
                    // elements: decls, expression
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 773:34: -> ^( QUANTIFY OP_SET_COMPREHENSION decls expression )
                    {
                        // JFSL.g:773:37: ^( QUANTIFY OP_SET_COMPREHENSION decls expression )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(QUANTIFY, "QUANTIFY"), root_1);

                        adaptor.addChild(root_1, (Node)adaptor.create(OP_SET_COMPREHENSION, "OP_SET_COMPREHENSION"));
                        adaptor.addChild(root_1, stream_decls.nextTree());
                        adaptor.addChild(root_1, stream_expression.nextTree());

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 11 :
                    // JFSL.g:774:5: '{' rangeExpression ( ',' rangeExpression )* '}'
                    {
                    char_literal301=(Token)match(input,144,FOLLOW_144_in_common4264); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_144.add(char_literal301);

                    pushFollow(FOLLOW_rangeExpression_in_common4266);
                    rangeExpression302=rangeExpression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_rangeExpression.add(rangeExpression302.getTree());
                    // JFSL.g:774:25: ( ',' rangeExpression )*
                    loop81:
                    do {
                        int alt81=2;
                        int LA81_0 = input.LA(1);

                        if ( (LA81_0==147) ) {
                            alt81=1;
                        }


                        switch (alt81) {
                    	case 1 :
                    	    // JFSL.g:774:26: ',' rangeExpression
                    	    {
                    	    char_literal303=(Token)match(input,147,FOLLOW_147_in_common4269); if (state.failed) return retval; 
                    	    if ( state.backtracking==0 ) stream_147.add(char_literal303);

                    	    pushFollow(FOLLOW_rangeExpression_in_common4271);
                    	    rangeExpression304=rangeExpression();

                    	    state._fsp--;
                    	    if (state.failed) return retval;
                    	    if ( state.backtracking==0 ) stream_rangeExpression.add(rangeExpression304.getTree());

                    	    }
                    	    break;

                    	default :
                    	    break loop81;
                        }
                    } while (true);

                    char_literal305=(Token)match(input,145,FOLLOW_145_in_common4275); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_145.add(char_literal305);



                    // AST REWRITE
                    // elements: rangeExpression
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 774:52: -> ^( OP_SET_COMPREHENSION_ENUM ( rangeExpression )+ )
                    {
                        // JFSL.g:774:55: ^( OP_SET_COMPREHENSION_ENUM ( rangeExpression )+ )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(OP_SET_COMPREHENSION_ENUM, "OP_SET_COMPREHENSION_ENUM"), root_1);

                        if ( !(stream_rangeExpression.hasNext()) ) {
                            throw new RewriteEarlyExitException();
                        }
                        while ( stream_rangeExpression.hasNext() ) {
                            adaptor.addChild(root_1, stream_rangeExpression.nextTree());

                        }
                        stream_rangeExpression.reset();

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 69, common_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "common"

    public static class primary_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "primary"
    // JFSL.g:777:1: primary : ( common | literal | typeDisambiguous | Identifier ( '.' Identifier )* ( typeParameters2 )? -> ^( AMBIGUOUS ( ^( IDENTIFIER Identifier ) )+ ( ^( TYPE_PARAMETERS typeParameters2 ) )? ) | typeName '@' Identifier -> ^( FIELD typeName ^( IDENTIFIER Identifier ) ) | Identifier arguments -> ^( CALL Identifier arguments ) );
    public final JFSLParser.primary_return primary() throws RecognitionException {
        JFSLParser.primary_return retval = new JFSLParser.primary_return();
        retval.start = input.LT(1);
        int primary_StartIndex = input.index();
        Node root_0 = null;

        Token Identifier309=null;
        Token char_literal310=null;
        Token Identifier311=null;
        Token char_literal314=null;
        Token Identifier315=null;
        Token Identifier316=null;
        JFSLParser.common_return common306 = null;

        JFSLParser.literal_return literal307 = null;

        JFSLParser.typeDisambiguous_return typeDisambiguous308 = null;

        JFSLParser.typeParameters2_return typeParameters2312 = null;

        JFSLParser.typeName_return typeName313 = null;

        JFSLParser.arguments_return arguments317 = null;


        Node Identifier309_tree=null;
        Node char_literal310_tree=null;
        Node Identifier311_tree=null;
        Node char_literal314_tree=null;
        Node Identifier315_tree=null;
        Node Identifier316_tree=null;
        RewriteRuleTokenStream stream_196=new RewriteRuleTokenStream(adaptor,"token 196");
        RewriteRuleTokenStream stream_Identifier=new RewriteRuleTokenStream(adaptor,"token Identifier");
        RewriteRuleTokenStream stream_140=new RewriteRuleTokenStream(adaptor,"token 140");
        RewriteRuleSubtreeStream stream_typeName=new RewriteRuleSubtreeStream(adaptor,"rule typeName");
        RewriteRuleSubtreeStream stream_arguments=new RewriteRuleSubtreeStream(adaptor,"rule arguments");
        RewriteRuleSubtreeStream stream_typeParameters2=new RewriteRuleSubtreeStream(adaptor,"rule typeParameters2");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 70) ) { return retval; }
            // JFSL.g:778:3: ( common | literal | typeDisambiguous | Identifier ( '.' Identifier )* ( typeParameters2 )? -> ^( AMBIGUOUS ( ^( IDENTIFIER Identifier ) )+ ( ^( TYPE_PARAMETERS typeParameters2 ) )? ) | typeName '@' Identifier -> ^( FIELD typeName ^( IDENTIFIER Identifier ) ) | Identifier arguments -> ^( CALL Identifier arguments ) )
            int alt85=6;
            alt85 = dfa85.predict(input);
            switch (alt85) {
                case 1 :
                    // JFSL.g:778:5: common
                    {
                    root_0 = (Node)adaptor.nil();

                    pushFollow(FOLLOW_common_in_primary4299);
                    common306=common();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, common306.getTree());

                    }
                    break;
                case 2 :
                    // JFSL.g:779:5: literal
                    {
                    root_0 = (Node)adaptor.nil();

                    pushFollow(FOLLOW_literal_in_primary4305);
                    literal307=literal();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, literal307.getTree());

                    }
                    break;
                case 3 :
                    // JFSL.g:780:5: typeDisambiguous
                    {
                    root_0 = (Node)adaptor.nil();

                    pushFollow(FOLLOW_typeDisambiguous_in_primary4311);
                    typeDisambiguous308=typeDisambiguous();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, typeDisambiguous308.getTree());

                    }
                    break;
                case 4 :
                    // JFSL.g:781:5: Identifier ( '.' Identifier )* ( typeParameters2 )?
                    {
                    Identifier309=(Token)match(input,Identifier,FOLLOW_Identifier_in_primary4317); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_Identifier.add(Identifier309);

                    // JFSL.g:781:16: ( '.' Identifier )*
                    loop83:
                    do {
                        int alt83=2;
                        int LA83_0 = input.LA(1);

                        if ( (LA83_0==140) ) {
                            int LA83_2 = input.LA(2);

                            if ( (LA83_2==Identifier) ) {
                                int LA83_3 = input.LA(3);

                                if ( (synpred128_JFSL()) ) {
                                    alt83=1;
                                }


                            }


                        }


                        switch (alt83) {
                    	case 1 :
                    	    // JFSL.g:781:17: '.' Identifier
                    	    {
                    	    char_literal310=(Token)match(input,140,FOLLOW_140_in_primary4320); if (state.failed) return retval; 
                    	    if ( state.backtracking==0 ) stream_140.add(char_literal310);

                    	    Identifier311=(Token)match(input,Identifier,FOLLOW_Identifier_in_primary4322); if (state.failed) return retval; 
                    	    if ( state.backtracking==0 ) stream_Identifier.add(Identifier311);


                    	    }
                    	    break;

                    	default :
                    	    break loop83;
                        }
                    } while (true);

                    // JFSL.g:781:34: ( typeParameters2 )?
                    int alt84=2;
                    alt84 = dfa84.predict(input);
                    switch (alt84) {
                        case 1 :
                            // JFSL.g:0:0: typeParameters2
                            {
                            pushFollow(FOLLOW_typeParameters2_in_primary4326);
                            typeParameters2312=typeParameters2();

                            state._fsp--;
                            if (state.failed) return retval;
                            if ( state.backtracking==0 ) stream_typeParameters2.add(typeParameters2312.getTree());

                            }
                            break;

                    }



                    // AST REWRITE
                    // elements: typeParameters2, Identifier
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 781:51: -> ^( AMBIGUOUS ( ^( IDENTIFIER Identifier ) )+ ( ^( TYPE_PARAMETERS typeParameters2 ) )? )
                    {
                        // JFSL.g:781:54: ^( AMBIGUOUS ( ^( IDENTIFIER Identifier ) )+ ( ^( TYPE_PARAMETERS typeParameters2 ) )? )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(AMBIGUOUS, "AMBIGUOUS"), root_1);

                        if ( !(stream_Identifier.hasNext()) ) {
                            throw new RewriteEarlyExitException();
                        }
                        while ( stream_Identifier.hasNext() ) {
                            // JFSL.g:781:66: ^( IDENTIFIER Identifier )
                            {
                            Node root_2 = (Node)adaptor.nil();
                            root_2 = (Node)adaptor.becomeRoot((Node)adaptor.create(IDENTIFIER, "IDENTIFIER"), root_2);

                            adaptor.addChild(root_2, stream_Identifier.nextNode());

                            adaptor.addChild(root_1, root_2);
                            }

                        }
                        stream_Identifier.reset();
                        // JFSL.g:781:92: ( ^( TYPE_PARAMETERS typeParameters2 ) )?
                        if ( stream_typeParameters2.hasNext() ) {
                            // JFSL.g:781:92: ^( TYPE_PARAMETERS typeParameters2 )
                            {
                            Node root_2 = (Node)adaptor.nil();
                            root_2 = (Node)adaptor.becomeRoot((Node)adaptor.create(TYPE_PARAMETERS, "TYPE_PARAMETERS"), root_2);

                            adaptor.addChild(root_2, stream_typeParameters2.nextTree());

                            adaptor.addChild(root_1, root_2);
                            }

                        }
                        stream_typeParameters2.reset();

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 5 :
                    // JFSL.g:782:5: typeName '@' Identifier
                    {
                    pushFollow(FOLLOW_typeName_in_primary4353);
                    typeName313=typeName();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_typeName.add(typeName313.getTree());
                    char_literal314=(Token)match(input,196,FOLLOW_196_in_primary4355); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_196.add(char_literal314);

                    Identifier315=(Token)match(input,Identifier,FOLLOW_Identifier_in_primary4357); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_Identifier.add(Identifier315);



                    // AST REWRITE
                    // elements: typeName, Identifier
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 782:29: -> ^( FIELD typeName ^( IDENTIFIER Identifier ) )
                    {
                        // JFSL.g:782:32: ^( FIELD typeName ^( IDENTIFIER Identifier ) )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(FIELD, "FIELD"), root_1);

                        adaptor.addChild(root_1, stream_typeName.nextTree());
                        // JFSL.g:782:49: ^( IDENTIFIER Identifier )
                        {
                        Node root_2 = (Node)adaptor.nil();
                        root_2 = (Node)adaptor.becomeRoot((Node)adaptor.create(IDENTIFIER, "IDENTIFIER"), root_2);

                        adaptor.addChild(root_2, stream_Identifier.nextNode());

                        adaptor.addChild(root_1, root_2);
                        }

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 6 :
                    // JFSL.g:783:5: Identifier arguments
                    {
                    Identifier316=(Token)match(input,Identifier,FOLLOW_Identifier_in_primary4377); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_Identifier.add(Identifier316);

                    pushFollow(FOLLOW_arguments_in_primary4379);
                    arguments317=arguments();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_arguments.add(arguments317.getTree());


                    // AST REWRITE
                    // elements: Identifier, arguments
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 783:26: -> ^( CALL Identifier arguments )
                    {
                        // JFSL.g:783:29: ^( CALL Identifier arguments )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(CALL, "CALL"), root_1);

                        adaptor.addChild(root_1, stream_Identifier.nextNode());
                        adaptor.addChild(root_1, stream_arguments.nextTree());

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 70, primary_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "primary"

    public static class selector_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "selector"
    // JFSL.g:787:1: selector : ( '.' Identifier arguments -> ^( CALL Identifier arguments ) | '.' Identifier -> ^( JOIN ^( IDENTIFIER Identifier ) ) | '.' primitiveType -> ^( JOIN primitiveType ) | '.' common -> ^( JOIN common ) | '.' '*' Identifier -> ^( JOIN_REFLEXIVE ^( IDENTIFIER Identifier ) ) | '.' '*' common -> ^( JOIN_REFLEXIVE common ) | '[' expression ']' -> ^( BRACKET expression ) | '[' ']' -> ^( BRACKET ) | '<' DecimalLiteral ( ',' DecimalLiteral )* '>' -> ^( PROJECTION ( DecimalLiteral )+ ) );
    public final JFSLParser.selector_return selector() throws RecognitionException {
        JFSLParser.selector_return retval = new JFSLParser.selector_return();
        retval.start = input.LT(1);
        int selector_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal318=null;
        Token Identifier319=null;
        Token char_literal321=null;
        Token Identifier322=null;
        Token char_literal323=null;
        Token char_literal325=null;
        Token char_literal327=null;
        Token char_literal328=null;
        Token Identifier329=null;
        Token char_literal330=null;
        Token char_literal331=null;
        Token char_literal333=null;
        Token char_literal335=null;
        Token char_literal336=null;
        Token char_literal337=null;
        Token char_literal338=null;
        Token DecimalLiteral339=null;
        Token char_literal340=null;
        Token DecimalLiteral341=null;
        Token char_literal342=null;
        JFSLParser.arguments_return arguments320 = null;

        JFSLParser.primitiveType_return primitiveType324 = null;

        JFSLParser.common_return common326 = null;

        JFSLParser.common_return common332 = null;

        JFSLParser.expression_return expression334 = null;


        Node char_literal318_tree=null;
        Node Identifier319_tree=null;
        Node char_literal321_tree=null;
        Node Identifier322_tree=null;
        Node char_literal323_tree=null;
        Node char_literal325_tree=null;
        Node char_literal327_tree=null;
        Node char_literal328_tree=null;
        Node Identifier329_tree=null;
        Node char_literal330_tree=null;
        Node char_literal331_tree=null;
        Node char_literal333_tree=null;
        Node char_literal335_tree=null;
        Node char_literal336_tree=null;
        Node char_literal337_tree=null;
        Node char_literal338_tree=null;
        Node DecimalLiteral339_tree=null;
        Node char_literal340_tree=null;
        Node DecimalLiteral341_tree=null;
        Node char_literal342_tree=null;
        RewriteRuleTokenStream stream_180=new RewriteRuleTokenStream(adaptor,"token 180");
        RewriteRuleTokenStream stream_146=new RewriteRuleTokenStream(adaptor,"token 146");
        RewriteRuleTokenStream stream_147=new RewriteRuleTokenStream(adaptor,"token 147");
        RewriteRuleTokenStream stream_148=new RewriteRuleTokenStream(adaptor,"token 148");
        RewriteRuleTokenStream stream_179=new RewriteRuleTokenStream(adaptor,"token 179");
        RewriteRuleTokenStream stream_DecimalLiteral=new RewriteRuleTokenStream(adaptor,"token DecimalLiteral");
        RewriteRuleTokenStream stream_Identifier=new RewriteRuleTokenStream(adaptor,"token Identifier");
        RewriteRuleTokenStream stream_140=new RewriteRuleTokenStream(adaptor,"token 140");
        RewriteRuleTokenStream stream_167=new RewriteRuleTokenStream(adaptor,"token 167");
        RewriteRuleSubtreeStream stream_expression=new RewriteRuleSubtreeStream(adaptor,"rule expression");
        RewriteRuleSubtreeStream stream_arguments=new RewriteRuleSubtreeStream(adaptor,"rule arguments");
        RewriteRuleSubtreeStream stream_common=new RewriteRuleSubtreeStream(adaptor,"rule common");
        RewriteRuleSubtreeStream stream_primitiveType=new RewriteRuleSubtreeStream(adaptor,"rule primitiveType");
        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 71) ) { return retval; }
            // JFSL.g:788:3: ( '.' Identifier arguments -> ^( CALL Identifier arguments ) | '.' Identifier -> ^( JOIN ^( IDENTIFIER Identifier ) ) | '.' primitiveType -> ^( JOIN primitiveType ) | '.' common -> ^( JOIN common ) | '.' '*' Identifier -> ^( JOIN_REFLEXIVE ^( IDENTIFIER Identifier ) ) | '.' '*' common -> ^( JOIN_REFLEXIVE common ) | '[' expression ']' -> ^( BRACKET expression ) | '[' ']' -> ^( BRACKET ) | '<' DecimalLiteral ( ',' DecimalLiteral )* '>' -> ^( PROJECTION ( DecimalLiteral )+ ) )
            int alt87=9;
            alt87 = dfa87.predict(input);
            switch (alt87) {
                case 1 :
                    // JFSL.g:788:5: '.' Identifier arguments
                    {
                    char_literal318=(Token)match(input,140,FOLLOW_140_in_selector4405); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_140.add(char_literal318);

                    Identifier319=(Token)match(input,Identifier,FOLLOW_Identifier_in_selector4407); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_Identifier.add(Identifier319);

                    pushFollow(FOLLOW_arguments_in_selector4409);
                    arguments320=arguments();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_arguments.add(arguments320.getTree());


                    // AST REWRITE
                    // elements: arguments, Identifier
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 788:30: -> ^( CALL Identifier arguments )
                    {
                        // JFSL.g:788:33: ^( CALL Identifier arguments )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(CALL, "CALL"), root_1);

                        adaptor.addChild(root_1, stream_Identifier.nextNode());
                        adaptor.addChild(root_1, stream_arguments.nextTree());

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 2 :
                    // JFSL.g:789:5: '.' Identifier
                    {
                    char_literal321=(Token)match(input,140,FOLLOW_140_in_selector4425); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_140.add(char_literal321);

                    Identifier322=(Token)match(input,Identifier,FOLLOW_Identifier_in_selector4427); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_Identifier.add(Identifier322);



                    // AST REWRITE
                    // elements: Identifier
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 789:20: -> ^( JOIN ^( IDENTIFIER Identifier ) )
                    {
                        // JFSL.g:789:23: ^( JOIN ^( IDENTIFIER Identifier ) )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(JOIN, "JOIN"), root_1);

                        // JFSL.g:789:30: ^( IDENTIFIER Identifier )
                        {
                        Node root_2 = (Node)adaptor.nil();
                        root_2 = (Node)adaptor.becomeRoot((Node)adaptor.create(IDENTIFIER, "IDENTIFIER"), root_2);

                        adaptor.addChild(root_2, stream_Identifier.nextNode());

                        adaptor.addChild(root_1, root_2);
                        }

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 3 :
                    // JFSL.g:790:5: '.' primitiveType
                    {
                    char_literal323=(Token)match(input,140,FOLLOW_140_in_selector4445); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_140.add(char_literal323);

                    pushFollow(FOLLOW_primitiveType_in_selector4447);
                    primitiveType324=primitiveType();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_primitiveType.add(primitiveType324.getTree());


                    // AST REWRITE
                    // elements: primitiveType
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 790:23: -> ^( JOIN primitiveType )
                    {
                        // JFSL.g:790:26: ^( JOIN primitiveType )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(JOIN, "JOIN"), root_1);

                        adaptor.addChild(root_1, stream_primitiveType.nextTree());

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 4 :
                    // JFSL.g:791:5: '.' common
                    {
                    char_literal325=(Token)match(input,140,FOLLOW_140_in_selector4461); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_140.add(char_literal325);

                    pushFollow(FOLLOW_common_in_selector4463);
                    common326=common();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_common.add(common326.getTree());


                    // AST REWRITE
                    // elements: common
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 791:16: -> ^( JOIN common )
                    {
                        // JFSL.g:791:19: ^( JOIN common )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(JOIN, "JOIN"), root_1);

                        adaptor.addChild(root_1, stream_common.nextTree());

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 5 :
                    // JFSL.g:792:5: '.' '*' Identifier
                    {
                    char_literal327=(Token)match(input,140,FOLLOW_140_in_selector4477); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_140.add(char_literal327);

                    char_literal328=(Token)match(input,167,FOLLOW_167_in_selector4479); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_167.add(char_literal328);

                    Identifier329=(Token)match(input,Identifier,FOLLOW_Identifier_in_selector4481); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_Identifier.add(Identifier329);



                    // AST REWRITE
                    // elements: Identifier
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 792:24: -> ^( JOIN_REFLEXIVE ^( IDENTIFIER Identifier ) )
                    {
                        // JFSL.g:792:27: ^( JOIN_REFLEXIVE ^( IDENTIFIER Identifier ) )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(JOIN_REFLEXIVE, "JOIN_REFLEXIVE"), root_1);

                        // JFSL.g:792:44: ^( IDENTIFIER Identifier )
                        {
                        Node root_2 = (Node)adaptor.nil();
                        root_2 = (Node)adaptor.becomeRoot((Node)adaptor.create(IDENTIFIER, "IDENTIFIER"), root_2);

                        adaptor.addChild(root_2, stream_Identifier.nextNode());

                        adaptor.addChild(root_1, root_2);
                        }

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 6 :
                    // JFSL.g:793:5: '.' '*' common
                    {
                    char_literal330=(Token)match(input,140,FOLLOW_140_in_selector4499); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_140.add(char_literal330);

                    char_literal331=(Token)match(input,167,FOLLOW_167_in_selector4501); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_167.add(char_literal331);

                    pushFollow(FOLLOW_common_in_selector4503);
                    common332=common();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_common.add(common332.getTree());


                    // AST REWRITE
                    // elements: common
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 793:20: -> ^( JOIN_REFLEXIVE common )
                    {
                        // JFSL.g:793:23: ^( JOIN_REFLEXIVE common )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(JOIN_REFLEXIVE, "JOIN_REFLEXIVE"), root_1);

                        adaptor.addChild(root_1, stream_common.nextTree());

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 7 :
                    // JFSL.g:794:5: '[' expression ']'
                    {
                    char_literal333=(Token)match(input,179,FOLLOW_179_in_selector4517); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_179.add(char_literal333);

                    pushFollow(FOLLOW_expression_in_selector4519);
                    expression334=expression();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) stream_expression.add(expression334.getTree());
                    char_literal335=(Token)match(input,180,FOLLOW_180_in_selector4521); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_180.add(char_literal335);



                    // AST REWRITE
                    // elements: expression
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 794:24: -> ^( BRACKET expression )
                    {
                        // JFSL.g:794:27: ^( BRACKET expression )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(BRACKET, "BRACKET"), root_1);

                        adaptor.addChild(root_1, stream_expression.nextTree());

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 8 :
                    // JFSL.g:795:5: '[' ']'
                    {
                    char_literal336=(Token)match(input,179,FOLLOW_179_in_selector4537); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_179.add(char_literal336);

                    char_literal337=(Token)match(input,180,FOLLOW_180_in_selector4539); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_180.add(char_literal337);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 795:13: -> ^( BRACKET )
                    {
                        // JFSL.g:795:16: ^( BRACKET )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(BRACKET, "BRACKET"), root_1);

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 9 :
                    // JFSL.g:796:5: '<' DecimalLiteral ( ',' DecimalLiteral )* '>'
                    {
                    char_literal338=(Token)match(input,146,FOLLOW_146_in_selector4552); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_146.add(char_literal338);

                    DecimalLiteral339=(Token)match(input,DecimalLiteral,FOLLOW_DecimalLiteral_in_selector4554); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_DecimalLiteral.add(DecimalLiteral339);

                    // JFSL.g:796:24: ( ',' DecimalLiteral )*
                    loop86:
                    do {
                        int alt86=2;
                        int LA86_0 = input.LA(1);

                        if ( (LA86_0==147) ) {
                            alt86=1;
                        }


                        switch (alt86) {
                    	case 1 :
                    	    // JFSL.g:796:25: ',' DecimalLiteral
                    	    {
                    	    char_literal340=(Token)match(input,147,FOLLOW_147_in_selector4557); if (state.failed) return retval; 
                    	    if ( state.backtracking==0 ) stream_147.add(char_literal340);

                    	    DecimalLiteral341=(Token)match(input,DecimalLiteral,FOLLOW_DecimalLiteral_in_selector4559); if (state.failed) return retval; 
                    	    if ( state.backtracking==0 ) stream_DecimalLiteral.add(DecimalLiteral341);


                    	    }
                    	    break;

                    	default :
                    	    break loop86;
                        }
                    } while (true);

                    char_literal342=(Token)match(input,148,FOLLOW_148_in_selector4563); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_148.add(char_literal342);



                    // AST REWRITE
                    // elements: DecimalLiteral
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 796:50: -> ^( PROJECTION ( DecimalLiteral )+ )
                    {
                        // JFSL.g:796:53: ^( PROJECTION ( DecimalLiteral )+ )
                        {
                        Node root_1 = (Node)adaptor.nil();
                        root_1 = (Node)adaptor.becomeRoot((Node)adaptor.create(PROJECTION, "PROJECTION"), root_1);

                        if ( !(stream_DecimalLiteral.hasNext()) ) {
                            throw new RewriteEarlyExitException();
                        }
                        while ( stream_DecimalLiteral.hasNext() ) {
                            adaptor.addChild(root_1, stream_DecimalLiteral.nextNode());

                        }
                        stream_DecimalLiteral.reset();

                        adaptor.addChild(root_0, root_1);
                        }

                    }

                    retval.tree = root_0;}
                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 71, selector_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "selector"

    public static class specUnaryOp_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "specUnaryOp"
    // JFSL.g:800:1: specUnaryOp : ( ( '@' '^' | '^' ) -> OP_CLOSURE | ( '@' '~' | '~' ) -> OP_TRANSPOSE );
    public final JFSLParser.specUnaryOp_return specUnaryOp() throws RecognitionException {
        JFSLParser.specUnaryOp_return retval = new JFSLParser.specUnaryOp_return();
        retval.start = input.LT(1);
        int specUnaryOp_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal343=null;
        Token char_literal344=null;
        Token char_literal345=null;
        Token char_literal346=null;
        Token char_literal347=null;
        Token char_literal348=null;

        Node char_literal343_tree=null;
        Node char_literal344_tree=null;
        Node char_literal345_tree=null;
        Node char_literal346_tree=null;
        Node char_literal347_tree=null;
        Node char_literal348_tree=null;
        RewriteRuleTokenStream stream_199=new RewriteRuleTokenStream(adaptor,"token 199");
        RewriteRuleTokenStream stream_186=new RewriteRuleTokenStream(adaptor,"token 186");
        RewriteRuleTokenStream stream_196=new RewriteRuleTokenStream(adaptor,"token 196");

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 72) ) { return retval; }
            // JFSL.g:801:3: ( ( '@' '^' | '^' ) -> OP_CLOSURE | ( '@' '~' | '~' ) -> OP_TRANSPOSE )
            int alt90=2;
            switch ( input.LA(1) ) {
            case 196:
                {
                int LA90_1 = input.LA(2);

                if ( (LA90_1==186) ) {
                    alt90=1;
                }
                else if ( (LA90_1==199) ) {
                    alt90=2;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return retval;}
                    NoViableAltException nvae =
                        new NoViableAltException("", 90, 1, input);

                    throw nvae;
                }
                }
                break;
            case 186:
                {
                alt90=1;
                }
                break;
            case 199:
                {
                alt90=2;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 90, 0, input);

                throw nvae;
            }

            switch (alt90) {
                case 1 :
                    // JFSL.g:801:5: ( '@' '^' | '^' )
                    {
                    // JFSL.g:801:5: ( '@' '^' | '^' )
                    int alt88=2;
                    int LA88_0 = input.LA(1);

                    if ( (LA88_0==196) ) {
                        alt88=1;
                    }
                    else if ( (LA88_0==186) ) {
                        alt88=2;
                    }
                    else {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        NoViableAltException nvae =
                            new NoViableAltException("", 88, 0, input);

                        throw nvae;
                    }
                    switch (alt88) {
                        case 1 :
                            // JFSL.g:801:6: '@' '^'
                            {
                            char_literal343=(Token)match(input,196,FOLLOW_196_in_specUnaryOp4587); if (state.failed) return retval; 
                            if ( state.backtracking==0 ) stream_196.add(char_literal343);

                            char_literal344=(Token)match(input,186,FOLLOW_186_in_specUnaryOp4589); if (state.failed) return retval; 
                            if ( state.backtracking==0 ) stream_186.add(char_literal344);


                            }
                            break;
                        case 2 :
                            // JFSL.g:801:16: '^'
                            {
                            char_literal345=(Token)match(input,186,FOLLOW_186_in_specUnaryOp4593); if (state.failed) return retval; 
                            if ( state.backtracking==0 ) stream_186.add(char_literal345);


                            }
                            break;

                    }



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 801:21: -> OP_CLOSURE
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_CLOSURE, "OP_CLOSURE"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 2 :
                    // JFSL.g:802:5: ( '@' '~' | '~' )
                    {
                    // JFSL.g:802:5: ( '@' '~' | '~' )
                    int alt89=2;
                    int LA89_0 = input.LA(1);

                    if ( (LA89_0==196) ) {
                        alt89=1;
                    }
                    else if ( (LA89_0==199) ) {
                        alt89=2;
                    }
                    else {
                        if (state.backtracking>0) {state.failed=true; return retval;}
                        NoViableAltException nvae =
                            new NoViableAltException("", 89, 0, input);

                        throw nvae;
                    }
                    switch (alt89) {
                        case 1 :
                            // JFSL.g:802:6: '@' '~'
                            {
                            char_literal346=(Token)match(input,196,FOLLOW_196_in_specUnaryOp4605); if (state.failed) return retval; 
                            if ( state.backtracking==0 ) stream_196.add(char_literal346);

                            char_literal347=(Token)match(input,199,FOLLOW_199_in_specUnaryOp4607); if (state.failed) return retval; 
                            if ( state.backtracking==0 ) stream_199.add(char_literal347);


                            }
                            break;
                        case 2 :
                            // JFSL.g:802:16: '~'
                            {
                            char_literal348=(Token)match(input,199,FOLLOW_199_in_specUnaryOp4611); if (state.failed) return retval; 
                            if ( state.backtracking==0 ) stream_199.add(char_literal348);


                            }
                            break;

                    }



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 802:21: -> OP_TRANSPOSE
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_TRANSPOSE, "OP_TRANSPOSE"));

                    }

                    retval.tree = root_0;}
                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 72, specUnaryOp_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "specUnaryOp"

    public static class setDeclOp_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "setDeclOp"
    // JFSL.g:805:1: setDeclOp : ( 'one' -> OP_SET_ONE | 'some' -> OP_SET_SOME | 'lone' -> OP_SET_LONE );
    public final JFSLParser.setDeclOp_return setDeclOp() throws RecognitionException {
        JFSLParser.setDeclOp_return retval = new JFSLParser.setDeclOp_return();
        retval.start = input.LT(1);
        int setDeclOp_StartIndex = input.index();
        Node root_0 = null;

        Token string_literal349=null;
        Token string_literal350=null;
        Token string_literal351=null;

        Node string_literal349_tree=null;
        Node string_literal350_tree=null;
        Node string_literal351_tree=null;
        RewriteRuleTokenStream stream_207=new RewriteRuleTokenStream(adaptor,"token 207");
        RewriteRuleTokenStream stream_208=new RewriteRuleTokenStream(adaptor,"token 208");
        RewriteRuleTokenStream stream_209=new RewriteRuleTokenStream(adaptor,"token 209");

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 73) ) { return retval; }
            // JFSL.g:806:3: ( 'one' -> OP_SET_ONE | 'some' -> OP_SET_SOME | 'lone' -> OP_SET_LONE )
            int alt91=3;
            switch ( input.LA(1) ) {
            case 207:
                {
                alt91=1;
                }
                break;
            case 208:
                {
                alt91=2;
                }
                break;
            case 209:
                {
                alt91=3;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 91, 0, input);

                throw nvae;
            }

            switch (alt91) {
                case 1 :
                    // JFSL.g:806:5: 'one'
                    {
                    string_literal349=(Token)match(input,207,FOLLOW_207_in_setDeclOp4636); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_207.add(string_literal349);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 806:13: -> OP_SET_ONE
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_SET_ONE, "OP_SET_ONE"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 2 :
                    // JFSL.g:807:5: 'some'
                    {
                    string_literal350=(Token)match(input,208,FOLLOW_208_in_setDeclOp4648); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_208.add(string_literal350);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 807:13: -> OP_SET_SOME
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_SET_SOME, "OP_SET_SOME"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 3 :
                    // JFSL.g:808:5: 'lone'
                    {
                    string_literal351=(Token)match(input,209,FOLLOW_209_in_setDeclOp4660); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_209.add(string_literal351);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 808:13: -> OP_SET_LONE
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_SET_LONE, "OP_SET_LONE"));

                    }

                    retval.tree = root_0;}
                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 73, setDeclOp_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "setDeclOp"

    public static class setUnaryOp_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "setUnaryOp"
    // JFSL.g:810:1: setUnaryOp : ( setDeclOp | 'no' -> OP_SET_NO | 'sum' -> OP_SET_SUM );
    public final JFSLParser.setUnaryOp_return setUnaryOp() throws RecognitionException {
        JFSLParser.setUnaryOp_return retval = new JFSLParser.setUnaryOp_return();
        retval.start = input.LT(1);
        int setUnaryOp_StartIndex = input.index();
        Node root_0 = null;

        Token string_literal353=null;
        Token string_literal354=null;
        JFSLParser.setDeclOp_return setDeclOp352 = null;


        Node string_literal353_tree=null;
        Node string_literal354_tree=null;
        RewriteRuleTokenStream stream_211=new RewriteRuleTokenStream(adaptor,"token 211");
        RewriteRuleTokenStream stream_210=new RewriteRuleTokenStream(adaptor,"token 210");

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 74) ) { return retval; }
            // JFSL.g:811:3: ( setDeclOp | 'no' -> OP_SET_NO | 'sum' -> OP_SET_SUM )
            int alt92=3;
            switch ( input.LA(1) ) {
            case 207:
            case 208:
            case 209:
                {
                alt92=1;
                }
                break;
            case 210:
                {
                alt92=2;
                }
                break;
            case 211:
                {
                alt92=3;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 92, 0, input);

                throw nvae;
            }

            switch (alt92) {
                case 1 :
                    // JFSL.g:811:5: setDeclOp
                    {
                    root_0 = (Node)adaptor.nil();

                    pushFollow(FOLLOW_setDeclOp_in_setUnaryOp4683);
                    setDeclOp352=setDeclOp();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, setDeclOp352.getTree());

                    }
                    break;
                case 2 :
                    // JFSL.g:812:5: 'no'
                    {
                    string_literal353=(Token)match(input,210,FOLLOW_210_in_setUnaryOp4689); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_210.add(string_literal353);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 812:11: -> OP_SET_NO
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_SET_NO, "OP_SET_NO"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 3 :
                    // JFSL.g:813:5: 'sum'
                    {
                    string_literal354=(Token)match(input,211,FOLLOW_211_in_setUnaryOp4700); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_211.add(string_literal354);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 813:13: -> OP_SET_SUM
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_SET_SUM, "OP_SET_SUM"));

                    }

                    retval.tree = root_0;}
                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 74, setUnaryOp_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "setUnaryOp"

    public static class setQuantOp_return extends ParserRuleReturnScope {
        Node tree;
        public Object getTree() { return tree; }
    };

    // $ANTLR start "setQuantOp"
    // JFSL.g:816:1: setQuantOp : ( setUnaryOp | '#' -> OP_SET_NUM | 'all' -> OP_SET_ALL | 'exists' -> OP_SET_EXISTS );
    public final JFSLParser.setQuantOp_return setQuantOp() throws RecognitionException {
        JFSLParser.setQuantOp_return retval = new JFSLParser.setQuantOp_return();
        retval.start = input.LT(1);
        int setQuantOp_StartIndex = input.index();
        Node root_0 = null;

        Token char_literal356=null;
        Token string_literal357=null;
        Token string_literal358=null;
        JFSLParser.setUnaryOp_return setUnaryOp355 = null;


        Node char_literal356_tree=null;
        Node string_literal357_tree=null;
        Node string_literal358_tree=null;
        RewriteRuleTokenStream stream_212=new RewriteRuleTokenStream(adaptor,"token 212");
        RewriteRuleTokenStream stream_213=new RewriteRuleTokenStream(adaptor,"token 213");
        RewriteRuleTokenStream stream_193=new RewriteRuleTokenStream(adaptor,"token 193");

        try {
            if ( state.backtracking>0 && alreadyParsedRule(input, 75) ) { return retval; }
            // JFSL.g:817:3: ( setUnaryOp | '#' -> OP_SET_NUM | 'all' -> OP_SET_ALL | 'exists' -> OP_SET_EXISTS )
            int alt93=4;
            switch ( input.LA(1) ) {
            case 207:
            case 208:
            case 209:
            case 210:
            case 211:
                {
                alt93=1;
                }
                break;
            case 193:
                {
                alt93=2;
                }
                break;
            case 212:
                {
                alt93=3;
                }
                break;
            case 213:
                {
                alt93=4;
                }
                break;
            default:
                if (state.backtracking>0) {state.failed=true; return retval;}
                NoViableAltException nvae =
                    new NoViableAltException("", 93, 0, input);

                throw nvae;
            }

            switch (alt93) {
                case 1 :
                    // JFSL.g:817:5: setUnaryOp
                    {
                    root_0 = (Node)adaptor.nil();

                    pushFollow(FOLLOW_setUnaryOp_in_setQuantOp4722);
                    setUnaryOp355=setUnaryOp();

                    state._fsp--;
                    if (state.failed) return retval;
                    if ( state.backtracking==0 ) adaptor.addChild(root_0, setUnaryOp355.getTree());

                    }
                    break;
                case 2 :
                    // JFSL.g:818:5: '#'
                    {
                    char_literal356=(Token)match(input,193,FOLLOW_193_in_setQuantOp4728); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_193.add(char_literal356);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 818:10: -> OP_SET_NUM
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_SET_NUM, "OP_SET_NUM"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 3 :
                    // JFSL.g:819:5: 'all'
                    {
                    string_literal357=(Token)match(input,212,FOLLOW_212_in_setQuantOp4739); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_212.add(string_literal357);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 819:13: -> OP_SET_ALL
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_SET_ALL, "OP_SET_ALL"));

                    }

                    retval.tree = root_0;}
                    }
                    break;
                case 4 :
                    // JFSL.g:820:5: 'exists'
                    {
                    string_literal358=(Token)match(input,213,FOLLOW_213_in_setQuantOp4752); if (state.failed) return retval; 
                    if ( state.backtracking==0 ) stream_213.add(string_literal358);



                    // AST REWRITE
                    // elements: 
                    // token labels: 
                    // rule labels: retval
                    // token list labels: 
                    // rule list labels: 
                    // wildcard labels: 
                    if ( state.backtracking==0 ) {
                    retval.tree = root_0;
                    RewriteRuleSubtreeStream stream_retval=new RewriteRuleSubtreeStream(adaptor,"rule retval",retval!=null?retval.tree:null);

                    root_0 = (Node)adaptor.nil();
                    // 820:14: -> OP_SET_EXISTS
                    {
                        adaptor.addChild(root_0, (Node)adaptor.create(OP_SET_EXISTS, "OP_SET_EXISTS"));

                    }

                    retval.tree = root_0;}
                    }
                    break;

            }
            retval.stop = input.LT(-1);

            if ( state.backtracking==0 ) {

            retval.tree = (Node)adaptor.rulePostProcessing(root_0);
            adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop);
            }
        }
        catch (RecognitionException re) {
            reportError(re);
            recover(input,re);
    	retval.tree = (Node)adaptor.errorNode(input, retval.start, input.LT(-1), re);

        }
        finally {
            if ( state.backtracking>0 ) { memoize(input, 75, setQuantOp_StartIndex); }
        }
        return retval;
    }
    // $ANTLR end "setQuantOp"

    // $ANTLR start synpred13_JFSL
    public final void synpred13_JFSL_fragment() throws RecognitionException {   
        // JFSL.g:436:55: ( ';' specCase )
        // JFSL.g:436:55: ';' specCase
        {
        match(input,139,FOLLOW_139_in_synpred13_JFSL1120); if (state.failed) return ;
        pushFollow(FOLLOW_specCase_in_synpred13_JFSL1122);
        specCase();

        state._fsp--;
        if (state.failed) return ;

        }
    }
    // $ANTLR end synpred13_JFSL

    // $ANTLR start synpred26_JFSL
    public final void synpred26_JFSL_fragment() throws RecognitionException {   
        // JFSL.g:456:5: ( ( 'public' | 'private' )? Identifier ':' declarationMult additiveExpression 'from' frame ( '|' expression )? )
        // JFSL.g:456:5: ( 'public' | 'private' )? Identifier ':' declarationMult additiveExpression 'from' frame ( '|' expression )?
        {
        // JFSL.g:456:5: ( 'public' | 'private' )?
        int alt98=2;
        int LA98_0 = input.LA(1);

        if ( ((LA98_0>=159 && LA98_0<=160)) ) {
            alt98=1;
        }
        switch (alt98) {
            case 1 :
                // JFSL.g:
                {
                if ( (input.LA(1)>=159 && input.LA(1)<=160) ) {
                    input.consume();
                    state.errorRecovery=false;state.failed=false;
                }
                else {
                    if (state.backtracking>0) {state.failed=true; return ;}
                    MismatchedSetException mse = new MismatchedSetException(null,input);
                    throw mse;
                }


                }
                break;

        }

        match(input,Identifier,FOLLOW_Identifier_in_synpred26_JFSL1379); if (state.failed) return ;
        match(input,161,FOLLOW_161_in_synpred26_JFSL1381); if (state.failed) return ;
        pushFollow(FOLLOW_declarationMult_in_synpred26_JFSL1383);
        declarationMult();

        state._fsp--;
        if (state.failed) return ;
        pushFollow(FOLLOW_additiveExpression_in_synpred26_JFSL1385);
        additiveExpression();

        state._fsp--;
        if (state.failed) return ;
        match(input,162,FOLLOW_162_in_synpred26_JFSL1387); if (state.failed) return ;
        pushFollow(FOLLOW_frame_in_synpred26_JFSL1389);
        frame();

        state._fsp--;
        if (state.failed) return ;
        // JFSL.g:456:92: ( '|' expression )?
        int alt99=2;
        int LA99_0 = input.LA(1);

        if ( (LA99_0==163) ) {
            alt99=1;
        }
        switch (alt99) {
            case 1 :
                // JFSL.g:456:93: '|' expression
                {
                match(input,163,FOLLOW_163_in_synpred26_JFSL1392); if (state.failed) return ;
                pushFollow(FOLLOW_expression_in_synpred26_JFSL1394);
                expression();

                state._fsp--;
                if (state.failed) return ;

                }
                break;

        }


        }
    }
    // $ANTLR end synpred26_JFSL

    // $ANTLR start synpred36_JFSL
    public final void synpred36_JFSL_fragment() throws RecognitionException {   
        JFSLParser.expression_return ff = null;


        // JFSL.g:479:34: ( '|' ff= expression )
        // JFSL.g:479:34: '|' ff= expression
        {
        match(input,163,FOLLOW_163_in_synpred36_JFSL1594); if (state.failed) return ;
        pushFollow(FOLLOW_expression_in_synpred36_JFSL1598);
        ff=expression();

        state._fsp--;
        if (state.failed) return ;

        }
    }
    // $ANTLR end synpred36_JFSL

    // $ANTLR start synpred40_JFSL
    public final void synpred40_JFSL_fragment() throws RecognitionException {   
        // JFSL.g:489:5: ( ( selector )* storeWildCard )
        // JFSL.g:489:5: ( selector )* storeWildCard
        {
        // JFSL.g:489:5: ( selector )*
        loop100:
        do {
            int alt100=2;
            int LA100_0 = input.LA(1);

            if ( (LA100_0==140) ) {
                int LA100_1 = input.LA(2);

                if ( (LA100_1==167) ) {
                    int LA100_3 = input.LA(3);

                    if ( (LA100_3==Identifier||LA100_3==144||LA100_3==150||LA100_3==186||LA100_3==196||(LA100_3>=199 && LA100_3<=206)) ) {
                        alt100=1;
                    }


                }
                else if ( (LA100_1==Identifier||LA100_1==144||LA100_1==150||(LA100_1>=171 && LA100_1<=178)||LA100_1==186||LA100_1==196||(LA100_1>=199 && LA100_1<=206)) ) {
                    alt100=1;
                }


            }
            else if ( (LA100_0==146||LA100_0==179) ) {
                alt100=1;
            }


            switch (alt100) {
        	case 1 :
        	    // JFSL.g:0:0: selector
        	    {
        	    pushFollow(FOLLOW_selector_in_synpred40_JFSL1690);
        	    selector();

        	    state._fsp--;
        	    if (state.failed) return ;

        	    }
        	    break;

        	default :
        	    break loop100;
            }
        } while (true);

        pushFollow(FOLLOW_storeWildCard_in_synpred40_JFSL1693);
        storeWildCard();

        state._fsp--;
        if (state.failed) return ;

        }
    }
    // $ANTLR end synpred40_JFSL

    // $ANTLR start synpred59_JFSL
    public final void synpred59_JFSL_fragment() throws RecognitionException {   
        // JFSL.g:533:39: ( ( '[' ']' ) )
        // JFSL.g:533:39: ( '[' ']' )
        {
        // JFSL.g:533:39: ( '[' ']' )
        // JFSL.g:533:40: '[' ']'
        {
        match(input,179,FOLLOW_179_in_synpred59_JFSL2015); if (state.failed) return ;
        match(input,180,FOLLOW_180_in_synpred59_JFSL2017); if (state.failed) return ;

        }


        }
    }
    // $ANTLR end synpred59_JFSL

    // $ANTLR start synpred61_JFSL
    public final void synpred61_JFSL_fragment() throws RecognitionException {   
        // JFSL.g:534:29: ( ( '[' ']' ) )
        // JFSL.g:534:29: ( '[' ']' )
        {
        // JFSL.g:534:29: ( '[' ']' )
        // JFSL.g:534:30: '[' ']'
        {
        match(input,179,FOLLOW_179_in_synpred61_JFSL2045); if (state.failed) return ;
        match(input,180,FOLLOW_180_in_synpred61_JFSL2047); if (state.failed) return ;

        }


        }
    }
    // $ANTLR end synpred61_JFSL

    // $ANTLR start synpred65_JFSL
    public final void synpred65_JFSL_fragment() throws RecognitionException {   
        // JFSL.g:564:7: ( '?' expression ':' expression )
        // JFSL.g:564:7: '?' expression ':' expression
        {
        match(input,181,FOLLOW_181_in_synpred65_JFSL2233); if (state.failed) return ;
        pushFollow(FOLLOW_expression_in_synpred65_JFSL2235);
        expression();

        state._fsp--;
        if (state.failed) return ;
        match(input,161,FOLLOW_161_in_synpred65_JFSL2237); if (state.failed) return ;
        pushFollow(FOLLOW_expression_in_synpred65_JFSL2239);
        expression();

        state._fsp--;
        if (state.failed) return ;

        }
    }
    // $ANTLR end synpred65_JFSL

    // $ANTLR start synpred66_JFSL
    public final void synpred66_JFSL_fragment() throws RecognitionException {   
        // JFSL.g:569:5: ( setQuantOp decls '|' expression )
        // JFSL.g:569:5: setQuantOp decls '|' expression
        {
        pushFollow(FOLLOW_setQuantOp_in_synpred66_JFSL2275);
        setQuantOp();

        state._fsp--;
        if (state.failed) return ;
        pushFollow(FOLLOW_decls_in_synpred66_JFSL2277);
        decls();

        state._fsp--;
        if (state.failed) return ;
        match(input,163,FOLLOW_163_in_synpred66_JFSL2280); if (state.failed) return ;
        pushFollow(FOLLOW_expression_in_synpred66_JFSL2282);
        expression();

        state._fsp--;
        if (state.failed) return ;

        }
    }
    // $ANTLR end synpred66_JFSL

    // $ANTLR start synpred67_JFSL
    public final void synpred67_JFSL_fragment() throws RecognitionException {   
        // JFSL.g:570:5: ( setQuantOp mdecls '|' expression )
        // JFSL.g:570:5: setQuantOp mdecls '|' expression
        {
        pushFollow(FOLLOW_setQuantOp_in_synpred67_JFSL2300);
        setQuantOp();

        state._fsp--;
        if (state.failed) return ;
        pushFollow(FOLLOW_mdecls_in_synpred67_JFSL2302);
        mdecls();

        state._fsp--;
        if (state.failed) return ;
        match(input,163,FOLLOW_163_in_synpred67_JFSL2304); if (state.failed) return ;
        pushFollow(FOLLOW_expression_in_synpred67_JFSL2306);
        expression();

        state._fsp--;
        if (state.failed) return ;

        }
    }
    // $ANTLR end synpred67_JFSL

    // $ANTLR start synpred73_JFSL
    public final void synpred73_JFSL_fragment() throws RecognitionException {   
        JFSLParser.exclusiveOrExpression_return a = null;


        // JFSL.g:598:9: ( '|' a= exclusiveOrExpression )
        // JFSL.g:598:9: '|' a= exclusiveOrExpression
        {
        match(input,163,FOLLOW_163_in_synpred73_JFSL2570); if (state.failed) return ;
        pushFollow(FOLLOW_exclusiveOrExpression_in_synpred73_JFSL2574);
        a=exclusiveOrExpression();

        state._fsp--;
        if (state.failed) return ;

        }
    }
    // $ANTLR end synpred73_JFSL

    // $ANTLR start synpred104_JFSL
    public final void synpred104_JFSL_fragment() throws RecognitionException {   
        // JFSL.g:733:5: ( '~' unaryExpression )
        // JFSL.g:733:5: '~' unaryExpression
        {
        match(input,199,FOLLOW_199_in_synpred104_JFSL3811); if (state.failed) return ;
        pushFollow(FOLLOW_unaryExpression_in_synpred104_JFSL3813);
        unaryExpression();

        state._fsp--;
        if (state.failed) return ;

        }
    }
    // $ANTLR end synpred104_JFSL

    // $ANTLR start synpred106_JFSL
    public final void synpred106_JFSL_fragment() throws RecognitionException {   
        // JFSL.g:735:5: ( castExpression )
        // JFSL.g:735:5: castExpression
        {
        pushFollow(FOLLOW_castExpression_in_synpred106_JFSL3851);
        castExpression();

        state._fsp--;
        if (state.failed) return ;

        }
    }
    // $ANTLR end synpred106_JFSL

    // $ANTLR start synpred113_JFSL
    public final void synpred113_JFSL_fragment() throws RecognitionException {   
        // JFSL.g:760:15: ( selector )
        // JFSL.g:760:15: selector
        {
        pushFollow(FOLLOW_selector_in_synpred113_JFSL4098);
        selector();

        state._fsp--;
        if (state.failed) return ;

        }
    }
    // $ANTLR end synpred113_JFSL

    // $ANTLR start synpred127_JFSL
    public final void synpred127_JFSL_fragment() throws RecognitionException {   
        // JFSL.g:780:5: ( typeDisambiguous )
        // JFSL.g:780:5: typeDisambiguous
        {
        pushFollow(FOLLOW_typeDisambiguous_in_synpred127_JFSL4311);
        typeDisambiguous();

        state._fsp--;
        if (state.failed) return ;

        }
    }
    // $ANTLR end synpred127_JFSL

    // $ANTLR start synpred128_JFSL
    public final void synpred128_JFSL_fragment() throws RecognitionException {   
        // JFSL.g:781:17: ( '.' Identifier )
        // JFSL.g:781:17: '.' Identifier
        {
        match(input,140,FOLLOW_140_in_synpred128_JFSL4320); if (state.failed) return ;
        match(input,Identifier,FOLLOW_Identifier_in_synpred128_JFSL4322); if (state.failed) return ;

        }
    }
    // $ANTLR end synpred128_JFSL

    // $ANTLR start synpred129_JFSL
    public final void synpred129_JFSL_fragment() throws RecognitionException {   
        // JFSL.g:781:34: ( typeParameters2 )
        // JFSL.g:781:34: typeParameters2
        {
        pushFollow(FOLLOW_typeParameters2_in_synpred129_JFSL4326);
        typeParameters2();

        state._fsp--;
        if (state.failed) return ;

        }
    }
    // $ANTLR end synpred129_JFSL

    // $ANTLR start synpred130_JFSL
    public final void synpred130_JFSL_fragment() throws RecognitionException {   
        // JFSL.g:781:5: ( Identifier ( '.' Identifier )* ( typeParameters2 )? )
        // JFSL.g:781:5: Identifier ( '.' Identifier )* ( typeParameters2 )?
        {
        match(input,Identifier,FOLLOW_Identifier_in_synpred130_JFSL4317); if (state.failed) return ;
        // JFSL.g:781:16: ( '.' Identifier )*
        loop103:
        do {
            int alt103=2;
            int LA103_0 = input.LA(1);

            if ( (LA103_0==140) ) {
                alt103=1;
            }


            switch (alt103) {
        	case 1 :
        	    // JFSL.g:781:17: '.' Identifier
        	    {
        	    match(input,140,FOLLOW_140_in_synpred130_JFSL4320); if (state.failed) return ;
        	    match(input,Identifier,FOLLOW_Identifier_in_synpred130_JFSL4322); if (state.failed) return ;

        	    }
        	    break;

        	default :
        	    break loop103;
            }
        } while (true);

        // JFSL.g:781:34: ( typeParameters2 )?
        int alt104=2;
        int LA104_0 = input.LA(1);

        if ( (LA104_0==146) ) {
            alt104=1;
        }
        switch (alt104) {
            case 1 :
                // JFSL.g:0:0: typeParameters2
                {
                pushFollow(FOLLOW_typeParameters2_in_synpred130_JFSL4326);
                typeParameters2();

                state._fsp--;
                if (state.failed) return ;

                }
                break;

        }


        }
    }
    // $ANTLR end synpred130_JFSL

    // Delegated rules

    public final boolean synpred128_JFSL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred128_JFSL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred59_JFSL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred59_JFSL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred66_JFSL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred66_JFSL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred65_JFSL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred65_JFSL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred40_JFSL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred40_JFSL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred129_JFSL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred129_JFSL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred106_JFSL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred106_JFSL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred113_JFSL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred113_JFSL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred73_JFSL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred73_JFSL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred13_JFSL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred13_JFSL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred26_JFSL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred26_JFSL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred104_JFSL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred104_JFSL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred61_JFSL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred61_JFSL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred67_JFSL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred67_JFSL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred130_JFSL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred130_JFSL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred36_JFSL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred36_JFSL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }
    public final boolean synpred127_JFSL() {
        state.backtracking++;
        int start = input.mark();
        try {
            synpred127_JFSL_fragment(); // can never throw exception
        } catch (RecognitionException re) {
            System.err.println("impossible: "+re);
        }
        boolean success = !state.failed;
        input.rewind(start);
        state.backtracking--;
        state.failed=false;
        return success;
    }


    protected DFA41 dfa41 = new DFA41(this);
    protected DFA45 dfa45 = new DFA45(this);
    protected DFA50 dfa50 = new DFA50(this);
    protected DFA73 dfa73 = new DFA73(this);
    protected DFA80 dfa80 = new DFA80(this);
    protected DFA82 dfa82 = new DFA82(this);
    protected DFA85 dfa85 = new DFA85(this);
    protected DFA84 dfa84 = new DFA84(this);
    protected DFA87 dfa87 = new DFA87(this);
    static final String DFA41_eotS =
        "\13\uffff";
    static final String DFA41_eofS =
        "\2\uffff\1\5\3\uffff\1\5\2\uffff\1\5\1\uffff";
    static final String DFA41_minS =
        "\1\171\1\uffff\3\171\1\uffff\1\171\1\u0093\2\171\1\u0093";
    static final String DFA41_maxS =
        "\1\u00b2\1\uffff\1\u00bb\1\171\1\u00b7\1\uffff\1\u00bb\1\u0094\1"+
        "\171\1\u00bb\1\u0094";
    static final String DFA41_acceptS =
        "\1\uffff\1\1\3\uffff\1\2\5\uffff";
    static final String DFA41_specialS =
        "\13\uffff}>";
    static final String[] DFA41_transitionS = {
            "\1\2\61\uffff\10\1",
            "",
            "\1\5\22\uffff\1\3\4\uffff\1\5\1\4\1\5\3\uffff\1\5\11\uffff"+
            "\3\5\17\uffff\1\1\10\5",
            "\1\6",
            "\1\7\74\uffff\2\5",
            "",
            "\1\5\22\uffff\1\3\4\uffff\1\5\1\4\1\5\3\uffff\1\5\11\uffff"+
            "\3\5\17\uffff\1\1\10\5",
            "\1\10\1\11",
            "\1\12",
            "\1\5\27\uffff\3\5\3\uffff\1\5\11\uffff\3\5\17\uffff\1\1\10"+
            "\5",
            "\1\10\1\11"
    };

    static final short[] DFA41_eot = DFA.unpackEncodedString(DFA41_eotS);
    static final short[] DFA41_eof = DFA.unpackEncodedString(DFA41_eofS);
    static final char[] DFA41_min = DFA.unpackEncodedStringToUnsignedChars(DFA41_minS);
    static final char[] DFA41_max = DFA.unpackEncodedStringToUnsignedChars(DFA41_maxS);
    static final short[] DFA41_accept = DFA.unpackEncodedString(DFA41_acceptS);
    static final short[] DFA41_special = DFA.unpackEncodedString(DFA41_specialS);
    static final short[][] DFA41_transition;

    static {
        int numStates = DFA41_transitionS.length;
        DFA41_transition = new short[numStates][];
        for (int i=0; i<numStates; i++) {
            DFA41_transition[i] = DFA.unpackEncodedString(DFA41_transitionS[i]);
        }
    }

    class DFA41 extends DFA {

        public DFA41(BaseRecognizer recognizer) {
            this.recognizer = recognizer;
            this.decisionNumber = 41;
            this.eot = DFA41_eot;
            this.eof = DFA41_eof;
            this.min = DFA41_min;
            this.max = DFA41_max;
            this.accept = DFA41_accept;
            this.special = DFA41_special;
            this.transition = DFA41_transition;
        }
        public String getDescription() {
            return "537:1: type : ( typeDisambiguous | typeName );";
        }
    }
    static final String DFA45_eotS =
        "\52\uffff";
    static final String DFA45_eofS =
        "\52\uffff";
    static final String DFA45_minS =
        "\1\171\10\0\41\uffff";
    static final String DFA45_maxS =
        "\1\u00d5\10\0\41\uffff";
    static final String DFA45_acceptS =
        "\11\uffff\1\3\36\uffff\1\1\1\2";
    static final String DFA45_specialS =
        "\1\uffff\1\0\1\1\1\2\1\3\1\4\1\5\1\6\1\7\41\uffff}>";
    static final String[] DFA45_transitionS = {
            "\7\11\20\uffff\1\11\5\uffff\1\11\21\uffff\13\11\4\uffff\1\11"+
            "\2\uffff\1\11\4\uffff\2\11\1\6\2\uffff\1\11\2\uffff\10\11\1"+
            "\1\1\2\1\3\1\4\1\5\1\7\1\10",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            ""
    };

    static final short[] DFA45_eot = DFA.unpackEncodedString(DFA45_eotS);
    static final short[] DFA45_eof = DFA.unpackEncodedString(DFA45_eofS);
    static final char[] DFA45_min = DFA.unpackEncodedStringToUnsignedChars(DFA45_minS);
    static final char[] DFA45_max = DFA.unpackEncodedStringToUnsignedChars(DFA45_maxS);
    static final short[] DFA45_accept = DFA.unpackEncodedString(DFA45_acceptS);
    static final short[] DFA45_special = DFA.unpackEncodedString(DFA45_specialS);
    static final short[][] DFA45_transition;

    static {
        int numStates = DFA45_transitionS.length;
        DFA45_transition = new short[numStates][];
        for (int i=0; i<numStates; i++) {
            DFA45_transition[i] = DFA.unpackEncodedString(DFA45_transitionS[i]);
        }
    }

    class DFA45 extends DFA {

        public DFA45(BaseRecognizer recognizer) {
            this.recognizer = recognizer;
            this.decisionNumber = 45;
            this.eot = DFA45_eot;
            this.eof = DFA45_eof;
            this.min = DFA45_min;
            this.max = DFA45_max;
            this.accept = DFA45_accept;
            this.special = DFA45_special;
            this.transition = DFA45_transition;
        }
        public String getDescription() {
            return "568:1: quantifiedExpression : ( setQuantOp decls '|' expression -> ^( QUANTIFY setQuantOp decls expression ) | setQuantOp mdecls '|' expression -> ^( QUANTIFY setQuantOp mdecls expression ) | logicalExpression );";
        }
        public int specialStateTransition(int s, IntStream _input) throws NoViableAltException {
            TokenStream input = (TokenStream)_input;
        	int _s = s;
            switch ( s ) {
                    case 0 : 
                        int LA45_1 = input.LA(1);

                         
                        int index45_1 = input.index();
                        input.rewind();
                        s = -1;
                        if ( (synpred66_JFSL()) ) {s = 40;}

                        else if ( (synpred67_JFSL()) ) {s = 41;}

                        else if ( (true) ) {s = 9;}

                         
                        input.seek(index45_1);
                        if ( s>=0 ) return s;
                        break;
                    case 1 : 
                        int LA45_2 = input.LA(1);

                         
                        int index45_2 = input.index();
                        input.rewind();
                        s = -1;
                        if ( (synpred66_JFSL()) ) {s = 40;}

                        else if ( (synpred67_JFSL()) ) {s = 41;}

                        else if ( (true) ) {s = 9;}

                         
                        input.seek(index45_2);
                        if ( s>=0 ) return s;
                        break;
                    case 2 : 
                        int LA45_3 = input.LA(1);

                         
                        int index45_3 = input.index();
                        input.rewind();
                        s = -1;
                        if ( (synpred66_JFSL()) ) {s = 40;}

                        else if ( (synpred67_JFSL()) ) {s = 41;}

                        else if ( (true) ) {s = 9;}

                         
                        input.seek(index45_3);
                        if ( s>=0 ) return s;
                        break;
                    case 3 : 
                        int LA45_4 = input.LA(1);

                         
                        int index45_4 = input.index();
                        input.rewind();
                        s = -1;
                        if ( (synpred66_JFSL()) ) {s = 40;}

                        else if ( (synpred67_JFSL()) ) {s = 41;}

                        else if ( (true) ) {s = 9;}

                         
                        input.seek(index45_4);
                        if ( s>=0 ) return s;
                        break;
                    case 4 : 
                        int LA45_5 = input.LA(1);

                         
                        int index45_5 = input.index();
                        input.rewind();
                        s = -1;
                        if ( (synpred66_JFSL()) ) {s = 40;}

                        else if ( (synpred67_JFSL()) ) {s = 41;}

                        else if ( (true) ) {s = 9;}

                         
                        input.seek(index45_5);
                        if ( s>=0 ) return s;
                        break;
                    case 5 : 
                        int LA45_6 = input.LA(1);

                         
                        int index45_6 = input.index();
                        input.rewind();
                        s = -1;
                        if ( (synpred66_JFSL()) ) {s = 40;}

                        else if ( (synpred67_JFSL()) ) {s = 41;}

                        else if ( (true) ) {s = 9;}

                         
                        input.seek(index45_6);
                        if ( s>=0 ) return s;
                        break;
                    case 6 : 
                        int LA45_7 = input.LA(1);

                         
                        int index45_7 = input.index();
                        input.rewind();
                        s = -1;
                        if ( (synpred66_JFSL()) ) {s = 40;}

                        else if ( (synpred67_JFSL()) ) {s = 41;}

                         
                        input.seek(index45_7);
                        if ( s>=0 ) return s;
                        break;
                    case 7 : 
                        int LA45_8 = input.LA(1);

                         
                        int index45_8 = input.index();
                        input.rewind();
                        s = -1;
                        if ( (synpred66_JFSL()) ) {s = 40;}

                        else if ( (synpred67_JFSL()) ) {s = 41;}

                         
                        input.seek(index45_8);
                        if ( s>=0 ) return s;
                        break;
            }
            if (state.backtracking>0) {state.failed=true; return -1;}
            NoViableAltException nvae =
                new NoViableAltException(getDescription(), 45, _s, input);
            error(nvae);
            throw nvae;
        }
    }
    static final String DFA50_eotS =
        "\17\uffff";
    static final String DFA50_eofS =
        "\1\1\16\uffff";
    static final String DFA50_minS =
        "\1\u0091\11\uffff\1\0\4\uffff";
    static final String DFA50_maxS =
        "\1\u00b9\11\uffff\1\0\4\uffff";
    static final String DFA50_acceptS =
        "\1\uffff\1\2\14\uffff\1\1";
    static final String DFA50_specialS =
        "\12\uffff\1\0\4\uffff}>";
    static final String[] DFA50_transitionS = {
            "\3\1\3\uffff\1\1\11\uffff\2\1\1\12\20\uffff\3\1\1\uffff\2\1",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\uffff",
            "",
            "",
            "",
            ""
    };

    static final short[] DFA50_eot = DFA.unpackEncodedString(DFA50_eotS);
    static final short[] DFA50_eof = DFA.unpackEncodedString(DFA50_eofS);
    static final char[] DFA50_min = DFA.unpackEncodedStringToUnsignedChars(DFA50_minS);
    static final char[] DFA50_max = DFA.unpackEncodedStringToUnsignedChars(DFA50_maxS);
    static final short[] DFA50_accept = DFA.unpackEncodedString(DFA50_acceptS);
    static final short[] DFA50_special = DFA.unpackEncodedString(DFA50_specialS);
    static final short[][] DFA50_transition;

    static {
        int numStates = DFA50_transitionS.length;
        DFA50_transition = new short[numStates][];
        for (int i=0; i<numStates; i++) {
            DFA50_transition[i] = DFA.unpackEncodedString(DFA50_transitionS[i]);
        }
    }

    class DFA50 extends DFA {

        public DFA50(BaseRecognizer recognizer) {
            this.recognizer = recognizer;
            this.decisionNumber = 50;
            this.eot = DFA50_eot;
            this.eof = DFA50_eof;
            this.min = DFA50_min;
            this.max = DFA50_max;
            this.accept = DFA50_accept;
            this.special = DFA50_special;
            this.transition = DFA50_transition;
        }
        public String getDescription() {
            return "()* loopback of 598:7: ( '|' a= exclusiveOrExpression -> ^( BINARY OP_BIT_OR $inclusiveOrExpression $a) )*";
        }
        public int specialStateTransition(int s, IntStream _input) throws NoViableAltException {
            TokenStream input = (TokenStream)_input;
        	int _s = s;
            switch ( s ) {
                    case 0 : 
                        int LA50_10 = input.LA(1);

                         
                        int index50_10 = input.index();
                        input.rewind();
                        s = -1;
                        if ( (synpred73_JFSL()) ) {s = 14;}

                        else if ( (true) ) {s = 1;}

                         
                        input.seek(index50_10);
                        if ( s>=0 ) return s;
                        break;
            }
            if (state.backtracking>0) {state.failed=true; return -1;}
            NoViableAltException nvae =
                new NoViableAltException(getDescription(), 50, _s, input);
            error(nvae);
            throw nvae;
        }
    }
    static final String DFA73_eotS =
        "\40\uffff";
    static final String DFA73_eofS =
        "\40\uffff";
    static final String DFA73_minS =
        "\1\171\1\0\1\uffff\1\0\34\uffff";
    static final String DFA73_maxS =
        "\1\u00ce\1\0\1\uffff\1\0\34\uffff";
    static final String DFA73_acceptS =
        "\2\uffff\1\2\1\uffff\1\4\31\uffff\1\1\1\3";
    static final String DFA73_specialS =
        "\1\uffff\1\0\1\uffff\1\1\34\uffff}>";
    static final String[] DFA73_transitionS = {
            "\7\4\20\uffff\1\4\5\uffff\1\3\21\uffff\13\4\4\uffff\1\2\2\uffff"+
            "\1\4\11\uffff\1\4\2\uffff\1\1\7\4",
            "\1\uffff",
            "",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            ""
    };

    static final short[] DFA73_eot = DFA.unpackEncodedString(DFA73_eotS);
    static final short[] DFA73_eof = DFA.unpackEncodedString(DFA73_eofS);
    static final char[] DFA73_min = DFA.unpackEncodedStringToUnsignedChars(DFA73_minS);
    static final char[] DFA73_max = DFA.unpackEncodedStringToUnsignedChars(DFA73_maxS);
    static final short[] DFA73_accept = DFA.unpackEncodedString(DFA73_acceptS);
    static final short[] DFA73_special = DFA.unpackEncodedString(DFA73_specialS);
    static final short[][] DFA73_transition;

    static {
        int numStates = DFA73_transitionS.length;
        DFA73_transition = new short[numStates][];
        for (int i=0; i<numStates; i++) {
            DFA73_transition[i] = DFA.unpackEncodedString(DFA73_transitionS[i]);
        }
    }

    class DFA73 extends DFA {

        public DFA73(BaseRecognizer recognizer) {
            this.recognizer = recognizer;
            this.decisionNumber = 73;
            this.eot = DFA73_eot;
            this.eof = DFA73_eof;
            this.min = DFA73_min;
            this.max = DFA73_max;
            this.accept = DFA73_accept;
            this.special = DFA73_special;
            this.transition = DFA73_transition;
        }
        public String getDescription() {
            return "732:1: unaryExpressionNotPlusMinus : ( '~' unaryExpression -> ^( UNARY OP_BIT_NOT_OR_TRANSPOSE unaryExpression ) | '!' unaryExpression -> ^( UNARY OP_NOT unaryExpression ) | castExpression | joinExpression );";
        }
        public int specialStateTransition(int s, IntStream _input) throws NoViableAltException {
            TokenStream input = (TokenStream)_input;
        	int _s = s;
            switch ( s ) {
                    case 0 : 
                        int LA73_1 = input.LA(1);

                         
                        int index73_1 = input.index();
                        input.rewind();
                        s = -1;
                        if ( (synpred104_JFSL()) ) {s = 30;}

                        else if ( (true) ) {s = 4;}

                         
                        input.seek(index73_1);
                        if ( s>=0 ) return s;
                        break;
                    case 1 : 
                        int LA73_3 = input.LA(1);

                         
                        int index73_3 = input.index();
                        input.rewind();
                        s = -1;
                        if ( (synpred106_JFSL()) ) {s = 31;}

                        else if ( (true) ) {s = 4;}

                         
                        input.seek(index73_3);
                        if ( s>=0 ) return s;
                        break;
            }
            if (state.backtracking>0) {state.failed=true; return -1;}
            NoViableAltException nvae =
                new NoViableAltException(getDescription(), 73, _s, input);
            error(nvae);
            throw nvae;
        }
    }
    static final String DFA80_eotS =
        "\37\uffff";
    static final String DFA80_eofS =
        "\1\2\36\uffff";
    static final String DFA80_minS =
        "\1\u008c\1\0\35\uffff";
    static final String DFA80_maxS =
        "\1\u00c6\1\0\35\uffff";
    static final String DFA80_acceptS =
        "\2\uffff\1\2\32\uffff\1\1\1\uffff";
    static final String DFA80_specialS =
        "\1\uffff\1\0\35\uffff}>";
    static final String[] DFA80_transitionS = {
            "\1\35\4\uffff\1\2\1\1\2\2\2\uffff\1\2\11\uffff\3\2\3\uffff\1"+
            "\2\13\uffff\1\35\15\2\1\uffff\5\2",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            ""
    };

    static final short[] DFA80_eot = DFA.unpackEncodedString(DFA80_eotS);
    static final short[] DFA80_eof = DFA.unpackEncodedString(DFA80_eofS);
    static final char[] DFA80_min = DFA.unpackEncodedStringToUnsignedChars(DFA80_minS);
    static final char[] DFA80_max = DFA.unpackEncodedStringToUnsignedChars(DFA80_maxS);
    static final short[] DFA80_accept = DFA.unpackEncodedString(DFA80_acceptS);
    static final short[] DFA80_special = DFA.unpackEncodedString(DFA80_specialS);
    static final short[][] DFA80_transition;

    static {
        int numStates = DFA80_transitionS.length;
        DFA80_transition = new short[numStates][];
        for (int i=0; i<numStates; i++) {
            DFA80_transition[i] = DFA.unpackEncodedString(DFA80_transitionS[i]);
        }
    }

    class DFA80 extends DFA {

        public DFA80(BaseRecognizer recognizer) {
            this.recognizer = recognizer;
            this.decisionNumber = 80;
            this.eot = DFA80_eot;
            this.eof = DFA80_eof;
            this.min = DFA80_min;
            this.max = DFA80_max;
            this.accept = DFA80_accept;
            this.special = DFA80_special;
            this.transition = DFA80_transition;
        }
        public String getDescription() {
            return "()* loopback of 760:15: ( selector )*";
        }
        public int specialStateTransition(int s, IntStream _input) throws NoViableAltException {
            TokenStream input = (TokenStream)_input;
        	int _s = s;
            switch ( s ) {
                    case 0 : 
                        int LA80_1 = input.LA(1);

                         
                        int index80_1 = input.index();
                        input.rewind();
                        s = -1;
                        if ( (synpred113_JFSL()) ) {s = 29;}

                        else if ( (true) ) {s = 2;}

                         
                        input.seek(index80_1);
                        if ( s>=0 ) return s;
                        break;
            }
            if (state.backtracking>0) {state.failed=true; return -1;}
            NoViableAltException nvae =
                new NoViableAltException(getDescription(), 80, _s, input);
            error(nvae);
            throw nvae;
        }
    }
    static final String DFA82_eotS =
        "\16\uffff";
    static final String DFA82_eofS =
        "\16\uffff";
    static final String DFA82_minS =
        "\1\u0090\11\uffff\1\171\1\u008c\2\uffff";
    static final String DFA82_maxS =
        "\1\u00ce\11\uffff\1\u00ce\1\u00c6\2\uffff";
    static final String DFA82_acceptS =
        "\1\uffff\1\1\1\2\1\3\1\4\1\5\1\6\1\7\1\10\1\11\2\uffff\1\13\1\12";
    static final String DFA82_specialS =
        "\16\uffff}>";
    static final String[] DFA82_transitionS = {
            "\1\12\5\uffff\1\1\43\uffff\1\11\11\uffff\1\11\2\uffff\1\11\1"+
            "\2\1\3\1\4\1\5\1\6\1\7\1\10",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "\1\13\6\14\20\uffff\1\14\5\uffff\1\14\21\uffff\13\14\4\uffff"+
            "\1\14\2\uffff\1\14\4\uffff\3\14\2\uffff\1\14\2\uffff\10\14",
            "\1\14\4\uffff\3\14\2\uffff\1\14\12\uffff\1\15\5\uffff\1\14"+
            "\13\uffff\1\14\11\uffff\1\15\3\14\1\uffff\5\14",
            "",
            ""
    };

    static final short[] DFA82_eot = DFA.unpackEncodedString(DFA82_eotS);
    static final short[] DFA82_eof = DFA.unpackEncodedString(DFA82_eofS);
    static final char[] DFA82_min = DFA.unpackEncodedStringToUnsignedChars(DFA82_minS);
    static final char[] DFA82_max = DFA.unpackEncodedStringToUnsignedChars(DFA82_maxS);
    static final short[] DFA82_accept = DFA.unpackEncodedString(DFA82_acceptS);
    static final short[] DFA82_special = DFA.unpackEncodedString(DFA82_specialS);
    static final short[][] DFA82_transition;

    static {
        int numStates = DFA82_transitionS.length;
        DFA82_transition = new short[numStates][];
        for (int i=0; i<numStates; i++) {
            DFA82_transition[i] = DFA.unpackEncodedString(DFA82_transitionS[i]);
        }
    }

    class DFA82 extends DFA {

        public DFA82(BaseRecognizer recognizer) {
            this.recognizer = recognizer;
            this.decisionNumber = 82;
            this.eot = DFA82_eot;
            this.eof = DFA82_eof;
            this.min = DFA82_min;
            this.max = DFA82_max;
            this.accept = DFA82_accept;
            this.special = DFA82_special;
            this.transition = DFA82_transition;
        }
        public String getDescription() {
            return "763:1: common : ( parExpression | 'return' -> RETURN_VAR | 'throw' -> THROW_VAR | 'this' -> THIS_VAR | 'super' -> SUPER_VAR | '_' -> LAMBDA_VAR | '@old' '(' expression ')' -> ^( OLD expression ) | '@arg' '(' integerLiteral ')' -> ^( ARGUMENT integerLiteral ) | relationalUnaryExpression | '{' decls '|' expression '}' -> ^( QUANTIFY OP_SET_COMPREHENSION decls expression ) | '{' rangeExpression ( ',' rangeExpression )* '}' -> ^( OP_SET_COMPREHENSION_ENUM ( rangeExpression )+ ) );";
        }
    }
    static final String DFA85_eotS =
        "\27\uffff";
    static final String DFA85_eofS =
        "\4\uffff\1\10\6\uffff\2\10\3\uffff\2\10\1\uffff\1\10\2\uffff\1\10";
    static final String DFA85_minS =
        "\1\171\3\uffff\1\u008c\1\uffff\2\171\1\uffff\2\171\2\u008c\1\0\1"+
        "\uffff\2\171\1\u008c\4\171\1\u008c";
    static final String DFA85_maxS =
        "\1\u00ce\3\uffff\1\u00c6\1\uffff\1\u00ce\1\u00d3\1\uffff\1\u00d5"+
        "\1\u00c0\2\u00c6\1\0\1\uffff\1\u00d5\1\u00d3\1\u00c6\1\u00c7\1\u00d3"+
        "\1\u00d5\1\u00c7\1\u00c6";
    static final String DFA85_acceptS =
        "\1\uffff\1\1\1\2\1\3\1\uffff\1\6\2\uffff\1\4\5\uffff\1\5\10\uffff";
    static final String DFA85_specialS =
        "\15\uffff\1\0\11\uffff}>";
    static final String[] DFA85_transitionS = {
            "\1\4\6\2\20\uffff\1\1\5\uffff\1\1\21\uffff\3\2\10\3\7\uffff"+
            "\1\1\11\uffff\1\1\2\uffff\10\1",
            "",
            "",
            "",
            "\1\6\4\uffff\1\10\1\7\2\10\1\uffff\1\5\1\10\11\uffff\3\10\3"+
            "\uffff\1\10\13\uffff\1\11\15\10\1\uffff\2\10\1\12\2\10",
            "",
            "\1\13\26\uffff\1\10\5\uffff\1\10\20\uffff\1\10\3\uffff\10\10"+
            "\7\uffff\1\10\11\uffff\1\10\2\uffff\10\10",
            "\1\14\6\10\20\uffff\1\10\1\uffff\1\10\3\uffff\1\10\21\uffff"+
            "\13\10\3\uffff\2\10\2\uffff\1\10\4\uffff\3\10\2\uffff\1\10\2"+
            "\uffff\15\10",
            "",
            "\7\10\20\uffff\1\10\5\uffff\1\10\21\uffff\13\10\1\uffff\1\15"+
            "\2\uffff\1\10\2\uffff\1\10\4\uffff\3\10\2\uffff\1\10\2\uffff"+
            "\17\10",
            "\1\16\101\uffff\1\10\3\uffff\2\10",
            "\1\6\4\uffff\1\10\1\7\2\10\1\uffff\2\10\11\uffff\3\10\3\uffff"+
            "\1\10\13\uffff\1\11\15\10\1\uffff\2\10\1\12\2\10",
            "\1\10\4\uffff\2\10\1\17\1\20\1\uffff\2\10\11\uffff\3\10\3\uffff"+
            "\1\10\13\uffff\16\10\1\uffff\5\10",
            "\1\uffff",
            "",
            "\1\21\6\10\20\uffff\1\10\5\uffff\1\10\21\uffff\13\10\4\uffff"+
            "\1\10\2\uffff\1\10\4\uffff\3\10\2\uffff\1\10\2\uffff\17\10",
            "\7\10\14\uffff\1\10\3\uffff\5\10\1\uffff\2\10\11\uffff\3\10"+
            "\3\uffff\14\10\1\11\20\10\1\22\17\10",
            "\1\10\5\uffff\1\10\1\24\1\23\1\uffff\2\10\13\uffff\1\10\3\uffff"+
            "\1\10\13\uffff\1\10\1\uffff\14\10\1\uffff\5\10",
            "\1\16\100\uffff\2\10\3\uffff\2\10\6\uffff\1\10",
            "\7\10\14\uffff\1\10\3\uffff\5\10\1\uffff\2\10\11\uffff\3\10"+
            "\3\uffff\14\10\1\11\20\10\1\25\17\10",
            "\1\26\6\10\20\uffff\1\10\5\uffff\1\10\21\uffff\13\10\4\uffff"+
            "\1\10\2\uffff\1\10\4\uffff\3\10\2\uffff\1\10\2\uffff\17\10",
            "\1\16\100\uffff\2\10\3\uffff\2\10\6\uffff\1\10",
            "\1\10\5\uffff\1\10\1\24\1\23\1\uffff\2\10\13\uffff\1\10\3\uffff"+
            "\1\10\13\uffff\1\10\1\uffff\14\10\1\uffff\5\10"
    };

    static final short[] DFA85_eot = DFA.unpackEncodedString(DFA85_eotS);
    static final short[] DFA85_eof = DFA.unpackEncodedString(DFA85_eofS);
    static final char[] DFA85_min = DFA.unpackEncodedStringToUnsignedChars(DFA85_minS);
    static final char[] DFA85_max = DFA.unpackEncodedStringToUnsignedChars(DFA85_maxS);
    static final short[] DFA85_accept = DFA.unpackEncodedString(DFA85_acceptS);
    static final short[] DFA85_special = DFA.unpackEncodedString(DFA85_specialS);
    static final short[][] DFA85_transition;

    static {
        int numStates = DFA85_transitionS.length;
        DFA85_transition = new short[numStates][];
        for (int i=0; i<numStates; i++) {
            DFA85_transition[i] = DFA.unpackEncodedString(DFA85_transitionS[i]);
        }
    }

    class DFA85 extends DFA {

        public DFA85(BaseRecognizer recognizer) {
            this.recognizer = recognizer;
            this.decisionNumber = 85;
            this.eot = DFA85_eot;
            this.eof = DFA85_eof;
            this.min = DFA85_min;
            this.max = DFA85_max;
            this.accept = DFA85_accept;
            this.special = DFA85_special;
            this.transition = DFA85_transition;
        }
        public String getDescription() {
            return "777:1: primary : ( common | literal | typeDisambiguous | Identifier ( '.' Identifier )* ( typeParameters2 )? -> ^( AMBIGUOUS ( ^( IDENTIFIER Identifier ) )+ ( ^( TYPE_PARAMETERS typeParameters2 ) )? ) | typeName '@' Identifier -> ^( FIELD typeName ^( IDENTIFIER Identifier ) ) | Identifier arguments -> ^( CALL Identifier arguments ) );";
        }
        public int specialStateTransition(int s, IntStream _input) throws NoViableAltException {
            TokenStream input = (TokenStream)_input;
        	int _s = s;
            switch ( s ) {
                    case 0 : 
                        int LA85_13 = input.LA(1);

                         
                        int index85_13 = input.index();
                        input.rewind();
                        s = -1;
                        if ( (synpred127_JFSL()) ) {s = 3;}

                        else if ( (synpred130_JFSL()) ) {s = 8;}

                         
                        input.seek(index85_13);
                        if ( s>=0 ) return s;
                        break;
            }
            if (state.backtracking>0) {state.failed=true; return -1;}
            NoViableAltException nvae =
                new NoViableAltException(getDescription(), 85, _s, input);
            error(nvae);
            throw nvae;
        }
    }
    static final String DFA84_eotS =
        "\40\uffff";
    static final String DFA84_eofS =
        "\1\2\37\uffff";
    static final String DFA84_minS =
        "\1\u008c\1\0\36\uffff";
    static final String DFA84_maxS =
        "\1\u00c6\1\0\36\uffff";
    static final String DFA84_acceptS =
        "\2\uffff\1\2\34\uffff\1\1";
    static final String DFA84_specialS =
        "\1\uffff\1\0\36\uffff}>";
    static final String[] DFA84_transitionS = {
            "\1\2\4\uffff\1\2\1\1\2\2\2\uffff\1\2\11\uffff\3\2\3\uffff\1"+
            "\2\13\uffff\16\2\1\uffff\5\2",
            "\1\uffff",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            ""
    };

    static final short[] DFA84_eot = DFA.unpackEncodedString(DFA84_eotS);
    static final short[] DFA84_eof = DFA.unpackEncodedString(DFA84_eofS);
    static final char[] DFA84_min = DFA.unpackEncodedStringToUnsignedChars(DFA84_minS);
    static final char[] DFA84_max = DFA.unpackEncodedStringToUnsignedChars(DFA84_maxS);
    static final short[] DFA84_accept = DFA.unpackEncodedString(DFA84_acceptS);
    static final short[] DFA84_special = DFA.unpackEncodedString(DFA84_specialS);
    static final short[][] DFA84_transition;

    static {
        int numStates = DFA84_transitionS.length;
        DFA84_transition = new short[numStates][];
        for (int i=0; i<numStates; i++) {
            DFA84_transition[i] = DFA.unpackEncodedString(DFA84_transitionS[i]);
        }
    }

    class DFA84 extends DFA {

        public DFA84(BaseRecognizer recognizer) {
            this.recognizer = recognizer;
            this.decisionNumber = 84;
            this.eot = DFA84_eot;
            this.eof = DFA84_eof;
            this.min = DFA84_min;
            this.max = DFA84_max;
            this.accept = DFA84_accept;
            this.special = DFA84_special;
            this.transition = DFA84_transition;
        }
        public String getDescription() {
            return "781:34: ( typeParameters2 )?";
        }
        public int specialStateTransition(int s, IntStream _input) throws NoViableAltException {
            TokenStream input = (TokenStream)_input;
        	int _s = s;
            switch ( s ) {
                    case 0 : 
                        int LA84_1 = input.LA(1);

                         
                        int index84_1 = input.index();
                        input.rewind();
                        s = -1;
                        if ( (synpred129_JFSL()) ) {s = 31;}

                        else if ( (true) ) {s = 2;}

                         
                        input.seek(index84_1);
                        if ( s>=0 ) return s;
                        break;
            }
            if (state.backtracking>0) {state.failed=true; return -1;}
            NoViableAltException nvae =
                new NoViableAltException(getDescription(), 84, _s, input);
            error(nvae);
            throw nvae;
        }
    }
    static final String DFA87_eotS =
        "\16\uffff";
    static final String DFA87_eofS =
        "\4\uffff\1\12\11\uffff";
    static final String DFA87_minS =
        "\1\u008c\2\171\1\uffff\1\u008c\1\171\10\uffff";
    static final String DFA87_maxS =
        "\1\u00b3\1\u00ce\1\u00d5\1\uffff\1\u00c6\1\u00ce\10\uffff";
    static final String DFA87_acceptS =
        "\3\uffff\1\11\2\uffff\1\4\1\3\1\10\1\7\1\2\1\1\1\5\1\6";
    static final String DFA87_specialS =
        "\16\uffff}>";
    static final String[] DFA87_transitionS = {
            "\1\1\5\uffff\1\3\40\uffff\1\2",
            "\1\4\26\uffff\1\6\5\uffff\1\6\20\uffff\1\5\3\uffff\10\7\7\uffff"+
            "\1\6\11\uffff\1\6\2\uffff\10\6",
            "\7\11\20\uffff\1\11\5\uffff\1\11\21\uffff\13\11\1\uffff\1\10"+
            "\2\uffff\1\11\2\uffff\1\11\4\uffff\3\11\2\uffff\1\11\2\uffff"+
            "\17\11",
            "",
            "\1\12\4\uffff\4\12\1\uffff\1\13\1\12\11\uffff\3\12\3\uffff"+
            "\1\12\13\uffff\16\12\1\uffff\5\12",
            "\1\14\26\uffff\1\15\5\uffff\1\15\43\uffff\1\15\11\uffff\1\15"+
            "\2\uffff\10\15",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            ""
    };

    static final short[] DFA87_eot = DFA.unpackEncodedString(DFA87_eotS);
    static final short[] DFA87_eof = DFA.unpackEncodedString(DFA87_eofS);
    static final char[] DFA87_min = DFA.unpackEncodedStringToUnsignedChars(DFA87_minS);
    static final char[] DFA87_max = DFA.unpackEncodedStringToUnsignedChars(DFA87_maxS);
    static final short[] DFA87_accept = DFA.unpackEncodedString(DFA87_acceptS);
    static final short[] DFA87_special = DFA.unpackEncodedString(DFA87_specialS);
    static final short[][] DFA87_transition;

    static {
        int numStates = DFA87_transitionS.length;
        DFA87_transition = new short[numStates][];
        for (int i=0; i<numStates; i++) {
            DFA87_transition[i] = DFA.unpackEncodedString(DFA87_transitionS[i]);
        }
    }

    class DFA87 extends DFA {

        public DFA87(BaseRecognizer recognizer) {
            this.recognizer = recognizer;
            this.decisionNumber = 87;
            this.eot = DFA87_eot;
            this.eof = DFA87_eof;
            this.min = DFA87_min;
            this.max = DFA87_max;
            this.accept = DFA87_accept;
            this.special = DFA87_special;
            this.transition = DFA87_transition;
        }
        public String getDescription() {
            return "787:1: selector : ( '.' Identifier arguments -> ^( CALL Identifier arguments ) | '.' Identifier -> ^( JOIN ^( IDENTIFIER Identifier ) ) | '.' primitiveType -> ^( JOIN primitiveType ) | '.' common -> ^( JOIN common ) | '.' '*' Identifier -> ^( JOIN_REFLEXIVE ^( IDENTIFIER Identifier ) ) | '.' '*' common -> ^( JOIN_REFLEXIVE common ) | '[' expression ']' -> ^( BRACKET expression ) | '[' ']' -> ^( BRACKET ) | '<' DecimalLiteral ( ',' DecimalLiteral )* '>' -> ^( PROJECTION ( DecimalLiteral )+ ) );";
        }
    }
 

    public static final BitSet FOLLOW_expression_in_clause785 = new BitSet(new long[]{0x0000000000000000L});
    public static final BitSet FOLLOW_EOF_in_clause787 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_declaration_in_specField794 = new BitSet(new long[]{0x0000000000000000L});
    public static final BitSet FOLLOW_EOF_in_specField796 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_frame_in_modifies803 = new BitSet(new long[]{0x0000000000000000L});
    public static final BitSet FOLLOW_EOF_in_modifies805 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_138_in_compilationUnit825 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_packageName_in_compilationUnit827 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000000800L});
    public static final BitSet FOLLOW_139_in_compilationUnit829 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x000000000000E000L});
    public static final BitSet FOLLOW_importDeclaration_in_compilationUnit835 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x000000000000E000L});
    public static final BitSet FOLLOW_typeDeclaration_in_compilationUnit841 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000000800L});
    public static final BitSet FOLLOW_139_in_compilationUnit843 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000000800L});
    public static final BitSet FOLLOW_EOF_in_compilationUnit850 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_Identifier_in_packageName875 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000001000L});
    public static final BitSet FOLLOW_140_in_packageName878 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_Identifier_in_packageName880 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000001000L});
    public static final BitSet FOLLOW_141_in_importDeclaration908 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_packageName_in_importDeclaration910 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000000800L});
    public static final BitSet FOLLOW_139_in_importDeclaration912 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000002000L});
    public static final BitSet FOLLOW_142_in_typeDeclaration939 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_143_in_typeDeclaration943 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_Identifier_in_typeDeclaration946 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000050000L});
    public static final BitSet FOLLOW_typeParameters_in_typeDeclaration948 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000010000L});
    public static final BitSet FOLLOW_144_in_typeDeclaration955 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L,0x000000000122E800L});
    public static final BitSet FOLLOW_typeBodyDeclaration_in_typeDeclaration957 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L,0x000000000122E800L});
    public static final BitSet FOLLOW_145_in_typeDeclaration960 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_146_in_typeParameters991 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_Identifier_in_typeParameters993 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000180000L});
    public static final BitSet FOLLOW_147_in_typeParameters996 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_Identifier_in_typeParameters998 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000180000L});
    public static final BitSet FOLLOW_148_in_typeParameters1002 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_146_in_typeParameters21024 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x0407FF0000410000L,0x0000000000007F90L});
    public static final BitSet FOLLOW_primary_in_typeParameters21026 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000180000L});
    public static final BitSet FOLLOW_147_in_typeParameters21029 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x0407FF0000410000L,0x0000000000007F90L});
    public static final BitSet FOLLOW_primary_in_typeParameters21031 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000180000L});
    public static final BitSet FOLLOW_148_in_typeParameters21035 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_139_in_typeBodyDeclaration1053 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_typeDeclaration_in_typeBodyDeclaration1060 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_149_in_typeBodyDeclaration1067 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000400000L});
    public static final BitSet FOLLOW_150_in_typeBodyDeclaration1069 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_expression_in_typeBodyDeclaration1071 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000800000L});
    public static final BitSet FOLLOW_151_in_typeBodyDeclaration1073 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_152_in_typeBodyDeclaration1087 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000400000L});
    public static final BitSet FOLLOW_150_in_typeBodyDeclaration1089 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L,0x0000000180000000L});
    public static final BitSet FOLLOW_declaration_in_typeBodyDeclaration1091 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000800000L});
    public static final BitSet FOLLOW_151_in_typeBodyDeclaration1093 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_Identifier_in_typeBodyDeclaration1107 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000400000L});
    public static final BitSet FOLLOW_150_in_typeBodyDeclaration1109 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L,0x0007F80000800000L});
    public static final BitSet FOLLOW_methodParameters_in_typeBodyDeclaration1111 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000800000L});
    public static final BitSet FOLLOW_151_in_typeBodyDeclaration1113 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000010000L});
    public static final BitSet FOLLOW_144_in_typeBodyDeclaration1115 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x000000007E020800L});
    public static final BitSet FOLLOW_specCase_in_typeBodyDeclaration1117 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000020800L});
    public static final BitSet FOLLOW_139_in_typeBodyDeclaration1120 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x000000007E020800L});
    public static final BitSet FOLLOW_specCase_in_typeBodyDeclaration1122 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000020800L});
    public static final BitSet FOLLOW_139_in_typeBodyDeclaration1127 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000020800L});
    public static final BitSet FOLLOW_145_in_typeBodyDeclaration1131 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_type_in_methodParameters1169 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_Identifier_in_methodParameters1171 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000080000L});
    public static final BitSet FOLLOW_147_in_methodParameters1174 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L,0x0007F80000000000L});
    public static final BitSet FOLLOW_type_in_methodParameters1176 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_Identifier_in_methodParameters1178 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000080000L});
    public static final BitSet FOLLOW_153_in_specCase1214 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000400000L});
    public static final BitSet FOLLOW_150_in_specCase1216 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_expression_in_specCase1222 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000800000L});
    public static final BitSet FOLLOW_151_in_specCase1224 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x000000001C000000L});
    public static final BitSet FOLLOW_154_in_specCase1235 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000400000L});
    public static final BitSet FOLLOW_150_in_specCase1237 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_expression_in_specCase1243 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000800000L});
    public static final BitSet FOLLOW_151_in_specCase1245 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000010000000L});
    public static final BitSet FOLLOW_155_in_specCase1250 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000400000L});
    public static final BitSet FOLLOW_150_in_specCase1252 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_expression_in_specCase1258 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000800000L});
    public static final BitSet FOLLOW_151_in_specCase1260 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000010000000L});
    public static final BitSet FOLLOW_156_in_specCase1271 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000400000L});
    public static final BitSet FOLLOW_150_in_specCase1273 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L,0x0400000000C10000L,0x0000000000007F90L});
    public static final BitSet FOLLOW_frame_in_specCase1279 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000800000L});
    public static final BitSet FOLLOW_151_in_specCase1281 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_157_in_specCase1332 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_158_in_specCase1347 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_159_in_declaration1371 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_160_in_declaration1375 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_Identifier_in_declaration1379 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000200000000L});
    public static final BitSet FOLLOW_161_in_declaration1381 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF6000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_declarationMult_in_declaration1383 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_additiveExpression_in_declaration1385 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000400000000L});
    public static final BitSet FOLLOW_162_in_declaration1387 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L,0x0400000800410000L,0x0000000000007F90L});
    public static final BitSet FOLLOW_frame_in_declaration1389 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_163_in_declaration1392 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_expression_in_declaration1394 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_159_in_declaration1426 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_160_in_declaration1430 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_Identifier_in_declaration1434 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000200000000L});
    public static final BitSet FOLLOW_161_in_declaration1436 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF6000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_declarationMult_in_declaration1438 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_additiveExpression_in_declaration1440 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_163_in_declaration1443 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_expression_in_declaration1445 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_164_in_quantIdMod1485 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_setDeclOp_in_declarationMult1512 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_165_in_declarationMult1518 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_166_in_declarationMult1528 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_storeRef_in_frame1559 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000080000L});
    public static final BitSet FOLLOW_147_in_frame1562 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L,0x0400000000490000L,0x0000000000007F90L});
    public static final BitSet FOLLOW_storeRef_in_frame1564 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000080000L});
    public static final BitSet FOLLOW_storePrimary_in_storeRef1589 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0008000000041000L});
    public static final BitSet FOLLOW_storeSelectors_in_storeRef1591 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000C00000000L});
    public static final BitSet FOLLOW_163_in_storeRef1594 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_expression_in_storeRef1598 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000400000000L});
    public static final BitSet FOLLOW_162_in_storeRef1603 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_expression_in_storeRef1607 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_Identifier_in_storePrimary1660 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_common_in_storePrimary1675 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_selector_in_storeSelectors1690 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0008000000041000L});
    public static final BitSet FOLLOW_storeWildCard_in_storeSelectors1693 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_selector_in_storeSelectors1700 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0008000000041000L});
    public static final BitSet FOLLOW_140_in_storeWildCard1721 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000008000000000L});
    public static final BitSet FOLLOW_167_in_storeWildCard1723 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_168_in_keywordLiteral1744 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_169_in_keywordLiteral1756 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_170_in_keywordLiteral1767 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_integerLiteral_in_literal1791 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_FloatingPointLiteral_in_literal1797 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_CharacterLiteral_in_literal1803 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_StringLiteral_in_literal1809 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_keywordLiteral_in_literal1815 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_set_in_integerLiteral0 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_171_in_primitiveType1869 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_172_in_primitiveType1878 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_173_in_primitiveType1888 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_174_in_primitiveType1900 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_175_in_primitiveType1911 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_176_in_primitiveType1922 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_177_in_primitiveType1932 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_178_in_primitiveType1943 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_Identifier_in_typeName1960 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000041000L});
    public static final BitSet FOLLOW_140_in_typeName1963 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_Identifier_in_typeName1965 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000041000L});
    public static final BitSet FOLLOW_typeParameters_in_typeName1969 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_primitiveType_in_typeDisambiguous2006 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0008000000000000L});
    public static final BitSet FOLLOW_179_in_typeDisambiguous2015 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0010000000000000L});
    public static final BitSet FOLLOW_180_in_typeDisambiguous2017 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0008000000000000L});
    public static final BitSet FOLLOW_typeName_in_typeDisambiguous2036 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0008000000000000L});
    public static final BitSet FOLLOW_179_in_typeDisambiguous2045 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0010000000000000L});
    public static final BitSet FOLLOW_180_in_typeDisambiguous2047 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0008000000000000L});
    public static final BitSet FOLLOW_typeDisambiguous_in_type2072 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_typeName_in_type2078 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_150_in_parExpression2112 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_expression_in_parExpression2115 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000800000L});
    public static final BitSet FOLLOW_151_in_parExpression2117 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_expression_in_expressionList2147 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000080000L});
    public static final BitSet FOLLOW_147_in_expressionList2150 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_expression_in_expressionList2153 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000080000L});
    public static final BitSet FOLLOW_150_in_arguments2174 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000C10000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_expressionList_in_arguments2176 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000800000L});
    public static final BitSet FOLLOW_151_in_arguments2179 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_conditionalExpression_in_expression2203 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_quantifiedExpression_in_conditionalExpression2220 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0020000000000000L});
    public static final BitSet FOLLOW_181_in_conditionalExpression2233 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_expression_in_conditionalExpression2235 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000200000000L});
    public static final BitSet FOLLOW_161_in_conditionalExpression2237 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_expression_in_conditionalExpression2239 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_setQuantOp_in_quantifiedExpression2275 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_decls_in_quantifiedExpression2277 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_163_in_quantifiedExpression2280 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_expression_in_quantifiedExpression2282 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_setQuantOp_in_quantifiedExpression2300 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L,0x0000001000000000L});
    public static final BitSet FOLLOW_mdecls_in_quantifiedExpression2302 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_163_in_quantifiedExpression2304 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_expression_in_quantifiedExpression2306 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_logicalExpression_in_quantifiedExpression2324 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_conditionalOrExpression_in_logicalExpression2340 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0040000000040000L});
    public static final BitSet FOLLOW_logicalOp_in_logicalExpression2353 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_conditionalOrExpression_in_logicalExpression2355 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_146_in_logicalOp2392 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0040000000000000L});
    public static final BitSet FOLLOW_182_in_logicalOp2394 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000100000L});
    public static final BitSet FOLLOW_148_in_logicalOp2396 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_146_in_logicalOp2404 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0080000000000000L});
    public static final BitSet FOLLOW_183_in_logicalOp2406 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000100000L});
    public static final BitSet FOLLOW_148_in_logicalOp2408 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_182_in_logicalOp2416 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000100000L});
    public static final BitSet FOLLOW_148_in_logicalOp2418 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_conditionalAndExpression_in_conditionalOrExpression2438 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0100000000000000L});
    public static final BitSet FOLLOW_184_in_conditionalOrExpression2453 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_conditionalAndExpression_in_conditionalOrExpression2457 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0100000000000000L});
    public static final BitSet FOLLOW_inclusiveOrExpression_in_conditionalAndExpression2494 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_185_in_conditionalAndExpression2509 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_inclusiveOrExpression_in_conditionalAndExpression2513 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_exclusiveOrExpression_in_inclusiveOrExpression2555 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_163_in_inclusiveOrExpression2570 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_exclusiveOrExpression_in_inclusiveOrExpression2574 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_andExpression_in_exclusiveOrExpression2611 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0400000000000000L});
    public static final BitSet FOLLOW_186_in_exclusiveOrExpression2626 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_andExpression_in_exclusiveOrExpression2630 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0400000000000000L});
    public static final BitSet FOLLOW_equalityExpression_in_andExpression2669 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0800000000000000L});
    public static final BitSet FOLLOW_bitAndOp_in_andExpression2684 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_equalityExpression_in_andExpression2688 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0800000000000000L});
    public static final BitSet FOLLOW_187_in_bitAndOp2725 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_instanceOfExpression_in_equalityExpression2747 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x00C0000000000000L});
    public static final BitSet FOLLOW_equalityOp_in_equalityExpression2764 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_instanceOfExpression_in_equalityExpression2768 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x00C0000000000000L});
    public static final BitSet FOLLOW_182_in_equalityOp2809 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0040000000000000L});
    public static final BitSet FOLLOW_182_in_equalityOp2811 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_183_in_equalityOp2819 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0040000000000000L});
    public static final BitSet FOLLOW_182_in_equalityOp2821 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_182_in_equalityOp2829 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_relationalExpression_in_instanceOfExpression2851 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x1000000000000000L});
    public static final BitSet FOLLOW_188_in_instanceOfExpression2866 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L,0x0007F80000000000L});
    public static final BitSet FOLLOW_type_in_instanceOfExpression2870 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_setUnaryExpression_in_relationalExpression2909 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x2080000000140000L});
    public static final BitSet FOLLOW_relationalOp_in_relationalExpression2926 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_setUnaryExpression_in_relationalExpression2930 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x2080000000140000L});
    public static final BitSet FOLLOW_189_in_inOp2969 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_146_in_relationalOp2988 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0040000000000000L});
    public static final BitSet FOLLOW_182_in_relationalOp2990 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_148_in_relationalOp2998 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0040000000000000L});
    public static final BitSet FOLLOW_182_in_relationalOp3000 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_146_in_relationalOp3008 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_148_in_relationalOp3016 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_inOp_in_relationalOp3024 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_183_in_relationalOp3028 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x2000000000000000L});
    public static final BitSet FOLLOW_189_in_relationalOp3030 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_setUnaryOp_in_setUnaryExpression3049 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_joinExpression_in_setUnaryExpression3051 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_shiftExpression_in_setUnaryExpression3067 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_rangeExpression_in_shiftExpression3085 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000140000L});
    public static final BitSet FOLLOW_shiftOp_in_shiftExpression3102 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_rangeExpression_in_shiftExpression3106 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000140000L});
    public static final BitSet FOLLOW_146_in_shiftOp3143 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000040000L});
    public static final BitSet FOLLOW_146_in_shiftOp3145 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_148_in_shiftOp3153 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000100000L});
    public static final BitSet FOLLOW_148_in_shiftOp3155 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000100000L});
    public static final BitSet FOLLOW_148_in_shiftOp3157 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_148_in_shiftOp3165 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000100000L});
    public static final BitSet FOLLOW_148_in_shiftOp3167 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_additiveExpression_in_rangeExpression3187 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x4000000000000000L});
    public static final BitSet FOLLOW_190_in_rangeExpression3199 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_additiveExpression_in_rangeExpression3203 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x4000000000000000L});
    public static final BitSet FOLLOW_sizeExpression_in_additiveExpression3241 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x8000000000000000L,0x0000000000000001L});
    public static final BitSet FOLLOW_additiveOp_in_additiveExpression3259 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_sizeExpression_in_additiveExpression3263 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x8000000000000000L,0x0000000000000001L});
    public static final BitSet FOLLOW_191_in_additiveOp3304 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_192_in_additiveOp3317 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_193_in_sizeExpression3336 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_joinExpression_in_sizeExpression3338 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_multiplicativeExpression_in_sizeExpression3354 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_setAdditiveExpression_in_multiplicativeExpression3372 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000008000000000L,0x000000000000000CL});
    public static final BitSet FOLLOW_multOp_in_multiplicativeExpression3389 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_setAdditiveExpression_in_multiplicativeExpression3393 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000008000000000L,0x000000000000000CL});
    public static final BitSet FOLLOW_167_in_multOp3423 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_194_in_multOp3436 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_195_in_multOp3449 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_overrideExpression_in_setAdditiveExpression3472 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000000000L,0x0000000000000010L});
    public static final BitSet FOLLOW_setAdditiveOp_in_setAdditiveExpression3492 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_overrideExpression_in_setAdditiveExpression3496 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000000000L,0x0000000000000010L});
    public static final BitSet FOLLOW_196_in_setAdditiveOp3538 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x8000000000000000L});
    public static final BitSet FOLLOW_191_in_setAdditiveOp3540 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_196_in_setAdditiveOp3551 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000000000L,0x0000000000000001L});
    public static final BitSet FOLLOW_192_in_setAdditiveOp3553 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_intersectionExpression_in_overrideExpression3576 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000000000L,0x0000000000000020L});
    public static final BitSet FOLLOW_197_in_overrideExpression3591 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_intersectionExpression_in_overrideExpression3595 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000000000L,0x0000000000000020L});
    public static final BitSet FOLLOW_composeExpression_in_intersectionExpression3639 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000000000L,0x0000000000000010L});
    public static final BitSet FOLLOW_196_in_intersectionExpression3654 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0800000000000000L});
    public static final BitSet FOLLOW_187_in_intersectionExpression3656 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_composeExpression_in_intersectionExpression3660 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000000000L,0x0000000000000010L});
    public static final BitSet FOLLOW_unaryExpression_in_composeExpression3700 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000000000L,0x0000000000000040L});
    public static final BitSet FOLLOW_198_in_composeExpression3715 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_unaryExpression_in_composeExpression3719 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000000000L,0x0000000000000040L});
    public static final BitSet FOLLOW_191_in_unaryExpression3762 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_unaryExpression_in_unaryExpression3764 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_192_in_unaryExpression3780 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_unaryExpression_in_unaryExpression3782 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_unaryExpressionNotPlusMinus_in_unaryExpression3798 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_199_in_unaryExpressionNotPlusMinus3811 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_unaryExpression_in_unaryExpressionNotPlusMinus3813 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_183_in_unaryExpressionNotPlusMinus3829 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_unaryExpression_in_unaryExpressionNotPlusMinus3831 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_castExpression_in_unaryExpressionNotPlusMinus3851 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_joinExpression_in_unaryExpressionNotPlusMinus3857 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_150_in_castExpression3877 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0007F80000000000L});
    public static final BitSet FOLLOW_primitiveType_in_castExpression3879 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000800000L});
    public static final BitSet FOLLOW_151_in_castExpression3881 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_unaryExpression_in_castExpression3883 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_Identifier_in_decls3909 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x2000000200000000L});
    public static final BitSet FOLLOW_inOp_in_decls3912 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF6000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_161_in_decls3916 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF6000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_declarationMult_in_decls3919 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_rangeExpression_in_decls3921 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000080000L});
    public static final BitSet FOLLOW_147_in_decls3925 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_Identifier_in_decls3928 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x2000000200000000L});
    public static final BitSet FOLLOW_inOp_in_decls3931 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF6000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_161_in_decls3935 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF6000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_declarationMult_in_decls3938 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_rangeExpression_in_decls3940 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000080000L});
    public static final BitSet FOLLOW_quantIdMod_in_mdecls3983 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_Identifier_in_mdecls3985 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x2000000200080000L});
    public static final BitSet FOLLOW_147_in_mdecls3988 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_Identifier_in_mdecls3990 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x2000000200080000L});
    public static final BitSet FOLLOW_inOp_in_mdecls3995 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF6000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_161_in_mdecls3999 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF6000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_declarationMult_in_mdecls4002 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_rangeExpression_in_mdecls4004 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_specUnaryOp_in_relationalUnaryExpression4043 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000400000L});
    public static final BitSet FOLLOW_parExpression_in_relationalUnaryExpression4045 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_specUnaryOp_in_relationalUnaryExpression4061 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_Identifier_in_relationalUnaryExpression4063 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_primary_in_joinExpression4096 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0008000000041000L});
    public static final BitSet FOLLOW_selector_in_joinExpression4098 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0008000000041000L});
    public static final BitSet FOLLOW_parExpression_in_common4127 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_200_in_common4133 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_201_in_common4143 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_202_in_common4154 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_203_in_common4166 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_204_in_common4177 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_205_in_common4192 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000400000L});
    public static final BitSet FOLLOW_150_in_common4194 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_expression_in_common4196 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000800000L});
    public static final BitSet FOLLOW_151_in_common4198 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_206_in_common4212 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000400000L});
    public static final BitSet FOLLOW_150_in_common4214 = new BitSet(new long[]{0x0000000000000000L,0xE000000000000000L});
    public static final BitSet FOLLOW_integerLiteral_in_common4216 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000800000L});
    public static final BitSet FOLLOW_151_in_common4218 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_relationalUnaryExpression_in_common4232 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_144_in_common4238 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_decls_in_common4240 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_163_in_common4242 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_expression_in_common4244 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000020000L});
    public static final BitSet FOLLOW_145_in_common4246 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_144_in_common4264 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_rangeExpression_in_common4266 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x00000000000A0000L});
    public static final BitSet FOLLOW_147_in_common4269 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_rangeExpression_in_common4271 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x00000000000A0000L});
    public static final BitSet FOLLOW_145_in_common4275 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_common_in_primary4299 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_literal_in_primary4305 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_typeDisambiguous_in_primary4311 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_Identifier_in_primary4317 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000041000L});
    public static final BitSet FOLLOW_140_in_primary4320 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_Identifier_in_primary4322 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000041000L});
    public static final BitSet FOLLOW_typeParameters2_in_primary4326 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_typeName_in_primary4353 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000000000L,0x0000000000000010L});
    public static final BitSet FOLLOW_196_in_primary4355 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_Identifier_in_primary4357 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_Identifier_in_primary4377 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000400000L});
    public static final BitSet FOLLOW_arguments_in_primary4379 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_140_in_selector4405 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_Identifier_in_selector4407 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000400000L});
    public static final BitSet FOLLOW_arguments_in_selector4409 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_140_in_selector4425 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_Identifier_in_selector4427 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_140_in_selector4445 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0007F80000000000L});
    public static final BitSet FOLLOW_primitiveType_in_selector4447 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_140_in_selector4461 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0400000000410000L,0x0000000000007F90L});
    public static final BitSet FOLLOW_common_in_selector4463 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_140_in_selector4477 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000008000000000L});
    public static final BitSet FOLLOW_167_in_selector4479 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_Identifier_in_selector4481 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_140_in_selector4499 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000008000000000L});
    public static final BitSet FOLLOW_167_in_selector4501 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0400000000410000L,0x0000000000007F90L});
    public static final BitSet FOLLOW_common_in_selector4503 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_179_in_selector4517 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_expression_in_selector4519 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0010000000000000L});
    public static final BitSet FOLLOW_180_in_selector4521 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_179_in_selector4537 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0010000000000000L});
    public static final BitSet FOLLOW_180_in_selector4539 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_146_in_selector4552 = new BitSet(new long[]{0x0000000000000000L,0x8000000000000000L});
    public static final BitSet FOLLOW_DecimalLiteral_in_selector4554 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000180000L});
    public static final BitSet FOLLOW_147_in_selector4557 = new BitSet(new long[]{0x0000000000000000L,0x8000000000000000L});
    public static final BitSet FOLLOW_DecimalLiteral_in_selector4559 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000180000L});
    public static final BitSet FOLLOW_148_in_selector4563 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_196_in_specUnaryOp4587 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0400000000000000L});
    public static final BitSet FOLLOW_186_in_specUnaryOp4589 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_186_in_specUnaryOp4593 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_196_in_specUnaryOp4605 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000000000000L,0x0000000000000080L});
    public static final BitSet FOLLOW_199_in_specUnaryOp4607 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_199_in_specUnaryOp4611 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_207_in_setDeclOp4636 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_208_in_setDeclOp4648 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_209_in_setDeclOp4660 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_setDeclOp_in_setUnaryOp4683 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_210_in_setUnaryOp4689 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_211_in_setUnaryOp4700 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_setUnaryOp_in_setQuantOp4722 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_193_in_setQuantOp4728 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_212_in_setQuantOp4739 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_213_in_setQuantOp4752 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_139_in_synpred13_JFSL1120 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x000000007E000000L});
    public static final BitSet FOLLOW_specCase_in_synpred13_JFSL1122 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_set_in_synpred26_JFSL1370 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_Identifier_in_synpred26_JFSL1379 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000200000000L});
    public static final BitSet FOLLOW_161_in_synpred26_JFSL1381 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF6000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_declarationMult_in_synpred26_JFSL1383 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_additiveExpression_in_synpred26_JFSL1385 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000400000000L});
    public static final BitSet FOLLOW_162_in_synpred26_JFSL1387 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L,0x0400000800410000L,0x0000000000007F90L});
    public static final BitSet FOLLOW_frame_in_synpred26_JFSL1389 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_163_in_synpred26_JFSL1392 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_expression_in_synpred26_JFSL1394 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_163_in_synpred36_JFSL1594 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_expression_in_synpred36_JFSL1598 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_selector_in_synpred40_JFSL1690 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0008000000041000L});
    public static final BitSet FOLLOW_storeWildCard_in_synpred40_JFSL1693 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_179_in_synpred59_JFSL2015 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0010000000000000L});
    public static final BitSet FOLLOW_180_in_synpred59_JFSL2017 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_179_in_synpred61_JFSL2045 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0010000000000000L});
    public static final BitSet FOLLOW_180_in_synpred61_JFSL2047 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_181_in_synpred65_JFSL2233 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_expression_in_synpred65_JFSL2235 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000200000000L});
    public static final BitSet FOLLOW_161_in_synpred65_JFSL2237 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_expression_in_synpred65_JFSL2239 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_setQuantOp_in_synpred66_JFSL2275 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_decls_in_synpred66_JFSL2277 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_163_in_synpred66_JFSL2280 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_expression_in_synpred66_JFSL2282 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_setQuantOp_in_synpred67_JFSL2300 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L,0x0000001000000000L});
    public static final BitSet FOLLOW_mdecls_in_synpred67_JFSL2302 = new BitSet(new long[]{0x0000000000000000L,0x0000000000000000L,0x0000000800000000L});
    public static final BitSet FOLLOW_163_in_synpred67_JFSL2304 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_expression_in_synpred67_JFSL2306 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_163_in_synpred73_JFSL2570 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_exclusiveOrExpression_in_synpred73_JFSL2574 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_199_in_synpred104_JFSL3811 = new BitSet(new long[]{0x0000000000000000L,0xFE00000000000000L,0x8487FF0000410000L,0x00000000003FFF93L});
    public static final BitSet FOLLOW_unaryExpression_in_synpred104_JFSL3813 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_castExpression_in_synpred106_JFSL3851 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_selector_in_synpred113_JFSL4098 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_typeDisambiguous_in_synpred127_JFSL4311 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_140_in_synpred128_JFSL4320 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_Identifier_in_synpred128_JFSL4322 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_typeParameters2_in_synpred129_JFSL4326 = new BitSet(new long[]{0x0000000000000002L});
    public static final BitSet FOLLOW_Identifier_in_synpred130_JFSL4317 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000041000L});
    public static final BitSet FOLLOW_140_in_synpred130_JFSL4320 = new BitSet(new long[]{0x0000000000000000L,0x0200000000000000L});
    public static final BitSet FOLLOW_Identifier_in_synpred130_JFSL4322 = new BitSet(new long[]{0x0000000000000002L,0x0000000000000000L,0x0000000000041000L});
    public static final BitSet FOLLOW_typeParameters2_in_synpred130_JFSL4326 = new BitSet(new long[]{0x0000000000000002L});

}